/*
 *  $Revision: 1.10 $
 *
 *  Definitions of the macro user interface.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


/*
 * Global variable storing the filename of the last macro executed or
 * recorded.  It is accessed from the vcMacPanel facility as well.
 */
LastMacro := "";


function SetLastMacro(new_filename)
    LastMacro = new_filename;

/*
 * Dynamic routine for the `mexec' command--the macro name from the `macro'
 * command is stored in a global variable `LastMacro'.  It is then passed on
 * to the `mexec' command as a default value for the macro name parameter.
 */
function GetLastMacro(d)
    LastMacro;


/*
 * Checks if the specified macro filename has already an extension.  If it
 * does, then it's reused.  Otherwise, the "vmc" extension is appended and the
 * macro file is reconstructed.  Note that the vscript.vsParse returns a
 * four-element list: device, pathname, filename and extension.
 */
function parse_macro_filename(macro)
{
    declare parse_list, ext;

    if (!null (parse_list = vscript.vsParse (macro)))
    {
        if (string_eq ("", (ext = cadddr (parse_list))))
            vscript.vsMerge (car (parse_list), cadr (parse_list),
                             caddr (parse_list), "vmc");
        else
            vscript.vsMerge (car (parse_list), cadr (parse_list),
                             caddr (parse_list), ext);
    }
    else
        vlEx.Throw2 (vlEx.FAILURE, "vcMac", vlErrSev.Error,
                     sprintf ("Unable to parse this macro name - %s", macro));
}


/*
 * Check routine for the "macro" command, which causes a confirmation
 * dialog to be put up if the specified macro file already exists.
 */
function check_mop(d, event, macro, units, origin)
{
    parsed_macro := parse_macro_filename (macro);
    filename     := vscript.vsGetini (parsed_macro);

    if (null (filename))        /* couldn't find macro */
        FALSE;
    else
        sprintf ("Macro file %s already exists and will be overwritten.",
                 filename);
}


/*
 * Action routine for the "macro" command which starts macro recording.
 */
function mop(d, event, macro, units, origin)
{
    LastMacro     = macro;
    parsed_macro := parse_macro_filename (macro);
    filename     := vscript.vsGetini (parsed_macro);

    if (filename == nil)
        filename = parsed_macro;

    vcMac.Start (filename, event, units, origin);
}


/*
 * If the user specifies a macro file that already exists, CmdWithConfirm
 * will get a message back from check_mop, and the user will get a
 * confirmation dialog.
 */
macro_cmd := vcWv.CmdWithConfirm (fac, mop, check_mop, FALSE,
                                  "macro", "Begin Macro Recording",
                       vcWv.Param ("Macro name", vlFileWriteVT,
                                   vcWv.Sticky, vcWv.NoDefault,
                                   filter <- "*.vmc"),
                       vcWv.Param ("Coordinate units", vcMacUserPhysVT,
                                   vcWv.Sticky, "User", vcWv.Secondary,
                                   "Pixels or facility specific units"),
                       vcWv.Param ("Origin point", vcMacAbsRelVT,
                                   vcWv.Sticky, "Abs", vcWv.Secondary,
                               "Lower left corner or current mouse position"));
vcCmd.EnableMacro (macro_cmd, vlOnOff.Off);
vcCmd.SetHelp (macro_cmd, "Start macro recording");


/*
 * Action routine for the `mclose' command.  Note that the vcMac.Close
 * function is always called.  In this way, if the user tries to close a macro
 * that hasn't been started yet, an appropriate alert box will be popped by
 * this function.
 */
function mcl(data, event)
    vcMac.Close (vcEvent.Window (event));

mclose_cmd := vcWv.Cmd (fac, mcl, "mclose");
vcCmd.EnableMacro (mclose_cmd, vlOnOff.Off);
vcCmd.SetHelp (mclose_cmd, "End macro recording");


/*
 * Action routine for the `mexec' command.  Note that the macro name is
 * saved in the LastMacro variable as a default for the `macro' command.
 */
function mex(d, e, filename)
{
    vcMac.Exec (filename, e);
    LastMacro = filename;
}
mexecute_cmd := vcWv.Cmd (fac, mex, "mexecute", "Execute Macro",
                          vcWv.Param ("Macro name:", vlStringVT,
                                      vcWv.Dynamic, GetLastMacro,
                                      vcFilePop.fac, filter <- "*.vmc"));
vcCmd.SetHelp (mexecute_cmd, "Replay a macro");


/*
 * The stop command allows one to terminate macro playback in midstream, so to
 * speak.  Comes in very handy in cross-process macros when network goes
 * berserk and macro system waits forever for a replay.
 */
mstop_cmd := vcWv.Cmd (fac, func(d, e) vcMac.StopExec ();, "mstop");
vcCmd.SetHelp (mstop_cmd, "Cancel macro playback");


vcBind.Set (vcWv.bindings,  '\_(',      macro_cmd);
vcBind.Set (vcWv.bindings,  '\_)',      mclose_cmd);
vcBind.Set (vcWv.bindings,  '\_\^E',    mexecute_cmd);
vcBind.Set (vcWv.bindings,  '\_\^S',    mstop_cmd);
