/*
 *  $Revision: 1.47 $
 *
 *  Viewscript UI definitions for the Core vcPref facility.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

function fac_has_pref(fac) {
    foreach (name, help, type, val, deflt, changed;
             vlPref.UserOptions ((vcFac.Package (fac)).prefs))
        return (TRUE);
    FALSE;
} /* function fac_has_pref */

package vcFacSelector(_Create, _Get) {
    function build_facility_list() {
        foreach (fac; vcFac.Loop ())
            if (fac_has_pref (fac))
                fac_list = cons (fac, fac_list);
        fac_list = reverse_destructive (fac_list);
        n_facs = length (fac_list);
    } /* function build_facility_list */

    fac_list := nil;
    n_facs := 0;

    function _Create(deflt, region) {
        if (fac_list == nil) {
            build_facility_list ();
            build_facility_list = nil;
        } /* if */

        p := vcRegion.Create ("", vcPicklist.fac, nil,
                              columns <- 5,
                              rows <- (n_facs + 4) / 5,
                              exclusive <- TRUE);

        i := 0;
        foreach (fac; vscript.list_iter (fac_list)) {
            vcPicklist.Add (p, vcFac.Name (fac), fac);
            if (fac == deflt)
                vcPicklist.SetPick (p, i);
            ++i;
        } /* foreach */
        vcRegion.AddChild (region, p);
        p;
    } /* function _Create */

    function _Get(p, region, ts)
        vcRegion.Get (p, ts);
} /* package vcFacSelector */

vcFac.CreateFromPackage ("vcFacSelector", vscmVT, vcFacSelector);

function do_facility(p, e, the_fac) {
    if (p.modified == nil) {
        set_fac (p, the_fac, (vcFac.Package (the_fac)).prefs);
        vcWin.SetTitle (vcRegion.Window (p.table),
                        "Preference Editor: " + vcFac.Name (p.fac));
    } /* if */
    else
        CreateWindow (p.window, the_fac);
} /* function do_facility */

fac_cmd := vcWv.Cmd (fac, do_facility, "facility", "Set Facility",
                     vcWv.Param ("", vcFacilityVT, vcWv.Dynamic,
                                 func(p) p.fac;, vcFacSelector.fac));
/*
 * Enable/disable commands at different times throughout execution.
 */
function enable_cmds(p, window) {
    if (window == nil)
        window = vcRegion.Window (p.region);
    vcCmd.Enable (apply_cmd, p.enable_apply);
    vcCmd.Enable (apply_all_cmd, p.enable_apply_all);
    vcCmd.Enable (install_cmd, p.enable_install);
    vcCmd.Enable (defaults_cmd, p.enable_defaults);
    vcCmd.Enable (restore_cmd, p.enable_restore);
    if (window != nil)
        vcWin.UpdateMenus (window);
} /* function enable_cmds */

function EnableCmds(pref_region, window)
    enable_cmds (vcRegion.GetData (pref_region), window);

/*
 * Every time we get FocusIn, update the command list.  This should
 * happen here and not _Initial because if the there's more than once
 * preference window up, they each may require a different set of
 * enabled commands.
 */
function _FocusIn(p, event)
    enable_cmds (p, (event == nil) ? nil : vcEvent.Window (event));
_Initial := _FocusIn;

                           
function do_apply(p, event) {
    option_num := 0;
    something_changed := FALSE;
    foreach (name, help, type, val, deflt, changed;
             p.option_iterator (p.prefs))
    {
        if (is_modified (p, name)) {
            something_changed = TRUE;
            vlPref.Setval (p.prefs, name,
                           vcRegion.Get (vcTable.Get (p.table, option_num, 2),
                                         type));
        } /* if */
        else if (changed)
            something_changed = TRUE;

        ++option_num;
    } /* if */
    p.enable_apply = FALSE;
    p.enable_restore = FALSE;
    p.enable_defaults = something_changed;
    _FocusIn (p, event);
    if (something_changed && (p.reset != nil))
        update_fac (p, p.window);
    p.modified = nil;
    something_changed;
} /* function do_apply */

apply_cmd := vcWv.Cmd (fac, do_apply, "apply");
vcCmd.SetHelp (apply_cmd,
               "Try the preference changes in the current window");

function _Get(p, pref_region, type) {
    option_num := 0;
    something_changed := FALSE;
    modified_prefs := nil;

    foreach (name, help, type, val, deflt, changed;
             p.option_iterator (p.prefs))
    {
        if (changed || is_modified (p, name))
            modified_prefs =
                cons (cons (string2symbol (name),
                            vcRegion.Get (vcTable.Get (p.table, option_num, 2),
                                          type)),
                      modified_prefs);
        ++option_num;
    } /* foreach */
    modified_prefs;
} /* function _Get */

function _Set(p, option_list) {
    option_num := 0;
    new_modified := nil;

    foreach (name, help, type, val, deflt, changed;
             p.option_iterator (p.prefs))
    {
        option := assq (string2symbol (name), option_list);
        if (option) {
            vcRegion.Set (vcTable.Get (p.table, option_num, 2),
                          convert (type, cdr (option)));
            new_modified = cons (name, new_modified);
        } /* if */
        else if (changed || is_modified (p, name))
            vcRegion.Set (vcTable.Get (p.table, option_num, 2), val);
        ++option_num;
    } /* foreach */
    p.modified = new_modified;
    enable_cmds (p, vcRegion.Window (p.table));
} /* function _Set */

function update_fac(p, w) {
    foreach (r; vcRegion.Children (vcWin.Region (w), TRUE)) {
        if (vcRegion.Fac (r) == p.fac) {
            data := vcRegion.Data (r, vcFac.Typespec (p.fac));
            if (p.reset != nil)
                p.reset (data, r, p.prefs, p.modified);
        } /* if */
    } /* if */
} /* function update_fac */

function do_apply_all(p, event) {
    if (do_apply (p, event) && (p.reset != nil)) {
        foreach (w; vcWin.Loop ())
            if (w != p.window)
                update_fac (p, w);
    } /* if */
    p.enable_apply_all = FALSE;
    _FocusIn (p, event);
} /* function do_apply_all */

apply_all_cmd := vcWv.Cmd (fac, do_apply_all, "apply_all");
vcCmd.SetHelp (apply_all_cmd,
               "Try the preference changes in all windows");

function do_defaults(p, event) {                 
    option_num := 0;
    something_changed := FALSE;
    foreach (name, help, type, val, deflt, changed; p.option_iterator (p.prefs))
    {
        if (changed) {
            /*print (changed);*/
            vlPref.Revert (p.prefs, name);
            something_changed = TRUE;
            p.modified = cons (name, p.modified);
        } /* if */
        save := p.modified; p.modified = nil;
        vcRegion.Set (vcTable.Get (p.table, option_num, 2), deflt);
        p.modified = save;
        ++option_num;
    } /* foreach */
    /*printf ("something_changed=%s\n", something_changed);*/
    p.enable_defaults = FALSE;
    p.enable_restore = FALSE;
    p.enable_install = TRUE;
    p.enable_apply = TRUE;
    p.enable_apply_all = TRUE;
    _FocusIn (p, event);
    if (something_changed)
        update_fac (p, p.window);
    p.modified = nil;
} /* function do_defaults */

defaults_cmd := vcWv.Cmd (fac, do_defaults, "defaults");
vcCmd.SetHelp (defaults_cmd,
               "Restore the preferences to their default values");

function do_restore(p, event) {
    option_num := 0;
    something_changed := FALSE;
    foreach (name, help, type, val, deflt, changed; p.option_iterator (p.prefs))
    {
        if (changed)
            something_changed = TRUE;
        if (is_modified (p, name))
            vcRegion.Set (vcTable.Get (p.table, option_num, 2), val);
        ++option_num;
    } /* foreach */
    p.enable_apply = FALSE;
    p.enable_apply_all = FALSE;
    p.enable_restore = FALSE;
    p.enable_defaults = something_changed;
    _FocusIn (p, event);
    p.modified = nil;
} /* function do_restore */

restore_cmd := vcWv.Cmd (fac, do_restore, "restore");
vcCmd.SetHelp (restore_cmd,
               "Restore the preferences to their previous values");



function do_install(p, event) {
    do_apply (p, event);
    fname := "powerview.ini";
    private_ini_file := FALSE;
    if (isbound (`IniFileName, vcFac.Package (p.fac))) {
        private_ini_file = TRUE;
        fname = (vcFac.Package (p.fac)).IniFileName;
    } /* if */
    path := vscript.vsGetini (fname);
    if (path == nil)
        path = fname;
    if (!vlPref.WriteIni (p.prefs, path, !private_ini_file)) {
        path = fname;
        include_package := !private_ini_file ||
            (p.prefs != (vcFac.Package (p.fac)).prefs);
        if (!vlPref.WriteIni (p.prefs, fname, include_package))
            vlEx.Throw2 (vlEx.FAILURE, "vcPref", vlErrSev.Error,
                         sprintf ("Could not write preferences to %s", fname));
    } /* if */
    p.enable_install = FALSE;
    _FocusIn (p, event);
    p.is_modified = FALSE;
    w := vcEvent.Window (event);
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, "Installed preferences in file " + path);
} /* function do_install */

install_cmd := vcWv.Cmd (fac, do_install, "install");
vcCmd.SetHelp (install_cmd,
               "Permanently install the changes by writing a new version of the initialization file");

function is_modified(p, name) {
    ret := FALSE;
    for (m := p.modified; (m != nil) && !ret; m = cdr (m))
        if (string_eq (car (m), name))
            ret = TRUE;
    ret;
} /* function is_modified */

function _Modified(p, region)
    p.is_modified;

/*
 * Constrain the default size of a region to a specific pixel width.
 * If the region is happy to be smaller, then let it be smaller.  If
 * it wants to be bigger, set its ideal size as specified in the
 * parameter to this size, but allow it to expand on a resize.
 * This is sort of painful to implement due to the lack of callbacks
 * for manipulating size structures.
 */
function constrain(r, width) {
    size := vgSize.Alloc ();
    vcRegion.Size (r, size);

    ideal_x := vgSize.GetIdealWidth (size);
    if (ideal_x > width) {
       vgSize.SetIdealWidth (size, width);
    }
    vcRegion.PrefSet (r, list (size <- size));

    r;
} /* function constrain */

function label(str, font) {
    r := vcRegion.Create (str, vcStaticText.fac, str);
    vcRegion.Font (r, font);
    r;
} /* function stattext */

function set_fac(p, fac, prefs) {
    p.fac = fac;
    p.prefs = prefs;
    if (isbound (`_ResetPrefs, vcFac.Package (p.fac)))
        p.reset = (vcFac.Package (p.fac))._ResetPrefs;
    else if (isbound (`_Reset, vcFac.Package (p.fac)))
    {
        pkg := vcFac.Package(p.fac);
        funk := pkg._Reset;
        p.reset = func(d, r, p, mod) funk(d, r);;
    }
    else
        p.reset = nil;

    n_options := 0;

    foreach (name, help, type, val, deflt, changed; p.option_iterator (p.prefs))
        ++n_options;
    
    p.table = vcTable.Create ("", n_options, 4);

/*
 *     foreach (col; vscript.list_iter (list (2, 3)))
 *         vcTable.Col (p.table, col, vcTable.Border.Adjustable, 3, 5, 7,
 *                     vgAlign.LowerLeft);
 */

    p.modified = nil;
    p.enable_apply    = FALSE;
    p.enable_apply_all= FALSE;
    p.enable_install  = FALSE;
    p.enable_defaults = FALSE;
    p.enable_restore  = FALSE;

    function add_modified(name) {
        if (p.modified == nil) {
            p.enable_apply      = TRUE;
            p.enable_apply_all  = TRUE;
            p.enable_install    = TRUE;
            p.enable_defaults   = TRUE;
            p.enable_restore    = TRUE;
            _FocusIn (p, nil);
            p.is_modified = TRUE;
        } /* if */
        if (!is_modified (p, name))
            p.modified = cons (name, p.modified);
    } /* function add_modified */

    declare splunge;
/*    function label(str) vcRegion.Create (str, vcLabel.fac, str); */
    foreach (name, help, type, val, deflt, changed; p.option_iterator (p.prefs)) {
        if (changed)
            p.enable_defaults = TRUE;
        vcTable.Add (p.table, label (name, vgFont.small));
        vcTable.Add (p.table, constrain (label (deflt, vgFont.smallb), 150));
        ts_editor := create_editor (p.prefs, name, type, val, add_modified);
        splunge := constrain (ts_editor, 300);
        vcTable.Add (p.table, splunge);
        vcTable.Add (p.table, label (help, vgFont.smalli));
    } /* foreach */

    if (n_options >= p.scroll_threshold)
        scr := vcTable.Scroll ("scr", p.table, p.table, 3, 8, n_options);
    else
        scr := vcRegion.Create ("", vcBorder.fac, p.table, rounded <- FALSE);
    vcTable.Set (p.big_table, 1, 0, scr);
    vcTable.AlignColumns (list (p.headers, p.table));
} /* function set_fac */

iterator all_prefs(p) {
    foreach (name, help, type, val, deflt, changed; vlPref.Options (p))
    {
        if ((help[0] == '~') && (help[1] == '~'))
            help = substring (help, 2, 0);
        iterate (name, help, type, val, deflt, changed);
    } /* foreach */
} /* iterator all_prefs */

function _Create(win_fac_prefs, region) {
    function l(str) label (str, vgFont.mediumb);

    p := struct {
        prefs = nil;
        table = nil;
        modified = nil;
        window = car (win_fac_prefs);
        region = region;
        fac = nil;
        reset = nil;
        big_table = nil;
        is_modified = FALSE;
        headers = vcTable.Make ("", 1, 4,
                           l ("Name"), l ("Default"), l ("Value"), l ("Help"));
        option_iterator = vlPref.UserOptions;
        scroll_threshold = vcRegion.PrefGet (region, "scroll_threshold");
        enable_apply    = FALSE;
        enable_apply_all= FALSE;
        enable_install  = FALSE;
        enable_defaults = FALSE;
        enable_restore  = FALSE};

    if (vcRegion.PrefGet (region, "all_prefs")) 
        p.option_iterator = all_prefs;
        
    p.big_table = vcTable.Make ("", 2, 1, p.headers);
    set_fac (p, cadr (win_fac_prefs), caddr (win_fac_prefs));
    vcRegion.AddChild (region, p.big_table);
    p;
} /* function _Create */

vlPref.Replace (prefs, "all_prefs", vlTrueFalseVT,
                "~~Show the programmer preferences in the Preference Editor, as well as the user preferencess",
                FALSE);
vlPref.Replace (prefs, "scroll_threshold", vlUnsignedIntVT,
                "~~Number of options below which we don't add a scrollbar",
                8);

menu := vcWv.MakeMenu ("Preference",
                        vcWin.RedSquareEntryNoPlotNoQuit,
                        list ("Set Facility", fac_cmd),
                        list ("Apply", apply_cmd),
                        list ("Apply All", apply_all_cmd),
                        list ("Defaults", defaults_cmd),
                        list ("Restore", restore_cmd),
                        list ("Install", install_cmd));
vcFac.SetMenus (fac, menu);

/*
 * Like CreateWindow, but takes an auxiliary preference database
 * instead of using the facility's default one.
 */
function CreateWindowAux(w, fac, prefs) {
    name := vcFac.Name (fac);
    r := vcRegion.Create (name, vcPref.fac, list (w, fac, prefs));
    w = vcWin.Create (w, r);
    vcWin.SetTitle (w, "Preference Editor: " + name);

    vcWin.SetConfirmMessage (w, "
 You have changed the preferences without saving 
 them permanently to the appropriate initialization
 file. You may continue and discard
 your changes, or you may cancel this dialog box, press 
 the Install button, and then dismiss the window.
");
} /* function CreateWindow */

/*
 * This function is called by Prefs, from vcWin.vs.
 */
function CreateWindow(w, fac) {
    if (!fac_has_pref (fac))
        fac = vcWv.fac;
    CreateWindowAux (w, fac, (vcFac.Package (fac)).prefs);
} /* function CreateWindow */

if (!isbound (`pref_editor_tables, current_package ())) {
    pref_editor_tables := vlSym.ObjCreate (11);
    vlSym.Dynamic (pref_editor_tables, 2, 2);
} /* if */

function SetEditor(args...) {
    prefs       := car (args);         args = cdr (args);
    option_name := car (args);         args = cdr (args);

    /* Now args contains (fac pref1 pref2 ...) */

    /* Get the table for this preference database */
    table := vlSym.Fetch (pref_editor_tables, prefs);

    /* If there is none, set one up. */
    if (table == nil) {
        table = vlSym.StrCreate (11);
        vlSym.Dynamic (pref_editor_tables, 2, 2);
        vlSym.Store (pref_editor_tables, prefs, table);
    } /* if */

    vlSym.Store (table, option_name, args);
} /* function SetEditor */

function create_editor(prefs, option_name, type, val, add_modified) {
    table := vlSym.Fetch (pref_editor_tables, prefs);
    declare fac, args, env;
    if (table != nil) {
        fac_and_args := vlSym.Fetch (table, option_name);
        if (fac_and_args != nil) {
            fac = car (fac_and_args);
            args = cdr (fac_and_args);
            env = vcFac.Package (fac);
        } /* if */
    } /* if */
    if (fac == nil) {
        env = vcCmd.TsToFac (type);
        fac = env.fac;
        args = nil;
    } /* if */

    editor := vcRegion.Create1 (option_name, fac, val, args);
    env.CallOnChange (editor, func(val) add_modified (option_name););
    editor;
} /* function create_editor */
