/*
 *      Viewlogic Proprietary Information
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 */

/*
 * $Log:	vdrc.vs,v $
Revision 1.33  94/03/29  16:17:25  josh
Assume spinname_works, because were at 5.2 now, and the test to see if
it works causes an error when run from the powerview process.

Revision 1.32  94/02/10  14:36:54  josh
Previous fix was for bug #12456

Revision 1.31  94/02/10  14:34:58  josh
Don't mutate a string to create "'", do it directly.  The direct
method didn't work due to a parsing problem in 5.1.  The mutating
version won't work in 5.2 because literal strings are constant.

Revision 1.30  93/12/13  15:42:56  josh
Use new ITC mechanism so everything works on the PC.

Revision 1.29  93/11/03  12:26:34  josh
_Modified takes the drc, not the region.

Revision 1.28  93/10/04  15:04:21  josh
Conditionalize a few things so this Viewscript file can be used
with 5.1.2 Viewdraw as well as 5.2.

Revision 1.27  93/09/25  02:02:49  josh
Commit to vcErrList already!
Hedge on the new ipc (variable use_new_itc, now FALSE), some code
inside ifs.
Assume viewdraw's spinname works now.
Use vsUnique instead of rolling our own filenames...
Deal with new preference pop_viewdraw, propogated to vcErrList

Revision 1.26  93/03/18  15:38:17  josh
When an error is selected, put it in the X cut buffer.

Revision 1.25  93/03/17  08:48:54  josh
Clean up the .ini file, which seems to have duplicate entries
in it after callling vlPref.WriteIniFile.

Revision 1.24  93/03/17  01:08:07  josh
Added unique_filename().

Revision 1.23  93/03/11  17:21:30  josh
Use vcWin.WithBusy so the user knows somethings happenin'

Revision 1.22  93/03/08  21:36:14  josh
Don't attempt to delete the VDRC_ attribute if the error was disabled.
It will mostly fail and pop up an alert box.

Revision 1.21  93/03/08  14:44:36  josh
Ignore stderr when invoking vdrc.

Revision 1.20  93/03/06  14:09:21  josh
Implement ShowDisabled and PcbFwd

Revision 1.19  93/03/04  19:48:59  josh
Use vcSetOrder.fac to edit the sort_order preference

Revision 1.18  93/03/01  13:28:00  josh
Disable the "Describe error" command when there is nothing selected.

Revision 1.17  93/03/01  13:23:00  josh
Add in vcErrList stuff

Revision 1.16  93/02/25  12:54:06  josh
Make the default size of the text window 80 columns.
Add in help.
Make a real close-window message.

Revision 1.15  93/02/22  20:32:58  josh
Non-macro version of add_attr.

Revision 1.14  93/02/01  11:04:08  josh
Re-arrange error menu entries

Revision 1.13  93/01/28  13:19:27  josh
Fix csh-quotify to make up for viewscript parsing bug.

Revision 1.12  93/01/08  19:52:55  josh
Update csh_quotify to use single-quote, which is a better quoting
mechanism than double-quote.

Revision 1.11  93/01/05  23:57:07  josh
Fix some spacing problems.

Revision 1.10  93/01/05  21:08:45  josh
Invoke vdrc with the -ipc switch which is no longer the default.

Revision 1.9  93/01/02  14:34:57  josh
Handle attribute creation on pins differently -- put the attribute
on the component with an embedded string identifying the pin.

Revision 1.8  92/12/30  16:23:49  josh
Make drcupdate take the design as an argument.

Revision 1.7  92/12/29  14:49:21  josh
Add disable-event code, which creates a Viewdraw macro to add an
attribute to the offending object.

Revision 1.6  92/12/17  17:30:50  josh
Specify vdrc.ini as the ini filename

Revision 1.5  92/12/16  18:10:32  josh
Major changes to implement standalone mode.  See prior version to
get server mode implementation.

 */

use_new_itc := TRUE;

if (!use_new_itc) {
    drcs := nil;

    function find_drc(design) {
        drc := nil;
        foreach (d; vscript.list_iter (drcs))
            if (string_eqi (d.the_design, design))
                drc = d;
        drc;
    } /* function find_drc */

    function drc_update(vdraw_data, ev, design, fname, notes, warnings, errors)
    {
        drc := find_drc (design);
        if (drc != nil) {
            msg := sprintf ("%s: DRC reported %d notes, %d warnings, %d errors\n",
                            vcWv.Time (), notes, warnings, errors);
            w := vcRegion.Window (drc.errs);
            if (w != nil) {
                vcWin.ClearStatus (vcEvent.Window (ev));
                vcWin.AddStatus (vcEvent.Window (ev), msg);
            } /* if */
            vcWin.WithBusy (vcEvent.Window (ev), func()
                            vcErrList.Update (drc.errs););
            drc.error_selected = FALSE;
            _Enter (drc, nil);
        } /* if */
    } /* function drc_update */

    vcWv.Cmd (vdraw.fac, drc_update,
              "drcupdate", "Drc Update",
              vcWv.Param ("Design", vlStringVT, vcWv.Sticky, ""),
              vcWv.Param ("Filename", vlStringVT, vcWv.Sticky, ""),
              vcWv.Param ("# Notes", vlIntVT, vcWv.Sticky, 0),
              vcWv.Param ("# Warnings", vlIntVT, vcWv.Sticky, 0),
              vcWv.Param ("# Errors", vlIntVT, vcWv.Sticky, 0));
} /* if */

/*
 * Takes a string and hack it so that it will pass thru csh's argument
 * parsing unmodified.
 */
function csh_quotify(str) {
    need_to_quote := FALSE;

    foreach (c; vscript.list_iter (string2list ("\\ \"'$!|^")))
        if (lindex (str, c)) {
            need_to_quote = TRUE;
            break;
        } /* if */

    if (need_to_quote) {
        q := "'"; 
        out_str := q;
        declare i;

        while (i = lindex (str, '\'')) {
            out_str += substring (str, 0, i + 1) + "\\''";
            str = substring (str, i + 1, 0);
        } /* while */

        out_str + str + q;
    } /* if */
    else
        str;
} /* function csh_quotify */

if (use_new_itc)
    declare vdraw_context;

pop_viewdraw_feature := isbound (`CompatibleVersion, vcWv);

function _Create(the_design, region) {
    /*
     * Check for altered preferences that may not have
     * been installed, and send them through the command line.
     */
    flags := "";
    foreach (name, help, type, val, deflt, changed; vlPref.Options (prefs)) {
        if (changed) {
            if ((type == vlYesNoVT) || (type == vlOnOffVT) ||
                (type == vlTrueFalseVT))
                flags = sprintf ("%s %c%s", flags, val ? '+' : '-', name);
            else
                flags = sprintf ("%s -%s %s", flags, name,
                                 csh_quotify (convert (vlStringVT, val)));
        } /* if */
    } /* foreach */

    foreach (name, help, type, val, deflt, changed;
             vlPref.Options (error_prefs))
        if (changed)
            flags = sprintf ("%s -vdrcError.%s %s",
                             flags, name, convert (vlStringVT, val));

    if (!use_new_itc)
        flags += " -ipc";

    drc := struct {
        the_design = the_design;
        sh = vcRegion.Create (vcRegion.Name (region) + "_sh",
                              vcShell.fac, "vdrc " + the_design + flags,
                              height <- 10);
        errs = vcRegion.Create ("drc_errors",
                                vcErrList.fac, the_design + ".drc",
                                macro_on <- TRUE);
        disabled_errors = FALSE;
        error_selected = FALSE;
        ini_file_dirty = FALSE;
        context = nil;
        handler = nil
    };

    if (pop_viewdraw_feature)
        vcRegion.PrefSet (drc.errs, list (pop_viewdraw <-
                                          vcRegion.PrefGet (region,
                                                            "pop_viewdraw")));

    if (use_new_itc) {
        if (vdraw_context == nil)
            vdraw_context = vitc.CntxCreate (vipc.GidDFLT, vipc.UidDFLT,
                                             vipc.FidDFLT, vitc.TidVDRAW,
                                             vitc.NameDFLT, vipc.HnmANY,
                                             vipc.PidANY, vipc.CidANY);

        drc.context = vitc.SrcCntxCreate (vitc.TidONLINE_DRC, the_design);
        function sheet_changed(mtype, src_cntx, mtch_cntx, data) {
            declare design_alias, design_name, sch_sym_flag, block_alias;
            declare block_name, sheet_number, check_succeeded, design_path;
            p := data;

            function get(var, do_check) {
                i := lindex (p, '\n');
                if (i) {
                    *var = (i == 0) ? "" : substring (p, 0, i);
                    p = substring (p, i + 1, 0);
                } /* if */
                else {
                    *var = p;
                    p = nil;
                    if (do_check)
                        error ("Invalid sheet changed msg", data);
                } /* else */
            } /* function get */

            get (&design_path, TRUE);
            get (&design_alias, TRUE);
            get (&design_name, TRUE);
            get (&sch_sym_flag, TRUE);
            get (&block_alias, TRUE);
            get (&block_name, TRUE);
            get (&sheet_number, TRUE);
            get (&check_succeeded, FALSE);

            is_schematic := string_eq (sch_sym_flag, "0");
            if (!is_schematic || string_eq (check_succeeded, "1")) {
                name := string_eq (block_alias, "") ? block_name :
                    block_alias + ":" + block_name;
                cmd := sprintf ("%s %s %d",
                                is_schematic ? "sheet" : "symbol",
                                string_upper (name),
                                convert (vlIntVT, sheet_number));
                send_drc (drc, cmd);
            } /* if */
        } /* function sheet_changed */

        drc.handler = vitc.Register (vitc.MsgSHEET_CHANGED, drc.context,
                                     vdraw_context, sheet_changed);

        sys := vcShell.Sys (drc.sh);
        vcSys.LineBuffer (sys);
        PATTERN := " written to ";
        PATTERN_LENGTH := string_length (PATTERN);
        vcSys.CallOnOutput (sys, func(str) {
            index := string_search (str, PATTERN);
            if (index != nil) {
                vcWin.WithBusy (vcRegion.Window (region), func()
                                vcErrList.Update (drc.errs););
                drc.error_selected = FALSE;
                _Enter (drc, nil);
            } /* if */
        });
    } /* if */

    vcErrList.Notify (drc.errs, func(e, on_off) {
        drc.error_selected = on_off;
        _Enter (drc, nil);

        catch (vlEx.FAILURE) {
            declare design, type, path;
            string_token (string_token (string_token (vlErr.Location (e),
                                                      &design, " "),
                                        &type, " "),
                          &path, " ");
            vcRegion.StoreCutBuffer (drc.errs, path);
            w := vcRegion.Window (drc.sh);
            vcWin.ClearStatus (w);
            vcWin.AddStatus (w, "Select/Cut Buffer: " + path);
        } /* catch */
        exception
            ;
    });

/*
    butns := vcRegion.Create ("butns", vcButnRow.fac,
                              vcErrList._ButtonRowEntries (),
                              macro_on <- vcRegion.MacroOn (region));
*/

    t := vcTable.Make ("", 2, 1, drc.errs, drc.sh);
    vcTable.Row (t, 1, vcTable.Border.Adjustable, 5, 7, 9,
                 vgAlign.MiddleCenter);
    vcRegion.AddChild (region, t);

    if (!use_new_itc)
        drcs = cons (drc, drcs);
    drc;
} /* function _Create */

function _Destroy(drc) {
    if (use_new_itc)
        vitc.Unregister (drc.handler);
    else
        drcs = vscript.delete_list_element (drcs, drc);
} /* function _Destroy */

function send_drc(drc, str)
    vcShell.TypeIn (drc.sh, str, TRUE);

/*
 * Maximum line-length to send to subprocess, to make things look cleaner
 * and to workaround some line-length limitations in Viewtext.
 */
MAX_LINE := 78;

function send_prefs_to_subprocess(drc, p, modified_list) {
    cmd := "";
    cmd_len := 0;
    sent_to_drc := FALSE;
    declare new, new_len;
    if (modified_list != nil) {
        foreach (name; vscript.list_iter (modified_list))
            if ((ui_pref_list == nil) || !member (name, ui_pref_list)) {
                new = sprintf ("; set_%s %s %s",
                               (p == prefs) ? "pref" : "err",
                               name,
                               convert (vlStringVT, vlPref.Val (p, name)));
                new_len = string_length (new);
                if ((cmd_len > 1) && ((new_len + cmd_len) > MAX_LINE)) {
                    send_drc (drc, substring (cmd, 2, 0));
                    sent_to_drc = TRUE;
                    cmd_len = new_len;
                    cmd = new;
                } /* if */
                else {
                    cmd += new;
                    cmd_len += new_len + 2;
                } /* else */
            } /* if */
        if (cmd_len > 1) {
            sent_to_drc = TRUE;
            send_drc (drc, substring (cmd, 2, 0));
        } /* if */
    } /* if */
    sent_to_drc;
} /* function send_prefs_to_subprocess */

function _ResetPrefs(drc, region, p, modified_list) {
    if (modified_list != nil) {
        if (send_prefs_to_subprocess (drc, p, modified_list))
            recheck (drc, nil);
        if (member ("pop_viewdraw", modified_list))
            vcRegion.PrefSet (drc.errs,
                              list (pop_viewdraw <-
                                    vlPref.Val (p, "pop_viewdraw")));
    } /* if */
} /* function _ResetPrefs */

function recheck(drc, event) {
    send_drc (drc, "recheck");
    drc.disabled_errors = FALSE;
    _Enter (drc, event);
} /* function recheck */

recheck_cmd := vcWv.Cmd (fac, recheck, "recheck");
vcCmd.SetHelp (recheck_cmd,
               "Re-invoke the DRC, taking into account interactively disabled errors");

function _Enter(drc, event) {
    vcCmd.Enable (update_ini_cmd, drc.ini_file_dirty);
    vcCmd.Enable (recheck_cmd, drc.disabled_errors);
    vcCmd.Enable (disable_cmd, drc.error_selected);
    vcCmd.Enable (disable_all_cmd, drc.error_selected);
    vcCmd.Enable (enable_cmd, drc.error_selected);
    vcCmd.Enable (err_help_cmd, drc.error_selected);
    vcWin.UpdateMenus (vcRegion.Window (drc.sh));
} /* function _Enter */

/*
 * This list of preference names will never be sent to the DRC subprocess,
 * because they apply only to the UI.
 */
ui_pref_list := list ("pop_viewdraw");

/*
vlPref.Add (prefs, "timeout", vlPositiveIntVT,
            "Number of seconds to wait when trying to contact DRC server",
            10);
*/

vcFac.CopyPrefs (vcErrList.fac, fac);   /* for pop_viewdraw */

error_prefs := vlPref.Create ("vdrcError");

/*
 * Add the preferences from the drcsrv executable by invoking it with the
 * -prefs option, which writes Viewscript to initialize each of its
 * preferences.  We load the generated Viewscript file
 * to "inherit" those preferences from the executable,
 * so the user can edit them before "really" invoking the executable.
 */
drc_prefs := "";
vcSys.Command ("vdrc -prefs", nil,
               func(str) drc_prefs += str;,
               func(str) printf ("vdrc error: %s\n", str);,
               func(mode, code) {
                   if ((mode == vcSysMode.Exited) && (code == 0)) {
                       setup_prefs :=
                           (vcWin.EvalPrint (drc_prefs, func(str);));
                       setup_prefs (prefs, error_prefs);
                       vlPref.Alphabetize (prefs);
                   } /* if */
               },
               nil);


if (!isbound (`vcPref, nil))
    vcFac.Create ("vcPref", vscmVT);
if (!isbound (`vcSetOrder, nil))
    vcFac.Create ("vcSetOrder", vscmVT);

vcPref.SetEditor (prefs, "sort_order", vcSetOrder.fac);

IniFileName := "vdrc.ini";

/*
 * So the error-severity preferences will work...
 */
if (!isbound (`vdrcSeverityVT, nil))
    vcWv.MakeEnum ("vdrcSeverityVT",
                   list ("Note", "Warning", "Error", "Disabled"));


/*
 * Find the coordinate-string of currently selected object by
 * doing a Viewdraw Object-Detail and parsing the output, looking
 * for the first line that begins with ' ', which seems to always
 * be a coordinate string for the object.  This function returns
 * the coordinates as an ascii string.
 */
function find_coords(cmd) {
    declare buf1, buf2;
    coords := nil;

    tmpfile :=  vscript.vsUnique (vscript.vsGetdef ());
    cmd ("odetail >" + tmpfile);
    f := fopen (tmpfile, "r");
    for (s := fgetline (f); (s != nil) && (coords == nil); s = fgetline (f))
        if ((s[0] == ' ') &&
            ((s = string_token (s, &buf1, " \n")) != nil) &&
            (string_token (s, &buf2, " \n") != nil))
            coords = buf1 + " " + buf2;

    fclose (f);
    vscript.vsDelete (tmpfile, vscript.vsFILE);
    coords;
} /* function find_coords */

function get_error_attr(test, location)
    sprintf ("VDRC_%s", test);

/* If viewdraw's spinname command doesn't work, the following must be used */
if (FALSE) {
    function get_error_attr(test, location) {
        attr := sprintf ("VDRC_%s", test);

        /*
         * If the location is a component pin, we can't really
         * navigate there, so we have to put the attribute on
         * the component, changing the attribute name to include
         * the pin name.  All of this info is available in the
         * location string.
         */
        declare design, type;
        location = string_token (location, &design, " ");
        location = string_token (location, &type, " ");
        if (string_eq (type, "pin"))
            attr += "_" + substring (location, lindex (location, '.') + 1, 0);
        attr;
    } /* get_error_attr */
} /* if */

function disable_current_error(drc, event) {
    vcErrList.Apply (drc.errs, func(cmd, w, test, location) {
        coords := find_coords (cmd);
        add_attr (w, get_error_attr (test, location), coords);
        drc.disabled_errors = TRUE;
        _Enter (drc, event);
    });
} /* function disable_current_error */

disable_cmd := vcWv.Cmd (fac, disable_current_error, "disable");
vcCmd.SetHelp (disable_cmd, "Disables selected error on this particular object, using a schematic attribute");

function enable_error(drc, event) {
    test := vcErrList.CurrentTest (drc.errs);
    if (vlPref.Val (error_prefs, test) == vdrcSeverity.Disabled) {
        if (vlPref.Default (error_prefs, test) == vdrcSeverity.Disabled)
            vlPref.Setval (error_prefs, test, vdrcSeverity.Error);
        else
            vlPref.Revert (error_prefs, test);
        send_prefs_to_subprocess (drc, error_prefs, list (test));
        drc.disabled_errors = TRUE;
        drc.ini_file_dirty = TRUE;
        _Enter (drc, event);
    } /* if */
    else {
        vcErrList.Apply (drc.errs, func(cmd, w, test, location) {
            cmd ("sattr " + get_error_attr (test, location) + " selected");
            cmd ("delete");
        });
    } /* else */
} /* function enable_error */

enable_cmd := vcWv.Cmd (fac, enable_error, "enable");
vcCmd.SetHelp (enable_cmd, "Enables error");

function show_disabled(drc, event) {
    vlPref.Setval (prefs, "show_disabled", TRUE);
    send_prefs_to_subprocess (drc, prefs, list ("show_disabled"));
    recheck (drc, event);
    vlPref.Setval (prefs, "show_disabled", FALSE);
    send_prefs_to_subprocess (drc, prefs, list ("show_disabled"));
    drc.disabled_errors = TRUE;
    vcWin.ClearStatus (vcEvent.Window (event));
    vcWin.AddStatus (vcEvent.Window (event),
                     "Viewing disabled errors, run Recheck to see enabled errors");
} /* function show_disabled */

show_disabled_cmd := vcWv.Cmd (fac, show_disabled, "show_disabled");
vcCmd.SetHelp (show_disabled_cmd, "Show only the disabled errors");

function disable_all(drc, event) {
    test := vcErrList.CurrentTest (drc.errs);
    vlPref.Setval (error_prefs, test, vdrcSeverity.Disabled);
    send_prefs_to_subprocess (drc, error_prefs, list (test));
    drc.disabled_errors = TRUE;
    drc.ini_file_dirty = TRUE;
} /* function disable_all */

disable_all_cmd := vcWv.Cmd (fac, disable_all, "disable_all");
vcCmd.SetHelp (disable_all_cmd, "Disables all occurrences of selected error by changing vdrc.ini");

function cleanup_ini_file(fname) {
    if (fname != nil) {
        f := fopen (fname, "r");
        declare s;
        l := nil;
        while ((s = fgetline (f)) != nil)
            l = cons (s, l);
        fclose (f);
        l = vscript.Sort (l, string_lei);
        f = fopen (fname, "w");
        s = "";
        foreach (s2; vscript.list_iter (l)) {
            if (!string_eqi (s2, s))
                fputline (f, s2);
            s = s2;
        } /* foreach */
        fclose (f);
    } /* if */
} /* function cleanup_ini_file */

function update_ini(drc, event) {
    path := vscript.vsGetini ("vdrc.ini");
    if (path == nil)
        path = "vdrc.ini";
    vlPref.WriteIni (error_prefs, path, TRUE);
    drc.ini_file_dirty = FALSE;
    cleanup_ini_file (vscript.vsGetini ("vdrc.ini"));
} /* function write_errors_to_ini_file */

update_ini_cmd := vcWv.Cmd (fac, update_ini, "update_ini");
vcCmd.SetHelp (update_ini_cmd,
             "Write out the vdrc.ini file with the updated error severities");

function edit_err_sev(drc, event) {
    if (!isbound (`vcPref, nil))
        vcFac.Create ("vcPref", vscmVT);
    vcPref.CreateWindowAux (vcEvent.Window (event), fac, error_prefs);
} /* function edit_err_sev */
edit_err_sev_cmd := vcWv.Cmd (fac, edit_err_sev, "edit_error_severities");
vcCmd.SetHelp (edit_err_sev_cmd, "Edit error severities");

function err_help(drc, event)
    vcAlert.SpawnHelp (vcEvent.Window (event), "vdrc",
                       vcErrList.CurrentTest (drc.errs));
err_help_cmd := vcWv.Cmd (fac, err_help, "err_help");
vcCmd.SetHelp (err_help_cmd, "Show detailed information on current error");

function pcbfwd(drc, event, config_file, level, suppress, write_params) {
    save_show_output := vlPref.Val (prefs, "pcb_show_output");
    save_pcb_test := vlPref.Val (prefs, "pcb_test");
    if (!write_params) {
        save_level := vlPref.Val (prefs, "level");
        save_config_file := vlPref.Val (prefs, "pcb_config_file");
        save_suppress := vlPref.Val (prefs, "pcb_suppress");
    } /* if */

    vlPref.Setval (prefs, "pcb_test", TRUE);
    vlPref.Setval (prefs, "pcb_config_file", config_file);
    vlPref.Setval (prefs, "level", level);
    vlPref.Setval (prefs, "pcb_show_output", TRUE);
    vlPref.Setval (prefs, "pcb_suppress", suppress);
    
    send_prefs_to_subprocess (drc, prefs,
                              list ("pcb_test", "pcb_config_file", "level",
                                    "pcb_show_output", "pcb_suppress"));
    recheck (drc, event);

    vlPref.Setval (prefs, "pcb_test", save_pcb_test);
    vlPref.Setval (prefs, "pcb_show_output", save_show_output);

    if (write_params)
        update_ini (drc, event);
    else {
        vlPref.Setval (prefs, "level", save_level);
        vlPref.Setval (prefs, "pcb_config_file", save_config_file);
        vlPref.Setval (prefs, "pcb_suppress", save_suppress);
    } /* else */

    send_prefs_to_subprocess (drc, prefs,
                              list ("pcb_test", "pcb_config_file", "level",
                                    "pcb_show_output", "pcb_suppress"));
    drc.disabled_errors = TRUE;
    _Enter (drc, event);
} /* function pcbfwd */

pcbfwd_cmd := vcWv.Cmd (fac, pcbfwd, "pcbfwd", "Parameters To PCBFWD",
    vcWv.Param ("Config file", vlStringVT, vcWv.Dynamic,
                func(drc) vlPref.Val (prefs, "pcb_config_file");),
    vcWv.Param ("Levels", vlStrArrayVT, vcWv.Dynamic,
                func(drc) vlPref.Val (prefs, "level");,
                vcWv.Secondary,
                "List of level attributes to stop hierarchical descent"),
    vcWv.Param ("Suppress", vlStringVT, vcWv.Dynamic,
                func(drc) vlPref.Val (prefs, "pcb_suppress");,
                vcWv.Secondary,
                "List of pcbfwd error messages to suppress"),
    vcWv.Param ("Write Ini File", vlYesNoVT, vcWv.Sticky, vlYesNo.No,
                "Write the vdrc.ini file with the specified changes"));
                

errors_menu := vcWv.MakeMenu ("errors",
                              vcWv.Pin (),
                              list ("Describe error", err_help_cmd),
                              list ("Disable selected", disable_cmd),
                              list ("Disable all", disable_all_cmd),
                              list ("Enable error", enable_cmd),
                              list ("Show disabled", show_disabled_cmd),
                              list ("Update Ini File", update_ini_cmd),
                              list ("Edit severities", edit_err_sev_cmd));
                              

main_menu := vcWv.MakeMenu ("vdrc",
                            vcWin.RedSquareEntryNoPlot,
                            list ("Preferences", vcWin.PrefsCmd),
                            list ("Errors", errors_menu),
                            list ("Visit(next)", vcErrList.visit_next_cmd),
                            list ("Recheck", recheck_cmd),
                            list ("Pcbfwd", pcbfwd_cmd));

vcFac.SetMenus (fac, main_menu);

function _Modified(drc, region) {
    subwindow_modified := FALSE;

    confirm_msg := "";

    if (drc.ini_file_dirty)
        confirm_msg +=
"You have changed error severities without saving
vdrc.ini using the menu entry `Errors -> Update Ini File'

";

    drc_window := vcRegion.Window (region);
    foreach (w; vcWin.Loop ())
        if ((vcWin.Parent (w) == drc_window) && vcWin.IsModified (w)) {
            confirm_msg += vcWin.ConfirmMessage (w) + "\n";
            subwindow_modified = TRUE;
        } /* if */

    vcWin.SetConfirmMessage (vcRegion.Window (region), confirm_msg);
    subwindow_modified || drc.ini_file_dirty;
} /* function _Modified */

function w_run(w, cmd, x, y)
    vcWin.RunCmdline (w, cmd,
                      vcEvent.Create (x, y, ' ', cmd, w,
                                      vcWin.OwnerRegion (w), nil));

/*
 * Adds an attribute to the schematic, at the specified position, using
 * "cmd" to send commands to Viewdraw.
 */
function add_attr(w, attrib, position) {
    declare x, y;
    position = string_token (position, &x, " ");
    position = string_token (position, &y, " ");

    x = convert (integer, x);
    y = convert (integer, y);

    vgU.UserToPixel (vcRegion.Viewport (vcWin.OwnerRegion (w), nil, 0),
                     &x, &y);

    w_run (w, "_attributeobject Visible No End_Command " + attrib, x, y);
    w_run (w, "MoveExecute", x, y);
} /* function add_attr */

