/*
 *  $Revision: 1.34 $
 *
 *  ViewScript file implementing the graphic ViewScript Interpreter.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */

/*
 * Facility's create function.  Note the facility's data object: it's a
 * structure with two fields--the facility's region and the shell region.
 */
function _Create(path, region)
{
    data := struct {
        region = region;
        shell  = vcRegion.Create (vcRegion.Name (region), vcTxShell.fac, "",
                                  prompt   <- "vscript> ",
                                  macro_on <- TRUE)
        };

    vcRegion.AddChild (region, data.shell);

    vcTxShell.CallOnReturn (data.shell, func(event, str) {
        function echo_output(str)
            vcTxShell.PrintString (data.shell, str);

        len := string_length (str);
        if (len > 0) {
            last := substring (str, len - 1, 0);
            if (!string_eq (last, ";") && !string_eq (last, "}"))
                str = str + ";";
            val := vcWin.EvalPrint (str, echo_output);
            if (!vcRegion.Destroyed (data.shell))
                vcTxShell.PrintString (data.shell, sprintf ("--> %s\n", val));
        }
    });

    data;
}


/*
 * Invoke the ViewScript Debugger.
 */
function vbugger(data, event, filename)
    vcWin.Create (vcRegion.Create ("ViewScript Debugger", vcVbg.fac,
                                   filename),
                  vcWin.NoMacros);


/*
 * Write the session into a log file.
 */
function write_log(data, event, filename)
    vtx.Writeto (data.region, filename);


/*
 * Call the load function to load and evaluate a ViewScript file.  This is
 * provided mainly as a shortcut so that you don't have to type the `load'
 * function call yourself.
 */
function load_file(data, event, filename, pkg)
    vcTxShell.TypeIn (data.shell, sprintf ("load (%s, %s);", filename, pkg),
                      TRUE, event);

/*
 * Build command objects for subsequent key and menu bindings.
 */
log_cmd         := vcWv.Cmd (fac, write_log, "log", "Write Log File",
                             vcWv.Param ("Log name", vlFileWriteVT,
                                         vcWv.Sticky, vcWv.NoDefault));
vbugger_cmd     := vcWv.Cmd (fac, vbugger, "debugger",
                             "Invoke ViewScript Debugger",
                             vcWv.Param ("ViewScript file:", vlFileReadVT,
                                         vcWv.Sticky, vcWv.NoDefault,
                                         vcFilePop.fac, filter <- "*.vs"));
vcCmd.Enable (vbugger_cmd, vlOnOff.Off);
load_file_cmd   := vcWv.Cmd (fac, load_file, "load_file", "Load File",
                             vcWv.Param ("Filename:", vlFileReadVT,
                                         vcWv.Sticky, vcWv.NoDefault,
                                         vcFilePop.fac, filter <- "*.vs"),
                             vcWv.Param ("Package:", vlStringVT, vcWv.Sticky,
                                         "nil", vcWv.Secondary));

/*
 * Construct the menu line.  Note that each menu has one option, thus in
 * reality they become buttons.
 */
menus := vcWv.MakeMenu ("vscmEval",
                        vcWin.RedSquareEntryNoPlotNoQuit,
                        list ("Log session",            log_cmd),
                        list ("Load a file",            load_file_cmd),
                        list ("ViewScript Debugger",    vbugger_cmd));

vcFac.SetMenus (fac, menus);
