
/******************************************************************************/
/* Filename: vsimutil.h - Interface to Viewsim <-> C Interface utilities      */
/******************************************************************************/
/* vsimutil.c contains global definitions, arrays and functions that may be
 * generally useful to models written using the Viewsim C interface.
 * This source code module can be complied and linked in with Viewsim
 * applications or cut back to only those pieces that are needed.
 */

#ifndef __vsimutil_h__
#define __vsimutil_h__

/*  Who knows if or where TRUE and FALSE may have been defined?  Whatever, we
 * need them to improve readability.
 */
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/*  Common attribute names are defined here.  The '=' character is included
 * so that strncmp( attr, ?_NAME, strlen( ?_NAME ) ) will guarantee that the
 * attribute name is an exact match.  For example, STR012 will not match STR01.
 */
#define TPLH_NAME   "TPLH="
#define TPHL_NAME   "TPHL="
#define DLYMOD_NAME "DELAYMOD="
#define STR01_NAME  "STR01="

/*  The following array can be used to map the Viewsim 28 states into
 * the corresponding 4 state enumerated values LOW4, X, HIGHZ4, HIGH4.
 */
extern vsc_pin_4values vsc_28to4_state[];

/*  The following array can be used to map the Viewsim 28 states into
 * the corresponding 3 state enumerated values LOW4, X4, HIGH4.
 */
extern vsc_pin_4values vsc_28to3_state[];

/* The following array can be used to invert the 4 state or 3 state
 * enumerated type values.
 */
extern vsc_pin_4values vsc_4state_not[];

/* The following array can be used to display the string representation
 * of a vsc_pin_values type.
 */
extern char * vsc_28state_display[];

/* The following array can be used to display the character
 * representation of a vsc_pin_4values type.
 */
extern char vsc_4state_display[];

#ifdef vscLONGLONG
#define vsc_time64_add( val1, val2, result ) (result) = (val1) + (val2);
#define vsc_time64_sub( val1, val2, result ) (result) = (val1) - (val2);
#define vsc_time64_eq( val1, val2 ) ((val1) == (val2));
#define vsc_time64_ne( val1, val2 ) ((val1) != (val2));
#define vsc_time64_ge( val1, val2 ) ((val1) >= (val2));
#define vsc_time64_gt( val1, val2 ) ((val1) > (val2));
#define vsc_time64_le( val1, val2 ) ((val1) <= (val2));
#define vsc_time64_lt( val1, val2 ) ((val1) < (val2));
#define vsc_int_to_time64( val_int, val_time ) (val_time) = (val_int);
#define vsc_time64_add_int( val1, val2, result ) (result) = (val1) + (val2);
#define vsc_time64_sub_int( val1, val2, result ) (result) = (val1) - (val2);

#else
#define vsc_time64_add( val1, val2, result ) \
    (result).low = (val1).low + (val2).low; \
    (result).high = (val1).high + (val2).high; \
    if ((result).low < (val1).low) \
        (result).high++;

#define vsc_time64_sub( val1, val2, result ) \
    (result).low = (val1).low - (val2).low; \
    (result).high = (val1).high - (val2).high; \
    if ((result).low > (val1).low) \
        (result).high--;

#define vsc_time64_eq( val1, val2 ) \
    (((val1).low==(val2).low) && ((val1).high == (val2).high))

#define vsc_time64_ne( val1, val2 ) \
    (((val1).low!=(val2).low) || ((val1).high != (val2).high))

#define vsc_time64_ge( val1, val2 ) \
    ((((val1).low>=(val2).low) && ((val1).high == (val2).high))|| \
      ((val1).high>(val2).high))

#define vsc_time64_gt( val1, val2 ) \
    ((((val1).low>(val2).low) && ((val1).high == (val2).high))|| \
      ((val1).high>(val2).high))

#define vsc_time64_le( val1, val2 ) \
    ((((val1).low<=(val2).low) && ((val1).high == (val2).high))|| \
      ((val1).high<(val2).high))

#define vsc_time64_lt( val1, val2 ) \
    ((((val1).low<(val2).low) && ((val1).high == (val2).high))|| \
      ((val1).high<(val2).high))

#define vsc_int_to_time64( val_int, val_time ) \
    (val_time).low = val_int; \
    if ((long) val_int < 0) \
        (val_time).high = -1; \
    else (val_time).high = 0;

#define vsc_time64_add_int( val1, val2, result ) \
    { \
        vsc_time_64 tmp; \
        vsc_int_to_time64(val2, tmp); \
        vsc_time64_add(val1, tmp, result); \
    }

#define vsc_time64_sub_int( val1, val2, result ) \
    vsc_time64_add(val1, (-(val2)), result);
#endif

#endif
/* end of file vsimutil.h */
