/*
 *  $Revision: 1.18 $
 *
 *  Implements the corner and tile areas of graphics worksheet.
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 */

package vsptico (create_tico_area, create_tile_paper_area, apply_changes,
                        reset_value) {

declare xcorner_cmdline, ycorner_cmdline;
declare xtile_cmdline, ytile_cmdline;

declare cur_cornerx;
declare cur_cornery;

/* 'row col' for tile
 */
declare cur_tiler;
declare cur_tilec;

vspCORNER := "CORNER";
vspTILE := "TILE";

XPRMPT := "X> ";
YPRMPT := " Y> ";
ROW_PRMPT := "Rows> ";
COL_PRMPT := " Cols> ";

CRNR_PRMPT := "";
TILE_PRMPT := "";

CDEF := "0.5";
TDEF := "1";


tile_us_papers := list (
cons ("A", "A"),
cons ("B", "B"),
cons ("C", "C"),
cons ("D", "D"),
cons ("E", "E"),
cons ("F", "F"),
cons ("ZSIZE", "ZSIZE"));

tile_met_papers := list (
cons ("A0", "A0"),
cons ("A1", "A1"),
cons ("A2", "A2"),
cons ("A3", "A3"),
cons ("A4", "A4"),
cons ("ZSIZE", "ZSIZE"));

us_paper_dims := list (
list ("A", list (8.5, 11)),
list ("B", list (11, 17)),
list ("C", list (17, 22)),
list ("D", list (22, 34)),
list ("E", list (34, 44)),
list ("F", list (28, 40)));


met_paper_dims := list (
list ("A0", list (841, 1189)),
list ("A1", list (594, 841)),
list ("A2", list (420, 594)),
list ("A3", list (297, 420)),
list ("A4", list (210, 297)));


PLOT_RANGE := .90;  /* Percentage of paper used for plotting */
cur_paper_sel := orig_paper_sel := 0;
PAPER_PKLST_TITLE := "Tiled Output Size";
cur_pap_lst := us_paper_dims;
declare tpap_pklst;
orig_tile_disabled := FALSE;

declare c_enable_lab, c_disable_lab, t_enable_lab, t_disable_lab;
declare c_enb_dis_ovly, t_enb_dis_ovly;

declare cur_corner_ovly, cur_tile_ovly;

/* Returns the list of dimensions that corresponds to the given index.
 */
function index2dims (index) {
        for (tp := cur_pap_lst, i:=0; i != index; ++i, tp = cdr (tp));

        cadr(car(tp));
}

/* Called when the cancel button is hit in graphics worksheet.  Tiling
 * takes effect immediately because it affects the scale factor and therefore
 * needs to be cancelled and not applied.
 */
function reset_value () {
        cur_paper_sel = orig_paper_sel;
        tlst := index2dims (cur_paper_sel);
        if (tlst != nil)
                fac_pack.vspSetTilePaper (car(tlst), cadr(tlst), PLOT_RANGE);
    
        if (orig_tile_disabled)
                fac_pack.set_doing_tile (FALSE);
        else
                fac_pack.set_doing_tile (TRUE);
}

/* The enabling and disabling and changing of tile output size affects scale
 */
function reflect_updated_scale () {
        fac_pack.vspUpdateScale ();
        vspscale.set_scale_cmdline_val(fac_pack.get_def_scale (vspFIRSTVAL));
        set_graphics_modified ();
}

/* Get tiled output size for user-defined size.  Called when zsize_cmd is run.
 */
function get_zsize_info (d, e, xval, yval) {
    if ((xval <= 0) || (yval <= 0))
        vlEx.Throw2 (vlEx.FAILURE, "vsptico", vlErrSev.Error,
                     sprintf ("Bad zsize values for tiling x=%s, y=%s",
                              number2string (xval), number2string (yval)));

        fac_pack.vspSetTilePaper (xval, yval, PLOT_RANGE);
        reflect_updated_scale ();
}

/* Prompts the user for an alternate tile output size
 */
zsize_cmd := vcWv.Cmd (fac, get_zsize_info, "zsize_tile_cmd",
                       "Enter ZSIZE Dimensions",
                        vcWv.Param ("X size", float, vcWv.Sticky, 8.5),
                        vcWv.Param ("Y size", float, vcWv.Sticky, 11.0));

/* Is called when a user selects an output tile size from picklist.  If
 * the user selects zsize, a window prompts for dimensions.  Otherwise,
 * the pre-defined sizes are used and set in the software.
 */
function get_selected_paper (val, on_off) {
        if (on_off == vlOnOff.On) {
                if (string_eq (val, "ZSIZE")) {
                        /* pop up input area */
                        vcCmd.Run (zsize_cmd, nil, 
                                vcEvent.Create (0, 0, 0, nil, graphics_win_id,
                                                tpap_pklst, nil),
                                nil, TRUE);
                        return (nil);
                }

                /* This is for a pre-defined output size pick */
                /* Must match the name with the dimensions and set it in
                   the software. */
                for (tp := cur_pap_lst, i:=0; tp != nil; tp = cdr (tp), ++i) {
                        if (string_eq (caar (tp), val)) {

                                /* Save current selection */
                                cur_paper_sel = i;

                                x := car (cadr (car (tp)));
                                y := cadr (cadr (car (tp)));

                                fac_pack.vspSetTilePaper (x, y, PLOT_RANGE);
                                reflect_updated_scale ();
                        }
                }
        }
}

/* Converts a list of paper dimensions to an index into picklist
 */
function tpap_size2index (sizes) {

        for (tp := cur_pap_lst, i:= 0; tp != nil; tp = cdr (tp), ++i) {
                if ((car(cadr(car(tp))) == car(sizes)) &&
                        (cadr(cadr(car(tp))) == cadr(sizes)))
                        return (i);
        }
        /* This assumes that no match will match zsize which is always
         * the last choice in picklist.
         */
        i;
}

/* Creates the picklist for tiling output sizes.
 */
function create_tile_paper_area () {

        /* Setup the output paper size lists according to metric mode */
        if (fac_pack.vspGetMetric ()) {
                cur_pap_lst = met_paper_dims;
                pap_lst := tile_met_papers;
        }
        else {
                cur_pap_lst = us_paper_dims;
                pap_lst := tile_us_papers;
        }

        /* Save original paper selection in case of cancel */
        orig_paper_sel = tpap_size2index (fac_pack.vspGetDefTilePaper());
        tlst := create_pklst (
                        func () tpap_size2index(
                                        fac_pack.vspGetDefTilePaper());,
                        `vertical, PAPER_PKLST_TITLE, pap_lst, 
                        get_selected_paper, TRUE);
        tpap_pklst = car (tlst);

        /* Make sure that the picklist is disabled if tiling is disabled */
        if (cur_tile_ovly == t_disable_lab) {
                orig_tile_disabled = TRUE;      
                vcRegion.Enable (tpap_pklst, FALSE, TRUE);
        }
        else
                orig_tile_disabled = FALSE;

        cadr (tlst);
}

/* The following code is to establish an enable/disable overly to be used
 * with a button that is coupled with a command line (or two).  There is
 * duplicated code, one for cornering and one for tiling.
 */
function create_overlay (which) {

        if (which == `corner_cmdline) {
        c_enable_lab = vcLabel.ButtonCreate ("CEnable.but", "Enable Corner",
                                func () hit_ovly_button(`corner_cmdline);,
                                list (macro_on <- FALSE));

        c_disable_lab = vcLabel.ButtonCreate ("CDisable.but", "Disable Corner",
                                func () hit_ovly_button(`corner_cmdline);,
                                list (macro_on <- FALSE));

        c_enb_dis_ovly = vcRegion.Create1 ("c_enb_dis.overlay", vcOverlay.fac, 
                                        vgAlign.LowerLeft, nil);

        vcRegion.AddChild (c_enb_dis_ovly, c_enable_lab);
        vcRegion.AddChild (c_enb_dis_ovly, c_disable_lab);
        }
        else {
        t_enable_lab = vcLabel.ButtonCreate ("TEnable.but", "Enable Tile",
                                func () hit_ovly_button(`tile_cmdline);,
                                list (macro_on <- FALSE));

        t_disable_lab = vcLabel.ButtonCreate ("TDisable.but", "Disable Tile",
                                func () hit_ovly_button(`tile_cmdline);,
                                list (macro_on <- FALSE));

        t_enb_dis_ovly = vcRegion.Create1 ("t_enb_dis.overlay", vcOverlay.fac, 
                                        vgAlign.LowerLeft, nil);

        vcRegion.AddChild (t_enb_dis_ovly, t_enable_lab);
        vcRegion.AddChild (t_enb_dis_ovly, t_disable_lab);
        }
}

/* Called when the 'Apply' button is hit on the plot options worksheet.
 * This is when all changes are actually effected in the plotting software.
 */
function apply_changes () {

        ret1 := ret2 := TRUE;

        if (cur_corner_ovly == c_enable_lab) {
                ret1 = fac_pack.sp_sapopt (string_append(vspCORNER, " ", 
                        vcRegion.Get (xcorner_cmdline, string), " ", 
                        vcRegion.Get (ycorner_cmdline, string)));
                if (ret1)
                        fac_pack.set_doing_corner (TRUE);
        }
        else
                fac_pack.set_doing_corner (FALSE);

        if (cur_tile_ovly == t_enable_lab) {
                ret2 = fac_pack.sp_sapopt (string_append(vspTILE, " ", 
                        vcRegion.Get (xtile_cmdline, string), " ", 
                        vcRegion.Get (ytile_cmdline, string)));
                if (ret2)
                        fac_pack.set_doing_tile (TRUE);
        }
        else
                fac_pack.set_doing_tile (FALSE);

        (ret1 && ret2);

}

/* Sets one of two tile or corner command lines to a value.  The
 * xy flag indicates the cmdline that has already been set and
 * _not_ the one to be set.  This should explain why the actual
 * actions which occur appear backwards.
 */
function set_one_cmdline (tico, xy, val) {
        if (tico == `corner_cmdline) {
                if (xy == `cornerx) {
                        cur_cornery = val;
                        vcCmdline.Set (ycorner_cmdline, val);
                }
                else {
                        cur_cornerx = val;
                        vcCmdline.Set (xcorner_cmdline, val);
                }
        }       
        else {
                if (xy == `tiler) {
                        cur_tilec = val;
                        vcCmdline.Set (ytile_cmdline, val);
                }
                else {
                        cur_tiler = val;
                        vcCmdline.Set (xtile_cmdline, val);
                }
        }
}

/* Called when either the x or y input areas for the CORNER field are
 * changed by the user.
 */
function set_corner (which, val) {

        if (string_eq (val, ""))
            val = "0";
        /* Make sure got a float type */
        if (!string_eqi (val, "none")) {
            catch (vlEx.FAILURE)
                convert(float, val);
            exception {
                if (lindex (val, '.') != 0) {
                    val = "0";
                    if (which == `cornerx)
                        vcCmdline.Set (xcorner_cmdline, val);
                    else
                        vcCmdline.Set (ycorner_cmdline, val);
                }
                        
            }
            /* If user typed in value, force both to be enabled */
            if (cur_corner_ovly != c_enable_lab) {
                cur_corner_ovly = switch_ovly (`corner_cmdline, 
                                    cur_corner_ovly);
                set_one_cmdline (`corner_cmdline, which, CDEF);
            }
        }

        set_graphics_modified ();
        if (which == `cornerx)
            cur_cornerx = val;
            
        else if (which == `cornery)
            cur_cornery = val;
}

/* Called when either the row or col. input areas for the TILE field
 * is changed by the user.
 */
function set_tile (which, val) {

        if (string_eq (val, ""))
            val = "0";
        /* Make sure got a float type */
        if (!string_eqi (val, "none")) {
            catch (vlEx.FAILURE)
                convert(float, val);
            exception {
                if (lindex (val, '.') != 0) {
                    val = "0";
                    if (which == `tiler)
                        vcCmdline.Set (xtile_cmdline, val);
                    else
                        vcCmdline.Set (ytile_cmdline, val);
                }
                        
            }
            /* If user typed in value, force both to be enabled */
            if (cur_tile_ovly != t_enable_lab) {
                cur_tile_ovly = switch_ovly (`tile_cmdline, 
                                    cur_tile_ovly);
                set_one_cmdline (`tile_cmdline, which, TDEF);
            }
        }

        set_graphics_modified ();
        if (which == `tiler)
            cur_tiler = val;
        else if (which == `tilec)
            cur_tilec = val;

}

/* Switches between enable and disable overlays for either the corner or
 * tile command line areas.  Also enables and disables the tiling paper
 * picklist as appropriate.
 */
function switch_ovly (which, cur_ovly) {

        if (which == `corner_cmdline) {
                if (cur_ovly == c_enable_lab)
                        cur := c_disable_lab;
                else
                        cur := c_enable_lab;
                vcOverlay.SetCurrent (c_enb_dis_ovly, cur);
        }
        else {
                if (cur_ovly == t_enable_lab) {
                        cur := t_disable_lab;
                        fac_pack.set_doing_tile (FALSE);
                        vcRegion.Enable (tpap_pklst, FALSE, TRUE);
                }
                else {
                        cur := t_enable_lab;
                        fac_pack.set_doing_tile (TRUE);
                        vcRegion.Enable (tpap_pklst, TRUE, TRUE);
                }
                reflect_updated_scale ();
                vcOverlay.SetCurrent (t_enb_dis_ovly, cur);
        }
        cur;
}

/* Sets either the corner or tile x/y command lines to the same value.
 */
function set_cmdlines (which, str) {
        if (which == `corner_cmdline) {
                vcCmdline.Set (xcorner_cmdline, str);
                vcCmdline.Set (ycorner_cmdline, str);
                cur_cornerx = cur_cornery = str;
        }
        else {
                vcCmdline.Set (xtile_cmdline, str);
                vcCmdline.Set (ytile_cmdline, str);
                cur_tiler = cur_tilec = str;
        }
}

/* This is called when either the corner or tile overlay button is hit.  It
 * switches the value of the overlay and sets the two (x/y) associated
 * command lines to the same default value.
 */
function hit_ovly_button (which) {

/* !! NOTE: Must enable/disable the command lines when functionality is in core
 */
        if (which == `corner_cmdline) {
                ovly := cur_corner_ovly = switch_ovly(which, cur_corner_ovly);
                if (ovly == c_enable_lab)
                        set_cmdlines (which, CDEF);
                else
                        set_cmdlines (which, "none");
        }
        else {
                ovly := cur_tile_ovly = switch_ovly(which, cur_tile_ovly);
                if (ovly == t_enable_lab)
                        set_cmdlines (which, TDEF);
                else
                        set_cmdlines (which, "none");
        }

        set_graphics_modified ();
}

/* This creates a horizontal area which is comprised of an overlay (enable/
 * disable) button and two (x/y) command lines.  Tile and Corner are treated
 * separately because cross-dependency problems resulted when they were
 * dealt with more generically.
 */
function create_complex_cmdlines (which, title, prmpt1, prmpt2, def_func,
        get_val_func, xsym, ysym, do_brdr) {

        create_overlay (which);

        ttab := vcTable.Create(string_append(title,".ov_tab"), 1, 2);

        if (which == `corner_cmdline) {

                /* Create the corner overlay button */
                corner_state := fac_pack.do_corner ();
                if (corner_state)
                        cur_corner_ovly = c_enable_lab;
                else
                        cur_corner_ovly = c_disable_lab;
                vcOverlay.SetCurrent (c_enb_dis_ovly, cur_corner_ovly);

                vcTable.Add (ttab, vcRegion.Create1 (string_append(title,
                        ".but_bord"), vcBorder.fac, c_enb_dis_ovly, nil));
        }
        else {
                /* Create the tile overlay button */
                tile_state := fac_pack.do_tile ();
                if (tile_state)
                        cur_tile_ovly = t_enable_lab;
                else
                        cur_tile_ovly = t_disable_lab;

                vcOverlay.SetCurrent (t_enb_dis_ovly, cur_tile_ovly);        

                vcTable.Add (ttab, vcRegion.Create1 (string_append(title,
                        ".but_bord"), vcBorder.fac, t_enb_dis_ovly, nil));
        }

        /* Create the x/y command lines side by side */
        cmdline_lst := create_double_cmdline_list (string_append (title,
                " ", prmpt1), prmpt2, def_func (vspFIRSTVAL), 
                def_func (vspSECONDVAL), get_val_func, xsym, ysym, do_brdr);

        vcTable.Add (ttab, caddr (cmdline_lst));

        /* Return a list consisting of x cmd. line, y cmd. line and
         * entire wrapped object of button and 2 command lines.
         */
        list (cadddr (cmdline_lst), cadr (cdddr (cmdline_lst)),
                vcRegion.Create1(string_append(title,".ov_tab_brdr"),
                vcBorder.fac, ttab, nil));
}

function create_tico_area () {

        /* Get default values for all widgets */
        cur_cornerx = fac_pack.get_def_corner (vspFIRSTVAL);
        cur_cornery = fac_pack.get_def_corner (vspSECONDVAL);

        cur_tiler = fac_pack.get_def_tile (vspFIRSTVAL);
        cur_tilec = fac_pack.get_def_tile (vspSECONDVAL);

        tico_area := vcTable.Create ("tico_area.tab", 2, 1);

        /* Create and Add the 2 Corner command lines */
        tlst := create_complex_cmdlines (`corner_cmdline, 
                CRNR_PRMPT, XPRMPT,
                YPRMPT, fac_pack.get_def_corner, set_corner,
                `cornerx, `cornery, FALSE);
        xcorner_cmdline = car (tlst);
        ycorner_cmdline = cadr (tlst);
        vcCmdline.CallOnChange (xcorner_cmdline, func (str) 
                                                        set_corner (`cornerx,
                                                                str););
        vcCmdline.CallOnChange (ycorner_cmdline, func (str)
                                                        set_corner (`cornery,
                                                                str););
        vcTable.Add (tico_area, caddr (tlst));

        /* Create and Add the 2 Tile command lines */
        tlst = create_complex_cmdlines (`tile_cmdline, 
                TILE_PRMPT, ROW_PRMPT,
                COL_PRMPT, fac_pack.get_def_tile, set_tile,
                `tiler, `tilec, FALSE);
        xtile_cmdline = car (tlst);
        ytile_cmdline = cadr (tlst);
        vcCmdline.CallOnChange (xtile_cmdline, func (str) 
                                                        set_tile (`tiler,
                                                                str););
        vcCmdline.CallOnChange (ytile_cmdline, func (str) 
                                                        set_tile (`tilec,
                                                                str););
        vcTable.Add (tico_area, caddr (tlst));

        tico_area;
}
}

