/* $Revision: 1.11 $
 *
 * General utility functions for plotting worksheets
 */

vspFIRSTVAL := 1;
vspSECONDVAL := 2;
vspTHIRDVAL := 3;
vspFOURTHVAL := 4;

/* Create a toggle button using an overlay.  Returns a list of overlay
 * and button.
 */
function create_toggle (name, first_lab, second_lab, cur_state)
{
        /* Create overlay itself */
        ovly := vcRegion.Create1 (string_append (name, ".ovly"),
                        vcOverlay.fac, vgAlign.LowerLeft, nil);

        /* Add labels as children of overlay */
        vcRegion.AddChild (ovly, first_lab);
        vcRegion.AddChild (ovly, second_lab);

        vcOverlay.SetCurrent (ovly, cur_state);

        /* Return a list of overlay and the object with a border around it */
        return (list (ovly, vcRegion.Create1 (string_append (name, ".border"),
                vcBorder.fac, ovly, nil)));
}

/* Create a command line which uses 'prompt_val' as its prompt, 'def_func' as
 * the function to call to get the default value for the command line, and
 * 'get_val_func' as the function to call when the user enters a value and a
 * <cr> is hit.
 */
function create_cmdline (prompt_val, def_val, get_val_func, do_border, desired_width, max_width) {

        /* Label region for prompt */
        prompt := vcRegion.Create1 (prompt_val, vcLabel.fac, prompt_val, nil);

        /* Command line input area */
        cmdline := vcRegion.Create1(string_append(prompt_val,".cmdline"), 
                                vcCmdline.fac, def_val,
                                list(cons(`max_width, max_width),
                                        cons(`width, desired_width),
                                        cons(`clear_on_return, FALSE), 
                                        cons(`macro_on, FALSE),
                                        cons(`max_size, 256)));

        /* Set up function to call when carriage return is hit */
        vcCmdline.CallOnReturn(cmdline, get_val_func);
/*
        data := struct {
                command_line = cmdline;
                fun = get_val_func};

        vcRegion.CallOnFocusChange (cmdline, func (d, new_r) {
                                                d.fun (nil,
                                                vcRegion.Get(d.command_line,
                                                        string));
                                                }, data);
 */

        /* Tie the two regions together */
        prompt_n_line := vcTable.Create (prompt_val, 1, 2);
        vcTable.Col (prompt_n_line, -1, vcTable.Border.Invisible,
                        2, 4, 6, vgAlign.LowerLeft);
        vcTable.Add (prompt_n_line, prompt);
        vcTable.Add (prompt_n_line, cmdline);

        if (do_border) {

                /* Wrap them with a border */
                border := vcRegion.Create1(string_append(prompt_val,".border"),
                        vcBorder.fac, prompt_n_line, nil);

                /* Return the entire widget */
                return(list (cmdline, border));
        }
        else
                return(list (cmdline, prompt_n_line));
}

/* Creates two side-by-side command lines with a title, and 2 prompts.
 * It optionally draws a border around them to tie them visually together.
 * Returns the region for this object.
 */
function vspCreateDoubleCmdlines (title, left_prompt, right_prompt, def_func, 
                            set_func, xsym, ysym, def_val1, def_val2, do_bdr) {

        tab_name := string_append(title, ".cmdlines");
        reg := vcTable.Create (tab_name, 1, 2);
        vcTable.Add(reg, cadr (create_cmdline(string_append(title,left_prompt),
                                def_func (def_val1),
                                func (ev,str) set_func (xsym, str);, FALSE,
                                8, 12)));
        vcTable.Add(reg, cadr (create_cmdline (right_prompt,
                                def_func (def_val2),
                                func (ev,str) set_func (ysym, str);, FALSE,
                                8, 12)));
        if (do_bdr) {
                border := vcRegion.Create1(string_append(tab_name,".border"),
                        vcBorder.fac, reg, nil);
                return (border);
        }
        else
                return (reg);
}

/* Create two command lines to type in possible generic values.  Returns
 * a list which contains (x_cmdline, y_cmdline, xy_cmdline_table).
 */
function create_double_cmdline_list (prmpt1, prmpt2, def_val1, def_val2, 
                        get_val_func, xsim, ysim, do_brdr) {

        /* Create and fill the table which holds 2 command lines
         * side by side.
         */
        ttab := vcTable.Create (string_append(prmpt1, prmpt2, ".tab"), 1, 2);
        xcmdline_list := create_cmdline(prmpt1, def_val1, 
                        func (data, str) get_val_func (xsim, str);,
                                 FALSE, 8, 12);
        xcmdline := car (xcmdline_list);
        xcmdline_tab := cadr (xcmdline_list);

        ycmdline_list := create_cmdline(prmpt2, def_val2,
                        func (data, str) get_val_func (ysim, str);,
                                 FALSE, 8, 12);
        ycmdline := car (ycmdline_list);
        ycmdline_tab := cadr (ycmdline_list);

        vcTable.Add (ttab, xcmdline_tab);
        vcTable.Add (ttab, ycmdline_tab);

        /* Return a list of the actual x and y command line regions as
         * well as the table ties them both together.
         */
        if (do_brdr)
                tlst := list (vcTable.Get(xcmdline_tab, 0, 1), 
                        vcTable.Get(ycmdline_tab, 0, 1),
                        vcRegion.Create1(string_append(prmpt1,prmpt2,".bord"),
                                vcBorder.fac, ttab, nil), xcmdline, ycmdline);
        else
                tlst := list (vcTable.Get(xcmdline_tab, 0, 1),
                        vcTable.Get(ycmdline_tab, 0, 1),
                        ttab, xcmdline, ycmdline);
        return(tlst);
}

/* Uses core's picklist as a basis and adds a title and a border.  Dimensions
 * are based on the structure of 'vals'.  Only one function gets called when 
 * an entry is selected.  A default value is set.  When titles can be added
 * to regions, then the orientation field can be removed.
 */
function create_pklst (default_fun, orientation, title, vals, get_val_func,
                        ret_list) {
        
        /* Create a table of title and picklist depending on orientation */
        if (orientation == `vertical) {
                ttab := vcTable.Create (string_append(title, ".tab"), 2, 1);
                pklst := vcRegion.Create1 (string_append(title, ".pklst"), 
                        vcPicklist.fac, vals, list (macro_on <- FALSE));
        }
        else {
                ttab := vcTable.Create (string_append(title, ".tab"), 1, 2);
                pklst := vcRegion.Create1 (string_append(title, ".pklst"), 
                        vcPicklist.fac, vals, list (macro_on <- FALSE));
        }

        /* Add title and picklist to table */
        vcTable.Add (ttab, vcRegion.Create1 (string_append(title, ".lab"),
                vcLabel.fac, title, nil));

        if (length (vals) > 10)
                vcTable.Add (ttab, vcPicklist.Scroll (string_append(title, 
                ".scroll"), pklst, 5, 10, 20));
        else
                vcTable.Add (ttab, pklst);

        /* Make sure the table is properly spaced out */
        if (orientation == `vertical)
                vcTable.Row (ttab, -1, vcTable.Border.Visible,
                        5, 10, 80, vgAlign.LowerLeft);
        else
                vcTable.Col (ttab, -1, vcTable.Border.Visible,
                        5, 10, 80, vgAlign.LowerLeft);

        /* Set the default/current value */
        def_val := default_fun ();
        if (def_val != nil)
                vcPicklist.SetPick (pklst, def_val);

        /* Setup function to be called when pick has been executed */
        vcPicklist.Notify (pklst, get_val_func);

        if (ret_list)
                return (list(pklst, vcRegion.Create1(
                        string_append(title, ".border"),
                        vcBorder.fac, ttab, nil)));
        else
                return (vcRegion.Create1(string_append(title, ".border"),
                        vcBorder.fac, ttab, nil));
}

/* Runs through a list of cons cells, trying to match 'val' to the
 * car of one of the cells.  Returns TRUE if a match is found,
 * FALSE otherwise.
 */
function is_mem_list (lst, val) {
        if (val == nil)
                return (FALSE);

        for (tp := lst; tp != nil; tp = cdr(tp)) {
                if (string_eqi (caar(tp), val))
                          return (TRUE);
        }
        return (FALSE);
}

/* Runs through a list of cons cells, trying to match 'val' to the
 * cdr of one of the cells.  Returns the string in the car of a cell
 * that matches.  Returns nil in event of no match.  This list must
 * be of the form: list(cons,cons,..,cons).
 */
function get_str_from_lst (lst, val) {
        
        for (tp := lst; tp != nil; tp = cdr(tp)) {
                if (cdar(tp) == val)
                          return (caar(tp));
        }
        return (nil);
}

/* Runs through a list of cons cells, trying to find the `val`th member.
 * Returns the string in the car of the matching cell.  Returns nil in 
 * event of no match.  This list must be of the form: 
 * list(cons,cons,..,cons).
 */
function get_nth_val_from_lst (lst, val) {

        for (tp := lst, i := 0; tp != nil; tp = cdr(tp), ++i) {
                if (val == i)
                        return (car(car(tp)));
        }

        return (nil);
}

/* Add a button to the given table.  It will have label 'str', will reside
 * in [row, col] entry, and will call 'fun' when it has been activated.
 */
function add_but (table, str, row, col, fun) {

        button := vcLabel.ButtonCreate (str, str, fun, 
                        list (macro_on <- FALSE));
        vcTable.Set (table, row, col, button);
}

/* This is called whenever the items in the paper picklist change, for
 * example as the result of changing devices or defining a new paper type
 */
function update_paper_pklst () {

        paper_pklst := vsppaper.create_paper_area ();
        vcRegion.Destroy (vcTable.Get (vspPlotTable, 0, 1));
        vcTable.Set(vspPlotTable, 0, 1, paper_pklst);

        /* Should look into refreshing just the picklist */
        vcRegion.Reframe(vspPlotTable, vcRegion.Area(vspPlotTable));
	}
