/*
 *  $Revision: 1.12.4.1 $
 *
 *  ViewScript UI definitions for the ViewScript Debugger.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


/*
 * When double clicking the left mouse button, select a word and then call the
 * DataValue function to check the variable's value.  The word is interpreted
 * as a variable identifier.
 */
function double_click_value(data, event)
{
    vtx_id := GetVtext (data);

    vtx._SelectWord (vtx_id, event);
    vtx.Update (vtx_id);

    if (vtx.HasSelection (vtx_id))
    {
        var_str := vtx.GetSelection (vtx_id);
        DataPrint (data, event, var_str);
        vtx.ClearSelection (vtx_id);
    }
}


/*****************************************************************************/
/*                              Define the L&F                               */
/*****************************************************************************/

/*
 * Build command objects for all callbacks that can be directly invoked by the
 * user either through key bindings, mouse clicks or commands in the command
 * input window.
 */

stack_backtrace_cmd := vcWv.Cmd (fac, StackBacktrace, "stack");
stack_down_cmd      := vcWv.Cmd (fac, StackDown, "down");
stack_up_cmd        := vcWv.Cmd (fac, StackUp, "up");
stack_top_cmd       := vcWv.Cmd (fac, StackTop, "top");
stack_bottom_cmd    := vcWv.Cmd (fac, StackBottom, "bottom");
stack_frame_cmd     := vcWv.Cmd (fac, StackFrame, "frame", "Stack Frame",
                                 vcWv.Param ("Frame number:", vlIntVT,
                                             vcWv.Sticky, 0));
show_err_cmd        := vcWv.Cmd (fac, ShowErr, "error");
eval_exp_cmd        := vcWv.Cmd (fac, EvalExp, "evalx", "Eval An Expression",
                                 vcWv.Param ("Expression:",
                                             vlStringRestOfLine1VT,
                                             vcWv.Sticky, ""));

data_print_cmd      := vcWv.Cmd (fac, DataPrint, "print", "Data Value",
                                 vcWv.Param ("Object name:", vlStringVT,
                                             vcWv.Sticky, ""));
data_type_cmd       := vcWv.Cmd (fac, DataType, "type", "Data Type",
                                 vcWv.Param ("Object name:", vlStringVT,
                                             vcWv.Sticky, ""));
double_click_cmd    := vcWv.Cmd (fac, double_click_value, "double_click");


vcCmd.SetHelp (stack_backtrace_cmd,     "Show the entire ViewScript stack");
vcCmd.SetHelp (stack_down_cmd,          "Show the next frame down the stack");
vcCmd.SetHelp (stack_up_cmd,            "Show the next frame up the stack");
vcCmd.SetHelp (stack_top_cmd,           "Show the top stack frame");
vcCmd.SetHelp (stack_bottom_cmd,        "Show the bottom stack frame");
vcCmd.SetHelp (stack_frame_cmd,         "Show a particular stack frame");
vcCmd.SetHelp (data_print_cmd,          "Show the value of a variable");
vcCmd.SetHelp (data_type_cmd,           "Show the type of a variable");


/*
 * Debugger's own key bindings.
 */
vcBind.Set (bindings, '\_d',        stack_down_cmd);
vcBind.Set (bindings, '\_u',        stack_up_cmd);
vcBind.Set (bindings, '\_t',        stack_backtrace_cmd);
vcBind.Set (bindings, '\leftdbl',   double_click_cmd);


/*
 * Build Debugger menus.
 */
stack_menu  := vcWv.MakeMenu ("stack",
                              list ("Stack",    stack_backtrace_cmd),
                              list ("Next",     stack_down_cmd),
                              list ("Prev",     stack_up_cmd),
                              list ("Top",      stack_top_cmd),
                              list ("Bottom",   stack_bottom_cmd),
                              list ("Frame #",  stack_frame_cmd),
                              list ("Error Msg",show_err_cmd));
                                   
data_menu   := vcWv.MakeMenu ("data",
                              list ("Value",    data_print_cmd),
                              list ("Type?",    data_type_cmd));

exec_menu   := vcWv.MakeMenu ("exec",
                              list ("Eval",     eval_exp_cmd));

main_menu   := vcWv.MakeMenu ("vcVbg",
                              vcWin.RedSquareEntryNoPlotNoQuit,
                              list ("Stack",    stack_menu),
                              list ("Data",     data_menu),
                              list ("Exec",     exec_menu));

vcFac.SetMenus (fac, main_menu);
