/*
 *  $Revision: 1.41.2.1 $
 *
 *  Help!
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 */

/*
 * Prefix conventions in variable names:
 *   "b"     --> boolean
 *   "i"     --> integer
 *   "s"     --> string
 *   "l"     --> list
 *   "e"     --> some enumerated type
 *   "v"     --> generic viewscript type
 *   "cmd"   --> command object
 *   "ev"    --> event object
 *   "menu"  --> menu object
 *   "rgn"   --> region object
 *   "win"   --> window object
 */

cmdWarpNext := vcWv.Cmd (fac, WarpNext, "WarpNext");
vcCmd.SetHelp (cmdWarpNext, "View the next screen of the current topic");
vcBind.Set (bindings, 'n', cmdWarpNext);
vcBind.Set (bindings, 'N', cmdWarpNext);
vcBind.Set (bindings, '\^N', cmdWarpNext);
vcBind.Set (bindings, '\_n', cmdWarpNext);
cmdWarpPrev := vcWv.Cmd (fac, WarpPrev, "WarpPrev");
vcCmd.SetHelp (cmdWarpPrev, "View the previous screen of the current topic");
vcBind.Set (bindings, 'p', cmdWarpPrev);
vcBind.Set (bindings, 'P', cmdWarpPrev);
vcBind.Set (bindings, '\^P', cmdWarpPrev);
vcBind.Set (bindings, '\_p', cmdWarpPrev);
cmdNextPage := vcWv.Cmd (fac, func(dpy, ev) vtx.NextPage (DpyVtx (dpy));,
                         "NextPage");
vcBind.Set (bindings, ' ', cmdNextPage);
cmdPrevPage := vcWv.Cmd (fac, func(dpy, ev) vtx.PrevPage (DpyVtx (dpy));,
                         "PrevPage");
vcBind.Set (bindings, '\177', cmdPrevPage);
vcBind.Set (bindings, '\^H', cmdPrevPage);


/**********************************************************************
 *
 * Functions to manage our windows
 *
 **********************************************************************/

winContents     := nil;
winTopics       := nil;
winErrMsgTopics := nil;
winNonErrMsgTopics := nil;
winSearchHits   := nil;
winMain         := nil;

/*
 * Since the main viewhelp window is created in C, this is
 * how we import its value into viewscript.
 *
 * DANGER:  Don't rename this function!
 */
function SetMainWindow(win)
{
    winMain = win;
    vcWin.DestroyNotify (win, WinDestroyed);

} /* SetMainWindow */

/*
 * NukeXXX -- destroy one of the popup windows
 *
 * We just have to vcWin.Destroy them here, the destroy notifier
 * WinDestroyed() will be called to correctly set the static
 * variables.
 */
function NukeContents()
    if (winContents != nil)
        vcWin.Destroy (winContents);

function NukeTopics()
    if (winTopics != nil)
        vcWin.Destroy (winTopics);

function NukeSearchHits()
    if (winSearchHits != nil)
        vcWin.Destroy (winSearchHits);

/*
 * WinDestroyed -- destroy notify handler
 */
function WinDestroyed(win)
{
    if (win == winContents)
        winContents = nil;
    else if (win == winTopics)
        winTopics = nil;
    else if (win == winErrMsgTopics)
        winErrMsgTopics = nil;
    else if (win == winNonErrMsgTopics)
        winNonErrMsgTopics = nil;
    else if (win == winSearchHits) {
        winSearchHits = nil;
        iSearchCurrent = nil;
        rgnSearchPicklist = nil;
    }
    else if (win == winMain) {
        /*NukeTopics ();*/
        /*NukeContents ();*/
        /*NukeSearchHits ();*/
        winMain = nil;
        lTopics = nil;
    }

} /* WinDestroyed */


/**********************************************************************
 *
 * Functions to handle dynamic Contents selection
 *
 **********************************************************************/

/*
 * ListContents -- show table of contents in a picklist
 */
function ListContents(dpy, ev)
{
    function GotoSelected(iIndex, eState)
        if (eState == vlOnOff.On)
            WarpNumber (dpy, ev, iIndex);


    /*** end of local functions ***/


    if (winContents != nil)
        vcWin.Show (winContents);
    else vcWin.WithBusy (vcEvent.Window (ev), func() {

        rgnLabel :=
            vcRegion.Create1 ("vhelp.toc.label", vcLabel.fac,
                              "Select a Section Title", nil);
        vcRegion.Font (rgnLabel, vgFont.mediumb);
        rgnPicklist :=
            vcRegion.Create ("vhelp.toc.pick", vcPicklist.fac,
                             GetContents (dpy), will_scroll <- TRUE);
        vcPicklist.Notify (rgnPicklist, GotoSelected);
        rgnPopup :=
            vcTable.Create ("Table of Contents", 3, 1);
        vcTable.Add (rgnPopup, rgnLabel);
        vcTable.Add (rgnPopup, vcPicklist.Scroll ("vhelp.toc.scroll",
                                                  rgnPicklist, 3, 10, 0));
        rgnButton := vcRegion.Create ("vhelp.toc.button", vcButnRow.fac,
            list (list ("Dismiss", "Dismiss", vcWin.wclose, nil)));
        vcTable.Add (rgnPopup, rgnButton);
        /*winContents = vcWin.Create (rgnPopup, vcWin.NoBindings);*/
        winContents = vcWin.Create (winMain, rgnPopup, vcWin.NoBindings);
                        /* + vcWin.Secondary flag when it becomes available */
        vcWin.DestroyNotify (winContents, WinDestroyed);
    });

} /* ListContents */

cmdListContents := vcWv.Cmd (fac, ListContents, "contents");
vcCmd.SetHelp (cmdListContents, "View the Table Of Contents");


/**********************************************************************
 *
 * Functions to handle dynamic Topic selection
 *
 **********************************************************************/

lTopics := nil;                 /* list of all topics (*.vhb) */
lErrTopics := nil;              /* list of error topics (*-err.vhb) */
lNonErrTopics := nil;           /* list of all topics except error topics */

/*
 * GetTopics -- search WDIR for all available help topics (.vhb files)
 */
function GetTopics()
{
    declare lTemp, iIndex;

    function Iter(sFilePath)
    {
        sTitle := FileGetTitle (sFilePath);
        lTemp = cons (list (sTitle, sFilePath, iIndex), lTemp);

    } /* Iter */

    function LookForHlpFiles (sSomeDir)
    {
        sWildcard := vscript.vsMerge ("", sSomeDir, "*", "vhb");
        /*printf ("sWildcard --> %s\n", sWildcard);*/
        vscript.vsLoop (sWildcard, vscript.vsFILE, Iter);

    } /* LookForHlpFiles */

    /*** end of local functions ***/

    iIndex := 0;
    LookForHlpFiles (".");
    foreach (sOneDir ; vscript.wdir_path ())
    {
        ++iIndex;
        if (!string_eq (sOneDir, "."))
            LookForHlpFiles (sOneDir);
    }

    if (lTemp != nil)
    {
        /* The list is sorted alphabetically by title (car(a)).  If titles
         * are equal, then we go by where we found it in WDIR (caddr(a)).
         */
        lTemp = vscript.Sort (lTemp,
                              func (a, b)
                                  string_lt (car (a), car (b))
                                  || (string_eq (car (a),car (b))
                                      && le (caddr (a), caddr (b))););

        /* Eliminate duplicate titles (where title==car(list_element)).
         */
        l := lTemp;
        while (cdr (l) != nil)
            if (string_eq (caar (l), caadr (l)))
                set_cdr (l, cddr (l)); /* but do NOT set l = cdr(l) */
            else
                l = cdr (l);
    }

    lTemp;

} /* GetTopics */

/*
 * ListTopics -- show available help topics in a picklist
 */
function ListTopics(dpy, ev)
{
    function GotoTopic(lValue, eState)
        if (eState == vlOnOff.On) {
            NukeContents ();
            NukeSearchHits ();
            WarpNewTopic (dpy, car (lValue), "");
        }


    /*** end of local functions ***/


    if (winTopics != nil)
        vcWin.Show (winTopics);
    else vcWin.WithBusy (vcEvent.Window (ev), func() {

        if (lTopics == nil)
            lTopics = GetTopics ();

        rgnLabel :=
            vcRegion.Create1 ("vhelp.topics.label", vcLabel.fac,
                              "Select a Topic", nil);
        vcRegion.Font (rgnLabel, vgFont.mediumb);
        rgnPicklist :=
            vcRegion.Create ("vhelp.topics.pick", vcPicklist.fac,
                             lTopics, will_scroll <- TRUE);
        vcPicklist.Notify (rgnPicklist, GotoTopic);
        rgnPopup :=
            vcTable.Create ("Available Help Topics", 3, 1);
        vcTable.Add (rgnPopup, rgnLabel);
        vcTable.Add (rgnPopup, vcPicklist.Scroll ("vhelp.topics.scroll",
                                                  rgnPicklist, 3, 5, 0));
        rgnButton := vcRegion.Create ("vhelp.topics.button", vcButnRow.fac,
            list (list ("Dismiss", "Dismiss", vcWin.wclose, nil)));
        vcTable.Add (rgnPopup, rgnButton);
        /*winTopics = vcWin.Create (rgnPopup, vcWin.NoBindings);*/
        winTopics = vcWin.Create (winMain, rgnPopup, vcWin.NoBindings);
                      /* + vcWin.Secondary when it becomes available */
        vcWin.DestroyNotify (winTopics, WinDestroyed);
    });

} /* ListTopics */

cmdListTopics := vcWv.Cmd (fac, ListTopics, "topics");
vcCmd.SetHelp (cmdListTopics, "View list of available Help topics");


/* PruneTopicsList -- Trim alist `alist', either removing all entries whose
 * CADR matches `regexp' (`bRemoveMatching' == TRUE), or leaving only entries
 * whose CADR matches `regexp' (`bRemoveMatching' == FALSE).
 *
 * `alist' was built with the GetTopics function.  We want to reuse that
 * functionality, and prune selected elements out of the list.
 */
function PruneTopicsList(alist, regexp, bRemoveMatching)
{
    declare prev_l, l;

    /* used to decide whether or not to remove a given element */
    function RemoveElementP(re, str)
        if (bRemoveMatching)
            vscript.re_exec (re, str);
        else
            ! vscript.re_exec (re, str);

    re := vscript.re_compile (regexp);

    /* remove all leading elements which fit our criteria */
    while (alist != nil && RemoveElementP (re, cadar (alist)))
        alist := cdr (alist);

    /* remove all non-leading elements which fit our criteria */
    prev_l = alist;
    l      = (alist != nil) ? cdr (alist) : nil;
    while (l != nil)
        if (RemoveElementP (re, cadar (l))) {
            set_cdr (prev_l, cdr (l));
            l = cdr (l);
        }
        else {
            prev_l = l;
            l = cdr (l);
        }

    alist;

} /* PruneTopicsList */

/*
 * ListErrMsgTopics -- show available help for error msgs in a picklist
 */
function ListErrMsgTopics(win, ev)
{
    declare selection;  /* selection made in the picklist */

    function GotoErrMsgTopic(lValue, eState)
        if (eState == vlOnOff.On)
            vcAlert.SpawnHelp (win, car (lValue), "");

    if (winErrMsgTopics != nil)
        vcWin.Show (winErrMsgTopics);
    else vcWin.WithBusy (vcEvent.Window (ev), func() {

        if (lErrTopics == nil) {

            /* Get list of all topics */
            if (lTopics == nil)
                lTopics = GetTopics ();

            /* Remove entries from lErrTopics that DO NOT match "*-err.vhb" */
            lErrTopics = copy_list (lTopics);
            lErrTopics = PruneTopicsList (lErrTopics, "-err.vhb$", FALSE);
        }

        rgnLabel :=
            vcRegion.Create1 ("vhelp.errtopics.label", vcLabel.fac,
                                  "Select a Topic", nil);
        vcRegion.Font (rgnLabel, vgFont.mediumb);
        rgnPicklist :=
            vcRegion.Create ("vhelp.errtopics.pick", vcPicklist.fac,
                             lErrTopics, will_scroll <- TRUE);
        vcPicklist.Notify (rgnPicklist, GotoErrMsgTopic);
        rgnPopup :=
            vcTable.Create ("Available Error Message References", 3, 1);
        vcTable.Add (rgnPopup, rgnLabel);
        vcTable.Add (rgnPopup, vcPicklist.Scroll ("vhelp.errtopics.scroll",
                                                  rgnPicklist, 3, 5, 0));
        rgnButton := vcRegion.Create ("vhelp.errtopics.button", vcButnRow.fac,
                list (list ("Dismiss", "Dismiss", vcWin.wclose, nil)));
        vcTable.Add (rgnPopup, rgnButton);
        /*winErrMsgTopics = vcWin.Create (rgnPopup, vcWin.NoBindings);*/
        winErrMsgTopics = vcWin.Create (win, rgnPopup, vcWin.NoBindings);
                      /* + vcWin.Secondary when it becomes available */
        vcWin.DestroyNotify (winErrMsgTopics, WinDestroyed);
    });

} /* ListErrMsgTopics */

/*
 * ListNonErrMsgTopics -- show available help (except error msgs) in a picklist
 */
function ListNonErrMsgTopics(win, ev)
{
    declare selection;  /* selection made in the picklist */

    function GotoNonErrMsgTopic(lValue, eState)
        if (eState == vlOnOff.On)
            vcAlert.SpawnHelp (win, car (lValue), "");

    if (winNonErrMsgTopics != nil)
        vcWin.Show (winNonErrMsgTopics);
    else vcWin.WithBusy (vcEvent.Window (ev), func() {

        if (lNonErrTopics == nil) {

            /* Get list of all topics */
            if (lTopics == nil)
                lTopics = GetTopics ();

            /* Remove entries from lNonErrTopics that DO match "*-err.vhb" */
            lNonErrTopics = copy_list (lTopics);
            lNonErrTopics = PruneTopicsList (lNonErrTopics,
                                             "-err.vhb$", TRUE);
        }

        rgnLabel :=
            vcRegion.Create1 ("vhelp.nonerrtopics.label", vcLabel.fac,
                                  "Select a Topic", nil);
        vcRegion.Font (rgnLabel, vgFont.mediumb);
        rgnPicklist :=
            vcRegion.Create ("vhelp.nonerrtopics.pick", vcPicklist.fac,
                             lNonErrTopics, will_scroll <- TRUE);
        vcPicklist.Notify (rgnPicklist, GotoNonErrMsgTopic);
        rgnPopup :=
            vcTable.Create ("Available Command References", 3, 1);
        vcTable.Add (rgnPopup, rgnLabel);
        vcTable.Add (rgnPopup, vcPicklist.Scroll ("vhelp.nonerrtopics.scroll",
                                                  rgnPicklist, 3, 5, 0));
        rgnButton := vcRegion.Create ("vhelp.nonerrtopics.button", vcButnRow.fac,
                list (list ("Dismiss", "Dismiss", vcWin.wclose, nil)));
        vcTable.Add (rgnPopup, rgnButton);
        /*winNonErrMsgTopics = vcWin.Create (rgnPopup, vcWin.NoBindings);*/
        winNonErrMsgTopics = vcWin.Create (win, rgnPopup, vcWin.NoBindings);
                        /* + vcWin.Secondary flag when it becomes available */
        vcWin.DestroyNotify (winNonErrMsgTopics, WinDestroyed);
    });

} /* ListNonErrMsgTopics */

/*
 * warpto -- interface through which alert box communicates w/ viewhelp.
 *
 * DANGER:  Don't rename this function!
 */
function warpto(dpy, ev, sFilename, sSection)
{
    NukeContents ();
    WarpNewTopic (dpy, sFilename, sSection);
    vcWin.Show  (vcRegion.Window (DpyRegion (dpy)));

} /* warpto */

cmdWarpto := vcWv.Cmd (fac, warpto, "warpto", "Switch to New Topic",
                       vcWv.Param ("Filename", vlStringVT, `sticky, ""),
                       vcWv.Param ("Section Title", vlStringVT, `sticky, ""));


/**********************************************************************
 *
 * Search.
 *
 **********************************************************************/

iSearchCurrent := nil;
rgnSearchPicklist := nil;

function caddddr(l)
    car (cddddr (l));

function cadddddr(l)
    car (cdr (cddddr (l)));

function Search(dpy, ev, sSearchFor, eSearchCaseSensitive,
                eSearchTopics, eSearchScope)
{
    function GotoHit(vWhatnot, eState, iIndex)
        if (eState == vlOnOff.On) {
            /*printf ("GotoHit: iIndex = %s, vWhatnot = %s\n",
                        iIndex, vWhatnot);*/
            iSearchCurrent = iIndex;
            HiliteSearchHit (dpy,
                             car (vWhatnot),  /* filespec */
                             cadr (vWhatnot),  /* section title */
                             caddr (vWhatnot),  /* section index */
                             cadddr (vWhatnot),  /* line # */
                             caddddr (vWhatnot),  /* start char # */
                             cadddddr (vWhatnot)); /* end char # */
        }

    function NextHit()
        if (iSearchCurrent == nil)
            vcPicklist.SetPick (rgnSearchPicklist, 0);
        else
            vcPicklist.SetPick (rgnSearchPicklist, iSearchCurrent + 1);


    /*** end of local functions ***/


    /*
     * Make sure we don't try to match every old thing...
     */
    if (string_eq (sSearchFor, ""))
        vlEx.Throw2 (vlEx.FAILURE, "vhelp", vlErrSev.Error,
                     "Can't search for nothing");

    /*
     * Nuke the "hit list" if it's around.
     */
    if (winSearchHits != nil)
        NukeSearchHits ();

    /*
     * Get the matches.
     */
    vcWin.Busy (vcEvent.Window (ev), TRUE);
    if ((lTopics == nil) && (eSearchTopics == vhSearchTopics.AllTopics))
        lTopics = GetTopics ();
    lMatches := GetSearchHits (dpy, sSearchFor, !eSearchCaseSensitive,
                               eSearchTopics, eSearchScope, lTopics);

    /*
     * Yell if there aren't any matches.
     */
    if (lMatches == nil) {
        vcWin.Busy (vcEvent.Window (ev), FALSE);
        vlEx.Throw2 (vlEx.FAILURE, "vhelp", vlErrSev.Error,
                     "No matches for search string");
    }

    rgnLabel :=
        vcRegion.Create1 ("vhelp.search.label", vcLabel.fac,
                          sprintf ("Matches found for \"%s\"", sSearchFor),
                          nil);
    vcRegion.Font (rgnLabel, vgFont.mediumb);
    rgnSearchPicklist =
        vcRegion.Create ("vhelp.search.pick", vcPicklist.fac,
                         lMatches,
                         enhanced_notify <- TRUE, will_scroll <- TRUE);
    vcPicklist.Notify (rgnSearchPicklist, GotoHit);
    rgnPopup :=
        vcTable.Create ("Search", 3, 1);
    vcTable.Add (rgnPopup, rgnLabel);
    vcTable.Add (rgnPopup, vcPicklist.Scroll ("vhelp.search.scroll",
                                              rgnSearchPicklist, 3, 10, 0));
    rgnButRow :=
        vcTable.Create ("buttons", 1, 2);
    rgnBut1 := vcWv.LabelButtonCreate ("vhelp.search.but1", "Search Next",
                                       nil,
                                       NextHit,
                                       nil);
    vcRegion.Align (rgnBut1, vgAlign.MiddleCenter);
    vcTable.Add (rgnButRow, rgnBut1);
    rgnBut2 := vcWv.LabelButtonCreate ("vhelp.search.but2", "Dismiss",
                                       nil,
                                       func() vcWin.Close (winSearchHits);,
                                       nil);
    vcRegion.Align (rgnBut2, vgAlign.MiddleCenter);
    vcTable.Add (rgnButRow, rgnBut2);
    vcTable.Add (rgnPopup, rgnButRow);
    /*winSearchHits = vcWin.Create (rgnPopup, vcWin.NoBindings);*/
    winSearchHits = vcWin.Create (winMain, rgnPopup, vcWin.NoBindings);
                      /* + vcWin.Secondary when it becomes available */
    vcWin.DestroyNotify (winSearchHits, WinDestroyed);

    vcWin.Busy (vcEvent.Window (ev), FALSE);

} /* Search */

cmdSearch := vcWv.Cmd (fac, Search, "Search", "Search",
                       vcWv.Param ("Search for string", vlStringNotEmptyVT,
                                   vcWv.Sticky, vcWv.NoDefault),
                       vcWv.Param ("Match upper/lower case", vlYesNoVT,
                                   vcWv.Sticky, vlYesNo.No, vcWv.Secondary,
                                   "Treat upper and lower case as distinct"),
                       vcWv.Param ("Limit searched topics to",
                                   vhSearchTopicsVT,vcWv.Sticky,
                                   vhSearchTopics.CurrentTopicOnly,
                                   vcWv.Secondary),
                       vcWv.Param ("Limit search scope to", vhSearchScopeVT,
                                   vcWv.Sticky, vhSearchScope.AllText,
                                   vcWv.Secondary));
vcCmd.SetHelp (cmdSearch, "Search for text strings");
vcCmd.EnableApply (cmdSearch, vlOnOff.On);
vcBind.Set (bindings, '\^S', cmdSearch);
vcBind.Set (bindings, '\_s', cmdSearch);

/**********************************************************************
 * Menus.
 */

function NotImplementedCmd(sCommand)
{
    function fFunc(dpy, ev)
        vlEx.Throw2 (vlEx.FAILURE, "vhelp", vlErrSev.Error,
                     sprintf ("`%s' is not implemented yet", sCommand));

    cmd := vcWv.Cmd (fac, fFunc);

    cmd;

} /* NotImplementedCmd */

menuNavigate := vcWv.MakeMenu ("navigate",
                                vcWv.Pin (),
                                list ("Next", cmdWarpNext),
                                list ("Prev", cmdWarpPrev),
                                /*list ("Backtrack",
                                      NotImplementedCmd ("Backtrack")),*/
                                list ("Search", cmdSearch));
menuMain     := vcWv.MakeMenu ("vref",
                                vcWin.RedSquareEntryNoPlotNoQuit,
                                list ("Topics...", cmdListTopics),
                                list ("Contents...", cmdListContents),
                                list ("Navigate", menuNavigate));
vcFac.SetMenus (fac, menuMain);
