;;;
;;;  $Revision: 1.40.1.1 $ $Date: 1994/07/27 18:02:31 $
;;;

(define boolean?   isboolean)
(define eqv?       eqv)
(define eq?        eq)
(define equal?     equal)
(define set-car!   set_car)
(define set-cdr!   set_cdr)
(define pair?      iscons)
(define list?      islist)
(define null?      null)
(define list-ref   list_ref)

(define list-tail 
    (lambda (x k)
        (if (<= k 0)
            x
            (list-tail (cdr x) (- k 1)))))

(define symbol?    issymbol)
(define reverse-destructive reverse_destructive)

(define number?    isnumber)
(define integer?   isinteger)
(define real?      isfloat)
(define complex?   (lambda (x) (or (iscomplex x) (real? x))))

(define rational?  integer?)
(define exact?     integer?)
(define inexact?   (lambda (x) (not (exact? x))))

(define mk-trans   (lambda (op)
                       (define trans
                           (lambda x (if (or (null? x) (null? (cdr x)))
                                         #t
                                         (and (op (car x) (cadr x))
                                              (apply trans (cdr x))))))
                       trans))

(define =          (mk-trans eqv))
(define <          (mk-trans lt))
(define >          (mk-trans gt))
(define <=         (mk-trans le))
(define >=         (mk-trans ge))

(define zero?      (lambda (x) (eqv? x 0)))
(define positive?  (lambda (x) (gt x 0)))
(define negative?  (lambda (x) (lt x 0)))
(define odd?       (lambda (x) (ne (mod x 2) 0)))
(define even?      (lambda (x) (eqv (mod x 2) 0)))

(define max2       (lambda (x y) (if (gt x y) x y)))
(define min2       (lambda (x y) (if (lt x y) x y)))

(define mk-assop   (lambda (op neutral)
                       (define assop
                           (lambda x 
                               (if (null? x)
                                    neutral
                                    (if (null? (cdr x))
                                        (car x)
                                        (op (car x) (apply assop (cdr x)))))))
                       assop))

(define max        (mk-assop max2 ()))
(define min        (mk-assop min2 ()))
(define +          (mk-assop add  0))
(define *          (mk-assop mul  1))

(define -          (lambda x (if (null? x)
                                 ()
                                 (if (null? (cdr x))
                                     (neg (car x))
                                     (sub (car x) (cadr x))))))
(define /          (lambda x (if (null? x)
                                 ()
                                 (if (null? (cdr x))
                                     (div 1 (car x))
                                     (div (car x) (cadr x))))))

(define gcd2
    (lambda (x y)
        (let loop ((x (abs x))
                   (y (abs y)))
            (if (< x y)
                (loop y x)
                (if (or (= x y) (zero? y))
                    x
                    (loop y (remainder x y)))))))

(define lcm2  (lambda (x y) (/ (abs (* x y)) (gcd2 x y))))

(define gcd        (mk-assop gcd2 0))
(define lcm        (mk-assop lcm2 1))

(define ceiling    ceil)
(define truncate   trunc)

(define make-rectangular complex)
(define make-polar polar)
(define real-part  real)
(define imag-part  imag)
(define magnitude  mag)
(define angle      phase)

(define exact->inexact   (lambda (x) x))
(define inexact->exact   round)

(define (number->string x . base)
    (if (or (real? x) (integer? x) (not (complex? x)))
        (apply number2string x base)
        (string-append (apply number2string (real-part x) base)
                       (if (negative? (imag-part x)) "-" "+")
                       (let ((im (abs (imag-part x))))
                            (if (= im 1)
                                ""
                                (apply number2string im base)))
                       "i") ))

(define char?      ischar)
(define char=?     char_eq)
(define char<?     char_lt)
(define char>?     char_gt)
(define char<=?    char_le)
(define char>=?    char_ge)

(define char-upcase   char_upper)
(define char-downcase char_lower)

(define char-ci=?     (lambda (x y) (char_eq (char-upcase x) (char-upcase y))))
(define char-ci<?     (lambda (x y) (char_lt (char-upcase x) (char-upcase y))))
(define char-ci>?     (lambda (x y) (char_gt (char-upcase x) (char-upcase y))))
(define char-ci<=?    (lambda (x y) (char_le (char-upcase x) (char-upcase y))))
(define char-ci>=?    (lambda (x y) (char_ge (char-upcase x) (char-upcase y))))

(define char->integer char2int)
(define integer->char int2char)

(define char-alphabetic? 
    (lambda (x)
        (or (and (char<=? #\A x) (char<=? x #\Z))
            (and (char<=? #\a x) (char<=? x #\z)))))

(define char-numeric? 
    (lambda (x)
        (and (char<=? #\0 x) (char<=? x #\9))))

(define char-whitespace?
    (lambda (x)
        (not (not (memv x '(#\space #\newline #'\t' #'\f' #'\r'))))))

(define char-upper-case? 
    (lambda (x)
        (and (char<=? #\A x) (char<=? x #\Z))))

(define char-lower-case?
    (lambda (x)
        (and (char<=? #\a x) (char<=? x #\z))))

(define string?       isstring)
(define string        string_var)
(define make-string   make_string)
(define string-length string_length)
(define string-ref    string_ref)
(define string-set!   string_set)
(define string=?      string_eq)
(define string-ci=?   string_eqi)
(define string<?      string_lt)
(define string-ci<?   string_lti)
(define string>?      string_gt)
(define string-ci>?   string_gti)
(define string<=?     string_le)
(define string-ci<=?  string_lei)
(define string>=?     string_ge)
(define string-ci>=?  string_gei)
(define string-append string_append)
(define string->list  string2list)
(define list->string  list2string)
(define string-copy   string_copy)
(define string-fill!  string_fill)

(define make-vector   make_vector)
(define vector?       isvector)
(define vector-length vector_length)
(define vector-ref    vector_ref)
(define vector-set!   vector_set)

(define vector-fill!
    (lambda (x val)
        (let loop ((i (- (vector-length x) 1)))
            (if (>= i 0)
                (begin (vector-set! x i val) (loop (- i 1)))))))

(define struct?       isstruct)
(define struct-define struct_define)
(define struct-create struct_create)
(define struct-ref    struct_ref)
(define struct-set!   struct_set)

(define procedure?    isprocedure)

(define (apply f . args) 
    (if (null? args)
        (f)
        ((lambda (rargs)
             (*internal-apply* f (append (reverse (cdr rargs)) (car rargs))) )
         (reverse-destructive args) ) ) )

(define (eval x) (*internal-eval* x (current_package)))

(define *internal-map* (evalq map vscript))
(define (map f . lists)
    (if (null? (car lists))
        ()
        (cons (*internal-apply* f (*internal-map* car lists))
              (apply map f (*internal-map* cdr lists))) ) )

(define (for-each f . lists)
    (if (null? (car lists))
        ()
        (begin (*internal-apply* f (*internal-map* car lists))
               (apply for-each f (*internal-map* cdr lists))) ) )

(define gensym 
    (let ((counter 0))
        (lambda () 
            (set! counter (+ counter 1))
            (string->symbol (string-append "*gensym*" 
                                           (number->string counter))))))

(define-macro let*
    (lambda (x)
        (let ((bindings (cadr x)))
            (if (null? bindings)
                (cons 'let (cdr x))
                (list 'let (list (car bindings))
                    (cons 'let* (cons (cdr bindings) (cddr x))))))))

(define-macro letrec
    (lambda (x)
        (let ((defs  (map (lambda (y) (cons 'define y)) (cadr x))))
            (cons 'let (cons '() (append defs (cddr x)))))))

(define-macro cond
    (lambda (x)
        (if (null? (cdr x))
            ''()
            (let ((clause (cadr x)))
                (if (eq? (car clause) 'else)
                    (cons 'begin (cdr clause))
                    (let ((id (gensym)))
                        (list 'let (list (list id (car clause )))
                              (list 'if id
                                    (if (eq? (cadr clause) '=>)
                                        (list (caddr clause) id)
                                        (if (null? (cdr clause))
                                            id
                                            (cons 'begin (cdr clause))))
                                    (cons 'cond (cddr x))))))))))

(define-macro case
    (lambda (x)
        (if (null? (cddr x))
            ''()
            (let ((id (gensym)))
                (list 'let (list (list id (cadr x)))
                    (cons 'cond 
                        (map (lambda (clause)
                                 (if (eq? (car clause) 'else)
                                     clause
                                     (cons (list 'memv id (list 'quote 
                                                                (car clause)))
                                           (cdr clause))))
                             (cddr x))))))))

(define vector->list
    (lambda (x)
        (let loop ((i (- (vector-length x) 1))
                   (result '()))
            (if (>= i 0)
                (loop (- i 1) (cons (vector-ref x i) result))
                result))))

(define list->vector (lambda (x) (apply vector x)))


;; quasiquote-expressions will be completely rewritten into a different
;; form that makes operations which are necessary to evaluate them
;; explicit. 

(define vscmqquote (package (construct-any)
    (begin

(define (qq? x)
  (and (pair? x) (eq? (car x) 'quasiquote) (= (length x) 2)))
(define (uq? x)
  (and (pair? x) (eq? (car x) 'unquote) (= (length x) 2)))
(define (uqs? x)
  (and (pair? x) (eq? (car x) 'unquote-splicing) (= (length x) 2)))

(define (substruct2 x lev)
  (let ((sub (construct-any (cadr x) lev)))
    (case (car sub)
      ((quote) (list 'quote x))
      ((splice) (list 'cons (list 'quote (car x))
                      (cadr sub)))
      (else (list 'cons
                  (list 'quote (car x))
                  (list 'cons
                        sub
                        ''()))))))

(define (construct-any x lev)
  (cond ((qq? x) (substruct2 x (+ lev 1)))
        ((uq? x)
         (if (zero? lev)
             (list 'identity (cadr x))
             (substruct2 x (- lev 1))))
        ((uqs? x)
         (if (zero? lev)
             (list 'splice (cadr x))
             (substruct2 x (- lev 1))))
        ((pair? x)
         (construct-pair x lev))
        ((vector? x)
         (construct-vector x lev))
        ((struct? x)
         (construct-struct x lev))
        (else (list 'quote x))))

(define (q-c-h x)
  (list 'quote (reverse x)))

(define (construct-vector v lev)
  (let* ((state 'quote)
         (subs (map (lambda (y)
                      (let ((r (construct-any y lev)))
                        (case (car r)
                          ((quote) r)
                          ((splice)
                           (set! state 'splice)
                           r)
                          (else
                           (if (not (eq? state 'splice))
                               (set! state 'collect))
                           r))))
                    (vector->list v))))
    (case state
      ((quote) (list 'quote v))
      ((collect)
       (cons 'vector subs))
      (else
       (list 'list->vector
             (letrec
                 ((recur
                   (lambda (constant-heading rest)
                     (cond ((not (pair? rest))
                            (q-c-h constant-heading))
                           ((eq? (caar rest) 'quote)
                            (recur
                             (cons (cadar rest)
                                   constant-heading)
                             (cdr rest)))
                           ((eq? (caar rest) 'splice)
                            (if (null? constant-heading)
                                (list 'append
                                      (list 'identity (cadar rest))
                                      (recur '() (cdr rest)))
                                (list
                                 'append
                                 (q-c-h constant-heading)
                                 (list 'append
                                       (list 'identity (cadar rest))
                                       (recur '() (cdr rest))))))
                           ((null? constant-heading)
                            (list 'cons
                                  (car rest)
                                  (recur '() (cdr rest))))
                           (else
                            (list 'append
                                  (q-c-h constant-heading)
                                  (list 'cons
                                        (car rest)
                                        (recur '() (cdr rest)))))))))
               (recur '() subs)))))))

(define (struct-values s)
   (let ((values nil))
       (struct_value_iter 
           (lambda (name val)
               (set! values (cons val values)))
           s)
       (reverse-destructive values)))

(define (construct-struct x lev)
   (list 'apply 'struct-create (list 'quote (typeof x))
         (construct-pair (struct-values x) lev)))

(define (construct-pair x lev)
  (letrec
      ((recur
        (lambda (constant-heading rest)
          (cond ((not (pair? rest))
                 (list 'quote
                       (letrec
                           ((loop (lambda (l r)
                                    (if (pair? l)
                                        (loop (cdr l)
                                              (cons (car l) r))
                                        r))))
                         (loop constant-heading rest))))
                ((and (zero? lev) (uq? rest))
                 (if (pair? constant-heading)
                     (list 'append
                           (q-c-h constant-heading)
                           (list 'identity (cadr rest)))
                     (list 'identity (cadr rest))))
                (else
                 (let ((item (construct-any (car rest) lev)))
                   (case (car item)
                     ((quote)
                      (recur (cons (cadr item) constant-heading)
                             (cdr rest)))
                     ((splice)
                      (if (null? constant-heading)
                          (list 'append
                                (list 'identity (cadr item))
                                (recur '() (cdr rest)))
                          (list 'append
                                (q-c-h constant-heading)
                                (list 'append
                                      (list 'identity (cadr item))
                                      (recur '() (cdr rest))))))
                     (else
                      (if (null? constant-heading)
                          (list 'cons
                                item
                                (recur '() (cdr rest)))
                          (list
                           'append
                           (q-c-h constant-heading)
                           (list 'cons
                                 item
                                 (recur '() (cdr rest)))))))))))))
    (recur '() x)))
)))
;;;
;;; End of package vscmqquote
;;;


(define identity (lambda (x) x))

(define-macro quasiquote
    (lambda (x)
        ((evalq construct-any vscmqquote) (cadr x) 0)))


(define-macro do
    (lambda (x)
        (let ((inits  (cadr x))
              (test   (caddr x))
              (comms  (cdddr x))
              (loopid (gensym)))
            `(let ,loopid ,(map (lambda (y) (list (car y) (cadr y)))
                                inits)
                 (if ,(car test) 
                     (begin ,@(cdr test))
                     (begin ,@comms 
                         (,loopid ,@(map (lambda (y) 
                                             (if (null? (cddr y))
                                                 (car y)
                                                 (caddr y)))
                                         inits))))))))

(define call-with-current-continuation
    (lambda (f)
        (let ((tag (gensym))
              (ret ()))
            (catchexc tag
                (f (lambda (x)
                       (set! ret x)
                       (throw tag)))
                ret))))

(define call/cc call-with-current-continuation)


(define (make-promise proc)
    (let ((result-ready? #f)
          (result #f))
        (lambda ()
            (if result-ready?
                result
                (let ((x (proc)))
                    (if result-ready?
                        result
                        (begin (set! result-ready? #t)
                               (set! result x)
                               result)))))))
(define-macro delay
    (lambda (x)
        `(make-promise (lambda () ,(cadr x)))))

(define (force object) (object))


;;;
;;; CFI extensions
;;;

(define cfi-interaction-environment current_package)

(define (cfi-eval exp env) (*internal-eval* exp env))

(define (cfi-eval-file filename env syntax) (load filename env))

(define (cfi-eval-string str env syntax)
    (if (eq? syntax 'scheme)
        (eval_string2 str env vscmLanguage|SCM)
        (eval_string2 str env syntax)))

(define-macro cfi-catch
    (lambda (x)
        (let ((oldthrow  (gensym))
              (result    (gensym))
              (throw-arg (gensym)))
            `(let ((,oldthrow   cfi-throw)
                   (,result     nil)
                   (,throw-arg  nil))
                 (set! cfi-throw (lambda (x)
                                     (set! ,throw-arg x)
                                     (throw '*cfi-exception-tag*)))
                 (unwind 
                     (set! ,result (catchexc '*cfi-exception-tag*
                                       ,(cadr x)
                                       (,(caddr x) ,throw-arg)))
                     (set! cfi-throw ,oldthrow))
                 ,result))))

(define-macro cfi-unwind-protect
    (lambda (x)
        (let ((result  (gensym)))
            `(let ((,result  nil))
                 (unwind 
                     (set! ,result ,(cadr x))
                     ,(caddr x))
                 ,result))))

(define cfi-throw (lambda (x) 
   (printf "CFI-style exception was raised but no handler was defined.\nThe argument of cfi-throw was:\n") 
   (write x)
   (newline)
   (throw '*cfi-exception-tag*)))

;;;
;;; Input/Output
;;;

(define *current-input-port* *stdin*)
(define *current-output-port* *stdout*)

(define (current-input-port) *current-input-port*)
(define (current-output-port) *current-output-port*)

(define (input-port? x)
    (and (eqv? (typeof x) vlFileVT)
         (not (not (lindex (fmode x) #\r)))))

(define (output-port? x) 
    (and (eqv? (typeof x) vlFileVT)
         (not (not (lindex (fmode x) #\w)))))

(define (open-input-file fname)  (fopen fname "rb"))
(define (open-output-file fname) (fopen fname "wb"))

(define (close-input-port f)  (fclose f))
(define (close-output-port f) (fclose f))

(define (call-with-input-file fname proc)
    (let* ((f (open-input-file fname))
           (result (proc f)))
        (close-input-port f)
        result))

(define (call-with-output-file fname proc)
    (let* ((f (open-output-file fname))
           (result (proc f)))
        (close-output-port f)
        result))

(define (with-input-from-file fname thunk)
    (let ((oldport *current-input-port*))
        (set! *current-input-port* (open-input-file fname))
        (let ((result (thunk)))
            (set! *current-input-port* oldport)
            result)))

(define (with-output-to-file fname thunk)
    (let ((oldport *current-output-port*))
        (set! *current-output-port* (open-output-file fname))
        (let ((result (thunk)))
            (set! *current-output-port* oldport)
            result)))

(define (make-input-port x)
    (if (null? x)
        *current-input-port*
        (car x)))

(define (make-output-port x)
    (if (null? x)
        *current-output-port*
        (car x)))

(define (read-char . port)
    (let ((c (fgetchar (make-input-port port))))
        (if (null? c) *eof-object* c)))

(define (peek-char . port)
    (let* ((f (make-input-port port))
           (c (fgetchar f)))
        (if (null? c)
            *eof-object*
            (fungetchar f c))))

(define (char-ready? . port) #t)

(define (read . port)
    (read_object (make-input-port port)))

(define (eof-object? x) (eq? x *eof-object*))

(define (write-char char . port)
    (fputchar (make-output-port port) char))

(define (write obj . port)
    (let ((f (make-output-port port)))
        (write_object f obj 0)))
    
(define (display obj . port)
    (let ((f (make-output-port port)))
        (write_generic obj #t 0 (lambda (s) (fputline f s)))))
    
(define (newline . port)
    (fputchar (make-output-port port) #\newline))

(define read-object    read_object)
(define read-string    read_string)
(define read-file      read_file)

(define write-generic  write_generic)
(define write-object   write_object)
(define write-string   write_string)
(define write-file     write_file)

(define *vscmslib-loaded* #t)

'done
