/*
 *  $Revision: 2.61.2.1 $
 *
 *  ViewScript UI definitions for the ViewText facility.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

declare tmpCmd;

/*
 * Keys which insert themselves
 */
tmpCmd = vcWv.Cmd (fac, _InsertChar, "insert");
foreach (i; vcWv.Printable ())
    vcBind.Set (bindings, i, tmpCmd);

vcBind.Set (bindings, '\tab',
            vcWv.Cmd (fac, func(vt, e) InsertChar (vt, '\t');, "tab"));
vcBind.Set (bindings, '\enter',
            vcWv.Cmd (fac, func(vt, e) InsertChar (vt, '\n');, "enter"));

vcBind.Set (bindings, '\^Z', vcWv.Cmd (fac, func(vt, e) Scroll (vt, 1);,
            "scroll_up"));
vcBind.Set (bindings, '\_z', vcWv.Cmd (fac, func(vt, e) Scroll (vt, -1);,
            "scroll_down"));

/*
 * Cut/paste/delete routines
 */
tmpCmd = vcWv.Cmd (fac, _AnchorToggle, "anchor");
vcBind.Set (bindings, '\F9', tmpCmd);
vcBind.Set (bindings, '\0', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _Delete, "del_next_char");
vcBind.Set (bindings, '\^D', tmpCmd);
vcBind.Set (bindings, '\delete', tmpCmd);
vcBind.Set (bindings, '\^Y', vcWv.Cmd (fac, _Paste, "yank"));
tmpCmd = vcWv.Cmd (fac, _DeletePrevChar, "del_prev_char");
vcBind.Set (bindings, '\177', tmpCmd);
vcBind.Set (bindings, '\^H', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _DeletePrevWord, "del_prev_word");
vcBind.Set (bindings, '\F7', tmpCmd);
vcBind.Set (bindings, '\_\177', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _DeleteNextWord, "del_next_word");
vcBind.Set (bindings, '\F8', tmpCmd);
vcBind.Set (bindings, '\_d', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _DeleteToEol, "del_to_eol");
vcBind.Set (bindings, '\F5', tmpCmd);
vcBind.Set (bindings, '\^K', tmpCmd);
vcBind.Set (bindings, '\F6', vcWv.Cmd (fac, _DeleteCurrLine, "del_cur_line"));

/*
 * Cursor movement
 */
tmpCmd = vcWv.Cmd (fac, _BeginningOfLine, "beg_of_line");
vcBind.Set (bindings, '\^A', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _EndOfLine, "end_of_line");
vcBind.Set (bindings, '\^E', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _PrevChar, "prev_char");
vcBind.Set (bindings, '\left', tmpCmd);
vcBind.Set (bindings, '\^B', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _NextChar, "next_char");
vcBind.Set (bindings, '\right', tmpCmd);
vcBind.Set (bindings, '\^F', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _PrevWord, "prev_word");
vcBind.Set (bindings, '\_b', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _NextWord, "next_word");
vcBind.Set (bindings, '\_f', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _PrevLine, "prev_line");
vcBind.Set (bindings, '\up', tmpCmd);
vcBind.Set (bindings, '\^P', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _NextLine, "next_line");
vcBind.Set (bindings, '\down', tmpCmd);
vcBind.Set (bindings, '\^N', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _PrevPage, "prev_page");
vcBind.Set (bindings, '\pgup', tmpCmd);
vcBind.Set (bindings, '\_v', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _NextPage, "next_page");
vcBind.Set (bindings, '\pgdn', tmpCmd);
vcBind.Set (bindings, '\^V', tmpCmd);

/**********************************************************************
 * Functions needed for File menu.
 */

/*
 * maybe_do -- if buffer modified, prompt before executing action_func(),
 *             otherwise just do it.
 */
function maybe_do(vt, ev, action_str, action_func)
    if (! IsModified(vt))
        action_func();
    else {
        declare popup;

        msg := sprintf ("\nText buffer %s has been modified.\nAre you sure you want to %s?\n\n", GetBufferName(vt), action_str);

        table := vcTable.Create ("table", 2, 1);
        vcTable.Add (table,
                     vcRegion.Create1 ("msg", vcStaticText.fac, msg, nil));

        buttons := vcRegion.Create ("Quit?", vcButnRow.fac,
                                    list (list ("Confirm",
                                                "Yes, discard edits",
                                                func() action_func();,
                                                list (close_window <- TRUE)),
                                          list ("cancel",
                                                "Cancel",
                                                func();,
                                                list (close_window <- TRUE))));
        vcTable.Add (table, buttons);
        popup = vcWin.Popup (vcEvent.Window(ev), table,
                             vcButnRow.GetDefault(buttons),
                             vcWin.NoCmdline+vcWin.NoStatus+vcWin.NoBindings);
        /*
         * This causes a warning message: 
         *  Warning: window=/etc/rc->edit vcWinDestroy() called on dead window
         * vcWin.Forward (vcEvent.Window(ev), popup);
         */
    }
/* function maybe_do */

clear_cmd := vcWv.Cmd (fac,
                       func(vt, ev) 
                           maybe_do (vt, ev, "clear", func() Clear(vt););,
                       "clear");
vcCmd.SetHelp (clear_cmd, "Erase the contents of the buffer");

revert_cmd := vcWv.Cmd (fac,
                        func(vt, ev)
                            maybe_do (vt, ev, "undo all edits",
                                func() Load(vt, GetBufferName(vt)););,
                        "revert");
vcCmd.SetHelp (revert_cmd, "Undo all edits, reloading the file from disk");

load_cmd := vcWv.Cmd (fac,
                      func(vt, ev, fname)
                          maybe_do (vt, ev, "load", func() Load(vt, fname););,
                      "load", "Load File",
                      vcWv.Param ("Filename", vlStringVT, vcWv.Sticky, "",
                                  vcFilePop.fac));
vcCmd.SetHelp (load_cmd,
    "Load a file into the text buffer, replacing the current contents");

include_cmd := vcWv.Cmd (fac,
                         func(vt, e, f) InsertFile (vt, f);,
                         "include", "Include File",
                         vcWv.Param ("Filename", vlStringVT, vcWv.Sticky, "",
                                     vcFilePop.fac));
vcCmd.SetHelp (include_cmd,
               "Insert the contents of a file into the buffer at the cursor");

/**********************************************************************
 * Misc
 */

Destroy := _Destroy;
UndoCmd := vcWv.Cmd (fac, _Undo, "undo");
vcCmd.SetHelp (UndoCmd, "Undo the last edit");
vcBind.Set (bindings, '\^U', UndoCmd);
tmpCmd = vcWv.Cmd (fac, _BeginningOfBuffer, "beg_of_buffer");
vcBind.Set (bindings, '\_<', tmpCmd);
/* OpenWindows apps use \home to go to the beginning of buffer */
vcBind.Set (bindings, '\home', tmpCmd);
tmpCmd = vcWv.Cmd (fac, _EndOfBuffer, "end_of_buffer");
vcBind.Set (bindings, '\_>', tmpCmd);
/* OpenWindows apps use \end to go to the end of buffer */
vcBind.Set (bindings, '\end', tmpCmd);

GotoLineCmd := vcWv.Cmd (fac,
                         func (vt, ev, line) Move (vt, line, 0);,
                         "goto_line", "Goto Line",
                         vcWv.Param ("Line number", vlPositiveIntVT,
                                     vcWv.Sticky, 1));
vcCmd.SetHelp (GotoLineCmd, "Move the cursor to a given line number");
vcBind.Set (bindings, '\^G', GotoLineCmd);

SelectLineCmd :=
    vcWv.Cmd (fac,
              func (vt, ev, lineno) SelectLine (vt, lineno);,
              "SelectLine", "Select Line",
              vcWv.Param ("Line number", vlPositiveIntVT, vcWv.Sticky, 1));
vcCmd.SetHelp (SelectLineCmd,
    "Move the cursor to a given line number and highlight the line");

WhatLineCmd :=
    vcWv.Cmd (fac,
              func (vt, ev) {
                  vcWin.ClearStatus (vcEvent.Window (ev));
                  vcWin.AddStatus (vcEvent.Window (ev),
                                   sprintf ("cursor is on line %d",
                                            GetCursorLine (vt)));
              });
vcCmd.SetHelp (WhatLineCmd, "Report what line the cursor is on");

WhatFilenameCmd :=
    vcWv.Cmd (fac,
              func (vt, ev) {
                  vcWin.ClearStatus (vcEvent.Window (ev));
                  vcWin.AddStatus (vcEvent.Window (ev),
                                   vscript.vsAbsPath (GetBufferName (vt)));
              });
vcCmd.SetHelp (WhatFilenameCmd,
               "Report the full pathname of the current buffer");

WritetoCmd := vcWv.Cmd (fac, _Writeto, "writeto", "Write To New Name",
                        vcWv.Param ("New filename:", vlStringVT,
                                    vcWv.Dynamic, GetBufferName));
vcCmd.SetHelp (WritetoCmd, "Save the buffer under a different name");

/*
 * WARNING * WARNING * WARNING
 * This function needs to be in VS code so that we can call
 * vcCmd.Run.  Don't delete or rename this function!
 */
function write_helper(vt, ev) {
    if (string_eq (GetBufferName (vt), ""))
        vcCmd.Run (WritetoCmd, vt, ev, nil, TRUE);
    else
        Write (vt);
}
WriteCmd := vcWv.Cmd (fac, write_helper, "write");
vcCmd.SetHelp (WriteCmd, "Save the buffer");

vcBind.Set (bindings, '\^W', WriteCmd);
vcBind.Set (bindings, '\^X', WritetoCmd);

/*
 * `has_moved_during_grab' and `evInitialClick' are used to
 * determine whether a \leftdown is the start of a swipe (meaning
 * clear selection and start it again) or just a click.
 * `save_bindings' is used to restore the binding table to its previous
 * value when the user releases the mouse.  This is necessary because
 * vcTxShell overrides the vtx bindings to make the "paste" command
 * go to the prompt area first, and we don't want to kill its bindings.
 */
declare has_moved_during_grab, evInitialClick, save_bindings;

function start_cut(vt, event)
  if (IsWithinBounds (vt, event)) {
    if (IsAnchorDown (vt))
        AnchorToggle (vt);

    _GotoPointer (vt, event);
    evInitialClick = event;
    has_moved_during_grab = FALSE;

    region := vtx.Region (vt);
    vcRegion.SetGrab (region, region);
    save_bindings = vcRegion.Bindings (region);
    vcRegion.SetBindings (region, cut_bindings);
  }
/* start_cut */

function cont_cut(vt, event) {
    if (has_moved_during_grab == FALSE
        && vcWv.ClickTogether (wv, evInitialClick, event) == FALSE)
    {
        has_moved_during_grab = TRUE;
        AnchorToggle (vt);
    }
    _GotoPointer (vt, event);
} /* cont_cut */

function end_cut(vt, event) {
    if (has_moved_during_grab) {
        str := GetSelection (vt);
        if (str == nil || string_eq (str, ""))
            AnchorToggle (vt);
        else
            vcRegion.StoreCutBuffer (vtx.Region (vt), str);
    }
    region := vtx.Region (vt);
    vcRegion.SetGrab (region, nil);
    vcRegion.SetBindings (region, save_bindings);
} /* end_cut */

function _ClearSelection(vt) {
    ClearSelection (vt);
    Update (vt);
}

copy_cmd :=  vcWv.Cmd (fac, func (vt, ev)
                              if (IsAnchorDown (vt))
                                  vcRegion.StoreCutBuffer(vtx.Region (vt),
                                                          GetSelection (vt));,
                       "Copy");
vcCmd.SetHelp (copy_cmd, "Copy the selected text to the clipboard");
paste_cmd := vcWv.Cmd (fac, func (vt, ev)
                              InsertString (vt, vcWv.FetchCutBuffer (wv));,
                       "Paste");
vcCmd.SetHelp (paste_cmd,
               "Insert the contents of the clipboard at the cursor");
cut_cmd :=   vcWv.Cmd (fac, func (vt, ev)
                              if (IsAnchorDown (vt)) {
                                  vcRegion.StoreCutBuffer (vtx.Region (vt),
                                                           GetSelection (vt));
                                  _Delete (vt, ev);
                              },
                       "Cut");
vcCmd.SetHelp (cut_cmd,
               "Delete the selected text and copy it to the clipboard");
select_word_cmd :=
    vcWv.Cmd (fac, func (vt, ev)
                       if (IsWithinBounds (vt, ev))
                           SelectWord (vt);,
              "SelectWord");

vcBind.Set (bindings, '\leftdown', vcWv.Cmd (fac, start_cut, "Select"));
vcBind.Set (bindings, '\leftdbl', select_word_cmd);
vcBind.Set (bindings, '\F14', UndoCmd);
vcBind.Set (bindings, '\F16', copy_cmd);
vcBind.Set (bindings, '\F18', paste_cmd);
vcBind.Set (bindings, '\middledown', paste_cmd);
vcBind.Set (bindings, '\F20', cut_cmd);

cut_bindings := vcBind.Create ();
tmpCmd := vcWv.Cmd (fac, cont_cut, "cont_cut");
/* we need this enabled because useful things happen during cont_cut()
   vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);*/
vcBind.Set (cut_bindings, '\move', tmpCmd);
vcBind.Set (cut_bindings, '\leftup', vcWv.Cmd (fac, end_cut, "SelectEnd"));
vcBind.Set (cut_bindings, '\leftdbl', select_word_cmd);


/**********************************************************************
 * Search + Replace functions
 */

SearchForwardCmd := vcWv.Cmd (fac, _SearchForward, "search_forward",
                              "String Search",
                              vcWv.Param ("String", vlStringVT,
                                          vcWv.Sticky, ""));
vcCmd.SetHelp (SearchForwardCmd,
               "Search forward for a string");
vcBind.Set (bindings, '\_s', SearchForwardCmd);
SearchNextCmd := vcWv.Cmd (fac, _SearchNext, "search_next");
vcCmd.SetHelp (SearchNextCmd,
               "Search forward for the next occurrence of the search string");
vcBind.Set (bindings, '\F10', SearchNextCmd);

function _QueryReplace(vt, ev, search_str, replacement_str)
    QueryReplace (vt, search_str, replacement_str);

function QueryReplace(vt, search_str, replacement_str)
{
    if (vcMac.IsReplayingOn () || vcMac.IsSuspended ()) {
        _RepSetup (vt, rep_bindings, search_str, replacement_str);
        while (_RepSearchNext (vt) == TRUE)
            _ReplaceMatch (vt);
        _RepCleanup (vt);
    }
    else {
        /* some init'ing */
        _RepSetup (vt, rep_bindings, search_str, replacement_str);

        /* goto the first occurrence */
        if (_RepSearchNext (vt) == TRUE)
            _RepHilite (vt);
        else
            _RepCleanup (vt);
    }
}

function _RepGlobal(vt, ev)
{
    _RepUnhilite (vt);
    _ReplaceMatch (vt);
    while (_RepSearchNext (vt) == TRUE) {
        _ReplaceMatch (vt);
    }
    _RepCleanup (vt);
}

function _RepDoOrDont(vt, yes_or_no) {
    _RepUnhilite (vt);
    if (yes_or_no == TRUE)
        _ReplaceMatch (vt);
    if (_RepSearchNext (vt) == FALSE)
        _RepCleanup (vt);
    else
        _RepHilite (vt);
}

function _RepDont(vt, ev)
    _RepDoOrDont (vt, FALSE);

function _RepDo(vt, ev)
    _RepDoOrDont (vt, TRUE);

function _RepEnd(vt, ev) {
    _RepUnhilite (vt);
    _RepCleanup (vt);
}

rep_bindings := vcBind.Create ();
tmpCmd := vcWv.Cmd (fac, _RepDo, "rep_do");
vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);
vcBind.Set (rep_bindings, 'y', tmpCmd);
vcBind.Set (rep_bindings, 'Y', tmpCmd);
vcBind.Set (rep_bindings, ' ', tmpCmd);
tmpCmd := vcWv.Cmd (fac, _RepDont, "rep_dont");
vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);
vcBind.Set (rep_bindings, 'n', tmpCmd);
vcBind.Set (rep_bindings, 'N', tmpCmd);
vcBind.Set (rep_bindings, '\177', tmpCmd);
tmpCmd := vcWv.Cmd (fac, _RepGlobal, "rep_global");
vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);
vcBind.Set (rep_bindings, 'g', tmpCmd);
vcBind.Set (rep_bindings, 'G', tmpCmd);
vcBind.Set (rep_bindings, '!', tmpCmd);
tmpCmd := vcWv.Cmd (fac, _RepEnd, "rep_end");
vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);
vcBind.Set (rep_bindings, 'q', tmpCmd);
vcBind.Set (rep_bindings, 'Q', tmpCmd);
vcBind.Set (rep_bindings, '\^C', tmpCmd);

QueryReplaceCmd :=
    vcWv.Cmd (fac, _QueryReplace, "query_replace", "Query Replace",
              vcWv.Param ("Search for:", vlStringVT, vcWv.Sticky, ""),
              vcWv.Param ("Replace with:", vlStringVT, vcWv.Sticky, ""));
vcCmd.SetHelp (QueryReplaceCmd,
               "Search and replace strings interactively");
vcBind.Set (bindings, '\^R', QueryReplaceCmd);

/**********************************************************************
 * Menus
 */
file_menu :=
    vcWv.MakeMenu ("file",
                    list ("Load File...", load_cmd),
                    list ("Save Current File", WriteCmd),
                    list ("Store as New File...", WritetoCmd),
                    list ("Include File...", include_cmd),
                    list ("Empty Document", clear_cmd));
edit_menu :=
    vcWv.MakeMenu ("edit",
                    vcWv.Pin(),
                    list ("Undo Last Edit", UndoCmd),
                    list ("Undo All Edits", revert_cmd),
                    list ("Copy", copy_cmd),
                    list ("Paste", paste_cmd),
                    list ("Cut", cut_cmd));
view_menu :=
    vcWv.MakeMenu ("view",
                    list ("Goto line number...", GotoLineCmd),
                    list ("Select line at number...", SelectLineCmd),
                    list ("What line?", WhatLineCmd),
                    list ("Buffer filename", WhatFilenameCmd));
find_menu :=
    vcWv.MakeMenu ("find",
                    list ("Search and Replace...", QueryReplaceCmd),
                    list ("Search forward...", SearchForwardCmd),
                    list ("Search next", SearchNextCmd));
top_menu :=
    vcWv.MakeMenu ("vtext",
                    vcWin.RedSquareEntryNoPlotNoQuit,
                    list ("File", file_menu),
                    list ("View", view_menu),
                    list ("Edit", edit_menu),
                    list ("Find", find_menu));
popup_menu :=
    vcWv.MakeMenu ("vtext",
                    list ("File", file_menu),
                    list ("View", view_menu),
                    list ("Edit", edit_menu),
                    list ("Find", find_menu));

vcFac.SetMenus (fac, top_menu);
tmpCmd := vcWv.Cmd (fac, func(data, event)
                             vcMenuPop.Create (popup_menu, event);,
                    "menu-textpane");
vcBind.Set (bindings, '\rightdown', tmpCmd);
vcCmd.EnableMacro (tmpCmd, vlOnOff.Off);

/**********************************************************************
 * Focus In + Out, etc.
 */
function _FocusIn(vt, event)
{
    SetCursor (vt, 1);
    EnableCmds (vt, event);
}

function _FocusOut(vt, event)
    SetCursor (vt, 2);

function _Get(vt, event, type)
    vtx.GetBuffer (vt);

/*
 * This is called the first time a window we own is mapped.
 */
function _Initial(vt, event)
    EnableCmds (vt, event);

/*
 * Enable/disable commands that appear in window/popup menus.
 */
function EnableCmds(vt, event)
{
    if (vtx.IsReadOnly (vt))
        state := vlOnOff.Off;
    else
        state := vlOnOff.On;
    vcCmd.Enable (load_cmd, state);
    vcCmd.Enable (WriteCmd, state);
    vcCmd.Enable (include_cmd, state);
    vcCmd.Enable (clear_cmd, state);
    vcCmd.Enable (UndoCmd, state);
    vcCmd.Enable (revert_cmd, state);
    vcCmd.Enable (paste_cmd, state);
    vcCmd.Enable (cut_cmd, state);
    vcCmd.Enable (QueryReplaceCmd, state);
    if (event != nil)
        vcWin.UpdateMenus (vcEvent.Window (event));

} /* EnableCmds */

/*
 * Load inremental search package (bound to ^S)
 */
load ("vtxisrch.vs");

/*
 * A command to toggle the read-only status.
 */
vcWv.Cmd (fac, func (vt, ev) ReadOnly (vt, ! IsReadOnly (vt));, "ReadOnly");

/*
 * load bindings file, menu file, help text file.
 */
/*
load ("vtxbt.vs");
load ("vtxmn.vs");
load ("vtxht.vs");
*/
/*
 * Do not load vtxus.vs here -- it will go into the main state file
 * that way.  It needs to be loaded at the end of initialization.
 */
