/*
 * $Revision: 1.3.1.1 $
 *
 * Facility for editing the "keys" preference to vcShowBind, to control
 * which keys the user sees in the binding display.
 */

/*
 * Set up a list with all the printable characters, using 10 columns
 */

picklist_format := nil;
function add_row(row) picklist_format = cons (row, picklist_format);

add_row (list (cons ("L", '\leftdown'), cons ("L2", '\leftdbl'),
               cons ("M", '\middledown'), cons ("M2", '\middledbl'),
               cons ("R", '\rightdown'), cons ("R2", '\rightdbl'),
               cons ("Move", '\move')));
add_row (list (cons ("F1", '\F1'), cons ("F2", '\F2'), cons ("F3", '\F3'),
               cons ("F4", '\F4'), cons ("F5", '\F5'), cons ("F6", '\F6'),
               cons ("F7", '\F7'), cons ("F8", '\F8'), cons ("F9", '\F9'),
               cons ("F10", '\F10')));
add_row (list (cons ("F11", '\F11'), cons ("F12", '\F12'),
               cons ("F13", '\F13'), cons ("F14", '\F14'),
               cons ("F15", '\F15'), cons ("F16", '\F16'),
               cons ("F17", '\F17'), cons ("F18", '\F18'),
               cons ("F19", '\F19'), cons ("F20", '\F20')));
add_row (list (cons ("Rubout", '\rubout'), cons ("Home", '\home'),
               cons ("End", '\end'), cons ("Insert", '\insert'),
               cons ("Delete", '\delete'), cons ("PgUp", '\pgup'),
               cons ("PgDn", '\pgdn'), cons ("Escape", '\e')));
add_row (list (cons ("Left", '\left'), cons ("Right", '\right'),
               cons ("Up", '\up'), cons ("Down", '\down'),
               cons ("Help", '\help'), cons ("Find", '\find'),
               cons ("Select", '\select'), cons ("Do", '\do'),
               cons ("Tab", '\tab'), cons ("Enter", '\enter')));

row := nil;
n_in_row := 0;
foreach (c; vcWv.printable ()) {
    if (n_in_row == 10) {
        add_row (reverse_destructive (row));
        row = nil;
        n_in_row = 0;
    } /* if */
    visual := (c == ' ') ? "Space" : sprintf ("%c", c);
    row = cons (cons (visual, c), row);
    ++n_in_row;
} /* foreach */

if (row != nil)
    add_row (reverse_destructive (row));

picklist_format = reverse_destructive (picklist_format);

function _Set(picklist, key_string) {
    vcPicklist.DeselectAll (picklist);
    keys := vlSym.ObjCreate (257);
    foreach (char_code, label; vcShowBind.ParseKeys (key_string))
        keys[char_code] = label;

    foreach (row, col, index, is_set, visual, char_code;
             vcPicklist.Items (picklist))
        if (keys[char_code] != nil)
            vcPicklist.SetPick (picklist, index);
} /* function _Set */

function _Create(key_string, region) {
    picklist := vcRegion.Create ("keys_picklist", vcPicklist.fac,
                                 picklist_format,
                                 exclusive <- FALSE,
                                 range <- TRUE);
    _Set (picklist, key_string);
    vcRegion.AddChild (region, picklist);
    picklist;
} /* function _Create */

function _Get(picklist, region, ts) {
    str := "";
    foreach (row, col, index, is_set, visual, data;
             vcPicklist.Items (picklist))
        if (is_set)
            str += sprintf (" %s=%c", visual, data);
    string_trim (str);
} /* function _Get */

function CallOnChange(key_edit_region, f) {
    vcPicklist.Notify (vcRegion.GetData (key_edit_region),
                       func(data, on_off) f (nil););
} /* function CallOnChange */
