/*  $Revision: 1.9.2.1 $
 *
 *  GUI definition for the vcVPCanvas facility.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


function cmd(args...) apply (vcWv.Cmd, cons (fac, args));

/*
 * Pan commands, on ALTed arrow keys...
 */
function make_pan_cmd(name, dx, dy) {
    function pan(canvas, event) {
        user := vcVPCanvas.GetUserspace (canvas);
        vgRect.Offset (user,
                       dx * (vgRect.Xmax (user) - vgRect.Xmin (user)),
                       dy * (vgRect.Ymax (user) - vgRect.Ymin (user)));
        vcVPCanvas.SetUserspace (canvas, user);
    }

    vcWv.Cmd (fac, pan, name);
}

PAN_FACTOR := .8;

vcBind.Set (bindings, '\_\left',    make_pan_cmd ("left", -PAN_FACTOR, 0));
vcBind.Set (bindings, '\_\right',   make_pan_cmd ("right", PAN_FACTOR, 0));
vcBind.Set (bindings, '\_\down',    make_pan_cmd ("down", 0, -PAN_FACTOR));
vcBind.Set (bindings, '\_\up',      make_pan_cmd ("up", 0, PAN_FACTOR));

MAG_FACTOR := 1.2;

function make_mag_cmd(name, mag) {
    mag /= 2;

    function do_mag(c, e) {
        user := vcVPCanvas.GetUserspace (c);
        center_x := (vgRect.Xmin (user) + vgRect.Xmax (user)) / 2;
        center_y := (vgRect.Ymin (user) + vgRect.Ymax (user)) / 2;
        sx := (vgRect.Xmax (user) - vgRect.Xmin (user) + 1) * mag;
        sy := (vgRect.Ymax (user) - vgRect.Ymin (user) + 1) * mag;
        vgRect.Set (user,
                    center_x - sx, center_y - sy,
                    center_x + sx, center_y + sy);
        vcVPCanvas.SetUserspace (c, user);
    }

    vcWv.Cmd (fac, do_mag, name);
}

InCmd  := make_mag_cmd ("in", 1/MAG_FACTOR);
OutCmd := make_mag_cmd ("out", MAG_FACTOR);

function do_zoom(c, e, x1, y1, x2, y2)
    SetUserspace (c, vgRect.Create (x1, y1, x2, y2));

function get_coord(f)
    func(c) f (GetUserspace (c));
;

ZoomToCoordsCmd := vcWv.Cmd (fac, do_zoom, "zoom", "Zoom to Coordinates",
    vcWv.Param ("X1", vlIntVT, vcWv.Dynamic, get_coord (vgRect.Xmin)),
    vcWv.Param ("Y1", vlIntVT, vcWv.Dynamic, get_coord (vgRect.Ymin)),
    vcWv.Param ("X2", vlIntVT, vcWv.Dynamic, get_coord (vgRect.Xmax)),
    vcWv.Param ("Y2", vlIntVT, vcWv.Dynamic, get_coord (vgRect.Ymax)));

function record_zoom(c, e, r)
    vcCmd.Run (ZoomToCoordsCmd, c, e,
               list (vgRect.Xmin (r), vgRect.Ymin (r),
                     vgRect.Xmax (r), vgRect.Ymax (r)),
               TRUE);

function start_zoom(c, e) {
    area := vcRegion.VisibleArea (vcEvent.Region (e));

    vcDrag.Create (e, area, vgColor.White,
                   func (new_area) {
                       vgU.PixelToUserRect (Viewport (c), new_area);
                       record_zoom (c, e, new_area);
                   },
                   vcDrag.Style.Box,
                   vgRect.Xmax (area) - vgRect.Xmin (area),
                   vgRect.Ymax (area) - vgRect.Ymin (area), nil);
}

function zoom_to_event(c, e) {
    area := vgRect.Copy (vcEvent.Rect (e));
    vgU.PixelToUserRect (Viewport (c), area);
    record_zoom (c, e, area);
}

ZoomToEventCmd := cmd (zoom_to_event, "ZoomTo");

function mouse_user_coords(c, e, x, y) {
    *x = vcEvent.UserX (e);
    *y = vcEvent.UserY (e);
}

function pan(c, e) {
    user := vcVPCanvas.GetUserspace (c);
    center_x := (vgRect.Xmin (user) + vgRect.Xmax (user)) / 2;
    center_y := (vgRect.Ymin (user) + vgRect.Ymax (user)) / 2;
    declare mouse_x, mouse_y;
    mouse_user_coords (c, e, &mouse_x, &mouse_y);
    vgRect.Offset (user, mouse_x - center_x, mouse_y - center_y);
    record_zoom (c, e, user);
}

function full(c, e)
    vcVPCanvas.SetUserspace (c, GetFull (c));

function refresh(c, e)
    vcRegion.QueueForRefresh (vcEvent.Region (e));

vcBind.Set (bindings, '\#\middledown', vcGesture.StartCmd);

vcBind.Set (bindings, '\F5', RefreshCmd := cmd (refresh));
vcBind.Set (bindings, '\F6', PanCmd := cmd (pan, "Pan"));
vcBind.Set (bindings, '\F7', InCmd);
vcBind.Set (bindings, '\F8', OutCmd);
vcBind.Set (bindings, '\F9', ZoomCmd := cmd (start_zoom));
vcBind.Set (bindings, '\F0', FullCmd := cmd (full, "full"));

vcGesture.Set (gestures, vcGesture.Z, ZoomCmd);
vcGesture.Set (gestures, vcGesture.Line, ZoomCmd);
vcGesture.Set (gestures, vcGesture.O, OutCmd);

ViewMenu := vcWv.MakeMenu ("view",
                           vcWv.Pin (),
                           list ("In",      InCmd),
                           list ("Out",     OutCmd),
                           list ("Pan",     PanCmd),
                           list ("Full",    FullCmd),
                           list ("Refresh", RefreshCmd));

vcWv.Interrupts (fac, '\F5', '\F6', '\F7', '\F8', '\F9', '\F0');

function arrow(key, name, dx, dy) {
    cmd := vcWv.Cmd (fac, 
                     func (c, e) vgU.MoveCursorDelta (Viewport (c), dx, dy);,
                     name);
    vcBind.Set (bindings, key, cmd);
    vcCmd.EnableMacro (cmd, FALSE);
}

arrow ('\left', "1-left", -1, 0);
arrow ('\right', "1-right", 1, 0);
arrow ('\up', "1-up", 0, 1);
arrow ('\down', "1-down", 0, -1);

arrow ('\^\left', "10-left", -10, 0);
arrow ('\^\right', "10-right", 10, 0);
arrow ('\^\up', "10-up", 0, 10);
arrow ('\^\down', "10-down", 0, -10);
