/*
 *  $Revision: 1.18.1.1 $
 *
 *  Definition of the vcMacPanel interface.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 *
 */


/*
 * Builds a table composed of a label and a list of enums.
 */
function build_enum_widget(prompt, default)
{
    t := vcTable.Create (prompt, 1, 2);

    vcTable.Add (t, vcRegion.Create ("", vcLabel.fac, prompt));
    vcTable.Add (t, vcRegion.Create ("", vcPickEnum.fac, default));

    t;
}


/*
 * Facility's _Create method.  Note that the new region is made the child of
 * the region passed in from Core.
 */
function _Create(event, region)
{
    main_table := vcTable.Create ("vcMacPanel", 2, 1);

    /*
     * Populate the first row:
     *
     * ----------------------------------
     * | vcLabel        | vcFilePop     |
     * ----------------------------------
     */
    row1 := vcTable.Create ("0:0", 1, 2);
    vcTable.Add (row1, vcRegion.Create ("", vcLabel.fac, "Macro filename:"));
    vcTable.Add (row1, vcRegion.Create ("", vcFilePop.fac, "",
                                        filter <- "*.vmc"));
    vcTable.Add (main_table, row1);

    /*
     * Populate the second row.
     *
     * --------------------------------------------------
     * |            | vcLabel   vcPickEnum  |           |
     * | vcIcon     | vcLabel   vcPickEnum  | vcIcon    |
     * |            | vcLabel   vcLabel     |           |
     * --------------------------------------------------
     */
    row2 := vcTable.Create ("1:0", 1, 3);
    vcTable.Col (row2, -1, vcTable.Border.Visible, 20, 20, 20, vgAlign.None);

    vcTable.Add (row2, vcRegion.Create ("", vcIcon.fac, "tapedck3"));
    
    coordinates := vcTable.Create ("", 3, 1);
    vcTable.Add (coordinates, build_enum_widget ("Coordinate units",
                                                 vcMacUserPhys.User));
    vcTable.Add (coordinates, build_enum_widget ("Origin point",
                                                 vcMacAbsRel.Absolute));
    title := vcRegion.Create ("", vcLabel.fac,
                              sprintf ("Current window: %s",
                                       vcWin.Title (vcEvent.Window (event))));
    vcTable.Add (coordinates, title);
    vcTable.Add (row2, coordinates);

    vcTable.Add (row2, vcRegion.Create ("", vcIcon.fac, "tapedck3"));

    vcTable.Add (main_table, row2);

    vcTable.Row (main_table, -1, vcTable.Border.Visible, 20, 20, 20,
                 vgAlign.None);
    vcTable.Col (main_table, -1, vcTable.Border.Visible, 20, 20, 20,
                 vgAlign.None);

    vcRegion.AddChild (region, main_table);

    struct {t = main_table; rec_timer = nil; rec_count = nil;
            exec_timer = nil; exec_count = nil; original_event = event;
            play_window = nil};
}


/*
 * Dynamic routine for the `macro' command to find out the value of the units
 * enum displayed in the Macro System Panel window.
 */
function get_units(data)
{
    coordinates := vcTable.Get (vcTable.Get (data.t, 1, 0), 0, 1);
    unit_widget := vcTable.Get (vcTable.Get (coordinates, 0, 0), 0, 1);
    
    vcRegion.Get (unit_widget, vcMacUserPhysVT);
}


/*
 * Dynamic routine for the `mrecord' command to find out the value of the origin
 * enum displayed in the Macro System Panel window.
 */
function get_origin(data)
{
    coordinates   := vcTable.Get (vcTable.Get (data.t, 1, 0), 0, 1);
    origin_widget := vcTable.Get (vcTable.Get (coordinates, 1, 0), 0, 1);

    vcRegion.Get (origin_widget, vcMacAbsRelVT);
}


/*
 * Action routine for the "mrecord" command which starts macro recording.  Note
 * that this function checks if there isn't a macro file with the same name
 * somewhere in the WDIR path.  If there is such a file, the user is prompted
 * whether the file can be overwritten or not.  Note also that the accept
 * action is written as a command, whereas the reject action is written as a
 * function.  This is legal and the vcWv.LabelButtonCreate function sorts
 * between the two modes.  It is always recommended to use the command-based
 * approach because it works well with macros.  In this case, the macro
 * "recordability" is not an issue.
 */
function mrecord(data, event, macro, units, origin)
{
    declare filename, parsed_macro, accept_cmd;

    parsed_macro = vcWin.parse_macro_filename (macro);

    function accept(d, e)
    {
        /*
         * Disable recording while recording.
         */
        vcCmd.Enable (mrecord_aux_cmd, vlOnOff.Off);
        vcCmd.Enable (vcWin.macro_cmd, vlOnOff.Off);
        vcCmd.Enable (mstop_cmd,        vlOnOff.On);

        vcWin.UpdateMenus (vcEvent.Window (e));

        /*
         * If there is an existing macro file, the `filename' variable will
         * contain the full pathname which should be used.  Otherwise, the
         * `macro' input argument will contain the filename.
         * NOTE the use of the local closure property for hanging on to the
         * original `event' object, rather than the one that will be passed in
         * to this function.  This one will contain information about the
         * window that will be created for the dialog box that prompts you for
         * an overwrite.  We want the original `event' structure, because we
         * need to have access to the window where the macro was started and,
         * possibly, the mouse coordinates for relative macro recording.
         */
        if (!null (filename))
            vcMac.Start (filename, data.original_event, units, origin);
        else
            vcMac.Start (macro, data.original_event, units, origin);

        vcWin.SetLastMacro (macro);
        vcCmdline.Set (vcTable.Get (vcTable.Get (data.t, 0, 0), 0, 1), macro);

        function flash()
        {
            if ((data.rec_count = data.rec_count + 1) > 3)
                data.rec_count = 1;

            new_icon := vcRegion.Create ("", vcIcon.fac,
                                         sprintf ("tapedck%d",
                                                  data.rec_count));

            vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 0, new_icon);
            vcRegion.Refresh (new_icon, vcRegion.VisibleArea (new_icon));
        }

        if (null (data.rec_timer))
        {
            data.rec_timer = vcWv.Timer (wv, 500, flash, TRUE);
            data.rec_count = 1;
            vcRegion.DestroyNotify (data.t,
                                    func(r) 
                                    if (!null (data.rec_timer))
                                    {
                                        vcWv.CancelTimer (data.rec_timer);
                                        data.rec_timer = nil;
                                    });
        }

        function stop_recording()
            mstop (data, event);

        vcMac.CallOnExit (stop_recording);
    }
    accept_cmd = vcWv.Cmd (fac, accept);


    /*
     * Check if there already exists a macro with the same name.  If it does,
     * then prompt the user if the macro can be overwritten.
     */
    if (!null (filename = vscript.vsGetini (parsed_macro)))
    {
        declare yes_but, no_but, buttons, query_t, str, text;

        buttons = vcRegion.Create ("Buttons", vcButnRow.fac,
            list (list ("yes", "Ok",
                        accept_cmd, list (close_window <- TRUE)),
                  list ("no", "Cancel",
                        func();, list (close_window <- TRUE))));

        query_t = vcTable.Create ("Overwrite?", 2, 1);
        
        str = sprintf ("Macro file %s already exists and will be overwritten.",
                       filename);
        text = vcRegion.Create ("over_text", vcStaticText.fac, str);
        vcTable.Add (query_t, text);
        vcTable.Add (query_t, buttons);

        vcWin.Popup (vcEvent.Window (event), query_t,
                     vcButnRow.GetDefault (buttons),
                     vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
    }
    else
        accept (data, event);

}
mrecord_cmd := vcWv.Cmd (fac, mrecord, "mrecord", "Begin Macro Recording",
                        vcWv.Param ("Macro name", vlFileWriteVT,
                                    vcWv.Sticky, vcWv.NoDefault),
                        vcWv.Param ("Coordinate units", vcMacUserPhysVT,
                                    vcWv.Dynamic, get_units, vcWv.Secondary,
                                    "Pixels or facility specific units"),
                        vcWv.Param ("Origin point", vcMacAbsRelVT,
                                    vcWv.Dynamic, get_origin, vcWv.Secondary,
                               "Lower left corner or current mouse position"));
vcCmd.EnableMacro (mrecord_cmd, vlOnOff.Off);


/*
 * Action routine for the auxillary mrecord command.  It tries to obtain the
 * argument values required by the real `mrecord' command.  If the values are
 * not available, the `mrecord' command will be called without them, thus
 * forcing Core to prompt the user to fill them in.
 */
function mrecord_aux(data, event)
{
    row1     := vcTable.Get (data.t, 0, 0);
    filename := vcRegion.Get (vcTable.Get (row1, 0, 1), vlStringVT);

    if (string_eq (filename, ""))
        vcCmd.Run (mrecord_cmd, data, event, nil, FALSE);
    else
    {
        row2 := vcTable.Get (data.t, 1, 0);
        coordinates := vcTable.Get (row2, 0, 1);
        unit_widget := vcTable.Get (vcTable.Get (coordinates, 0, 0), 0, 1);
        origin_widget := vcTable.Get (vcTable.Get (coordinates, 1, 0), 0, 1);

        units  := vcRegion.Get (unit_widget,   vcMacUserPhysVT);
        origin := vcRegion.Get (origin_widget, vcMacAbsRelVT);
        vcCmd.Run (mrecord_cmd, data, event, list (filename, units, origin),
                   FALSE);
    }
}
mrecord_aux_cmd := vcWv.Cmd (fac, mrecord_aux);
vcCmd.SetHelp (mrecord_aux_cmd, "Start macro recording");


/*
 * Action routine for the `mstop' command.  If the record timer is on, cancel
 * it.  Also, switch to the neutral version of the tape deck icon.
 */
function mstop(data, event)
{
    /*
     * Enable the recording command.
     */
    vcCmd.Enable (mrecord_aux_cmd, vlOnOff.On);
    vcCmd.Enable (vcWin.macro_cmd, vlOnOff.On);
    vcCmd.Enable (mstop_cmd,       vlOnOff.Off);
    if (vcWin.IsAlive (vcEvent.Window (event)))
        vcWin.UpdateMenus (vcEvent.Window (event));

    if (!null (data.rec_timer))
    {
        vcWv.CancelTimer (data.rec_timer);
        data.rec_timer = nil;

        /*
         * Make sure that it's the last of the three tape deck icons that is
         * showing when the macro stops recording.  The other two have the
         * word "Recording" which might confuse users.
         */
        icon := vcRegion.Create ("", vcIcon.fac, "tapedck3");
        vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 0, icon);
        vcRegion.Refresh (icon, vcRegion.VisibleArea (icon));
    }

    e := data.original_event;
    vcMac.Close (vcEvent.Window (e));

}
mstop_cmd := vcWv.Cmd (fac, mstop);
vcCmd.Enable (mstop_cmd, vlOnOff.Off);
vcCmd.SetHelp (mstop_cmd, "End macro recording");


/*
 * Package calls to enable/disable Record and Stop Rec commands.
 */
function turn_on_off(flag, win)
{
    if (flag == vlOnOff.On)
    {
        vcCmd.Enable (mrecord_aux_cmd,  flag);
        vcCmd.Enable (mstop_cmd,        vlOnOff.Off);
        vcCmd.Enable (mplay_aux_cmd,    flag);
        vcCmd.Enable (mcancel_cmd,      vlOnOff.Off);

    }
    else
    {
        vcCmd.Enable (mrecord_aux_cmd,  flag);
        vcCmd.Enable (mstop_cmd,        flag);
        vcCmd.Enable (mplay_aux_cmd,    vlOnOff.On);
        vcCmd.Enable (mcancel_cmd,      vlOnOff.On);
    }
    if (vcWin.IsAlive (win))
        vcWin.UpdateMenus (win);
}


/*
 * Action routine for the `mplay' command.  Note that the macro name is
 * saved in the LastMacro variable as a default for the `mrecord' command.
 */
function mplay(data, event, filename)
{
    function flash()
        if (vcMac.IsReplayingOn ())
        {
            if ((data.exec_count = data.exec_count - 1) < 3)
                data.exec_count = 5;

            new_icon := vcRegion.Create ("", vcIcon.fac,
                                       sprintf ("tapedck%d", data.exec_count));

            vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 2, new_icon);
            vcRegion.Refresh (new_icon, vcRegion.VisibleArea (new_icon));
        }
        else
        {
            if (!null (data.exec_timer))
                vcWv.CancelTimer (data.exec_timer);
            data.exec_timer = nil;
            new_icon := vcRegion.Create ("", vcIcon.fac, "tapedck3");

            vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 2, new_icon);
            vcRegion.Refresh (new_icon, vcRegion.VisibleArea (new_icon));
        }
                    
        
    if (null (data.exec_timer))
    {
        data.exec_timer = vcWv.Timer (wv, 500, flash, TRUE);
        data.exec_count = 5;
        vcRegion.DestroyNotify (data.t,
                                func(r)
                                    if (!null (data.exec_timer))
                                    {
                                        vcWv.CancelTimer (data.exec_timer);
                                        data.exec_timer = nil;
                                    });
    }

    /*
     * Turn off the Record and Stop Rec buttons while a macro is being played
     * out.  Note that we are using a different window, the one that was
     * stored in the facility's data structure in the mplay_aux command.
     * Note also the trick with the lexical scoping in the turn_on function.
     */
    turn_on_off (vlOnOff.Off, data.play_window);

    function turn_on()
        turn_on_off (vlOnOff.On, data.play_window);

    vcMac.CallOnExit (turn_on);

    /*
     * Execute the macro.
     */
    win := vcEvent.Window (event);
    vcWin.RunCmdline (win, "mexecute " + filename, event);

    if (vcWin.IsAlive (win))
    {
        vcWin.SetLastMacro (filename);
        vcCmdline.Set (vcTable.Get (vcTable.Get (data.t, 0, 0), 0, 1),
                       filename);
    }
}
mplay_cmd := vcWv.Cmd (fac, mplay, "mplay", "Play macro",
                          vcWv.Param ("Macro name:", vlFileNameVT,
                                      vcWv.Dynamic, vcWin.GetLastMacro));

vcCmd.EnableMacro (mplay_cmd, vlOnOff.Off);

/*
 * Action routine for the auxillary `mplay' command.  It queries the file
 * browser for a macro filename.  If there is one, run the real `play' command
 * with that filename.  Otherwise, run the command without arguments, thus
 * forcing Core to prompt the user for the filename.
 *
 * Note that because we are passing the original event to the mplay command,
 * not the one that was used to invoke *this* command AND we want to use the
 * turn_on_off function, we need to save the current event in the facility's
 * data structure.  Actually, we only need the current window.
 */
function mplay_aux(data, event)
{
    row1     := vcTable.Get (data.t, 0, 0);
    filename := vcRegion.Get (vcTable.Get (row1, 0, 1), vlStringVT);

    /*
     * Save the current window for use by the mplay command.
     */
    data.play_window = vcEvent.Window (event);

    if (string_eq (filename, ""))
        vcCmd.Run (mplay_cmd, data, data.original_event, nil, TRUE);
    else
        vcCmd.Run (mplay_cmd, data, data.original_event, list (filename),
                   TRUE);
}
mplay_aux_cmd := vcWv.Cmd (fac, mplay_aux);
vcCmd.SetHelp (mplay_aux_cmd, "Replay a macro");    

/*
 * Action routine for the Cancel button.  Aside from stoping the macro
 * playback with a call to vcMac.StopExec, check if the exec timer doesn't
 * need to be stopped.
 */
function mcancel(data, event)
{
    if (!null (data.exec_timer))
    {
        vcWv.CancelTimer (data.exec_timer);
        data.exec_timer = nil;

        new_icon := vcRegion.Create ("", vcIcon.fac, "tapedck3");

        vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 2, new_icon);
        vcRegion.Refresh (new_icon, vcRegion.VisibleArea (new_icon));
    }

    if (!null (data.rec_timer))
    {
        vcWv.CancelTimer (data.rec_timer);
        data.rec_timer = nil;

        icon := vcRegion.Create ("", vcIcon.fac, "tapedck3");
        vcTable.Set (vcTable.Get (data.t, 1, 0), 0, 0, icon);
        vcRegion.Refresh (icon, vcRegion.VisibleArea (icon));
    }

    turn_on_off (vlOnOff.On, vcEvent.Window (event));
    vcMac.StopExec ();
}
mcancel_cmd := vcWv.Cmd (fac, mcancel);
vcCmd.Enable (mcancel_cmd, vlOnOff.Off);
vcCmd.SetHelp (mcancel_cmd, "Cancel macro playback");


menus := vcWv.MakeMenu ("vcMacPanel",
                        vcWin.RedSquareEntryNoPlotNoQuit,
                        list ("&Record",     mrecord_aux_cmd),
                        list ("&Stop",       mstop_cmd),
                        list ("&Play",       mplay_aux_cmd),
                        list ("&Cancel",     mcancel_cmd));

vcFac.SetMenus (fac, menus);
