/*
 *  $Revision: 1.11.6.1 $
 *
 *  Translation table for the vcMacTrans facility.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 *
 *  This is a companion file to the vcMacTrans facility.  It contains the
 *  translation table used to translate the 4.x command.  The table is
 *  implemented as a hash table, so the search time is constant, no matter the
 *  number of entries.  Only the time to load this file increases with the
 *  size of the table.
 *
 *  This file is loaded by the `trans' command defined in the vcMacTrans.vs
 *  file.  It will be loaded only once, when the user issues the command for
 *  the first time.  Subsequent translations will be faster, because the table
 *  will have been instantiated.
 */


/*
 *
 *  This is the command translation table.  Each entry is of the format
 *  described below.  Note that the table matches a string into a ViewScript
 *  object which happens to be a list of 4 elements.  If you ever decide that
 *  4 elements just doesn't cut it, add the new elements at the end of the
 *  list, so that you don't have to modify the translator's engine too much.
 *  See the comments in the vcMacTrans.vs file on how the translator is
 *  structured.
 *
 *  Note also that the hash table is dynamic, i.e. it will grow according to
 *  the need, but not through the expansion of buckets, but by dynamically
 *  resizing itself to keep the number of buckets to maximum of 2.
 *
 *  old_name        - 4.x command name
 *  new_name        - Powerview command name
 *                    If nil, ignore the 4.x command.
 *  nargs           - required 4.x arguments
 *                    If -1, the next argument is a procedure pointer to a
 *                    special case function to be called.
 *  opt_args        - a list of additional arguments required by the Powerview
 *                    command.
 *                    If the first element is -1, the 4.x args are put after
 *                    the optional arguments.
 *  line_action     - action list for each of the 4.x lines that might follow
 *                    a 4.x command:
 *                    IGNORE  -> ignore the line
 *                    APPEND  -> append the line to the command string
 *                    "50cmd" -> additional Powerview command to be added
 *                               after the 4.x command has been translated 
 *
 *  `nargs' tells the translator for how many arguments it should look for in
 *  the 4.x macro file.  An argument is defined as any string that doesn't
 *  represent a key stroke or a coordinate.  Remember that a 4.x command with
 *  two arguments, for examples, can be recorded legally in three different
 *  configurations: everything on one line, everything on separate lines
 *  (three lines), a mix (two lines).  The translator handles all of these
 *  scenarios and any key strokes that might be recorded in between.
 *
 *  If `nargs' is -1, it means that there is a special processor function
 *  written to handle the command.  This is done when the 4.x command is very
 *  complicated and/or requires some extra processing.  If you add a special
 *  processor, there is a section in the vcMacTrans.vs file where you can add
 *  it.  It's recommended to prefix the function name with `process_' to keep
 *  things consistent.
 *
 *
 *  `opt_args' is a list of strings which represent additional command
 *  arguments that are required in Powerview.  Those arguments will be
 *  appended at the end of the command strings, unless, the first element in
 *  the list is -1.  See the "attr" command for an example of this scenario.
 *
 *
 *  `line_actions' is a list of integers and/or strings.  A 4.x command might
 *  have some lines always recorded after the command.  For each of those
 *  possible lines, an action item is presented in this list.  IGNORE skips
 *  the line, APPEND appends the line to the command string, and a string
 *  stands for a Powerview command that must always follow.  See the "attr"
 *  command for an example.
 */


/*
 *  NOTES:
 *
 *  check   - see write below.
 *  color   - cannot translate the argument.  Powerview will put up a dialog
 *            box asking you to choose the color. (Can hard code a color in
 *            the Powerview macro).
 *  write   - leaves the `check' info window, if check detects errors.
 */

Table = vlSym.StrCreate (101);


function tadd(old, new, nargs, opt_args, line_actions)
    vlSym.Store (Table, old,
                 list (new, nargs, opt_args, line_actions));


tadd ("]1",         "", -1, process_right_butt, nil);
tadd ("]4",         "", -1, process_left_butt, nil);
tadd ("$absolute",  nil, nil, nil, nil);
tadd ("ainvis",     "ainvis", 0, list ("*"), nil);
tadd ("ainvi",      "ainvis", 0, list ("*"), nil);
tadd ("alist",      "alist", 0, nil, nil);
tadd ("annoinit",   "annoinit", 1, nil, nil);
tadd ("annooff",    "annoff", 0, nil, nil);
tadd ("annoon",     "annoon", 1, nil, nil);
tadd ("annostep",   "annstep", 0, nil, nil);
tadd ("anvis",      "anvis", 0, nil, nil);
tadd ("aoff",       "aoff", 0, nil, nil);
tadd ("aon",        "aon", 0, nil, nil);
tadd ("aroute",     "aroute", 0, nil, nil);
tadd ("asize",      "asize", 0, nil, nil);
tadd ("a0size",     "a0size", 0, nil, nil);
tadd ("a1size",     "a1size", 0, nil, nil);
tadd ("a2size",     "a2size", 0, nil, nil);
tadd ("a3size",     "a3size", 0, nil, nil);
tadd ("a4size",     "a4size", 0, nil, nil);
tadd ("attr",       "_attributeobject", 1,
                    list (-1, "Visible", "No", "End_Command"),
                    list ("MoveExecute"));
tadd ("attrib",     "_attributeobject", 1,
                    list (-1, "Visible", "No", "End_Command"),
                    list ("MoveExecute"));
tadd ("avis",       "avisibility", 0, list ("*", "Visible", "All"), nil);
tadd ("avvis",      "avvis", 1, nil, nil);
tadd ("avvi",       "avvis", 1, nil, nil);
tadd ("bannotate",  "btype", 0, list ("Annotate"), nil);
tadd ("bcomposite", "btype", 0, list ("Composite"), nil);
tadd ("bmodule",    "btype", 0, list ("Module"), nil);
tadd ("boff",       "boff", 0, nil, nil);
tadd ("bon",        "bon", 0, nil, nil);
tadd ("bpin",       "btype", 0, list ("Pin"), nil);
tadd ("bsize",      "bsize", 0, nil, nil);
tadd ("cc",         "_ccomp", 1, nil, nil);
tadd ("ccomp",      "_ccomp", 1, nil, nil);
tadd ("check",      "check", 0, nil, list (IGNORE));/* Leaves check window */
tadd ("cinst",      "cinst", 1, nil, nil);
tadd ("cleanup",    "cleanup", 0, nil, nil);
tadd ("clrdeflib",  nil, nil, nil, nil);
tadd ("coff",       "coff", 0, nil, nil);
tadd ("color",      "color", 0, nil, list (IGNORE));
tadd ("con",        "con", 0, nil, nil);
tadd ("comp",       "comp", 1, nil, list ("MoveExecute"));
tadd ("copy",       "copy", 0, nil, nil);
tadd ("csize",      "csize", 0, nil, nil);
tadd ("ctoff",      "ctoff", 0, nil, nil);
tadd ("cton",       "cton", 0, nil, nil);
tadd ("dboxoff",    nil, nil, nil, nil);
tadd ("dboxon",     nil, nil, nil, nil);
tadd ("d",          "delete", 0, nil, nil);
tadd ("del",        "delete", 0, nil, nil);
tadd ("delete",     "menu-delete", 0, nil, nil);
tadd ("dir",        "dir", 0, nil, nil);
tadd ("dirsch",     "dirsch", 0, nil, nil);
tadd ("dirsym",     "dirsym", 0, nil, nil);
tadd ("doff",       "doff", 0, nil, nil);
tadd ("don",        "don", 0, nil, nil);
tadd ("dsize",      "dsize", 0, nil, nil);
tadd ("esize",      "esize", 0, nil, nil);
tadd ("fstyle",     "fstyle", 0, nil, nil);     /* Currently disabled */
tadd ("global",     "global", 0, nil, nil);
tadd ("goff",       "goff", 0, nil, nil);
tadd ("gon",        "gon", 0, nil, nil);
tadd ("hoff",       "hoff", 0, nil, nil);
tadd ("hon",        "hon", 0, nil, nil);
tadd ("label",      "labelobject", 1,
                    list ("No", "Visible", "Local", "NotInverted"),
                    list ("MoveExecute"));
tadd ("linvis",     "linvis", 0, list ("*"), nil);
tadd ("llist",      "llist", 0, nil, nil);
tadd ("local",      "local", 0, nil, nil);
tadd ("loff",       "loff", 0, nil, nil);
tadd ("lon",        "lon", 0, nil, nil);
tadd ("lvis",       "lvis", 0, nil, nil);
tadd ("mc",         nil, nil, nil, nil);
tadd ("mcl",        nil, nil, nil, nil);
tadd ("mclose",     nil, nil, nil, nil);
tadd ("mex",        "mexecute", -1, process_mex, nil);
tadd ("mexec",      "mexecute", -1, process_mex, nil);
tadd ("mexecute",   "mexecute", -1, process_mex, nil);
tadd ("nainvi",     "nainvis", 0, nil, nil);
tadd ("name",       "name", 1, nil, nil);
tadd ("nanv",       "nanvis", 0, nil, nil);
tadd ("navi",       "navi", 0, nil, nil);
tadd ("navvis",     "navvis", 0, nil, nil);
tadd ("noff",       "noff", 0, nil, nil);
tadd ("non",        "non", 0, nil, nil);
tadd ("ocount",     "ocount", 0, nil, nil);
tadd ("odetail",    "odetail", 0, nil, nil);
tadd ("oroute",     "oroute", 0, nil, nil);
tadd ("$physical",  nil, nil, nil, nil);
tadd ("pinnum",     "pinnum", 1, nil, nil);
tadd ("poff",       "poff", 0, nil, nil);
tadd ("pon",        "pon", 0, nil, nil);
tadd ("pop",        "_pop", 0, list ("1"), nil);
tadd ("$prompt",    nil, nil, nil, nil);
tadd ("$PROMPT",    nil, nil, nil, nil);
tadd ("psch",       "psch", 0, nil, nil);
tadd ("psh",        "psheet", 0, nil, nil);
tadd ("psheet",     "psheet", 0, nil, nil);
tadd ("psy",        "psym", 0, nil, nil);
tadd ("psym",       "psym", 0, nil, nil);
tadd ("q",          "quit", 0, list ("yes"), nil);
tadd ("quit",       "quit", 0, list ("yes"), nil);
tadd ("read",       "read", 0, nil, nil);
tadd ("refdes",     "refdes", 1, nil, nil);
tadd ("refresh",    "refresh", 0, nil, nil);
tadd ("$relative",  nil, nil, nil, nil);
tadd ("roff",       "roff", 0, nil, nil);
tadd ("ron",        "ron", 0, nil, nil);
tadd ("sattr",      "sattr", 0, list ("*", "All"), nil);
tadd ("save",       "save", 0, nil, nil);
tadd ("scomp",      "scomp", 1, nil, nil);
tadd ("sgroup",     "", -1, process_sgroup, nil);
tadd ("slab",       "slabel", 1, list ("All"), nil);
tadd ("slabel",     "slabel", 1, list ("All"), nil);
tadd ("slot",       "slot", 1, nil, nil);
tadd ("sname",      "sname", 1, nil, nil);
tadd ("soff",       "soff", 0, nil, nil);
tadd ("son",        "son", 0, nil, nil);
tadd ("sortoff",    "sortoff", 0, nil, nil);
tadd ("sorton",     "sorton", 0, nil, nil);
tadd ("sp_spoolit", nil, nil, nil, nil);
tadd ("spwplot",    nil, nil, nil, nil);
tadd ("sroute",     "sroute", 0, nil, nil);
tadd ("stext",      "stext", 0, list ("*", "All"), nil);
tadd ("string",     "_string", 1,
                    list (-1, "String", "No", "Apply_Full_String"), nil);
tadd ("svalue",     "svalue", 1, list ("All"), nil);
tadd ("sys",        "", -1, process_sys_line, nil);
tadd ("system",     "", -1, process_sys_line, nil);
tadd ("text",       "text", 1, nil, list ("MoveExecute"));
tadd ("toff",       "toff", 0, nil, nil);
tadd ("ton",        "ton", 0, nil, nil);
tadd ("uoff",       "uoff", 0, nil, nil);
tadd ("uon",        "uon", 0, nil, nil);
tadd ("unoff",      "unoff", 0, nil, nil);
tadd ("unon",       "unon", 0, nil, nil);
tadd ("value",      "value", 1, nil, nil);
tadd ("$virtual",   nil, nil, nil, nil);
tadd ("voff",       "voff", 0, nil, nil);
tadd ("von",        "von", 0, nil, nil);
tadd ("wpop",       nil, nil, nil, nil);
tadd ("wpush",      nil, nil, nil, nil);
tadd ("wri",        "write", 0, nil, list (IGNORE));/* Leaves check window */
tadd ("write",      "write", 0, nil, list (IGNORE));/* Leaves check window */
tadd ("writenew",   "writenew", 1, nil, nil);
tadd ("writeto",    "_writeto", 1, nil, nil);
tadd ("wrn",        "wrnodes", 1, nil, list (IGNORE));
tadd ("wrnode",     "wrnodes", 1, nil, list (IGNORE));
tadd ("wrnodes",    "wrnodes", 1, nil, list (IGNORE));
tadd ("wte",        "wtext", 1, nil, nil);
tadd ("wtext",      "wtext", 1, nil, nil);
tadd ("zsel",       "zselect-menu", 0, nil, nil);
tadd ("zselect",    "zselect-menu", 0, nil, nil);
