
function _Create (init, rgn) {
    t := vcTable.Create ("", 10, 3);
    s := vcRegion.Create ("", vcScroll.fac, t, 
                     show_border <- TRUE,
                     inner_border <- TRUE,
                     vertical <- vcScroll.Presence.Always);
    vcRegion.AddChild (rgn, s);

    ae := struct {table = t;
            rgn = rgn;
            class;
            Options = vcRegion.PrefGet (rgn, "attrs");
            attrs
           };
    set (ae, init);
    ae;
}

struct AttrVT {vlStringVT name; vcRegionVT r; vlTypespecVT ts};

function set_class (ae, class) {
if (!equal (class, ae.class)) {
    /* first, clean up the old table */
    declare rows, cols;
    vcTable.Dimensions (ae.table, &rows, &cols);
    vcTable.DeleteRows (ae.table, 0, rows);
    ae.attrs = nil;
    if (class != nil) {
        foreach (name, ts, deflt, fac, prefs; ae.Options (class)) {
            label := vcRegion.Create ("", vcLabel.fac, name + ":");
            thefac := (null (fac)) ? (vcCmd.TsToFac (ts)).fac : fac;

            editor := vcRegion.Create1 ("", thefac, deflt, prefs);
            vcTable.Add (ae.table, label);
            vcTable.Add (ae.table, editor);
            vcTable.Add (ae.table, vcRubber.Create ("", 0, 0));
            a := struct AttrVT {name, editor, ts};
            ae.attrs = cons (a, ae.attrs);
        }
    vcTable.Row (ae.table, vcTable.All, vcTable.Border.Invisible,
                     1, 2, 4, vgAlign.UpperLeft);
    vcTable.Col (ae.table, vcTable.All, vcTable.Border.Invisible,
                     1, 2, 4, vgAlign.UpperLeft);
    vcRegion.RequestSize (ae.rgn);
    ae.class = class;
    }
}
}
     
function SetClass (rgn, class){
    ae := vcRegion.GetData (rgn);
    set_class (ae, class);
}

function _Get (ae, rgn, ts) {
    declare ret;
    if (ts == vlStringVT) {
        ret = ae.class;
        foreach (elem; vscript.iter (ae.attrs)) {
            val := vcRegion.Get (elem.r, vlStringVT);
            if (!equal (val, ""))
                ret = ret + "\n" + elem.name + "\n" + val;
        }
    } else {
        foreach (elem; vscript.iter (ae.attrs)) {
            val := vcRegion.Get (elem.r, elem.ts);
            ret = cons (cons (elem.name, val), ret);
        }
        ret = cons (ae.class, ret);
    }
        
    ret;
}

/* set the value in the attributes, given the list of strings */
function set (ae, attr) {
    /* if string, convert it to list form */
    if (istype (attr, vlStringVT) && !equal (attr, "")) {
        declare token, l;
        attr = string_token (attr, &token, "\n");
        class := token;
        while (attr != nil) {
            attr = string_token (attr, &token, "\n");
            name := token;
            if (attr != nil) {
                attr = string_token (attr, &token, "\n");
                l = cons (cons (name, token), l);
            }
        }
        attr = cons (class, l);
    }

    if (!null (attr) && !equal (attr, "")) {
        set_class (ae, car (attr));
        /* compare all combinations of our new settings and available 
         * attributes. For each match, set the region's value.
         */
        foreach (elem; vscript.iter (cdr (attr)))
            foreach (attr; vscript.iter (ae.attrs))
                if (equal (car (elem), attr.name))
                    vcRegion.Set (attr.r, cdr (elem));
    }
}

function Set(rgn, attr)
    set (vcRegion.GetData (rgn), attr);
    
function _Size (ae, sz, r, wv)
    vgSize.Set (sz, 300, 100, 200, 100, 600, 400);

vlPref.Replace (prefs, "attrs", vscmVT, "~~iterator to list options", nil);
                           




