/*
 * $Id: vflCanvas.vs,v 1.7.1.1 1995/04/13 17:42:32 akoren Stab $
 * 
 * Wrapper around vge specific for viewflow.
 */

function _Create (mf, rgn) {
    vge_rgn := vcRegion.Create ("", vge.fac, mf);
    vcRegion.SetBindings (vge_rgn, vcBind.Create ());
    vcRegion.AddChild (rgn, vge_rgn);
    vcRegion.ViewportForward (vge_rgn, rgn);

    /* turn off the obnoxious vpcanvas gestures */
    canvas := vge.GetCanvas (vge_rgn);
    b := vcBind.Duplicate (vcRegion.Bindings (canvas));
    vcBind.Set (b, '\#\middledown', nil);
    vcRegion.SetBindings (canvas, b);

    struct {
        vge_rgn = vge_rgn;
        vpcanvas = vcRegion.FindFac (vge_rgn, vcVPCanvas.fac);
        draw = vge.GetDrawing (vge_rgn);
        rgn = rgn;
        call_on_start_drag;
        call_on_complete;        /* used with modes */
        call_on_query_drop;
        call_on_drop;
        call_on_activate;
        call_on_popup;
        first_event    /* used with vcWv.ClickTogether */
    };
}

function Drawing(rgn) {
    vg := vcRegion.GetData (rgn);
    vg.draw;
}

function Display(rgn) {
    vg := vcRegion.GetData (rgn);
    vcRegion.GetData (vg.vge_rgn);
}

function VPCanvas(rgn){
    vg := vcRegion.GetData (rgn);
    vg.vpcanvas;
}
function grab(vg, b) {
    vcRegion.SetGrab (vg.rgn, vg.rgn);
    vcRegion.SetBindings (vg.rgn, b);
}

function ungrab (vg) {
    vcRegion.SetGrab (vg.rgn, nil);
    vcRegion.SetBindings (vg.rgn, bindings);
}

function CallOnStartDrag(rgn, proc) {
    vg := vcRegion.GetData (rgn);
    vg.call_on_start_drag = proc;
}

function CallOnActivate(rgn, proc) {
    vg := vcRegion.GetData (rgn);
    vg.call_on_activate = proc;
}

function CallOnPopup(rgn, proc) {
    vg := vcRegion.GetData (rgn);
    vg.call_on_popup = proc;
}

function activate(vg, ev) {
    x := vcEvent.UserX (ev);
    y := vcEvent.UserY (ev);
    declare o;
    catch (vlEx.FAILURE) {
        o = vge.DrawFindObjectXY (vge.GetDrawing (vg.vge_rgn), x, y);
    } exception {
        w := vcEvent.Window (ev);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, sprintf ("There's no object at (%d,%d)", x, y));
    }
    if (vg.call_on_activate != nil && o != nil)
        vg.call_on_activate (o, ev);
}
ActivateCmd := vcWv.Cmd (fac, activate, "Activate");

function popup(vg, ev) {
    x := vcEvent.UserX (ev);
    y := vcEvent.UserY (ev);
    declare o;
    catch (vlEx.FAILURE) {
        o = vge.DrawFindObjectXY (vge.GetDrawing (vg.vge_rgn), x, y);
    } exception {
        w := vcEvent.Window (ev);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, sprintf ("There's no object at (%d,%d)", x, y));
    }
    /* select it; this makes things easier to deal with later */
    vge.DrawUnselectAll (vge.GetDrawing (vg.vge_rgn));
    if (o != nil)
        vge.DrawSelectObject (vge.GetDrawing (vg.vge_rgn), o);
    if (vg.call_on_popup != nil && o != nil)
        vg.call_on_popup (o, ev);
}
PopupCmd := vcWv.Cmd (fac, popup, "Popup");

function start_drag (vg, ev) {
    vp := vge.GetViewport (vg.vge_rgn);
    x := vcEvent.X (ev);
    y := vcEvent.Y (ev);
    vgU.PixelToUser (vp, &x, &y);
    errstring := nil;

    catch (vlEx.FAILURE) {
        o := vge.DrawFindObjectXY (vg.draw, x, y);
        data := vge.ObjectGetData (o);

        icon := nil;
        if (!null (vg.call_on_start_drag))
            icon = vg.call_on_start_drag (data);
        if (null (icon))
            errstring = sprintf ("Unable to drag object %s.", 
                                       vge.ObjectName (o));
        else {
            d := vcWv.StartDrag (wv, data, "motif_head");
            vcWv.ModifyDrag (d, erase_color <- vgAllColor.Background);
        }
    } exception {
        errstring := sprintf ("There's no object at (%d,%d)", x, y);
        d := vcRegion.GetData (vcWin.Region (vcEvent.Window (ev)));
        vcCmd.Run (vcGesture.StartCmd, d, ev, nil, FALSE);
    }
    if (!null (errstring)) {
        w := vcEvent.Window (ev);
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, errstring);
    }
}
DragCmd := vcWv.Cmd (fac, start_drag, "Gesture/Drag");

/* this allows you to enter a mode, from which you may finish or cancel */
ModeBindings := vcBind.Create ();

function EnterMode(rgn, proc, helpstr) {
    vg := vcRegion.GetData (rgn);
    vg.call_on_complete = proc;
    w := vcRegion.Window (rgn);
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, helpstr);

    grab (vg, ModeBindings);
    vcRegion.SetCursor (vg.rgn, vgCursor.UpArrow);
}

function cancel_mode (vg, ev) {
    vcWin.ClearStatus (vcEvent.Window (ev));
    ungrab (vg);
    vcRegion.SetCursor (vg.rgn, vgCursor.Default);
}
CancelCmd := vcWv.Cmd (fac, cancel_mode, "Cancel");

function complete_mode (vg, ev) {
    cancel_mode (vg, ev);
    vg.call_on_complete (ev);
}
CompleteCmd := vcWv.Cmd (fac, complete_mode, "Complete");
vcBind.Set (ModeBindings, '\middledown', CompleteCmd);
vcBind.Set (ModeBindings, '\rightdown', CancelCmd);

function _QueryDrop (vg, ev, data, onoff) {
    declare ret;
    if (vg.call_on_query_drop != nil && 
        vg.call_on_query_drop (data))
        ret = vcWvDropAccept.YesAll;
    else
        ret = vcWvDropAccept.No;
}

function _CompleteDrop (vg, ev, data)
   if (!null (vg.call_on_drop))
       vg.call_on_drop (ev, data);

function CallOnQueryDrop (r, proc) {
    vg := vcRegion.GetData (r);
    vg.call_on_query_drop = proc;
}

function CallOnDrop(r, proc) {
    vg := vcRegion.GetData (r);
    vg.call_on_drop = proc;
}

vcBind.Set (bindings, 's', vge.StretchCmd);
    
vcBind.Set (bindings, '\leftdown',  vge.SelectCmd);
vcBind.Set (bindings, '\^\leftdown', vge.SelectAddCmd);
vcBind.Set (bindings, '\#\middledown', vcGesture.StartCmd);
vcBind.Set (bindings, '\middledown', vcGesture.StartCmd);
vcBind.Set (bindings, '\leftdbl', ActivateCmd);
vcBind.Set (bindings, '\rightdown', PopupCmd);


vcGesture.Set (gestures, vcGesture.Z,       vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.Line,    vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.UpsideDownCheck, vcVPCanvas.FullCmd);
vcGesture.Set (gestures, vcGesture.P,       vcVPCanvas.PanCmd);



