/*
 * $Revision: 1.5.1.1 $
 *
 * Viewflow global symbl editor
 */


struct EntryVT {
    vcRegionVT name, value;
    vscmVT sym;
    vlStringVT help;
};

AlternatePicklistBindings := vcBind.Duplicate (vcPicklist.bindings);
vcBind.Set (AlternatePicklistBindings, '\enter', nil);

function create_entry (sym, n, v, s, hlp) {
    fill := vcRegion.Create ("", vcFstyle.fac, vgFill.X1);
    if (sym.mode == vflowMode.Edit)
        name := vcRegion.Create ("", vcCmdline.fac, n, max_size <- 0);
    else
        name := vcRegion.Create ("", vcStaticText.fac, n);

    readonly := (sym.mode == vflowMode.Run && s == vflGsymType.Constant);
    if (readonly) {
        value := vcRegion.Create ("", vcStaticText.fac, v);
        vcRegion.Font (value, vgFont.medium);
    } else
        value := vcRegion.Create ("", vcCmdline.fac, v,  max_size <- 0);
        
    if (sym.mode == vflowMode.Edit) { 
        symr := vcRegion.Create ("", vcPickEnum.fac, s);
        p := vcRegion.FindFac (symr, vcPicklist.fac);
        vcRegion.Font (p, vgFont.tiny);
        vcRegion.SetBindings (p, AlternatePicklistBindings);
    } else {
        r := vcRegion.Create ("", vcLabel.fac, convert (vlStringVT, s));
        vcRegion.Font (r, vgFont.tinyb);
        symr := vcRegion.Create ("", vcBorder.fac, r,
                           border_style <- vcBorderStyle.EtchedOut);
    }

    tab := vcTable.Make ("", 1, 4, fill, name, value, symr);
    e := struct EntryVT {name, value, symr, hlp};
    if (sym.mode == vflowMode.Run)
        e.sym = s;

    /* now align columnds */
    declare rows, cols;
    vcPicklist.Dimensions (sym.pick, &rows, &cols);
    tablist := list (sym.header, tab);
    foreach (r, c, ix, set, table, d; vcPicklist.Items (sym.pick))
        tablist = cons (table, tablist);
    vcTable.AlignColumns (tablist);
    list (tab, e);
}

function create_empty_entry (sym)
    create_entry (sym, "", "", vflGsymType.Constant, "");

function add_entry (sym) {
    <tab, e> := create_empty_entry (sym);
    vcPicklist.Add (sym.pick, tab, e);
}

function insert_entry (sym, ix) {
    <tab, e> := create_empty_entry (sym);
    vcPicklist.InsertRows (sym.pick, ix, 1);
    vcPicklist.Set (sym.pick, tab, e, ix, 0);
}

iterator symbol_entries(l)
    foreach (elem; vscript.list_iter (l))
        iterate (car (elem), cadr (elem), caddr (elem), cadddr (elem));

function _Create (init, rgn) {
    p := vcRegion.Create ("", vcPicklist.fac, nil, 
                          columns <- 1, will_scroll <- TRUE);
    vcRegion.SetBindings (p, AlternatePicklistBindings);

    s := vcPicklist.Scroll ("", p, 4, 7, 12);
    header := vcTable.Make ("", 1, 4, 
                 vcRubber.Create ("", 0, 0),
                 vcRegion.Create ("", vcStaticText.fac, "Symbol"),
                 vcRegion.Create ("", vcStaticText.fac, "Value"),
                 vcRegion.Create ("", vcStaticText.fac, "Type"));
    table := vcTable.Make ("", 2, 1, header, s);

    mode := vcRegion.PrefGet (rgn, "mode");
    sym := struct {
        rgn = rgn;
        pick = p;
        mode = mode;
        header = header
    };

    if (init == nil && mode == vflowMode.Edit)
        add_entry (sym);
    else
        foreach (n, v, s, hlp; symbol_entries (init)) {
            <tab, e> := create_entry (sym, n, v, s, hlp);
            vcPicklist.Add (p, tab, e);
        }

    vcRegion.AddChild (rgn, table);

    enable_commands (sym);
    vcPicklist.Notify (p, func (d, flg) enable_commands (sym););

    function highlight (entry, flg, ix)
        show_help (rgn, entry);
    vcPicklist.HighlightNotify (p, highlight);

    function activate (entry, pick)
        vcCmd.Run (SetHelpCmd, sym, vcEvent.Create (0, 0, ' ', nil, 
                            vcRegion.Window (rgn), rgn, nil), nil, TRUE);
    if (mode == vflowMode.Edit)
        vcPicklist.ActivateNotify (p, activate);

    vcRegion.FocusInhibit (rgn);
    sym;
} 

function help(ev)
    vcAlert.SpawnHelp (vcEvent.Window (ev), "vflow", "GlobalSymbols");
function ButtonRow (mode)    
    if (mode == vflowMode.Edit)
        brow := vcRegion.Create ("", vcButnRow.fac, 
                  list (list ("", "Accept", vflow.SetGlSymCmd, nil),
                        list ("", "Insert", InsertCmd, nil),
                        list ("", "Delete", DeleteCmd, nil),
                        list ("", "Set Help", SetHelpCmd, nil),
                        list ("", "Cancel", func(ev)nil;,
                                           list (close_window <- TRUE)),
                        list ("", "Help", help, nil)));
    else
        brow := vcRegion.Create ("", vcButnRow.fac, 
                  list (list ("", "Accept", vflow.ChangeSymbolsCmd, nil),
                        list ("", "Cancel", func(ev)nil;,
                                           list (close_window <- TRUE)),
                        list ("", "Help", help, nil)));

function Create(parent, init, mode) {
    s := vcRegion.Create ("", fac, init, mode <- mode);
    brow := ButtonRow (mode);
    t := vcTable.Make ("Edit Global Symbols", 2, 1, s, brow);
    if (parent == nil)
        vcWin.Create (t, vcWin.NoMacros);
    else
        vcWin.Create (parent, t, vcWin.NoMacros);
}

function insert(sym, ev) {
    declare rows, cols;
    vcPicklist.Dimensions (sym.pick, &rows, &cols);
    for (i := 0; i < rows; ++i)
        if (vcPicklist.IsSet (sym.pick, i)) {
            insert_entry (sym, i);
            break;
        }
}

InsertCmd := vcWv.Cmd (fac, insert, "Insert");
vcCmd.SetHelp (InsertCmd, "Insert a new symbol above the selected one.");

function delete(sym, ev) {
    declare rows, cols;
    vcPicklist.Dimensions (sym.pick, &rows, &cols);
    for (i := 0; i < rows; ++i)
        if (vcPicklist.IsSet (sym.pick, i)) {
            vcPicklist.DeleteRows (sym.pick, i, 1);
            break;
        }
    tablist := list (sym.header);
    foreach (r, c, ix, set, table, d; vcPicklist.Items (sym.pick))
        tablist = cons (table, tablist);
    vcTable.AlignColumns (tablist);
    if (rows == 1)
        add_entry (sym);
  
}
function confirm_delete(sym, ev) {
    e := vcRegion.Get (sym.pick, vscmVT);
    ret := FALSE;
    if (e != nil) {
        str := vcCmdline.Get (e.name);
        if (!equal (str, ""))
            ret = sprintf ("Delete symbol %s?", str);
    }
    ret;
}
DeleteCmd := vcWv.CmdWithConfirm (fac, delete, confirm_delete,
               FALSE, "Delete");
vcCmd.SetHelp (DeleteCmd, "Delete the currently selected symbol.");

function show_help (rgn, entry) {
    w := vcRegion.Window (rgn);
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, entry.help);
}
function set_help(sym, ev, s, help) {
    entry := vcRegion.Get (sym.pick, vscmVT);
    entry.help = help;
    show_help (sym.pick, entry);
}
function get_symbol (sym) {
    entry := vcRegion.Get (sym.pick, vscmVT);
    str := vcCmdline.Get (entry.name);
    if (equal (str, ""))
        str = "<unnamed>";
    str;
}
function get_help (sym) {
    entry := vcRegion.Get (sym.pick, vscmVT);
    entry.help;
}
SetHelpCmd := vcWv.Cmd (fac, set_help, "SetHelp", 
                     "Set Symbol Help String",
             vcWv.Param ("Symbol", vlStringVT, vcWv.Dynamic, get_symbol,
                                vcStaticText.fac),
             vcWv.Param ("Help", vlStringVT, vcWv.Dynamic, get_help,
                               vcCmdline.fac, max_size <- 0,
                                 width <- 60));
vcCmd.SetHelp (SetHelpCmd, "Set symbol's help string.");

disabled := "No symbols are selected.";
vcCmd.SetDisabledMsg (InsertCmd, disabled);
vcCmd.SetDisabledMsg (DeleteCmd, disabled);
vcCmd.SetDisabledMsg (SetHelpCmd, disabled);

function enable_commands (sym) {
    enable := (vcRegion.Get (sym.pick, vscmVT) != nil) && 
                    sym.mode == vflowMode.Edit;
    vcCmd.Enable (InsertCmd, enable);
    vcCmd.Enable (DeleteCmd, enable);
    vcCmd.Enable (SetHelpCmd, enable);
}

iterator Entries(r) {
    sym := vcRegion.GetData (r);
    foreach (row, col, ix, set, visual, e; vcPicklist.Items (sym.pick)) {
        if (typeof (e.sym) == vcRegionVT)
            s := vcRegion.Get (e.sym, vflGsymTypeVT);
        else
            s := e.sym;
        iterate (vcRegion.Get (e.name, vlStringVT), 
                 vcRegion.Get (e.value, vlStringVT),
                 s, e.help);
    }
}

function after (sym, ev)
    enable_commands (sym);

function advance(sym, ev) {
    /* put together a list of all focus-able regions */
    current := vcRegion.GetFocus (sym.rgn);
    declare r_list;
    foreach (r, c, ix, s, t, entry; vcPicklist.Items (sym.pick)) {
        if (vcRegion.Fac (entry.name) == vcCmdline.fac)
            r_list = cons (entry.name, r_list);
        if (vcRegion.Fac (entry.value) == vcCmdline.fac)
            r_list = cons (entry.value, r_list);
        if (typeof (entry.sym) == vcRegionVT)
            r_list = cons (vcRegion.FindFac (entry.sym, vcPicklist.fac), 
                           r_list);
    }        
    r_list = reverse_destructive (r_list);
    if (current == sym.pick && r_list != nil)
        dest := car (r_list);
    else {
        dest := sym.pick;
        while (r_list != nil) {
            if (car (r_list) == current && cdr (r_list) != nil) {
                dest = cadr (r_list);
                r_list = nil;
           } else 
                r_list = cdr (r_list);
       }
    }
    vcRegion.SetFocus (sym.rgn, dest);
}
AdvanceCmd := vcWv.Cmd (fac, advance, "advance");

vcBind.Set (bindings, '\enter', AdvanceCmd);
vlPref.Replace (prefs, "mode", vflowModeVT,
                 "~~Permit edit-mode behavior", vflowMode.Run);
