/*
 * $Revision: 1.36.1.9 $
 *
 * The main viewflow facility
 */
define_enum ("vflowModeVT", list ("Run", "Edit"));

InstanceScale := 1.0;

/**************
 * vtext syntax highlighting preferences:
 * because vtext highlighting preferences that are programmatically
 * defined cannot be customized from ini files & command lines,
 * I have to put this preference wrapper around them.
 * This has the nice side effect of sparing us pref names like
 * Regexp1_color. All this must be done before loading in all the 
 * facilities I use.
 */
colors := vlPref.Create ("vflowcolor");
vlPref.Add (colors, "log_msg", vgColorVT, 
             "Log window error and status messages", vgColor.Yellow);
vlPref.Add (colors, "data_names", vgColorVT,
             "Color of data object names in info region", vgColor.Cyan);
vlPref.Add (colors, "task_names", vgColorVT,
             "Color of task object names in info region", vgColor.Magenta);
vlPref.Add (colors, "global_syms", vgColorVT,
             "Color of global symbols in info region", vgColor.Blue);
vlPref.Add (colors, "running", vgColorVT,
             "Color of `Running' in info region", vgColor.Yellow);
vlPref.Add (colors, "completed", vgColorVT,
             "Color of `Completed' in info region", vgColor.Green);
vlPref.Add (colors, "failed", vgColorVT,
             "Color of task failure messages in info region", 
             vgColor.LightRed);
            
vlPref.Add (prefs, "data_classes", vlStrArrayVT, "Shortcut data class list",
                      "AnyFileOrDir vdmCollection");
vlPref.Add (prefs, "task_classes", vlStrArrayVT, "Shortcut task class list",
                      "System Subflow");
/* we have to parse now, because we will check these prefs 
 * during facility create time */
vlPref.Parse (nil, wv);

vcWv.RequireFac (wv, "vcMultOpts");
vcWv.RequireFac (wv, "vflAttrEdit");
vcWv.RequireFac (wv, "vflCanvas");
vcWv.RequireFac (wv, "vflInfo");
vcWv.RequireFac (wv, "vflLog");
vcWv.RequireFac (wv, "vflSym");
vcWv.RequireFac (wv, "vflSubsym");
vcWv.RequireFac (wv, "vdata");

register_facility(vflSymbolVT, vflSubsym.fac);

/* we keep track, globally, of all flows that we know about */
declare FLOW_LIST;

/* used with vcWv.Dynamic */
function get_flow_list() {
    declare ret;
    foreach (fi; vscript.iter (FLOW_LIST))
         ret = cons (fi.name, ret);
    ret;
}

function vflow_error(num, sev, text)
    vlEx.Throw (vlEx.FAILURE, 
         vlErr.Create (nil, "vflow", num, sev, text));

/* create an all-inclusive vdm context */
VdmCntx := vdm.CntxCreate ("ViewFlow");
vflFlowClass := vdm.CntxFindClass (VdmCntx, "vflFlow");
/* if we can't get this class, we really can't continue */
if (vflFlowClass == nil)
    vflow_error (10, vlErrSev.Fatal, 
            "Vdm class vflFlow is not defined\n" + 
            "                 (perhaps an obsolete vdmMaster.scm).");


/* one of these records is stored for each flow that we've loaded */
struct vflFlowInfoVT {
    vlStringVT name;
    vflFlowVT flow;
    vgeDrawingVT draw;
    vflowModeVT mode;
    vscmVT modified;          /* TRUE or FALSE */
    vscmVT overview_win;      /* vcWinVT or nil */
    vdmObjVT obj
};

function create_flow_info (vf, name, flow, drawing, mode, obj) {
    fi := struct vflFlowInfoVT {
          name, flow, drawing, mode, FALSE, nil, obj};
    FLOW_LIST = cons (fi, FLOW_LIST);
    fi;
}

/* facility data object  type */
struct vflowVT {
    vcRegionVT region, canvas, info, cmd;
    vscmVT initial;
    vscmVT flow_info;         /* vflFlowInfoVT or nil */
    vscmVT flow_list;         /* listof vflFlowInfoVT */
    vscmVT flow_stack;        /* stack of pushed/popped vflFlowInfoVTs */
    vdmCntxVT dm;

    vscmVT global_sym_w;             /* window or nil */
    vcRegionVT vdata_rgn;
    vcWindowVT vdata_win;
    vscmVT log_rgn, log_win;         /* may be nil */

    vscmVT chosen_flow;              /* used with the flow_list dialog */    
    vscmVT flow_picks;         /* vcWinVT  or nil */
    vscmVT data_names, task_names, gsym_names;   /* lists of strings */
    vscmVT curved_depends;           /* TRUE or FALSE */
    vscmVT firstobj                 /* first obj selected (for attach) */
};

define_string ("vflTaskNameVT");
define_string ("vflDataNameVT");
define_enum ("vflLogStyleVT", list ("Panel", "Window", "None"));

/*******************************/
/* we attach this structure to the object while we are placing it,
 * thus allowing us to identify what we want to do with it
 * when it gets placed on the canvas.
 */
struct PlaceInfoVT {
    vlTypespecVT ts;
    vscmVT info
};

/* info is one of the following structs: */
struct PlaceDataVT {
    vlStringVT name, version, class, attr, loc, editor, icon;
};

struct PlaceTaskVT {
    vlStringVT name, class, command, success, icon;
    vscmVT attrs;
};

struct PlaceDependVT {
    vlIntVT x1, y1, x2, y2;
};

/****************************************/
/* this structure type is used as the generic data for the
 * vfl objects, allowing the GUI to identify their vge objects */

struct VgesVT {
    vgeObjectVT symbol;
    vscmVT label   /* might be nil */
};

/* get the VgesVT for a vfl object: it might return nil */
function get_vges (o) {
    declare v;
    if (typeof (o) == vflNodeVT)
        o = vflNode.Object (o);

    if (typeof (o) == vflDataVT)
        v = vflData.GenericData (o);
    else if (typeof (o) == vflTaskVT)
        v = vflTask.GenericData (o);
    else if (typeof (o) == vflDependVT)
        v = vflDepend.GenericData (o);
    else if (typeof (o) == vflStartStopVT)
        v = vflStartStop.GenericData (o);
    v;
}

/* get the id for any type of object */
function get_id (o) {
    declare id;
    if (typeof (o) == vflNodeVT)
        o = vflNode.Object (o);

    if (typeof (o) == vflDataVT)
        id = vflData.Id (o);
    else if (typeof (o) == vflTaskVT)
        id = vflTask.Id (o);
    else if (typeof (o) == vflDependVT)
        id = vflDepend.Id (o);
    else if (typeof (o) == vfStartStopVT)
        id = vflStartStop.Id (o);
}

/* get the symbol object associated with a vfl object: might return nil */
function GetSymbol (o) {
    v := get_vges (o);
    if (typeof (v) == VgesVT)
        v.symbol;
    else
        nil;
}

/* get the label object associated with the vfl object: might return nil */
function GetLabel (o) {
    v := get_vges (o);
    if (typeof (v) == VgesVT)
        v.label;
    else
        nil;
}


/* make GetSymbol accessable from C */
RegisterGetSymbol (GetSymbol);


/********************************************/


/* converts a vlStrArrayVT to a list of strings */
function strarray_to_list (strarray) {
    declare ret;
    for (i := 0; i < length (strarray); ++i)
        ret = cons (strarray[i], ret);
    reverse_destructive (ret);
}


declare FLOW; /* for debugging */

/* given a vge object, gets the corresponding vfl object out of the flow */
function vge_to_vfl(flow, o) {
    id := vge.ObjectGetData (o);
    if (typeof (id) == vflIdVT)
        vflFlow.GetFromId (flow, id);
    else
        nil;
}

/* called whenever a vge obejct is selected: fill out the
 * vflInfo region.
 *
 * Things may get selected/unselected in flows not other
 * than the one we are displaying, so we check the draw.
 */
function ReportSelected (vf, anobj, draw) {
    if (vf.flow_info != nil && draw == vf.flow_info.draw) {
        vflInfo.Clear (vf.info);
        foreach (vgeobj; vge.DrawLoopSelected (draw)) {
        d := (vgeobj == nil) ? nil : vge_to_vfl (vf.flow_info.flow, vgeobj);
        if (typeof (d) == vflTaskVT)
            vflInfo.AddLine (vf.info, vflTask.Description (d));
        else if (typeof (d) == vflDataVT)
            vflInfo.AddLine (vf.info, vflData.Description (d));
        else if (typeof (d) == vflDependVT)
            vflInfo.AddLine (vf.info, vflDepend.Description (d));
        else if (typeof (d) == vflStartStopVT)
            vflInfo.AddLine (vf.info, vflStartStop.Description (d));
        vflInfo.AddLine (vf.info, "\n");
    } /* foreach */
        enable_commands (vf);
    }
}

function HandleQueryDrop (vf, data)
    (vf.flow_info != nil &&
     vf.flow_info.mode == vflowMode.Edit &&
     typeof (data) == vdmDragObjVT);

/* something was dropped on the vflCanvas: handle it */
function HandleDrop (vf, ev, data) {
    x := vcEvent.UserX (ev);
    y := vcEvent.UserY (ev);
    obj := vdm.CntxGetObjFromLogicalName (vf.dm, data, vdm.VersDefault);
    name := vdm.ObjGetName (obj);
    vers := vdm.ObjGetVersion (obj);
    class := vdm.ClassGetName (c := vdm.ObjGetClass (obj));
    loc := vpm.LocationGetAlias (vdm.ObjGetLocation (obj)); 
    attr := class + "\n";
    foreach (name; vscript.iter (vdm.ClassGetAttrList (c))) {
        attr += name + "\n";
        attr += vdm.ObjGetAttr (obj, name) + "\n";
    }
    if (equal (class, "vflFlow")) {
        attr := list ("Subflow", cons ("LOCATION", loc));
        vcCmd.Run (AddTaskAtCmd, vf, ev,
                   list (x, y, name, "Subflow", name, attr, "", ""), TRUE);
    } else {
        icon := (vdm.ObjIsCollection (obj) ? "vflcoll" : "");
        vcCmd.Run (AddDataAtCmd, vf, ev, 
                   list (x, y, name, vers, class, attr, loc, "", icon), TRUE);
    }

}

function HandlePopup(vf, o, ev) {
    obj := vge_to_vfl (vf.flow_info.flow, o);
    if (typeof (obj) == vflTaskVT) 
        vcMenuPop.Create (TaskMenu, ev);
    else if (typeof (obj) == vflDataVT)
        vcMenuPop.Create (DataMenu, ev);
    else if (typeof (obj) == vflStartStopVT)
        vcMenuPop.Create (StartStopMenu, ev);
    else if (typeof (obj) == vflDependVT)
        vcMenuPop.Create (DependMenu, ev);
}

/* wrappers around depend set-from/to that handles implicit start/stop.
 * i.e., if the endpoint is nil, we fill in the flow's starting/stopping
 * point, if the startstop preference is FALSE
 */
function depend_set_from(vf, dep, n) {
    if (n == nil && !vcRegion.PrefGet (vf.region, "startstop"))
        n = vflFlow.Start (vf.flow_info.flow);
    vflDepend.SetFrom (dep, n);
}
function depend_set_to(vf, dep, n) {
    if (n == nil && !vcRegion.PrefGet (vf.region, "startstop"))
        n = vflFlow.Stop (vf.flow_info.flow);
    vflDepend.SetTo (dep, n);
}

/* recheck both endpoints of the dependency. Called when the dep moves,
 * or when a node is removed from the canvas
 */
function reevaluate_dependency(dep, vf, obj) {
    /* determine our new endpoints */
    p1 := GetVgePoint (obj, FALSE);
    p2 := GetVgePoint (obj, TRUE);
    n1 := ChooseNode (vf.flow_info.flow, car (p1), cdr (p1));
    n2 := ChooseNode (vf.flow_info.flow, car (p2), cdr (p2));

    /* detatch ourselves */
    catch (vlEx.FAILURE) {
        s := GetSymbol (vflDepend.From (dep));
        if (s != nil)
            vge.ObjectDetachFrom (obj, s);
        s := GetSymbol (vflDepend.To (dep));
        if (s != nil)
            vge.ObjectDetachFrom (obj, s);
    } exception;
    /* under some unusual circumstances involving dependencies that
     * attach a thing to itself, we may get detatched without that being
     * fully updated here. Thus we ignore that exception
     */

    /* set ourselves to our new place */
    depend_set_from (vf, dep, n1);
    depend_set_to (vf, dep, n2);
    show_depend_validity (dep, obj);
    attach_dependency (dep, n1, FALSE);
    attach_dependency (dep, n2, TRUE);
}

/* something moved on the canvas - update ourselves */
function handle_move (vf, obj, draw) {
    o := vge_to_vfl (vf.flow_info.flow, obj);

    vf.flow_info.modified = TRUE;
    check_bounds (vf, vf.flow_info);
    /* if a node moved, see if it was put on top of an otherwise
     * unattached dependency, and attach it
     */
    if (typeof (o) == vflDataVT ||
        typeof (o) == vflTaskVT ||
        typeof (o) == vflStartStopVT) {
        /* check each unattached end of each dependency, to see
         * if we can hook it 
         */
        l := FindUnattachedDepends (vf.flow_info.flow, vge.ObjectMbb (obj));
        foreach (dep; vscript.iter (car (l))) {
            depend_set_from (vf, dep, o);
            attach_dependency (dep, o, FALSE);
            show_depend_validity (dep, GetSymbol (dep));
        }
        foreach (dep; vscript.iter (cdr (l))) {
            depend_set_to (vf, dep, o);
            attach_dependency (dep, o, TRUE);
            show_depend_validity (dep, GetSymbol (dep));
        }
        vcRegion.QueueForRefresh (vf.canvas);
    } else if (typeof (o) == vflDependVT) {
        /* if a dependency moved, see where it is connected now */

        reevaluate_dependency (o, vf, obj);
        vcRegion.QueueForRefresh (vf.canvas);
    }

}

function handle_activate(vf, o, ev) {
    vge.UnselectAll (vf.canvas);
    if (o != nil)
        vge.DrawSelectObject (vf.flow_info.draw, o);
    if (vf.flow_info != nil && o != nil) {
        if (vf.flow_info.mode == vflowMode.Edit)
            vcCmd.Run (ModifySlctCmd, vf, ev, nil, TRUE);
        else {
            obj := vge_to_vfl (vf.flow_info.flow, o);
            if (typeof (obj) == vflDataVT)
                vcCmd.Run (EditSlctCmd, vf, ev, nil, TRUE);
            else if (typeof (obj) == vflTaskVT)
                vcCmd.Run (RunSlctCmd, vf, ev, nil, TRUE);
        }
    }
}

/* task output goes to the log window, if any */
/* else send to the txshell */
function handle_task_output(vf, task, str) {
    if (task == nil)
        vcTxShell.PrintString (vf.cmd, str + "\n");
    else if (vf.log_rgn == nil)
        vcTxShell.PrintString (vf.cmd, 
                 vflMaster.PrepareTaskOutput (task, str));
    else {
        vflLog.AddEntry (vf.log_rgn, vflTask.Name (task), task);
        vflLog.AddString (vf.log_rgn, task, str);
    }
}

/* a task error goes as follows:
 * if no task, put it in the command shell
 * if no log window exists, put it in the command shell,
 *           with message saying which task caused it
 * else put in the log window
 *
 * in cmdshell, use statusline (which defaults to concise)
 * in log window, use stderr (which defaults to verbose)
 */
function handle_task_error(vf, task, e) {
    sev := vlErr.Severity (e);
    if (task == nil)
        vcTxShell.PrintString (vf.cmd, 
                      vlErr.String (e, vlErrCntx.StatusLine) + "\n");
    else if (vf.log_rgn == nil) {
        e2 := vflMaster.PrepareTaskError (task, e);
        vcTxShell.PrintString (vf.cmd, 
                      vlErr.String (e2, vlErrCntx.StatusLine) + "\n");
    } else {
        vflLog.AddEntry (vf.log_rgn, vflTask.Name (task), task);
        /* yeah, i shouldn't hard code this # in, but what the hell.
         * this ensures that the task log switches to the currently
         * running task. The # is the "task started" message.
         */
        if (vlErr.Num (e) == 2510)
            vcNoteTab.SelectByString (vf.log_rgn, vflTask.Name (task));
        vflLog.AddString (vf.log_rgn, task, "\n" +  
                                   vlErr.String (e, vlErrCntx.Stderr) + "\n");
    }

    if (sev == vlErrSev.Error || sev == vlErrSev.Fatal)
        vcErr.Report (vcRegion.Window (vf.region), nil, e);
}

function handle_task_change(vf, task, state, draw, flow) {
    name := choose_task_icon (vf, vflTask.Icon (task));
    icon := vfl.ConvertIconName (name, state);
    /* if we want a different icon for this state, do the change */
    if (!equal (name, icon))
        replace_symbol (task, icon, vf, draw, flow);
}
 
/* add the optional values to the task */
/* this is used both when creating & modifying tasks */
function finish_create_task (t, success, icon, attrs) {
    if (equal (success, "Default for Class"))
        success = "";
    vflTask.SetSuccess (t, success);
    vflTask.SetIcon (t, icon);
    /* now the attributes */
    foreach (elem; vscript.iter (cdr (attrs)))
        vflTask.SetAttr (t, car (elem), cdr (elem));

}

/* create a task object from stuff typed in dialog box */
function create_task (PlaceTaskVT stuff) {
    class := vflTaskClass.Lookup (stuff.class);
    t := vflTask.Create (stuff.name, class, stuff.command);
    finish_create_task (t, stuff.success, stuff.icon, stuff.attrs);
    t;
}

/* fill in optional data stuff. used when creating and modifying data */
function finish_create_data (d, loc, editor, version) {
    vflData.SetLocation (d, loc);
    if (equal (editor, "Default for Class"))
        editor = "";
    vflData.SetEditor (d, editor);
    if (equal (version, "Default"))
        version = vdm.VersDefault;
    else if (equal (version, "Latest"))
        version = vdm.VersLatest;
    vflData.SetVersion (d, version);
}

/* fill in data object from dialog box info */
function create_data (PlaceDataVT stuff) {
    d := vflData.Create (stuff.name, stuff.class, stuff.attr);
    finish_create_data (d, stuff.loc, stuff.editor, stuff.version);
    d;
}

/* uses the vge object of the dependency to show the validity */
function show_depend_validity(vflDependVT dep, vgeObjectVT obj) {
    pattern := (vflDepend.IsValid (dep)) ? vgLinePattern.Solid :
                                           vgLinePattern.Dash;
    vge.ObjectSetLine (obj, pattern, vgLineCap.Butt, vgLineJoin.Miter, 1);
}

/* create a dependecy from a vge object placement */
/* as soon as we create it, we will get a MoveCompelete notification
 * from vge. At that point we will figure out the correct endpoints.
 * no need to do it twice, so we just use nil,nil here
 */
function create_dependency(flow, obj, stuff)
    vflDepend.Create (nil, nil);

/* attach the dependency to the node, at the from or to point */
function attach_dependency(d, node, last) {
    /* if the dep connects an object to itself, don't attach it. It
     * will end up attaching on both points, which will cause
     * a Pvect recursive save state to occur
     */
    from := vflDepend.From (d);
    to := vflDepend.To (d);
    if (node != nil && (from == nil || to == nil || 
                        !vflNode.Equal (from, to))) {
        dep_sym := GetSymbol (d);
        node_sym := GetSymbol (node);
        if (dep_sym != nil && node_sym != nil) {
            p := GetVgePoint (dep_sym, last);
            vge.ObjectAttachTo (dep_sym, node_sym, car (p), cdr(p));
        }
    }
}

/* adds the dependency to flow, and sets the vge object up */
function add_dependency(vf, d, obj) {
    vflFlow.AddDepend (vf.flow_info.flow, d);
    vge.ObjectSetData (obj, vflDepend.Id (d));
    set_depend_vges (d, obj, nil);
    attach_dependency (d, vflDepend.From (d), FALSE);
    attach_dependency (d, vflDepend.To (d), TRUE);
    vge.ObjectSetColor (obj, vcRegion.PrefGet (vf.region, "depend_color"));
    show_depend_validity (d, obj);
}


/* functions to create, place, modify labels */
function create_label(str, x, y, col, align, size, font) {
    o := vge.TextCreateXY (x, y + 1, str, 0, align,
                                vgeScore.None, size, font);
    vge.ObjectSetColor (o, col);
    o;
}

/* given a vge object, aligns a label as appropriate for it.
 * returns (x y align)
 */
function align_label (vf, o) {
    declare ret;
    align := vcRegion.PrefGet (vf.region, "label_align");
    mbb := vge.ObjectMbb (o);
    cx := (vgRect.Xmax (mbb) + vgRect.Xmin (mbb)) / 2;
    cy := (vgRect.Ymax (mbb) + vgRect.Ymin (mbb)) / 2;

    if (align == vgAlign.UpperLeft)
        ret = list (vgRect.Xmin (mbb), vgRect.Ymax (mbb), vgAlign.LowerRight);
    else if (align == vgAlign.MiddleLeft)
        ret = list (vgRect.Xmin (mbb), cy, vgAlign.MiddleRight);
    else if (align == vgAlign.LowerLeft)
        ret = list (vgRect.Xmin (mbb), vgRect.Ymin (mbb), vgAlign.UpperRight);
    else if (align == vgAlign.LowerCenter)
        ret = list (cx, vgRect.Ymin (mbb), vgAlign.UpperCenter);
    else if (align == vgAlign.LowerRight)
        ret = list (vgRect.Xmax (mbb), vgRect.Ymin (mbb), vgAlign.UpperLeft);
    else if (align == vgAlign.MiddleRight)
        ret = list (vgRect.Xmax (mbb), cy, vgAlign.MiddleLeft);
    else if (align == vgAlign.UpperRight)
        ret = list (vgRect.Xmax (mbb), vgRect.Ymax (mbb), vgAlign.LowerLeft);
    else if (align == vgAlign.UpperCenter)
        ret = list (cx, vgRect.Ymax (mbb), vgAlign.LowerCenter);
    else
        ret = list (cx, cy, vgAlign.MiddleCenter);
}

function attach_label (label, obj) {
    vge.ObjectAttachTo (label, obj, 
                                vge.ObjectGetX (label),
                                vge.ObjectGetY (label));
}

/* change the symbol for the object
 * note that symbols may get replaced in flows other than the one
 * we are currently displaying, viz. traffic light changes.
 * That is why we get the draw & flow separately, and *not* assume
 * that the one currently stored in the vf is the one to use
 */
function replace_symbol(obj, str, vf, draw, flow) {
    vges := get_vges (obj);
    type := typeof (obj);
    if (typeof (vges) == VgesVT) {
        old := vges.symbol;
        x := vge.ObjectGetX (old);
        y := vge.ObjectGetY (old);
        is_selected := FALSE;
        /*if the old obj is selected, we should select the new one */
        foreach (o; vge.DrawLoopSelected (draw))
            if (o == old)
                is_selected = TRUE;

        if (typeof (obj) == vflTaskVT)
            str = choose_initial_task_icon (vf, str);
        else
            str = choose_initial_data_icon (vf, str);
        /* this will throw if the file doesn't exist.
         * So do it before we remove the old guy */
        new := vge.InstanceCreateXY (x, y, str, 0, FALSE, InstanceScale);
        vge.DrawRemoveObject (draw, old);
        vge.DrawAddObject (draw, new);
        vges.symbol = new;
        vge.ObjectSetData (new, get_id (obj));
        if (is_selected)
            vge.DrawSelectObject (draw, new);
        vge.DrawRefreshDisplays (draw);
        if (vges.label != nil) 
            attach_label (vges.label, new);

        /* attach all of the dependency symbols */
        if (type == vflDataVT || type == vflTaskVT) {
            node := vflNode.Create (obj);
            foreach (dep; vflFlow.Depends (flow)) {
                from := vflDepend.From (dep);
                if (from != nil &&vflNode.Equal (node, from))
                    attach_dependency (dep, node, FALSE);
                to := vflDepend.To (dep);
                if (to != nil && vflNode.Equal (node, to))
                    attach_dependency (dep, node, TRUE);
            }
        }
    }
}

/* replace the label for the object with 'str' */
function replace_label(obj, str, vf) {
    v := get_vges (obj);
    if (typeof (v) == VgesVT) {
        if (v.label != nil)
            vge.DrawRemoveObject (vf.flow_info.draw, v.label);
        mbb := vge.ObjectMbb (v.symbol);
        if (typeof (obj) == vflTaskVT)
            col := vcRegion.PrefGet (vf.region, "task_label_color");
        else
            col := vcRegion.PrefGet (vf.region, "data_label_color");
        size := vcRegion.PrefGet (vf.region, "label_size");
        font := vcRegion.PrefGet (vf.region, "label_font");
        l := align_label (vf, v.symbol);
        v.label = create_label (str, car (l), cadr (l), col, caddr (l),
                                     size, font);
        vge.DrawAddObject (vf.flow_info.draw, v.label);
        vge.ObjectAttachTo (v.label, v.symbol, car (l), cadr (l));
        vge.ObjectSetData (v.label, vge.ObjectGetData (v.symbol));
        if (!vcRegion.PrefGet (vf.region, "move_labels"))
            vge.ObjectSetSelectStyle (v.label,  vgeSelectStyle.None);
    }
}

/* create and add a VgesVT object to the data object */    
function set_data_vges (d, sym, vf) {
    v := struct VgesVT {sym, nil};
    vflData.SetGenericData (d, v);
    replace_label (d, vflData.Name (d), vf);
}

/* create and add a VgesVT object to the task object */    
function set_task_vges (t, sym, vf) {
    v := struct VgesVT {sym, nil};
    vflTask.SetGenericData (t, v);
    replace_label (t, vflTask.Name (t), vf);
}

/* create and add a VgesVT object to the dependency */    
function set_depend_vges (vflDependVT d, sym, label) {
    v := struct VgesVT {sym, label};
    vflDepend.SetGenericData (d, v);
}

/* set all the correct highlighting for the flow in the info region */
function set_info_highlighting (vf) {
    vf.data_names = nil;    
    foreach (d; vflFlow.Datas (vf.flow_info.flow))
        vf.data_names = cons (vflData.Name (d), vf.data_names);
    vf.task_names = nil;    
    foreach (t; vflFlow.Tasks (vf.flow_info.flow))
        vf.task_names = cons (vflTask.Name (t), vf.task_names);
    foreach (gs; vflFlow.Gsyms (vf.flow_info.flow))
        vf.gsym_names = cons (vflGsym.Name (gs), vf.gsym_names);
    vflInfo.SetNames (vf.info, vf.data_names, vf.task_names, vf.gsym_names);
}

/* ensure that the bounds are large enough to include the entire flow */
function check_bounds(vf, fi) {
    mbb := vge.DrawGetMbb (fi.draw);
    bounds := vcVPCanvas.GetBounds (vf.canvas);
    if (!vgRect.Obscures (bounds, mbb)) {
        newbounds := vgRect.Create (0, 0, 0, 0);
        vgRect.Max (bounds, mbb, newbounds);
        vcRegion.PrefSet (vflCanvas.VPCanvas (vf.canvas),
                          list (bounds <- newbounds));
    }
}
    
/* an object has been placed on the draw. Update it, create the node
 * or dependency, add it to the flow 
 */
function place_object (vf, obj, draw) {
    info := vge.ObjectGetData (obj);
    if (typeof (info) == PlaceInfoVT) {
        stuff := info.info;
        if (info.ts == vflDataVT) {
            d := create_data (stuff);
            vflFlow.AddData (vf.flow_info.flow, d);
            vge.ObjectSetData (obj, vflData.Id (d));
            set_data_vges (d, obj, vf);
            vf.data_names = cons (stuff.name, vf.data_names);
        } else if (info.ts == vflTaskVT) {
            t := create_task (stuff);
            vflFlow.AddTask (vf.flow_info.flow, t);
            vge.ObjectSetData (obj, vflTask.Id (t));
            set_task_vges (t, obj, vf);
            vf.task_names = cons (stuff.name, vf.task_names);
        } else if (info.ts == vflDependVT) {
            d := create_dependency (vf.flow_info.flow, obj, stuff);
            add_dependency (vf, d, obj);
        }
        /* if we've moved a node, see if we put it on top of an 
         * unattached dependency */
        handle_move (vf, obj, draw);
        ReportSelected (vf, nil, vf.flow_info.draw);
        /* update the keyword highlighting */
        vflInfo.SetNames (vf.info, vf.data_names, 
                          vf.task_names, vf.gsym_names);
    }
}

/* alas, some of the alt keys I want to use with the menus
 * are taken for obscure vtx commands. Since none of my vtx
 * regions are really text editting regions, I'll go ahead
 * and remove those bindings
 */
function fix_vtx_bindings (rgn) {
    b := vcBind.Duplicate (vtx.bindings);
    vcBind.Set (b, '\_f', nil);
    vcBind.Set (b, '\_v', nil);
    vcBind.Set (b, '\_d', nil);
    b2 := vcBind.Duplicate (vcTxShell.bindings);
    vcBind.Set (b2, '\_d', nil);

    foreach (child; vcRegion.Children (rgn, TRUE))
        if (vcRegion.Fac (child) == vtx.fac)
            vcRegion.SetBindings (child, b);
        else if (vcRegion.Fac (child) == vcTxShell.fac)
            vcRegion.SetBindings (child, b2);

}

function setup_drawing (vf, draw) {
    vge.DrawCallOnSelect (draw,
                func (obj, draw) ReportSelected (vf, obj, draw););
    vge.DrawCallOnSelectNone (draw, TRUE);
    vge.DrawCallOnCreate (draw,
                             func (obj, draw) place_object (vf, obj, draw););
    vge.DrawCallOnFinishMove (draw,
                             func (obj, draw) handle_move (vf, obj, draw););
}

function setup_flow (vf, flow, draw) {
    vflFlow.CallOnOutput (flow, func (t, str) 
                     handle_task_output (vf, t, str););
    vflFlow.CallOnOutputError (flow, func (t, e) 
                     handle_task_error (vf, t, e););
    vflFlow.CallOnTaskChange (flow, func (t, s)
                     handle_task_change (vf, t, s, draw, flow););
    vflFlow.CallOnMakeDone (flow, func (f, mode, str) {
                    ev := vcEvent.Create (0, 0, ' ', nil, 
                            vcRegion.Window (vf.region), vf.region, nil);
                    fi := find_flow (vf, f);
                    if (fi != nil) 
                        pop_flow (vf, ev, fi);
                    enable_commands (vf);});
}

function push_flow(vf, ev, fi) {
    flow_stack := cons (fi, vf.flow_stack);
    switch_to_flow (vf, ev, fi);
    vf.flow_stack = flow_stack;
}
function pop_flow(vf, ev, fi) {
    top := car (vf.flow_stack);
    newlist := vf.flow_stack;
    if (top == fi)
        newlist = cdr (newlist);
    if (newlist == nil)
        newlist = list (fi);

    /* have we changed? */
    if (car (newlist) != top)
        switch_to_flow (vf, ev, car (newlist));
    vf.flow_stack = newlist;
}

function find_flow_by_name(vf, name) {
    declare found;
    foreach (fi; vscript.iter (FLOW_LIST))
        if (equal (fi.name, name))
            found = fi;
    found;
}
function find_flow_by_name_ex(vf, name) {
    fi := find_flow_by_name (vf, name);
    if (fi == nil)
        vflow_error (20, vlErrSev.Error, 
                      sprintf ("There is no flow with name %s.", name));
    fi;
}
function find_flow(vf, f) {
    declare found;
    foreach (fi; vscript.iter (FLOW_LIST))
        if (fi.flow == f)
            found = fi;
    found;
}

function add_subflow (vf, f, disp, obj) {
    if (disp != vflSubflowDisplay.None) {
        fi := find_flow (vf, f);
        if (fi == nil) {
            name := vflFlow.Name (f);
            draw := vge.DrawCreate (name + ".mf");
            vge.DrawSetDoModify (draw, FALSE);
            setup_drawing (vf, draw);
            setup_flow (vf, f, draw);
            fi = create_flow_info (vf, name, f, draw, vflowMode.Run, obj);
            connect_vges (vf, fi);
        }
        if (disp == vflSubflowDisplay.Switch) {
            ev := vcEvent.Create (0, 0, ' ', nil, 
                           vcRegion.Window (vf.region), vf.region,nil);
            push_flow (vf, ev, fi);
        }
    }
}
    
/* _Create method */
function _Create (name, rgn) {
    /* create the regions making up this table */
    canvas := vcRegion.Create ("", vflCanvas.fac, "vflcvs",
                   size <- vgSize.Create (400, 400, 100, 100, 900, 900));
    size := vcRegion.PrefGet (rgn, "canvas_size");
    rect := vgRect.Create (-size, -size, size, size);
    vcRegion.PrefSet (vflCanvas.VPCanvas (canvas), list (bounds <-  rect));

    vflCanvas.CallOnDrop (canvas, 
            func (ev, data) HandleDrop (vf, ev, data););
    vflCanvas.CallOnQueryDrop (canvas,
            func (data) HandleQueryDrop (vf, data););
    vflCanvas.CallOnActivate (canvas, 
            func (o, ev) handle_activate (vf, o, ev););
    vflCanvas.CallOnPopup (canvas,
            func (o, ev) HandlePopup (vf, o, ev););

    info := vcRegion.Create ("", vflInfo.fac, nil);
    cmd := vcRegion.Create ("", vcTxShell.fac, "", 
                         height <- 10, width <- 40,
                         commands <- TRUE,
                         prompt <- "vflow> "); 

    declare log_rgn, log_win;
    logstyle := vcRegion.PrefGet (rgn, "log");
    if (logstyle != vflLogStyle.None)
        log_rgn = vcRegion.Create ("Viewflow Task Log", vflLog.fac, nil);
    if (logstyle == vflLogStyle.Window) {
        log_win = vcWin.Create (log_rgn, vcWin.Hidden);
        vcWin.CallOnClose (log_win, vcWin.Hide);
    }

    cmdtable := vcTable.Make ("", 1, 2, cmd, info);
    vcTable.Col (cmdtable, 1, vcTable.Border.Adjustable, 1, 1, 1,
                              vgAlign.MiddleCenter);
    if (log_rgn != nil && log_win == nil) {
        tab := vcTable.Make ("", 3, 1, canvas, log_rgn, cmdtable);
    vcTable.Row (tab, 2, vcTable.Border.Adjustable, 1, 1, 1,
                                  vgAlign.MiddleCenter);
    } else
        tab := vcTable.Make ("", 2, 1, canvas, cmdtable);
    vcTable.Row (tab, 1, vcTable.Border.Adjustable, 1, 1, 1,
                                  vgAlign.MiddleCenter);

    vcRegion.AddChild (rgn, tab);
    vcRegion.ViewportForward (canvas, rgn);
    fix_vtx_bindings (rgn);

    vcRegion.FocusInhibit (rgn);

    /* create a viewdata region & window */
    vdata := vcRegion.Create ("ViewData", vdata.fac, VdmCntx);
    vdata_win := vcWin.Create (vdata, vcWin.Hidden);
    vcWin.CallOnClose (vdata_win, vcWin.Hide);
    /* I wait until now to do this, so that we can read the vpm files
     * and see if it's in there first. Otherwise it may end up 
     * appearing twice. */
    map := vdm.CntxGetMap (VdmCntx);
    if (vpm.MapFindLocation (map, "vflFlow") == nil)
        vpm.MapAddLocation (map, vfl.Root);

    vflMaster.SetSearchForFlow (
           func (name) {fi := find_flow_by_name (vf, name); 
                        (fi == nil) ? nil : fi.flow;});
    vflMaster.SetAddFlow (
           func (f, disp, obj) add_subflow (vf, f, disp, obj););

    vf := struct vflowVT {
            rgn, canvas, info, cmd, name,
            nil, nil, nil,
            VdmCntx,
            /*class_library = create_class_library (dm);*/
            nil, vdata, vdata_win, log_rgn, log_win, 
            nil, nil, nil, nil, nil, FALSE, nil};

    vf;

}
    
/****************** OBJECT MENU *********************/

/********** ADD submenu *************/

/* lists of classes and other things that are of importance to us */
declare TaskClassList;
foreach (class; vflTaskClass.Classes())
    TaskClassList = cons (vflTaskClass.Name (class), TaskClassList);
TaskClassList = vscript.Sort (TaskClassList, string_lei);

declare SuccessFunctionList;
foreach (name, s; vfl.SuccessFunctions ())
    SuccessFunctionList = cons (name, SuccessFunctionList);
SuccessFunctionList = vscript.Sort (SuccessFunctionList, string_lei);
    
declare DataClassList;
foreach (cf, name; vdm.CntxLoopClassFiles (VdmCntx))
        foreach (class, name; vdm.ClassFileLoop (cf))
             DataClassList = cons (name, DataClassList);
DataClassList = vscript.Sort (DataClassList, string_lei);

declare EditorList;
foreach (name, ed; vfl.EditorFunctions ())
    EditorList = cons (name, EditorList);
EditorList = vscript.Sort (EditorList, string_lei);

/*************************/
/* reusable parameter lists */

/* thw following are reused by all modify/create data and task
 * dialog boxes. We have here parameter lists, as well as 
 * DialogCreate and VerifyParam functions 
 */
labels := strarray_to_list (vlPref.Val (prefs, "task_classes"));

/* when the task class selection changes, update the task attributes */
iterator loop_task_class_attr (classname) {
    catch (vlEx.FAILURE) {
        taskclass := vflTaskClass.Lookup (classname);
        p := vflTaskClass.Attributes (taskclass);
        foreach (name, help, ts, value, dflt, ch; vlPref.Options (p)) {
            fac := nil;
            prefs := nil;
            if (equal (classname, "Subflow") && 
                equal (name, "LOCATION")){
                fac = vdmLocSelect.fac;
                prefs := list (cntx <- VdmCntx, allow_empty <- TRUE);
            }
            iterate (name, ts, value, fac, prefs);
        }
    } exception {
        /* couldn't find task class, just do nothing */
        ;
    }
}
/* the list of parameters needed to define a task.
 * These will be resused by several commands 
 */
TaskParams := list (
        vcWv.Param ("Name", vlStringNotEmptyVT, vcWv.Constant, vcWv.NoDefault),
        vcWv.Param ("Class", vlStringVT, vcWv.Sticky, "System",
                        vcMultOpts.fac, 
                        labels <- labels,
                        browse <- vcMultOpts.CreatePicklistFunc
                               ("Select Task Class", TaskClassList),
                        completion <- TaskClassList),
        vcWv.Param ("Command", vlStringVT, vcWv.Constant, "", 
                        vcCmdline.fac, max_size <- 0),
        vcWv.Param ("Attributes", vscmVT, vcWv.Constant, nil,
                           vflAttrEdit.fac, attrs <- loop_task_class_attr),
        vcWv.Param ("Success", vlStringVT, vcWv.Constant, "Default for Class",
                       vcWv.Secondary, 
                       vcMultOpts.fac, labels <- list ("Default for Class"),
                        browse <- vcMultOpts.CreatePicklistFunc
                            ("Select Success Function", SuccessFunctionList),
                        completion <- SuccessFunctionList),
        vcWv.Param ("Icon", vlStringVT, vcWv.Sticky, "", vcWv.Secondary));


class_labels := strarray_to_list (vlPref.Val (prefs, "data_classes"));

/* when the data class changes, update the data obj attribute list */
iterator loop_classes(classname) {
    class := vdm.CntxFindClass (VdmCntx, classname);
    if (!null (class))
        foreach (name; vscript.iter (vdm.ClassGetAttrList (class))) {
            declare thefac;
            if (equal (classname, "AnyFileOrDir") ||
                equal (classname, "vdmCollection"))
                thefac := vcFilePop.fac;
            iterate (name, vlStringVT, "", thefac, nil);
        }
}

/* ensures that class is a valid task class name */
function CheckTaskClass (class)
    vflTaskClass.Lookup (class); 

DataParams := list (
        vcWv.Param ("Name", vlStringNotEmptyVT, vcWv.Constant, 
                               vcWv.NoDefault),
        vcWv.Param ("Version", vlStringVT, vcWv.Sticky, "Latest",
                               vcMultOpts.fac,
                               labels <- list ("Default", "Latest")),
        vcWv.Param ("Class", vlStringVT, vcWv.Sticky, "AnyFileOrDir",
                      vcMultOpts.fac, 
                      labels <- class_labels,
                      browse <- vcMultOpts.CreatePicklistFunc
                              ("Select Data Class", DataClassList),
                      completion <- DataClassList),
        vcWv.Param ("Attributes", vlStringVT, vcWv.Constant, "",
                           vflAttrEdit.fac, attrs <- loop_classes),
        vcWv.Param ("Location", vlStringVT, vcWv.Sticky, "",
                      vdmLocSelect.fac, cntx <- VdmCntx,
                                        allow_empty <- TRUE),
        vcWv.Param ("Editor", vlStringVT, vcWv.Constant, "Default for Class",
                     vcWv.Secondary, vcMultOpts.fac,
                     labels <- list ("Default for Class"),
                     browse <- vcMultOpts.CreatePicklistFunc
                              ("Select Editor", EditorList),
                     completion <- EditorList),
        vcWv.Param ("Icon", vlStringVT, vcWv.Sticky, "", vcWv.Secondary));

/* for use with the above parameter lists */
/* relates the multopt 'class' region with the attedit 'attributes' rgn */
function SetupAttrDialog(classrgn, attr) {
     function set_class () {
        class :=  vcRegion.Get (classrgn, vlStringVT);
        vflAttrEdit.SetClass (attr, class);
    }
    vcMultOpts.CallOnChange (classrgn, set_class);
    set_class ();
}
/*********************************/

function add_mf (name) {
    if (name != nil && !equal (name, "")) {
        l := vscript.vsParse (name);
        if (l != nil && equal (cadddr (l), ""))
            name = vscript.vsMerge (car (l), cadr (l), caddr (l), "mf");
    }
    name;
}

/* given a task, get the icon name for the symbol */
function choose_task_icon(vf, name) {
    declare icon;
    if (name == nil || equal (name, ""))
        icon = vcRegion.PrefGet (vf.region, "task_icon");
    else
        icon = name;
    add_mf (icon);
}
/* choose task icon, setting up for the NotRun state */
function choose_initial_task_icon(vf, name)
    vfl.ConvertIconName (choose_task_icon (vf, name), vflTaskState.NotRun);

/* this sets the data  of the object that is being placed */
function set_new_obj (draw, info) {
    vge.ObjectSetData (o := vge.DrawGetOneSelectedObject (draw), info);
}

/* add a task object: store the info & begin placing an icon */
function add_task(vf, ev, name, class, command, attr, success, icon) {
    canvas := vf.canvas;
    i := choose_initial_task_icon (vf, icon);
    vcCmd.Run (vge.InstCmd, vflCanvas.Display (canvas), ev,
           list (i, 0, FALSE, InstanceScale), TRUE);
    set_new_obj (vf.flow_info.draw, 
             struct PlaceInfoVT {vflTaskVT,
                   struct PlaceTaskVT {name, class, command, 
                                        success, icon, attr}});
}
AddTaskCmd := apply (vcWv.Cmd, append (list (fac, add_task, "AddTask", 
                                    "Create New Task"), TaskParams));
vcCmd.CallOnDialogCreate (AddTaskCmd, 
              func (vf, ev, n, class, c, attr,  s, i)
                  SetupAttrDialog (class, attr););
vcCmd.VerifyParams (AddTaskCmd, 
              func (vf, ev, n, class, c, a, s, i) 
                  CheckTaskClass (class););
vcCmd.EnableApply (AddTaskCmd, TRUE);

/* this should only be anabled if the one and only selected object
 * is a task.
 */
function modify_task(vf, ev, name, class, command, attr, success, icon) {
    vf.flow_info.modified = TRUE;
    t := vge_to_vfl (vf.flow_info.flow, 
                    vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflTask.Reset (t, vf.flow_info.flow);
    if (!equal (name, vflTask.Name (t)))
        replace_label (t, name, vf);
    if (!equal (icon, vflTask.Icon (t)))
        replace_symbol (t, icon, vf, vf.flow_info.draw, vf.flow_info.flow);
    vflTask.SetName (t, name);
    vflTask.SetClass (t, vflTaskClass.Lookup (class));
    vflTask.SetCommand (t, command);
    finish_create_task (t, success, icon, attr);
    set_info_highlighting (vf);
    ReportSelected (vf, nil, vf.flow_info.draw);
}
/* I really should use vcWv.Dynamic to accomplish this, but this way
 * I can reuse by TaskParam paramter list */
function setup_modify_task_dialog(vf, ev, name, class, 
                                 command, attr, success, icon) {
    t := vge_to_vfl (vf.flow_info.flow, vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vcRegion.Set (name, vflTask.Name (t));
    cl := vflTask.Class (t);
    clname := vflTaskClass.Name (cl);
    vcRegion.Set (class, clname);
    vcRegion.Set (command, vflTask.Command (t));
    s := vflTask.Success (t);
    vcRegion.Set (success, (equal (s, "")) ? "Default for Class" : s);
    i := vflTask.Icon (t);
    vcRegion.Set (icon, i);

    /* set up the attribute editor */
    a := vflTaskClass.Attributes (cl);
    declare attr_list;
    foreach (name, h, ts, val, d, ch; vlPref.Options (a))
        attr_list = cons (cons (name, vflTask.GetAttr (t, name)), attr_list);
    vcRegion.Set (attr, cons (clname, attr_list));
    SetupAttrDialog (class, attr);
}

ModifyTaskCmd := apply (vcWv.Cmd, append (list (fac, modify_task, 
                                    "ModifyTask", 
                                    "Modify Task"), TaskParams));
vcCmd.VerifyParams (AddTaskCmd, 
              func (vf, ev, n, class, c, a, s, i) 
                  CheckTaskClass (class););
vcCmd.CallOnDialogCreate (ModifyTaskCmd, setup_modify_task_dialog);

function copy_task(vf, ev) {
    t := vge_to_vfl (vf.flow_info.flow, vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vcCmd.Run (AddTaskCmd, vf, ev, 
                 list (vflTask.Name (t) + "_2",
                       vflTaskClass.Name (vflTask.Class (t)),
                       vflTask.Command (t),
                       vflTask.GetAttrList (t),
                       vflTask.Success (t),
                       vflTask.Icon (t)), TRUE);
}
CopyTaskCmd := vcWv.Cmd (fac, copy_task, "copy_task");

                       
/* this would generally only be run programatically */
function add_task_at(vf, ev, x, y, name, class, command, attr,
                                      success, icon) {
    i := choose_initial_task_icon (vf, icon);
    obj := vge.InstanceCreateXY (x, y, i, 0, FALSE, InstanceScale);

    /* do far, we've put the object with the lower left corner at
     * the x, y spot. I'd rather put the upper left at that spot,
     * to make drag/drop work better. Thus, the move. 
     */
    mbb := vge.ObjectMbb (obj);
    dy := vgRect.Ymax (mbb) - vgRect.Ymin (mbb);
    vge.ObjectMove (obj, 0, -dy);

    pi := struct PlaceInfoVT {vflTaskVT,
                   struct PlaceTaskVT {name, class, command, 
                                        success, icon, attr}};
    vge.ObjectSetData (obj, pi);
    vge.DrawAddObject (vf.flow_info.draw, obj);
}

AddTaskAtCmd := apply (vcWv.Cmd, append (list (fac, add_task_at, "AddTaskAt", 
                                    "Create New Task",
                          vcWv.Param ("X", vlIntVT, vcWv.Sticky, 0),
                          vcWv.Param ("Y", vlIntVT, vcWv.Sticky, 0)),
                       TaskParams));

/********************** data objects *************/

/* choose the data icon for the data object */
function choose_data_icon(vf, name) {
    declare icon;
    if (name == nil || equal (name, ""))
        icon = vcRegion.PrefGet (vf.region, "data_icon");
    else
        icon = name;
    add_mf (icon);
}
choose_initial_data_icon := choose_data_icon;
    
/* add data object: store dialog info and begin placing an icon */
function add_data(vf, ev, handle, vers, class, attr, loc, editor, icon) {
    canvas := vf.canvas;
    i := choose_initial_data_icon (vf, icon);
    vcCmd.Run (vge.InstCmd, vflCanvas.Display (canvas), ev,
           list (i, 0, FALSE, InstanceScale), TRUE);
    pi := struct PlaceInfoVT {vflDataVT, 
                struct PlaceDataVT {handle, vers, class, 
                                        attr, loc, editor, icon}};
    set_new_obj (vf.flow_info.draw, pi);
}

AddDataCmd := apply (vcWv.Cmd, append (list (fac, add_data, 
                      "AddData", "Create New Data Object"),
                      DataParams));
vcCmd.CallOnDialogCreate (AddDataCmd, 
              func (vf, ev, h, vers, class, attr, l, e, i)
                  SetupAttrDialog (class, attr););
vcCmd.EnableApply (AddDataCmd, TRUE);

function modify_data(vf, ev, handle, vers, class, attr, loc, editor, icon) {
    vf.flow_info.modified = TRUE;
    d := vge_to_vfl (vf.flow_info.flow, 
                 vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflData.Reset (d);
    if (!equal (handle, vflData.Name (d)))
        replace_label (d, handle, vf);
    if (!equal (icon, vflData.Icon (d)))
        replace_symbol (d, icon, vf, vf.flow_info.draw, vf.flow_info.flow);
    vflData.SetName (d, handle);
    vflData.SetClass (d, class);
    vflData.SetAttrString (d, attr);
    finish_create_data (d, loc, editor, vers);
    set_info_highlighting (vf);
    ReportSelected (vf, nil, vf.flow_info.draw);
}
/* same as modify_task, I'm not using vcWv.Dynamic */
function setup_modify_data_dialog(vf, ev, handle, vers, class, 
                                     attr, loc, editor, icon) {
    d := vge_to_vfl (vf.flow_info.flow, 
                   vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vcRegion.Set (handle, vflData.Name (d));
    vcRegion.Set (class, vflData.Class (d));
    vcRegion.Set (attr, vflData.AttrString (d));
    l := vflData.Location (d);
    vcRegion.Set (loc, l);
    e := vflData.Editor (d);
    i := vflData.Icon (d);
    vcRegion.Set (icon, i);
    vcRegion.Set (editor, (equal (e, "")) ? "Default for Class" : e);
    v := vflData.Version (d);
    if (equal (v, vdm.VersLatest))
        v = "Latest";
    else if (equal (v, vdm.VersDefault))
        v = "Default";
    vcRegion.Set (vers, v);
    SetupAttrDialog (class, attr);
}

ModifyDataCmd := apply (vcWv.Cmd, append (list (fac, modify_data, 
                      "ModifyData", "Modify Data"),
                      DataParams));
vcCmd.CallOnDialogCreate (ModifyDataCmd, setup_modify_data_dialog);

function copy_data(vf, ev) {
    d := vge_to_vfl (vf.flow_info.flow, 
                   vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vcCmd.Run (AddDataCmd, vf, ev, list (
                     vflData.Name (d) + "_2",
                     vflData.Version (d),
                     vflData.Class (d),
                     vflData.AttrString (d),
                     vflData.Location (d),
                     vflData.Editor (d),
                     vflData.Icon (d)), TRUE);
}
CopyDataCmd := vcWv.Cmd (fac, copy_data, "copy_data");

/* this would generally only be run programatically */
function add_data_at(vf, ev, x, y, handle, vers, class, 
                                  attr, loc, editor, icon) {
    i := choose_initial_data_icon (vf, icon);
    obj := vge.InstanceCreateXY (x, y, i, 0, FALSE, InstanceScale);

    /* do far, we've put the object with the lower left corner at
     * the x, y spot. I'd rather put the upper left at that spot,
     * to make drag/drop work better. Thus, the move. 
     */
    mbb := vge.ObjectMbb (obj);
    dy := vgRect.Ymax (mbb) - vgRect.Ymin (mbb);
    vge.ObjectMove (obj, 0, -dy);
    pi := struct PlaceInfoVT {vflDataVT, 
                struct PlaceDataVT {handle, vers, class, 
                                        attr, loc, editor, icon}};
    vge.ObjectSetData (obj, pi);
    vge.DrawAddObject (vf.flow_info.draw, obj);
}
AddDataAtCmd := apply (vcWv.Cmd, append (list (fac, add_data_at, 
                      "AddDataAt", "Add New Data Entry",
                      vcWv.Param ("X", vlIntVT, vcWv.Sticky, 0),
                      vcWv.Param ("Y", vlIntVT, vcWv.Sticky, 0)),
                   DataParams));
vcCmd.CallOnDialogCreate (AddDataAtCmd, 
              func (vf, ev, x, y, h, v, class, attr, l, e, i)
                  SetupDataDialog (class, attr););


/********** end Add submenu *******/

/* modify the selected object: call either ModifyTask or ModifyData */
function modify_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil) {
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (typeof (obj) == vflTaskVT)
            vcCmd.Run (ModifyTaskCmd, vf, ev, nil, TRUE);
        else if (typeof (obj) == vflDataVT)
           vcCmd.Run (ModifyDataCmd, vf, ev, nil, TRUE);
        else if (typeof (obj) == vflDependVT)
            vcCmd.Run (ModifyDependCmd, vf, ev, nil, TRUE);
        else if (vge.ObjectGetData (o) == nil)
            /* just a graphics addon */
            vcCmd.Run (vge.StyleCmd, vflCanvas.Display (vf.canvas), 
                      ev, nil, TRUE);
    } 
}
ModifySlctCmd := vcWv.Cmd (fac, modify_slct, "ModifySlct");

function copy_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil) {
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (typeof (obj) == vflTaskVT)
            vcCmd.Run (CopyTaskCmd, vf, ev, nil, TRUE);
        else if (typeof (obj) == vflDataVT)
           vcCmd.Run (CopyDataCmd, vf, ev, nil, TRUE);
        else if (vge.ObjectGetData (o) == nil) {
            /* if menu pick, make it work nicely */
            if (vcEvent.Ch (ev) != vcCmd.Key (CopySlctCmd)) {
                user := sprintf ("%d %d", vge.ObjectGetX (o),
                                          vge.ObjectGetY (o));
                vcEvent.SetUserCoords (ev, user);
            }
            vcCmd.Run (vge.CopyCmd, vflCanvas.Display (vf.canvas),
                              ev, nil, TRUE);
        }
    }
}
CopySlctCmd := vcWv.Cmd (fac, copy_slct, "CopySlct");

function move(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil) {
        if (vcEvent.Ch (ev) != vcCmd.Key (MoveCmd)) {
            user := sprintf ("%d %d", vge.ObjectGetX (o),
                                      vge.ObjectGetY (o));
            vcEvent.SetUserCoords (ev, user);
        }
        vcCmd.Run (vge.MoveCmd, vflCanvas.Display (vf.canvas),
                             ev, nil, TRUE);
    }
}
MoveCmd := vcWv.Cmd (fac, move, "Move");

/* remove selected object */
function delete_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);

    if (obj != nil) {
        vflFlow.Remove (vf.flow_info.flow, obj);
        vge.DrawRemoveObject (vf.flow_info.draw, o);
        lab := GetLabel (obj);
        if (lab != nil)
            vge.DrawRemoveObject (vf.flow_info.draw, lab);

        /* if we removed a node, we might affect some dependencies */
        if (typeof (obj) != vflDependVT)
            foreach (dep; vflFlow.Depends (vf.flow_info.flow)) {
                sym := GetSymbol (dep);
                if (sym != nil)
                    reevaluate_dependency (dep, vf, sym);
            }
        set_info_highlighting (vf);
        vcRegion.QueueForRefresh (vf.canvas);
    } else if (vge.ObjectGetData (o) == nil) {
        vge.DrawRemoveObject (vf.flow_info.draw, o);
        vcRegion.QueueForRefresh (vf.canvas);
    }

} 

/* dialog box: are you sure you want to delete? */
function confirm_delete (vf, ev) { 
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);
    declare type, name;

    if (typeof (obj) == vflDataVT) {
        type = "data object";
        name = vflData.Name (obj);
    } else if (typeof (obj) == vflTaskVT) {
        type = "task object";
        name = vflTask.Name (obj);
    } else if (typeof (obj) == vflDependVT) {
        type = "dependency";
        name = vflDepend.Name (obj);
    }

    if (type == nil)
        FALSE;
    else
        sprintf ("Delete %s %s?", type, name);
}
DeleteSlctCmd := vcWv.CmdWithConfirm (fac, delete_slct, confirm_delete,
                  TRUE, "DeleteSlct");

/* look for the object with this id & select it */
function select_by_id(vf, id) {
    obj := vflFlow.GetFromId (vf.flow_info.flow, id);
    o := GetSymbol (obj);
    vge.UnselectAll (vf.canvas);
    if (o != nil)
        vge.DrawSelectObject (vf.flow_info.draw, o);
    o;
}


/* find the obejct with the name and select it */
function select_by_name(vf, name) {
    obj := vflFlow.GetFromName (vf.flow_info.flow, name);
    o := GetSymbol (obj);
    vge.UnselectAll (vf.canvas);
    if (o != nil)
        vge.DrawSelectObject (vf.flow_info.draw, o);
    o;
}

/* select object with id, then run modify command */
function modify_id(vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (ModifySlctCmd, vf, ev, nil, TRUE);
}
ModifyIdCmd := vcWv.Cmd (fac, modify_id, "ModifyId", 
                      "Modify Object By Id",
                 vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));
/* select object with id, then run delete command */
function delete_id(vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (DeleteSlctCmd, vf, ev, nil, TRUE);
}
DeleteIdCmd := vcWv.Cmd (fac, delete_id, "DeleteId", 
                      "Delete Object By Id",
                 vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));


/* select by name, run modify command */
function modify(vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (ModifySlctCmd, vf, ev, nil, TRUE);
}
ModifyCmd := vcWv.Cmd (fac, modify, "Modify", 
                      "Modify Object",
                 vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

/* select by name, then tun delete */
function delete(vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (DeleteSlctCmd, vf, ev, nil, TRUE);
}
DeleteCmd := vcWv.Cmd (fac, delete, "Delete", 
                      "Delete Object",
                 vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

/* zoom to the vge object */
function zoom_to(canvas, o) {
    mbb := vge.ObjectMbb (o);
    dx := vgRect.Xmax (mbb) - vgRect.Xmin (mbb);
    dy := vgRect.Ymax (mbb) - vgRect.Ymin (mbb);

    uspace := vgRect.Create (vgRect.Xmin (mbb) - 3 * dx,
                             vgRect.Ymin (mbb) - 3 * dy,
                             vgRect.Xmax (mbb) + 3 * dx,
                             vgRect.Ymax (mbb) + 3 * dy);
    vcVPCanvas.SetUserspace (canvas, uspace);
}
function find_id(vf, ev, id) {
    o := select_by_id (vf, id);
    zoom_to (vf.canvas, o);
}
FindIdCmd := vcWv.Cmd (fac, find_id, "FindId", "Find Object By Id", 
              vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));
function find(vf, ev, name) {
    o := select_by_name (vf, name);
    zoom_to (vf.canvas, o);
}
FindCmd := vcWv.Cmd (fac, find, "Find", "Find Object By Name", 
              vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

/* some edit mode stuff */

function edit_subflow(vf, ev, task) {
    l := vflTask.GetAttr (task, "LOCATION");
    l = vflFlow.ReplaceSymbols (vf.flow_info.flow, l);
    vcCmd.Run (OpenCmd, vf, ev, 
                 list (vflTask.Command (task), 
                       l,
                       vf.flow_info.mode), FALSE);
}
function edit_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);

    if (typeof (obj) == vflTaskVT)
        edit_subflow (vf, ev, obj);
    else {
        function output (str)
            vflFlow.Output (vf.flow_info.flow, nil, str);
        vflData.Edit (obj, vf.flow_info.flow, output, output, nil);
    }
}
EditSlctCmd := vcWv.Cmd (fac, edit_slct, "EditSlct");
    
function edit_id (vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (EditSlctCmd, vf, ev, nil, TRUE);
}
EditIdCmd := vcWv.Cmd (fac, edit_id, "EditId", "Edit Object With Id",
            vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));
function edit (vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (EditSlctCmd, vf, ev, nil, TRUE);
}
EditCmd := vcWv.Cmd (fac, edit, "Edit", "Edit Object With Name",
              vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

function manage_slct (vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);
    vdm_obj := vflData.GetVdmObject2 (obj, vf.flow_info.flow);
    vdata.SelectObject (vf.vdata_rgn, vdm_obj);
    vcWin.Show (vf.vdata_win);
}
ManageSlctCmd := vcWv.Cmd (fac, manage_slct, "ManageSlct");

function manage_id (vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (ManageSlctCmd, vf, ev, nil, TRUE);
}
ManageIdCmd := vcWv.Cmd (fac, manage_id, "ManageId", "Manage Object With Id",
            vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));
function manage (vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (ManageSlctCmd, vf, ev, nil, TRUE);
}
ManageCmd := vcWv.Cmd (fac, manage, "Manage", "Manage Object With Name",
              vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

function update_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);

    vflFlow.UpdateTask (vf.flow_info.flow, obj);
}
UpdateSlctCmd := vcWv.Cmd (fac, update_slct, "UpdateSlct");

function update_id (vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (UpdateSlctCmd, vf, ev, nil, TRUE);
}
UpdateIdCmd := vcWv.Cmd (fac, update_id, "UpdateId", "Update Object With Id",
            vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));

function update (vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (UpdateSlctCmd, vf, ev, nil, TRUE);
}
UpdateCmd := vcWv.Cmd (fac, update, "Update", "Update Task With Name",
              vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

function reset_slct (vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    obj := vge_to_vfl (vf.flow_info.flow, o);
    if (typeof (obj) == vflDataVT)
        vflData.Reset (obj);
    else
        vflTask.Reset (obj, vf.flow_info.flow);
}
ResetSlctCmd := vcWv.Cmd (fac, reset_slct, "ResetSlct");

function reset_id (vf, ev, id) {
    select_by_id (vf, id);
    vcCmd.Run (ResetSlctCmd, vf, ev, nil, TRUE);
}
ResetIdCmd := vcWv.Cmd (fac, reset_id, "ResetId", "Reset Object With Id",
            vcWv.Param ("Id", vlPositiveIntVT, vcWv.Sticky, 1));
function reset (vf, ev, name) {
    select_by_name (vf, name);
    vcCmd.Run (ResetSlctCmd, vf, ev, nil, TRUE);
}
ResetCmd := vcWv.Cmd (fac, reset, "Reset", "Reset Object With Name",
              vcWv.Param ("Name", vlStringRestOfLineVT, vcWv.Sticky, ""));

/********************* DEPENDENCY MENU *********************/

/*********** ADD submenu ****/

/* create a poly-vector for use with dependency */
function create_pvect (points) {
    o := vge.PvectCreateXY (vgArrow.Point2, 15, 20, vgFill.Solid, points);
    vge.ObjectSetColor (o, vgColor.Yellow);
    o;
}

/* create and setup a pvect dependency */
function make_dep_pline (points) {
    o := create_pvect (points);
    if (length (points) > 1) {
        p1 := vge.PvectGetPointXY (o, 0);
        p2 := vge.PvectGetPointXY (o, vge.PvectGetLastPoint (o));
        vge.ObjectSetData (o, 
              struct PlaceInfoVT {vflDependVT, 
               struct PlaceDependVT {car (p1), cdr (p1), car (p2), cdr (p2)}});
    }
    o;
}

DependencyPlineCmd := vge.make_pline (make_dep_pline, "DependPline");

/* crete a dependency spline */
function create_spline(points) {
    o := vge.SplineCreateXY (vgSplineOrder.Smooth, vgSplineType.Open, 20,
                                         vgArrow.Point2, 15, 20,
                                         vgFill.Solid, points);
    vge.ObjectSetColor (o, vgColor.Yellow);
    o;
}
/* create & setup a dependecy */
function make_dep_spline (points) {
    o := create_spline (points);
    if (length (points) > 0) {
        p1 := vge.SplineGetPointXY (o, 0);
        p2 := vge.SplineGetPointXY (o, vge.SplineGetLastPoint (o));
        vge.ObjectSetData (o, 
              struct PlaceInfoVT {vflDependVT, 
               struct PlaceDependVT {car (p1), cdr (p1), car (p2), cdr (p2)}});
    }
    o;
}

DependencySplineCmd := vge.make_pline (make_dep_spline, "DependSpline");

/* run one or the other dependecy drawing commands */
DepPlineAuxCmd := vcWv.Cmd (fac, func(vf, ev) {
       display := vflCanvas.Display (vf.canvas);
       if (vf.curved_depends)
           vcCmd.Run (DependencySplineCmd, display, ev,
                           list (nil), TRUE);
       else
           vcCmd.Run (DependencyPlineCmd, display, ev,
                           list (nil), TRUE); }, 
    "AddDepend");

function add_line (vf, ev) {
    function run_starter_cmd (ev)
        vcCmd.Run (DepPlineAuxCmd, vf, ev, nil, TRUE);
    vflCanvas.EnterMode (vf.canvas, run_starter_cmd, 
                   "Begin drawing a dependency.");
}
AddLineCmd := vcWv.Cmd (fac, add_line, "Dependency");

function dep_mode(vf, ev, flag) {
    vcBind.Set (vflCanvas.bindings, '\middledown', 
                  (flag) ? nil : vcGesture.StartCmd);
    vcBind.Set (bindings, '\middledown',
                  (flag) ? DepPlineAuxCmd : nil);
}
DepModeAuxCmd := vcWv.Cmd (fac, dep_mode, "DepMode", "Dependency Mode",
                vcWv.Param ("Mode", vlOnOffVT, vcWv.Sticky, FALSE));
dep_mode_entry := vcWv.CheckedOption ("Dependency Mode", 
                                            DepModeAuxCmd, FALSE);
DepModeCmd := cadr (dep_mode_entry);
  
/* get the vge object at the event */
function get_event_object (vf, ev) {
    vp := vcVPCanvas.Viewport (vf.canvas);
    x := vcEvent.UserX (ev);
    y := vcEvent.UserY (ev);
    vge.DrawFindObjectXY (vf.flow_info.draw, x, y);
}

/* create a dependency between the two objects */
function add_dependency_between (vf, o1, o2) {
    s1 := GetSymbol (o1);
    s2 := GetSymbol (o2);
    vf.flow_info.modified = TRUE;
    if (s1 != nil && s2 != nil) {
        d := vflDepend.Create (o1, o2);
        m1 := vge.ObjectMbb (s1);
        m2 := vge.ObjectMbb (s2);
        /* create the vge obejct at the right point */
        if (vf.curved_depends)
           obj := create_spline (ChooseAttachPoints (m1, m2));
        else
            obj := create_pvect (ChooseAttachPoints (m1, m2));
        vge.DrawAddObject (vf.flow_info.draw, obj);
        vcRegion.QueueForRefresh (vf.canvas);
        add_dependency (vf, d, obj);
    }
}

/* attach from the selected node: go into select-node mode */
function attach_from (vf, ev) {
    function finish_attach_from (ev) {
        o := get_event_object (vf, ev);
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (obj != nil) 
            add_dependency_between (vf, vf.firstobj, obj);
    }
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil) {
        vf.firstobj = vge_to_vfl (vf.flow_info.flow, o);
        vflCanvas.EnterMode (vf.canvas, finish_attach_from, 
                      "Select object to which dependency goes.");
    }
}

AttachFromCmd := vcWv.Cmd (fac, attach_from, "AttachFrom");

/* attach from the selected node: go into select-node mode */
function attach_to (vf, ev) {
    function finish_attach_to (ev) {
        o := get_event_object (vf, ev);
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (obj != nil) 
            add_dependency_between (vf, obj, vf.firstobj);
    }
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil) {
        vf.firstobj = vge_to_vfl (vf.flow_info.flow, o);
        vflCanvas.EnterMode (vf.canvas, finish_attach_to, 
             "Select object from which dependency comes.");
    }
}

AttachToCmd := vcWv.Cmd (fac, attach_to, "AttachTo");

/* attach dependency between 2 nodes by id */
function depend_id (vf, ev, from, to) {
    o := select_by_id (vf, from);
    n1 := vge_to_vfl (vf.flow_info.flow, o);
    o := select_by_id (vf, to);
    n2 := vge_to_vfl (vf.flow_info.flow, o);

    add_dependency_between (vf, n1, n2);
}
DependIdCmd := vcWv.Cmd (fac, depend_id, "DependId",
               "Add Dependency By Id",
           vcWv.Param ("From", vlPositiveIntVT, vcWv.Sticky, 1),
           vcWv.Param ("To", vlPositiveIntVT, vcWv.Sticky, 1));

/* attach dependency between 2 nodes by name */
function depend_name (vf, ev, from, to) {
    o := select_by_name (vf, from);
    n1 := vge_to_vfl (vf.flow_info.flow, o);
    o := select_by_name (vf, to);
    n2 := vge_to_vfl (vf.flow_info.flow, o);

    add_dependency_between (vf, n1, n2);
}
DependNameCmd := vcWv.Cmd (fac, depend_name, "Depend",
               "Add Dependency By Name",
           vcWv.Param ("From", vlStringVT, vcWv.Sticky, ""),
           vcWv.Param ("To", vlStringVT, vcWv.Sticky, ""));

/******** end Add submenu */
function start_stretch (vf, ev) {
    function stretch (ev)
        vcCmd.Run (vge.StretchCmd, vflCanvas.Display (vf.canvas),
                   ev, nil, TRUE);
    vflCanvas.EnterMode (vf.canvas, stretch, 
           "Select object to stretch.");
}
StretchCmd := vcWv.Cmd (fac, start_stretch, "Stretch");
vcCmd.SetKey (StretchCmd, 's');

/* modify dependency command */
function modify_depend(vf, ev, name, type, req) {
    vf.flow_info.modified = TRUE;
    dep := vge_to_vfl (vf.flow_info.flow,
                       vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflDepend.SetType (dep, type);
    vflDepend.SetRequired (dep, req);
    o := GetSymbol (dep);
    if (o != nil) {
        show_depend_validity (dep, o);
        declare color;
        if (type == vflDependType.OnPass)
            color = vcRegion.PrefGet (vf.region, "depend_color");
        else if (type == vflDependType.OnFail)
            color = vcRegion.PrefGet (vf.region, "depend_fail_color");
        else if (type == vflDependType.Always)
            color = vcRegion.PrefGet (vf.region, "depend_always_color");
        else if (type == vflDependType.Never)
            color = vcRegion.PrefGet (vf.region, "depend_never_color");
        vge.ObjectSetColor (o, color);
    }
}
function get_dep_name(vf) {
    dep := vge_to_vfl (vf.flow_info.flow,
                       vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflDepend.Name (dep);
}
function get_dep_type(vf) {
    dep := vge_to_vfl (vf.flow_info.flow,
                       vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflDepend.Type (dep);
}
function get_dep_req(vf) {
    dep := vge_to_vfl (vf.flow_info.flow,
                       vge.DrawGetOneSelectedObject (vf.flow_info.draw));
    vflDepend.Required (dep);
}
dep_type_data := list (list ("On Pass", vflDependType.OnPass),
                       list ("On Fail", vflDependType.OnFail),
                       list ("Always",  vflDependType.Always),
                       list ("Never",  vflDependType.Never));
ModifyDependCmd := vcWv.Cmd (fac, modify_depend, "ModifyDepend",
                             "Modify Dependency",
        vcWv.Param ("Name", vlStringVT, vcWv.Dynamic, get_dep_name,
               vcStaticText.fac),
        vcWv.Param ("Type", vflDependTypeVT, vcWv.Dynamic, get_dep_type,
               vcRadioBox.fac, data <- dep_type_data),
        vcWv.Param ("Required", vlTrueFalseVT, vcWv.Dynamic, get_dep_req));

/* toggle between splines and pvects */
function curved_depend (vf, ev, state)
    vf.curved_depends = state;
CurvedDependCmd := vcWv.Cmd (fac, curved_depend, "CurvedDepend", 
                   "Use Curved Dependencies",
                   vcWv.Param ("Curved", vlTrueFalseVT, vcWv.Sticky, FALSE));


/*********** wrappers around vpcanvas commands ********/
function pan (vf, ev) {
    function do_pan (ev)
        vcCmd.Run (vcVPCanvas.PanCmd, 
                   vcRegion.Data (vf.region, vcFac.Typespec (vcVPCanvas.fac)),
                   ev, nil, TRUE);
    vflCanvas.EnterMode (vf.canvas, do_pan, "Pan to mouse click.");
}
PanCmd := vcWv.Cmd (fac, pan, "Pan");
/* make sure the key appears in the menu */
vcCmd.SetKey (PanCmd, '\f6');

function start_zoom (vf, ev) {
    function do_pan (ev)
        vcCmd.Run (vcVPCanvas.ZoomCmd, 
                   vcRegion.Data (vf.region, vcFac.Typespec (vcVPCanvas.fac)),
                   ev, nil, TRUE);
    vflCanvas.EnterMode (vf.canvas, do_pan, "Begin zooming.");
}
StartZoomCmd := vcWv.Cmd (fac, start_zoom, "Zoom");
vcCmd.SetKey (StartZoomCmd, '\f9');

OverviewBindings := vcBind.Create ();
vcBind.Set (OverviewBindings, '\leftdown', vge.SelectCmd);
vcBind.Set (OverviewBindings, '\^\leftdown', vge.SelectAddCmd);
vcBind.Set (OverviewBindings, '\middledown', vcGesture.StartCmd);

/* produce a little baby vge region */
function create_overview_window(parent, fi) {
    if (fi.overview_win == nil) {
        title := sprintf ("Overview: %s\n", fi.name);
        vg := vcRegion.Create (title, vge.fac, 
                        vge.DrawName (fi.draw),
                        size <- vgSize.Create (300, 300, 100, 100, 800, 800));
        vcRegion.SetBindings (vg, OverviewBindings);
        fi.overview_win = vcWin.Create (parent, vg, 
                      vcWin.NoBindings + vcWin.NoStatus + vcWin.NoMenus);
        vcWin.CallOnClose (fi.overview_win, vcWin.Hide);
    } else
        vcWin.Show (fi.overview_win);
}
   

/* create overview of the current flow */
function overview (vf, ev)
    create_overview_window (vcEvent.Window (ev), vf.flow_info);
OverviewCmd := vcWv.Cmd (fac, overview, "Overview");

/**************** FLOW MENU *************/

/* running a task */
function run_slct(vf, ev) {
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil)
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (typeof (obj) == vflTaskVT)
            vflFlow.RunTask (vf.flow_info.flow, obj);
}
function confirm_run_slct(vf, e) {
    ret := FALSE;
    o := vge.DrawGetOneSelectedObject (vf.flow_info.draw);
    if (o != nil)
        obj := vge_to_vfl (vf.flow_info.flow, o);
        if (typeof (obj) == vflTaskVT && 
            vflTask.GetState (obj) == vflTaskState.Running) {
            remote := vflTask.RemoteRun (obj);
            if (length (remote) == 0)
                vflow_error (40, vlErrSev.Error, 
                      sprintf ("Task %s is already running.", 
                                                        vflTask.Name (obj)));
            else
                ret := sprintf ("Task %s is being run by user %s.\n\n" +
                                "Continue?", vflTask.Name (obj), remote);
        }
    ret;        
}
RunSlctCmd := vcWv.CmdWithConfirm (fac, run_slct, confirm_run_slct,
                                      TRUE, "RunSlct", "run_slct");

function global_symbols (vf, ev) {
    if (vf.global_sym_w == nil) {
        declare initial;
        foreach (gs; vflFlow.Gsyms (vf.flow_info.flow)) {
            entry := list (vflGsym.Name (gs),
                           vflGsym.Value (gs),
                           vflGsym.Type (gs),
                           vflGsym.Help (gs));
            initial = cons (entry, initial);
        }
        initial = reverse_destructive (initial);
        vf.global_sym_w = vflSym.Create (vcEvent.Window (ev), 
                                        initial, vf.flow_info.mode);
        vcWin.DestroyNotify (vf.global_sym_w, 
                                   func (w) vf.global_sym_w = nil;);
    } else
        vcWin.Show (vf.global_sym_w);
}
GlobalSymbolCmd := vcWv.Cmd (fac, global_symbols, "GlobalSymbols");

/* commands run by the vflSym dialog box in Edit mode:
 * define all of the symbols
 */
function set_global_symbols(vf, ev) {
    sym := vcRegion.FindFac (vcWin.OwnerRegion (vf.global_sym_w),
                             vflSym.fac);
    vflFlow.ClearGsyms (vf.flow_info.flow);
    foreach (name, val, type, help; vflSym.Entries (sym)) {
        if (length (name) > 0) {
            gs := vflGsym.Create (name, val, type);
            vflGsym.SetHelp (gs, help);
            vflFlow.AddGsym (vf.flow_info.flow, gs);
        }
    }
    set_info_highlighting (vf);
    vcWin.Close (vf.global_sym_w);
}

SetGlSymCmd := vcWv.Cmd (fac, set_global_symbols, "set_gl_sym");

/* the command that gets run in Run mode */
/* check to see if identifying symbols have changed */ 
function confirm_change_symbols(vf, ev) {
    sym := vcRegion.FindFac (vcWin.OwnerRegion (vf.global_sym_w),
                             vflSym.fac);
    /* the assumption here is that the vflSym.Entries iterator
     * and the vflFlow.Gsyms iterator produce the same order.
     * This is guarranteed by the way the original dialog is created 
     */
    declare newvals;
    foreach (name, val, type, help; vflSym.Entries (sym))
        newvals = cons (val, newvals);
    newvals = reverse_destructive (newvals);

    changed := FALSE;
    changed_list := "The following indentifying symbols have changed:\n";
    foreach (gs; vflFlow.Gsyms (vf.flow_info.flow)) {
        if (vflGsym.Type (gs) == vflGsymType.Identifying &&
            !equal (car (newvals), vflGsym.Value (gs))) {
            changed = TRUE;
            changed_list += "    " + vflGsym.Name (gs) + "\n";
        }
        newvals = cdr (newvals);
    }
    changed_list += "\nThis will reset the flow.\nContinue?";
    (changed) ? changed_list : FALSE;
}

function change_symbols(vf, ev) {
    sym := vcRegion.FindFac (vcWin.OwnerRegion (vf.global_sym_w),
                             vflSym.fac);
    declare newvals;
    foreach (name, val, type, help; vflSym.Entries (sym))
        newvals = cons (val, newvals);
    newvals = reverse_destructive (newvals);

    /* have identifying symbols changed? */
    changed := FALSE;
    foreach (gs; vflFlow.Gsyms (vf.flow_info.flow)) {
        if (vflGsym.Type (gs) != vflGsymType.Constant &&
            !equal (car (newvals), vflGsym.Value (gs))) {
            vflGsym.SetValue (gs, car (newvals));
            if (vflGsym.Type (gs) == vflGsymType.Identifying)
                changed = TRUE;
        }
        newvals = cdr (newvals);
    }
    if (changed)
        reset_all (vf, ev);
    vcWin.Close (vf.global_sym_w);
}
ChangeSymbolsCmd := vcWv.CmdWithConfirm (fac, change_symbols, 
                      confirm_change_symbols, TRUE, "change_symbols");

function viewdata (vf, ev)
    vcWin.Show (vf.vdata_win);
ViewdataCmd := vcWv.Cmd (fac, viewdata, "ViewData");

function log_window(vf, ev)
    vcWin.Show (vf.log_win);
LogWindowCmd := vcWv.Cmd (fac, log_window, "LogWindow");

function update_all(vf, ev)
    foreach (t; vflFlow.Tasks (vf.flow_info.flow))
        vflFlow.UpdateTask (vf.flow_info.flow, t);
UpdateAllCmd := vcWv.Cmd (fac, update_all, "UpdateAll");

function reset_the_flow (flow) {
    foreach (t; vflFlow.Tasks (flow))
        vflTask.Reset (t, flow);
    foreach (d; vflFlow.Datas (flow))
        vflData.Reset (d);
}
function reset_all(vf, ev)
    reset_the_flow (vf.flow_info.flow);

ResetAllCmd := vcWv.Cmd (fac, reset_all, "ResetAll");

function make(vf, ev)
    vflFlow.Make (vf.flow_info.flow);
function confirm_make(vf, ev) {
    user := vflFlow.RemoteMake (vf.flow_info.flow);
    if (length (user) == 0)
        FALSE;
    else
        sprintf ("Make is currently being run on\n" +
                 "this flow by user %s.\n\nContinue?", user);
}
MakeCmd := vcWv.CmdWithConfirm (fac, make, confirm_make, TRUE, "Make", "");
  
function cancel_make(vf, ev)
    vflFlow.CancelMake (vf.flow_info.flow);
CancelMakeCmd := vcWv.Cmd (fac, cancel_make, "CancelMake");

function customize_vflow(vf, ev) {
    vcWv.RequireFac (wv, "vcPref");
    vcPref.CreateWindow (vcEvent.Window (ev), fac);
}
CustomizeVflowCmd := vcWv.Cmd (fac, customize_vflow, "CustomizeVflow");

function customize_vfl (vf, ev) {
    vcWv.RequireFac (wv, "vcPref");
    w := vcPref.CreateWindowAux (vcEvent.Window (ev), fac, vfl.prefs);
    /* since vfl is not a facility, there is no _Reset method, thus...*/
    vcWin.DestroyNotify (w, func (w) vfl.ResetPreferences (););
}
CustomizeVflCmd := vcWv.Cmd (fac, customize_vfl, "CustomizeVfl");

function customize_colors (vf, ev) {
    vcWv.RequireFac (wv, "vcPref");
    w := vcPref.CreateWindowAux (vcEvent.Window (ev), fac, colors);
}
CustomizeTextColorsCmd := vcWv.Cmd (fac, customize_colors, 
                                         "CustomizeColors");

function icon_editor (vf, ev, name) {
    r := vcRegion.Create ("Icon Editor", vge.fac, name);
    vcWin.Create (vcEvent.Window (ev), r);
}
IconEditorCmd := vcWv.Cmd (fac, icon_editor, "IconEditor", "Edit Icon",
                vcWv.Param ("Name", vlStringVT, vcWv.Sticky, "",
                              vcFilePop.fac, filter <- "*.mf"));


/*****************************************/
/*       the check flow dialog           */
/*****************************************/

function check(vf, ev) {
    declare result;
    vcWin.WithBusy (vcEvent.Window (ev), func () 
        result = vflFlow.Check (vf.flow_info.flow););

    /* first, the invalid dependency picklist */
    invalid := car (result);
    result = cdr (result);
    declare invalid_pick_data;
    foreach (elem; vscript.iter (invalid)) 
        invalid_pick_data = cons (cons (vflDepend.Name (elem), elem),
                                  invalid_pick_data);
    invalid_pick := vcRegion.Create ("", vcPicklist.fac, invalid_pick_data,
                                          will_scroll <- TRUE);
    invalid_scroll := vcPicklist.Scroll ("", invalid_pick, 5, 9, 25);
    function highlight_single(obj, flag) {
        vge.UnselectAll (vf.canvas);
        if (flag) {
            sym := GetSymbol (obj);
            if (sym != nil)
                vge.DrawSelectObject (vf.flow_info.draw, sym);
        }
    }
    vcPicklist.Notify (invalid_pick, highlight_single);

    /* now the duplicates picklist */
    duplicates := car (result);
    result = cdr (result);
    /* each member of the duplicates list is a list of duplicates */
    /* did you get that? try this picture: */
    /* ((thing thing thing) (other other)) */
    declare duplicates_data;
    foreach (elem; vscript.iter (duplicates)) {
        first := car (elem);
        name := (typeof (first) == vflTaskVT) ? vflTask.Name (first) :
                                               vflData.Name (first);
        /* remember, by definition, all members of this list have same name */
        duplicates_data = cons (cons (name, elem), duplicates_data);
    }
    dup_pick := vcRegion.Create ("", vcPicklist.fac, duplicates_data,
                                                    will_scroll <- TRUE);
    dup_scroll  := vcPicklist.Scroll ("", dup_pick, 5, 9, 25);
             
    function highlight_list(l, flag) {
        vge.UnselectAll (vf.canvas);
        if (flag) {
            foreach (elem; vscript.iter (l)) {
                sym := GetSymbol (elem);
                if (sym != nil) {
                    vge.DrawSelectObject (vf.flow_info.draw, sym);
                }
            }
        }
    }
    vcPicklist.Notify (dup_pick, highlight_list);

    /* now the unreachables */
    unreach := car (result);
    result = cdr (result);
    declare unreach_data;
    foreach (elem; vscript.iter (unreach)) {
        name := (typeof (elem) == vflTaskVT) ? vflTask.Name (elem) :
                                               vflData.Name (elem);
        unreach_data = cons (cons (name, elem), unreach_data);
    }
    unreach_pick := vcRegion.Create ("", vcPicklist.fac, unreach_data,
                            will_scroll <- TRUE);
    unreach_scroll  := vcPicklist.Scroll ("", unreach_pick, 5, 9, 25);
    vcPicklist.Notify (unreach_pick, highlight_single);

    circle := car (result);
    result = cdr (result);
    declare circle_data;
    foreach (elem; vscript.iter (circle))
        circle_data = cons (cons (vflNode.Name (car (elem)), elem),
                             circle_data);
    circle_pick := vcRegion.Create ("", vcPicklist.fac, circle_data,
                                        will_scroll <- TRUE);
    circle_scroll  := vcPicklist.Scroll ("", circle_pick, 5, 9, 25);
    vcPicklist.Notify (circle_pick, highlight_list);

    note := vcRegion.Create ("", vcNotebook.fac, vgAlign.UpperCenter);
    vcNotebook.AddEntry (note, "Dependencies", invalid_scroll);
    vcNotebook.AddEntry (note, "Duplicate Names", dup_scroll);
    vcNotebook.AddEntry (note, "Unreachable", unreach_scroll);
    vcNotebook.AddEntry (note, "Circularities", circle_scroll);

    /* only allow selection of entries with errors */
    vcNoteTab.EnableByString (note, "Dependencies", invalid != nil);
    vcNoteTab.EnableByString (note, "Duplicate Names", duplicates != nil);
    vcNoteTab.EnableByString (note, "Unreachable", unreach != nil);
    vcNoteTab.EnableByString (note, "Circularities", circle != nil);

    /* if no errors, don't bother with dialog */
    if (invalid == nil && duplicates == nil &&
        unreach == nil && circle == nil)
        vflow_error (25, vlErrSev.Note, "No errors found in flow.");

    function help(ev)
        vcAlert.SpawnHelp (vcEvent.Window (ev), "vflow", "Check");
    brow := vcRegion.Create ("", vcButnRow.fac, 
              list (list ("", "Continue", func (ev)nil;, 
                                            list (close_window <- TRUE)),
                    list ("", "Help", help, nil)));
    t := vcTable.Make ("Check Flow", 2, 1, note, brow);
    vcTable.Row (t, 1, vcTable.Border.Visible, 1, 1, 1, vgAlign.MiddleCenter);
    w := vcWin.Create (vcEvent.Window (ev), t);

}
CheckCmd := vcWv.Cmd (fac, check, "Check");

/***************** FILE MENU **************/

function set_vge_symbol(obj, sym) {
    v := struct VgesVT {sym, nil};
    if (typeof (obj) == vflTaskVT)
        vflTask.SetGenericData (obj, v);
    else if (typeof (obj) == vflDataVT)
        vflData.SetGenericData (obj, v);
    else if (typeof (obj) == vflDependVT)
        vflDepend.SetGenericData (obj, v);
    else if (typeof (obj) == vflStartStopVT)
        vflStartStop.SetGenericData (obj, v);
}

function set_vge_label (obj, label) {
    v := get_vges (obj);
    if (typeof (v) == VgesVT) {
        v.label = label;
        attach_label (label, v.symbol);
    }
}

/* setup all the connections between vfl and vge objects as necessary */
function connect_vges(vf, fi) {
    /* first setup all the symbols & set them */
    foreach (o; vge.DrawLoop (fi.draw))
        if (vge.ObjectType (o) != vgeType.Text) {
            id := vge.ObjectGetData (o);
            if (typeof (id) == vflIdVT) {
                obj := vflFlow.GetFromId (fi.flow, id);
                set_vge_symbol (obj, o);
            }
        }

    /* now try the labels */
    foreach (o; vge.DrawLoop (fi.draw))
        if (vge.ObjectType (o) == vgeType.Text) {
            id := vge.ObjectGetData (o);
            if (typeof (id) == vflIdVT) {
                obj := vflFlow.GetFromId (fi.flow, id);
                set_vge_label (obj, o);
                if (!vcRegion.PrefGet (vf.region, "move_labels"))
                    vge.ObjectSetSelectStyle (o,  vgeSelectStyle.None);
            }
        }
    foreach (dep; vflFlow.Depends (fi.flow)) {
        attach_dependency (dep, vflDepend.From (dep), FALSE);
        attach_dependency (dep, vflDepend.To (dep), TRUE);
    }
}

function set_win_title (vf) {
    mode := (vf.flow_info.mode == vflowMode.Run) ? "Runn" : "Edit";
    vcWin.SetTitle (vcRegion.Window (vf.region), 
                     sprintf ("%sing flow %s", mode,
                            vf.flow_info.name));

}

/* place the start/stop icons on the canvas */
function place_start_stop(vf, fi) {
    if (vcRegion.PrefGet (vf.region, "startstop")) {
        uspace := vcVPCanvas.GetUserspace (vf.canvas);
        start := vflFlow.Start (fi.flow);
        starti := vge.InstanceCreateXY (vgRect.Xmin (uspace), 
                                       vgRect.Ymax (uspace),
                           vcRegion.PrefGet (vf.region, "start_icon") + ".mf",
                                   0, FALSE, 2.0);
        vge.ObjectSetData (starti, vflStartStop.Id (start));
        vflStartStop.SetGenericData (start, struct VgesVT {starti, nil});
        stop := vflFlow.Stop (fi.flow);
        stopi := vge.InstanceCreateXY (vgRect.Xmax (uspace) - 20, 
                                       vgRect.Ymin (uspace) + 20,
                           vcRegion.PrefGet (vf.region, "stop_icon") + ".mf",
                                   0, FALSE, 2.0);
        vflStartStop.SetGenericData (stop, struct VgesVT {stopi, nil});
        vge.ObjectSetData (stopi, vflStartStop.Id (stop));
        vge.DrawAddObject (fi.draw, starti);
        vge.DrawAddObject (fi.draw, stopi);
    }
}

/* strip the .vfl extension from the name, since it is added by
 * the vflFlow class 
 */
function strip_extension(name) {
    len := length (name);
    if (len > 4 && equal (substring (name, len - 4, len), ".vfl"))
        substring (name, 0, len - 4);
    else
         name;
}


/* used with verify_params to ensure that a valid location name is used */
function verify_location(vf, loc) {
    if (length (loc) > 0 &&
          vpm.MapFindLocation (vdm.CntxGetMap (vf.dm), loc) == nil)
        vflow_error (30, vlErrSev.Error,
                       sprintf ("No location defined for alias `%s.'", loc));
}

function new(vf, ev, name, location) {
    verify_location (vf, location);
    name = strip_extension (name);
    fi := find_flow_by_name (vf, name);
    if (fi == nil) {
        loc := vfl.Root;
        if (length (location) > 0)
            loc = vpm.MapFindLocation (vdm.CntxGetMap (vf.dm), location);
        o := vdm.ObjCreate (name, vdm.VersLatest, vflFlowClass, 
                                                  loc, vf.dm, nil);
        flow := vflFlow.Create (name, vf.dm);
        draw := vge.DrawCreate (name + ".mf");
        vge.DrawSetDoModify (draw, FALSE);
        fi := create_flow_info (vf, name, flow, draw, vflowMode.Edit, o);
        setup_drawing (vf, draw);
        setup_flow (vf, flow, draw);
        place_start_stop (vf, fi);
    }
    switch_to_flow (vf, ev, fi);
        
}
NewCmd := vcWv.Cmd (fac, new, "New", "Create New Flow",
             vcWv.Param ("Name", vlStringVT, vcWv.Sticky, "", vcFilePop.fac,
                                filter <- "*.vfl", edit_filter <- FALSE),
             vcWv.Param ("Location", vlStringVT, vcWv.Sticky, "",
                           vdmLocSelect.fac, cntx <- VdmCntx,
                                             allow_empty <- TRUE));
vcCmd.VerifyParams (NewCmd, func (vf, ev, name, loc)         
                                      verify_location (vf, loc););

function open(vf, ev, name, location, mode) {
    declare o;
    verify_location (vf, location);
    name = strip_extension (name);
    fi := find_flow_by_name (vf, name);
    if (fi == nil) {
       loc := vfl.Root;
        if (length (location) > 0) {
            verify_location (vf, location);
            loc := vpm.MapFindLocation (vdm.CntxGetMap (vf.dm), location);
        }
        found := FALSE;
        if (length (location) == 0) {
            foreach (l; vpm.MapLoopLocation (vdm.CntxGetMap (vf.dm))) {
                if (!found) {
                    o = vdm.ObjCreate (name, vdm.VersLatest, 
                                          vflFlowClass, l, vf.dm, nil);
                    found = vdm.ObjFind (o, nil, l);
                }
            }
        }
        if (!found) 
            o = vdm.ObjCreate (name, vdm.VersLatest, 
                                       vflFlowClass, loc, vf.dm, nil);
        /* I can't use := within the WithBusy - that's a new scope */
        declare f;
        declare draw;
        vcWin.WithBusy (vcEvent.Window (ev), func () {
            f = vflFlow.Read (o, vf.dm);
            draw = vge.DrawCreate (name + ".mf");
        });
        vge.DrawSetDoModify (draw, FALSE);
        fi = create_flow_info (vf, name, f, draw, mode, o);
        setup_drawing (vf, draw);
        setup_flow (vf, f, draw);
        connect_vges (vf, fi);
   } else 
       /* old flow, but possibly new mode */
       fi.mode = mode;
   switch_to_flow (vf, ev, fi);
}
OpenCmd := vcWv.Cmd (fac, open, "Open", "Open Flow",
             vcWv.Param ("Name", vlStringVT, vcWv.Sticky, "", vcFilePop.fac,
                                filter <- "*.vfl", edit_filter <- FALSE),
             vcWv.Param ("Location", vlStringVT, vcWv.Sticky, "",
                           vdmLocSelect.fac, cntx <- VdmCntx,
                                             allow_empty <- TRUE),
             vcWv.Param ("Mode", vflowModeVT, vcWv.Constant, 
                                 vflowMode.Run));
vcCmd.VerifyParams  (OpenCmd, func (vf, ev, name, loc, mode)         
                                      verify_location (vf, loc););

function save_the_flow (flow_info, ev) {
    w := vcEvent.Window (ev);
    /* we don't want to save the colored traffic lights */
    vcWin.WithBusy (w, func () {
        reset_the_flow (flow_info.flow);
        vflFlow.Write (flow_info.flow, flow_info.obj);
        /* I have to explicitle state the filename * not just say nil,
         * because otherwise I get a 'no changes to save' message 
         */
        vge.DrawWrite (flow_info.draw, 
                       vge.DrawName (flow_info.draw), 
                       vcEvent.Window (ev));
    });
    flow_info.modified = FALSE;
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, sprintf ("Wrote flow %s.", flow_info.name));
}

function write(vf, ev)
    save_the_flow (vf.flow_info, ev);
WriteCmd := vcWv.Cmd (fac, write, "Write");

function write_to (vf, ev, name, location) {
    name = strip_extension (name);
    loc := vfl.Root;
    if (length (location) > 0)
        loc = vpm.MapFindLocation (vdm.CntxGetMap (vf.dm), location);
    o := vdm.ObjCreate (name, vdm.VersLatest, vflFlowClass, 
                                                  loc, vf.dm, nil);
    w := vcEvent.Window (ev);
    vcWin.WithBusy (w, func () {
        reset_the_flow (vf.flow_info.flow);
        vflFlow.Write (vf.flow_info.flow, o);
        vge.DrawWrite (vf.flow_info.draw, 
                       name + ".mf",
                       vcEvent.Window (ev));
    });
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, 
        sprintf ("Wrote flow %s to %s.", vf.flow_info.name, name));
}
WriteToCmd := vcWv.Cmd (fac, write_to, "WriteTo", "Write To New Name",
            vcWv.Param ("Name", vlStringNotEmptyVT, vcWv.Sticky, 
                            vcWv.NoDefault, vcFilePop.fac, filter <- "vfl"),
            vcWv.Param ("Location", vlStringVT, vcWv.Sticky, "",
                              vdmLocSelect.fac, cntx <- VdmCntx,
                                                allow_empty <- TRUE));
vcCmd.VerifyParams (WriteToCmd, func (vf, ev, name, loc)         
                                      verify_location (vf, loc););


/****************************/
/* the flow list dialog box */

function switch_to_flow(vf, ev, fi) {
    if (vf.flow_info != nil)
        vge.PopDrawing (vf.canvas);
    vf.flow_info = fi;
    vge.PushDrawing (vf.canvas, vf.flow_info.draw);
    set_info_highlighting (vf);
    /* clear the info area */
    ReportSelected (vf, nil, vf.flow_info.draw);
    set_win_title (vf);
    /* set the vpcanvas to 'full' */
    /* we may want to add something to the draw before we do this
     * 'full', so we'll put it off like so. */
    vcWv.Delay (wv, func () {
        check_bounds (vf, fi);
        vcWin.RunCmdline (vcEvent.Window (ev), "full", ev);
        });
    vf.flow_stack = list (fi);
    FLOW = fi.flow;
}

/* command run by the 'apply' button */
function switch_flow(vf, ev)
    switch_to_flow (vf, ev, vf.chosen_flow);
SwitchFlowCmd := vcWv.Cmd (fac, switch_flow, "switch_flow");

/* same as above, but close the window */
function switch_flow_bye_window(vf, ev) {
    if (vf.chosen_flow != nil)
        switch_flow (vf, ev);
    vcWin.Close (vcRegion.Window (vf.flow_picks));
}
SwitchFlowByeWindowCmd := vcWv.Cmd (fac, switch_flow_bye_window, 
                             "switch_flow_bye_window");

function current_flow_name(vf)
    vf.flow_info.name;

function switch(vf, ev, name){
    fi := find_flow_by_name_ex (vf, name);
    switch_to_flow (vf, ev, fi);
}
SwitchCmd := vcWv.Cmd (fac, switch, "Switch", "Switch To Flow",
                   vcWv.Param ("Flow", vlStringVT, vcWv.Dynamic,
                          current_flow_name, vcPickEdit.fac,
                          data <- get_flow_list));

function overview_flow(vf, ev)
    create_overview_window (vcEvent.Window (ev), vf.chosen_flow);
OverviewFlowCmd := vcWv.Cmd (fac, overview_flow, "overview_flow");

function view(vf, ev, name){
    fi := find_flow_by_name_ex (vf, name);
    create_overview_window (vcEvent.Window (ev), fi);
}

ViewCmd := vcWv.Cmd (fac, view, "View", "View a Flow",
                   vcWv.Param ("Flow", vlStringVT, vcWv.Dynamic,
                          current_flow_name, vcPickEdit.fac,
                          data <- get_flow_list));

function format_flow_name (fi)
        sprintf ("%-40s\011%-13s\016%s", 
                  fi.name, convert (vlStringVT,  fi.mode),
                  (fi.modified) ? "\023yes" : "");

function update_flow_list(vf, update_flow) {
    declare form;
    foreach (row, col, ix, set, visual, fi; 
                        vcPicklist.Items (vf.flow_picks)) {
        if (fi == update_flow)
            vcPicklist.Set (vf.flow_picks, 
                          form = format_flow_name (fi), fi, row, col);
    }
    vcPicklist.SetByString (vf.flow_picks, form);
}
function switchmode_flow(vf, ev) {
    vf.chosen_flow.mode = 
            (vf.chosen_flow.mode == vflowMode.Edit) ? vflowMode.Run :
                                                      vflowMode.Edit;
    if (vf.chosen_flow == vf.flow_info)
        set_win_title (vf);
    update_flow_list (vf, vf.chosen_flow);
}
function switchmode_confirm(vf, ev)
    sprintf ("Switch flow %s to %s Mode?", vf.chosen_flow.name,
         convert (vlStringVT, 
         (vf.chosen_flow.mode == vflowMode.Edit) ? vflowMode.Run :
                                                   vflowMode.Edit));
SwitchmodeFlowCmd := vcWv.CmdWithConfirm (fac, switchmode_flow,
                       switchmode_confirm, TRUE, "switchmode_flow");

function mode(vf, ev, name, mode) {
    if (equal (name, "."))
        fi := vf.flow_info;
    else
        fi := find_flow_by_name_ex (vf, name);
    fi.mode = mode;
    set_win_title (vf);
}
ModeCmd := vcWv.Cmd (fac, mode, "Mode", "Change Flow Mode",
                   vcWv.Param ("Flow", vlStringVT, vcWv.Dynamic,
                          current_flow_name, vcPickEdit.fac,
                          data <- get_flow_list),
                   vcWv.Param ("Mode", vflowModeVT, vcWv.Constant,
                          vflowMode.Run));

function remove_a_flow (vf, fi) {
    if (vf.flow_info == fi)
        vflow_error (21, vlErrSev.Error, 
                               "Cannot remove flow currently in use.");
    /* first remove it from the flowlist */
    declare newlist;
    foreach (thefi; vscript.iter (FLOW_LIST))
        if (thefi != fi)
            newlist = cons (thefi, newlist);
    FLOW_LIST = newlist;
    vge.DrawForget (fi.draw);
}

function remove_flow(vf, ev) {
    remove_a_flow (vf, vf.chosen_flow);
    /* remove it from the picklist */
    declare rem_row;
    foreach (row, col, ix, set, visual, fi; 
                        vcPicklist.Items (vf.flow_picks)) {
        if (fi == vf.chosen_flow)
            rem_row = row;
    }
    vcPicklist.DeleteRows (vf.flow_picks, rem_row, 1);
    vf.chosen_flow = nil;
}

function remove_a_flow_confirm (vf, fi)
    if (vf.flow_info == fi || !fi.modified)
        FALSE; /* will throw later if chosen flow == flow_info */
    else
        sprintf ("Flow %s has been modified.\n\nRemove anyway?", 
                     fi.name);

function remove_flow_confirm(vf, ev)
    remove_a_flow_confirm (vf, vf.chosen_flow);

RemoveFlowCmd := vcWv.CmdWithConfirm (fac, remove_flow,
                       remove_flow_confirm, TRUE, "remove_flow");

function remove(vf, ev, name) {
    fi := find_flow_by_name_ex (vf, name);
    remove_a_flow (vf, fi);
}
function remove_confirm(vf, ev, name) {
    fi := find_flow_by_name_ex (vf, name);
    r := remove_a_flow_confirm (vf, fi);
    r;
}
RemoveCmd := vcWv.CmdWithConfirm (fac, remove, remove_confirm,
                           TRUE, "Remove", "Remove Flow",
                   vcWv.Param ("Flow", vlStringVT, vcWv.Dynamic,
                          current_flow_name, vcPickEdit.fac,
                          data <- get_flow_list));
       

function save_flow(vf, ev) {
    save_the_flow (vf.chosen_flow, ev);
    update_flow_list (vf, vf.chosen_flow);
}
SaveFlowCmd := vcWv.Cmd (fac, save_flow, "save_flow");

function save_all(vf, ev)
    foreach (elem; vscript.iter (FLOW_LIST)) {
        catch (vlEx.FAILURE) {
            save_the_flow (elem, ev);
            update_flow_list (vf, elem);
        } exception {
            vcErr.Out (vcEvent.Window (ev), nil, vlErr.CurrentError ());
        }
    }
SaveAllCmd := vcWv.Cmd (fac, save_all, "SaveAll");

function manage_flow(vf, ev) {
    vdata.SelectObject (vf.vdata_rgn, vf.chosen_flow.obj);
    vcWin.Show (vf.vdata_win);
}
ManageFlowCmd := vcWv.Cmd (fac, manage_flow, "manage_flow");

function flow_list(vf, ev) {

    declare pick_data, initial;
    foreach (fi; vscript.iter (FLOW_LIST)) {
        form := format_flow_name (fi);
        pick_data = cons (cons (form, fi), pick_data);
        if (fi == vf.flow_info)
            initial = form;
    }

    header := vcRegion.Create ("", vcStaticText.fac,
                      " \002Flow\016                          " +
                      "          \002Mode\016        \002Modified\016",
                      formatted <- TRUE);
    /* this allows us to line up with the picklist */
    p := vcRegion.Create ("", vcPicklist.fac, pick_data, 
                 will_scroll <- TRUE,
                 formatted <- TRUE);

    vcPicklist.Notify (p, func (fi, flag) vf.chosen_flow = fi;);
    if (initial != nil)
        vcPicklist.SetByString (p, initial);

    /* give both the same font, so the formatting works correctly */
    vcRegion.Font (header, vgFont.medium);
    vcRegion.Font (p, vgFont.medium);
    s := vcPicklist.Scroll ("", p, 5, 9, 15);

    function help (ev)
        vcAlert.SpawnHelp (vcEvent.Window (ev), "vflow", "FlowList");

    /* add the button row */
    brow := vcRegion.Create ("", vcButnRow.fac, 
             list (list ("", "Accept", SwitchFlowByeWindowCmd, nil),
                   list ("", "Apply", SwitchFlowCmd, nil),
                   list ("", "View", OverviewFlowCmd, nil),
                   list ("", "Change Mode", SwitchmodeFlowCmd, nil),
                   list ("", "Manage", ManageFlowCmd, nil),
                   list ("", "Remove", RemoveFlowCmd, nil),
                   list ("", "Save", SaveFlowCmd, nil),
                   list ("", "Save All", SaveAllCmd, nil),
                   list ("", "Cancel", func(ev)nil;, 
                              list (close_window <- TRUE)),
                   list ("", "Help", help, nil)));

    t := vcTable.Make ("Select Flow", 3, 1, header, s, brow);
    parent := vcEvent.Window (ev);
    w := vcWin.Popup (parent, t, p, vcWin.NoBindings);
    vcWin.Forward (parent, w);
    vf.flow_picks = p;
    vcWin.DestroyNotify (w, func (w) vf.flow_picks = nil;);
}
FlowListCmd := vcWv.Cmd (fac, flow_list, "FlowList");
          
/******************************************/

/********* enabling commands ************/

/* these are all the vge commands we had better disable in run mode: */
/* we fill this in when we create the Graphics menu */
declare VgeCommandList;

function enable_commands (vf) {
    has_flow := (vf.flow_info != nil);
    edit_mode := (has_flow && vf.flow_info.mode == vflowMode.Edit);

    o := (has_flow) ? vge.DrawGetOneSelectedObject (vf.flow_info.draw) : nil;
    obj := (o == nil) ? nil : vge_to_vfl (vf.flow_info.flow, o);
    t := typeof (obj);

    object_selected := (obj != nil);
    node_selected := (t == vflDataVT || t == vflTaskVT || 
                      t == vflStartStopVT);
    task_selected := (t == vflTaskVT);
    data_selected := (t == vflDataVT);
    stst_selected := (t == vflStartStopVT);
    dep_selected  := (t == vflDependVT);
    subflow_selected := (task_selected && 
                            vflTask.Class (obj) == vflMaster.Subflow);
    graphics_selected := (o != nil && vge.ObjectGetData (o) == nil);

    vcCmd.Enable (AddTaskCmd, edit_mode);
    vcCmd.Enable (AddTaskAtCmd, edit_mode);
    vcCmd.Enable (ModifyTaskCmd, edit_mode && task_selected);
    vcCmd.Enable (AddDataCmd, edit_mode);
    vcCmd.Enable (AddDataAtCmd, edit_mode);
    vcCmd.Enable (ModifyDataCmd, edit_mode && data_selected);
    vcCmd.Enable (ModifyDependCmd, edit_mode && dep_selected);

    vcCmd.Enable (ModifyIdCmd, edit_mode);
    vcCmd.Enable (DeleteIdCmd, edit_mode);
    vcCmd.Enable (ModifyCmd, edit_mode);
    vcCmd.Enable (DeleteCmd, edit_mode);
    vcCmd.Enable (FindCmd, has_flow);
    vcCmd.Enable (FindIdCmd, has_flow);
    vcCmd.Enable (EditCmd, has_flow);
    vcCmd.Enable (EditIdCmd, has_flow);
    vcCmd.Enable (ManageCmd, has_flow);
    vcCmd.Enable (ManageIdCmd, has_flow);
    vcCmd.Enable (UpdateCmd, has_flow);
    vcCmd.Enable (UpdateIdCmd, has_flow);
    vcCmd.Enable (ResetCmd, has_flow);
    vcCmd.Enable (ResetIdCmd, has_flow);
    vcCmd.Enable (ModifySlctCmd, edit_mode && (data_selected || 
                                 task_selected || dep_selected ||
                                 graphics_selected));
    vcCmd.Enable (CopySlctCmd, edit_mode && ((node_selected &&
                                 !stst_selected) || graphics_selected));
    vcCmd.Enable (DeleteSlctCmd, edit_mode && ((object_selected && 
                                 !stst_selected) || graphics_selected));

    vcCmd.Enable (AddLineCmd, edit_mode);
    vcCmd.Enable (DepModeCmd, edit_mode);
    vcCmd.Enable (AttachFromCmd, edit_mode && node_selected);
    vcCmd.Enable (AttachToCmd, edit_mode && node_selected);
    vcCmd.Enable (DependIdCmd, edit_mode);
    vcCmd.Enable (DependNameCmd, edit_mode);

    vcCmd.Enable (StretchCmd, edit_mode);
    vcCmd.Enable (MoveCmd, edit_mode);
    vcCmd.Enable (vge.MoveCmd, edit_mode);
    vcCmd.Enable (vge.StretchCmd, edit_mode);
    
    vcCmd.Enable (OverviewCmd, has_flow);
    vcCmd.Enable (GlobalSymbolCmd, has_flow);
    vcCmd.Enable (ResetAllCmd, has_flow);
    vcCmd.Enable (UpdateAllCmd, has_flow);
    vcCmd.Enable (CheckCmd, edit_mode);

    vcCmd.Enable (WriteCmd, edit_mode);
    vcCmd.Enable (WriteToCmd, edit_mode);

    vcCmd.Enable (RunSlctCmd, task_selected);
    vcCmd.Enable (UpdateSlctCmd, task_selected);
    vcCmd.Enable (EditSlctCmd, data_selected || subflow_selected);
    vcCmd.Enable (ManageSlctCmd, data_selected);
    vcCmd.Enable (ResetSlctCmd, node_selected);

    vcCmd.Enable (LogWindowCmd, vf.log_win != nil);

    making := (vf.flow_info != nil && 
                        vflFlow.MakeInProgress (vf.flow_info.flow));
    vcCmd.Enable (MakeCmd, has_flow && !making);
    vcCmd.Enable (CancelMakeCmd, has_flow && making);

    vcCmd.Enable (FlowListCmd, FLOW_LIST != nil);

    /* the buttons in the flow list dialog */
    flow_slct := (vf.flow_picks != nil && vf.chosen_flow != nil);
    vcCmd.Enable (SwitchFlowCmd, flow_slct);
    vcCmd.Enable (OverviewFlowCmd, flow_slct);
    vcCmd.Enable (SwitchmodeFlowCmd, flow_slct);
    vcCmd.Enable (RemoveFlowCmd, flow_slct);

    /* global symbol dialog */
    vcCmd.Enable (SetGlSymCmd, has_flow &&  vf.global_sym_w != nil);

    /* rather than enable/disable a whole list of vge
     * menu items, we just make the menu inaccessable.
     */
    foreach (cmd; vscript.iter (VgeCommandList))
        vcCmd.Enable (cmd, edit_mode);
}
function after (vf, ev)
    enable_commands (vf);
function _Initial (vf, ev) {
    if (FirstWindow == nil)
        FirstWindow = vcEvent.Window (ev);
    enable_commands (vf);   
    if (length (vf.initial) > 0)  {
        /* we use a delay so we can get in one refresh before
         * doing the time-consuming load */
        l := vcRegion.PrefGet (vf.region, "location");
        m := vcRegion.PrefGet (vf.region, "mode");
        vcWv.Delay (wv, func () {
            success := FALSE;
            catch (vlEx.FAILURE) {
                open (vf, ev, vf.initial, l, m);
                success = TRUE;
            } exception {
                if (m != vflowMode.Edit) /* can't `new' in run mode */
                    vlEx.Throw (vlEx.FAILURE, vlErr.CurrentError ());
            }
            if (!success)
                new (vf, ev, vf.initial, l);
         });
    }
}

function quit_message() {
    num_modified := 0;
    message := "";
    foreach (fi; vscript.iter (FLOW_LIST)) {
        if (fi.modified) {
            ++num_modified;
            message += "     " + fi.name + "\n";
        }
    }
    if (num_modified == 0)
        message = "";
    else if (num_modified == 1)
        message = "The following flow has been modified:\n" + message;
    else
        message = "The following flows have been modified:\n" + message;
    message;
}

function _Modified (vf, r) {
    q := quit_message ();
    vcWin.SetConfirmMessage (vcRegion.Window (r), quit_message ());
    length (q) > 0;
}
vcQuit.AddFunction (quit_message);

/* set up the LogWindow creation function that will be used by subflows */
function create_subflow_log_window(flow, task, subflow) {
    log := vcRegion.Create (sprintf ("Log for subflow %s", 
                                                 vflTask.Name (task)),
                          vflLog.fac, nil);
    w := vcWin.Create (log);
    vflFlow.CallOnOutput (subflow, func (t, str)
                   if (t == nil)
                       vflFlow.Output (flow, task, str);
                   else {
                       vflLog.AddEntry (log, vflTask.Name (t), t);
                       vflLog.AddString (log, t, str);
                   });
    vflFlow.CallOnOutputError (subflow, func (t, e)
                   if (t == nil)
                       vflFlow.OutputError (flow, task, e);
                   else {
                       vflLog.AddEntry (log, vflTask.Name (t), t);
                       vflLog.AddString (log, t, "\n" + 
                                   vlErr.String (e, vlErrCntx.Stderr) + "\n");
                   });
}
vflMaster.SetLogFunction (create_subflow_log_window);

declare FirstWindow;
vfl.CallOnWait (func(flg) {
                      if (FirstWindow != nil)
                          vcWin.Busy (FirstWindow, flg);
                      if (flg)
                           vcRegion.ExecuteRefreshQueue ();});
load ("vflowmn.vs");
load ("vflowbt.vs");
load ("vflowht.vs");

vlPref.Replace (prefs, "startstop", vlOnOffVT, 
        "Display explicit starting/stopping points.", TRUE);
vlPref.Replace (prefs, "log", vflLogStyleVT, 
                    "Display of the task log.", vflLogStyle.Window);
vlPref.Replace (prefs, "data_icon", vlStringVT,
                       "Default data icon", "vfldata");
vlPref.Replace (prefs, "task_icon", vlStringVT,
                       "Default task icon", "vfltask?");
vlPref.Replace (prefs, "start_icon", vlStringVT,
                       "Default starting point icon", "vflstart");
vlPref.Replace (prefs, "stop_icon", vlStringVT,
                       "Default stopping point icon", "vflstop");
vlPref.Replace (prefs, "depend_color", vgColorVT,
                    "Color of dependencies", vgColor.Yellow);
vlPref.Replace (prefs, "depend_fail_color", vgColorVT,
                    "Color of on-fail dependencies", vgColor.LightRed);
vlPref.Replace (prefs, "depend_always_color", vgColorVT,
                    "Color of always-type dependencies", 
                    vgColor.LightBlue);
vlPref.Replace (prefs, "depend_never_color", vgColorVT,
                    "Color of never-type dependencies", 
                    vgColor.LightGrey);
vlPref.Replace (prefs, "task_label_color", vgColorVT,
                    "Color of task object labels", vgColor.Magenta);
vlPref.Replace (prefs, "data_label_color", vgColorVT,
                    "Color of data object labels", vgColor.Cyan);
vlPref.Replace (prefs, "label_size", vlPositiveIntVT,
                    "Size of labels", 35);
vlPref.Replace (prefs, "label_font", vgStkVT,
                    "Font of labels", vgStk.SansSerif);
vlPref.Replace (prefs, "label_align", vgAlignVT,
                    "Label alignment", vgAlign.MiddleRight);
vlPref.Replace (prefs, "move_labels", vlTrueFalseVT,
                    "Allow movement of labels", FALSE);
vlPref.Replace (prefs, "canvas_size", vlPositiveIntVT,
                    "Logical size of graphical canvas", 2000);
vlPref.Replace (prefs, "mode", vflowModeVT, 
                    "Mode of flow specified on command line.",
                    vflowMode.Run);
vlPref.Replace (prefs, "location", vlStringVT,
                    "Vdm location of flow specified on command line.",
                      "");
vlPref.Parse (nil, wv);

vcsa.LoadCustomizations ("vflow",  current_package ());

