/*
 *  $Revision: 1.46.1.4 $
 *
 *  ViewScript file implementing the graphic ViewScript Interpreter.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */

struct vscmEvalVT {
    vscmLanguageVT      language;
    vcRegionVT          region;
    vcRegionVT          shell;
}; /* struct vscmEvalVT */

/*
 * Facility's create function.  Note the facility's data object: it's a
 * structure with two fields--the facility's region and the shell region.
 */
function _Create(path, region)
{
    initial_language := vcRegion.PrefGet (region, "language");

    data := struct vscmEvalVT {
        initial_language,
        region,
        vcRegion.Create (vcRegion.Name (region), vcTxShell.fac, "",
                         prompt   <- what_prompt(initial_language),
                         macro_on <- TRUE)
    };

    vcRegion.AddChild (region, data.shell);

    vcTxShell.CallOnReturn (data.shell, func(event, str) {
        function echo_output(str)
            vcTxShell.PrintString (data.shell, str);

        str = string_trim (str);
        len := string_length (str);
        if (len > 0) {
            last := substring (str, len - 1, 0);
            if (!string_eq (last, ";") && !string_eq (last, "}"))
                str = str + ";";
            catch (vlEx.FAILURE) {
                val := vcWin.EvalPrint2 (str, echo_output, data.language);
                       

            if (! vcRegion.Destroyed (data.shell)) {
/*
                if (data.language == vscmLanguage.SCM)
                    write_generic (val, FALSE, 75, func(str) {
                        vcTxShell.PrintString (data.shell, str);
                    });
                else
*/
                    foreach (str; print_iter (val,
                                              vscmPrint.Fields +
                                              vscmPrint.Newlines +
                                              vscmPrint.Pretty))
                        vcTxShell.PrintString (data.shell, str);
                vcTxShell.PrintString (data.shell, "\n");
            } /* if */
           } exception
                vcTxShell.PrintString (data.shell, 
                     vlErr.String (vlErr.CurrentError (), 
                                   vlErrCntx.Stderr) + "\n");
        };
    });

    data;
}


function _Reset(data, region)
{
    data.language = vcRegion.PrefGet (region, "language");
    vcRegion.PrefSet (data.shell, list (prompt <- what_prompt(data.language)));
}


function what_prompt(l)
    if (l == vscmLanguage.SCM)
        "scm> ";
    else
        "vs> ";


vlPref.Replace (prefs, "language", vscmLanguageVT, "Language Selection",
                vscmLanguage.VS);

/*
 * Invoke the ViewScript Debugger.
 */
function vbugger(data, event, filename)
    vcWin.Create (vcRegion.Create ("ViewScript Debugger", vcVbg.fac,
                                   filename),
                  vcWin.NoMacros);


/*
 * Write the session into a log file.
 */
function write_log(data, event, filename)
    vtx.Writeto (data.region, filename);


/*
 * Call the load function to load and evaluate a ViewScript file.  This is
 * provided mainly as a shortcut so that you don't have to type the `load'
 * function call yourself.
 */
function load_file(data, event, filename, pkg)
    vcTxShell.TypeIn (data.shell, sprintf ("load (%s, %s);", filename, pkg),
                      TRUE, event);


declare vscm_class, window_class, region_class, root_window_class;

/*
 * This block of code loads the class definitions for the browser.  It
 * is run only once, then the function is nulled out.  If vcTree is not
 * linked into the executable, then this function is nulled out immediately
 * and the browse command is disabled.
 */
function load_browser() {
    LIMIT := 10000;

    iterator limited_iter(l, iter) {
        i := 0;
        foreach (args...; iter (l)) {
            apply (iterate, cons (i, args));
            if (i == LIMIT)
                return (nil);
            ++i;
        } /* foreach */
    } /* iterator limited_iter */

    function is_list(x, limit) {
        if ((limit == 0) || (x == nil))
            TRUE;
        else if (istype (x, consVT))
            is_list (cdr (x), limit - 1);
        else
            FALSE;
    } /* function is_list */

    function unwrap_object(parent) {
        if (istype (parent, consVT) &&
            car (parent) == `wrapper)
            list (cadr (parent), caddr (parent));
        else
            list (parent, nil);
    } /* function unwrap_object */

    function wrap_object(obj, label)
        list (`wrapper, obj, label);

    iterator vscm_children(parent, sub_class) {
        <x, label> := unwrap_object (parent);
        if (is_list (x, LIMIT) || istype (x, vectorVT))
            foreach (i, child; limited_iter (x, vscript.iter))
                iterate (wrap_object (child, i), vscm_class);
        else if (istype (x, consVT)) {
            iterate (wrap_object (car (x), "Car"), vscm_class);
            iterate (wrap_object (cdr (x), "Cdr"), vscm_class);
        } /* else if */
        else if (isstruct (x))
            foreach (name, val; struct_value_iter (x))
                iterate (wrap_object (val, name), vscm_class);
        else if (istype (x, vlSymTableVT) || istype (x, tableVT))
            foreach (i, name, val; limited_iter (x, vlSym.Loop)) {
                if (isstring (name) || istype (name, symbolVT))
                    iterate (wrap_object (val, name), vscm_class);
                else {
                    iterate (wrap_object (name, sprintf ("Symbol %d", i)),
                             vscm_class);
                    iterate (wrap_object (val, sprintf ("Value  %d", i)),
                             vscm_class);
                } /* else */
            } /* foreach */
        else if (istype (x, symbolVT)) {
            if (isbound (x, nil))
                iterate (wrap_object (eval_string (sprintf ("#\"%s\";", x),
                                                   nil),
                                      "Global Value"),
                         vscm_class);
            else
                iterate (wrap_object (`#"**Unbound**", nil), vscm_class);
        } /* else if */
        else {
            class := vcBrowse.TypeGetClass (typeof (x));
            if (class != nil)
                iterate (x, class);
        } /* else */
    } /* iterator vscm_children */

    /*
     * Used when generating a printed representation of a preference value list
     * to make the output more like what a human programmer would type.  Thisis
     * is completely cosmetic, but without this routine you get pref lists
     * like:
     *      block_cursor <- #<vlTrueFalseVT False>,
     *      rows <- #<vlPositiveIntVT 5>
     * instead of
     *      block_cursor <- FALSE,
     *      rows <- 5
     * The parsing implications are a wash.  Either way, someone has to parse
     * the string into the object, it's just a question of whether it's done
     * by the viewscript parser or by the typespec convert routine.
     */
    function hack_pref_val(val) {
        if (istype (val, vlTrueFalseVT) || istype (val, vlYesNoVT) ||
            istype (val, vlOnOffVT))
            val ? TRUE : FALSE;
        else if (istype (val, vlIntVT) || istype (val, vlUnsignedIntVT) ||
                 istype (val, vlPositiveIntVT))
            convert (fixnumVT, val);
        else if (istype (val, vlFloatVT) || istype (val, vlPositiveFloatVT))
            convert (flonumVT, val);
    /*
        else if (istype (val, typespecVT))
            convert (vlStringVT, val);
    */
        else if (isstruct (val))
            convert (vlStringVT, typeof (val));

        else if (is_list (val, LIMIT))
            sprintf ("(%d element list)", length (val));

        else if (isvector (val))
            sprintf ("[%d element vector]", vector_length (val));

        else
            sprintf ("%s", val);
    } /* function hack_pref_val */

    function string_limit(s, limit) {
        if (string_length (s) > limit)
            substring (s, 0, limit) + "...";
        else
            s;
    } /* function string_limit */

    function vscm_name(parent) {
        <x, label> := unwrap_object (parent);
        s := hack_pref_val (x);
        if (label != nil)
            s = sprintf ("%s: %s", label, s);
        else
            s = sprintf ("%s", s);
        string_limit (s, 40);
    } /* function vscm_name */

    function vscm_hier_name(parent) {
        <x, label> := unwrap_object (parent);
        sprintf ("%s", label);
    } /* function vscm_hier_name */

    iterator vscm_info(parent) {
        <x, label> := unwrap_object (parent);
        iterate (sprintf ("Type: %s", convert (vlStringVT, typeof (x))));
    /*    iterate (string_limit (sprintf ("Value: %s", x), 50)); */
    } /* iterator vscm_info */

    function vscm_parse(s)
        eval_string (s + ";", nil);

    vscm_class =
        vcBrowse.ClassCreate ("vscm",
                              children <- vscm_children,
                              get_name <- vscm_name,
                              get_hier_name <- vscm_hier_name,
                              get_info <- vscm_info,
                              parse <- vscm_parse,
                        vertex_prefs <- list (text_color <- vgColor.Yellow));


    iterator region_children(r, sub_class) {
        if ((sub_class == nil) || (sub_class == region_class))
            foreach (child; vcRegion.Children (r, FALSE))
                iterate (child, region_class);
        if ((sub_class == nil) || (sub_class == vscm_class))
            iterate (vcRegion.GetData (r), vscm_class);
    } /* iterator region_children */

    function region_name(r)
        vcFac.Name (vcRegion.Fac (r)) + ":" + vcRegion.Name (r);

    struct RegionInfoVT {
        vcRegionVT parent;
        vgRectVT area, visible_area;
        vlTrueFalseVT macro_on, is_focus, show_focus_cursor, is_enabled;
        vlTrueFalseVT is_entered, has_been_reframed;
    };

    iterator region_info(r) {
        iterate (sprintf ("Parent = %s", vcRegion.Parent (r)));
        iterate (sprintf ("Area = %s", vcRegion.Area (r)));
        declare data;
        catch (vlEx.FAILURE) {
            data = vcRegion.Get (r, vlStringVT);
            iterate ("Data = %s" + data);
        } /* catch */
        exception
            ;
        iterate (sprintf ("VisibleArea = %s", vcRegion.VisibleArea (r)));
        iterate (sprintf ("MacroOn = %s", vcRegion.MacroOn (r)));
        iterate (sprintf ("IsFocus = %s", vcRegion.IsFocus (r, r)));
        iterate (sprintf ("ShouldShowFocusCursor = %s",
                          vcRegion.ShouldShowFocusCursor (r)));
        iterate (sprintf ("IsEnabled = %s", vcRegion.IsEnabled (r)));
        iterate (sprintf ("IsEntered = %s", vcRegion.IsEntered (r)));
        iterate (sprintf ("HasBeenReframed = %s", vcRegion.HasBeenReframed (r)));
    } /* iterator region_info */

    region_class =
        vcBrowse.ClassCreate ("Region",
                              children <- region_children,
                              get_name <- region_name,
                              get_info <- region_info,
                              vertex_prefs <- list (text_color <- vgColor.Red),
                              type <- vcRegionVT);

    function window_name(w) (w == nil) ? "(Root)" : vcWin.Name (w);

    iterator window_children(w, sub_class) {
        if ((sub_class == nil) || (sub_class == window_class))
            foreach (child; vcWin.Loop ())
                if (vcWin.Parent (child) == w)
                    iterate (child, window_class);
        if (((sub_class == nil) || (sub_class == region_class)) && (w != nil))
        {
            iterate (vcWin.OwnerRegion (w), region_class);
            iterate (vcWin.Region (w), region_class);
        } /* if */
    } /* iterator window_children */

    iterator window_info(w) {
        if (w != nil) {
            iterate (sprintf ("Title = %s", vcWin.Title (w)));
            iterate (sprintf ("Modified = %s", vcWin.IsModified (w)));
            iterate (sprintf ("Busy = %s", vcWin.IsBusy (w)));
            iterate (sprintf ("IsHidden = %s", vcWin.IsHidden (w)));
            iterate (sprintf ("IsMapped = %s", vcWin.IsMapped (w)));
            iterate (sprintf ("IsIconified = %s", vcWin.IsIconified (w)));
            iterate (sprintf ("IsDecorated = %s", vcWin.IsDecorated (w)));
            iterate (sprintf ("ConfirmMessage = %s", vcWin.ConfirmMessage (w)));
            iterate (sprintf ("Parent = %s", vcWin.Parent (w)));
            iterate (sprintf ("Ancestor = %s", vcWin.Ancestor (w)));
            iterate (sprintf ("HasFocus = %s", vcWin.HasFocus (w)));
            iterate (sprintf ("DefaultAction = %s", vcWin.GetDefaultAction (w)));
        } /* if */
    } /* iterator window_info */

    window_class =
        vcBrowse.ClassCreate ("Window",
                              children <- window_children,
                              get_name <- window_name,
                              get_info <- window_info,
                              vertex_prefs <-
                                  list (text_color <- vgColor.LightBlue),
                              type <- vcWindowVT);

    iterator root_window_children(data, sub_class) {
        foreach (w; vcWin.Loop ())
            if (vcWin.Parent (w) == nil)
                iterate (w, window_class);
    } /* iterator root_window_children */

    root_window_class =
        vcBrowse.ClassCreate ("Root Window",
                              parse <- (func(name) nil;),
                              children <- root_window_children);
} /* function load_browser */

/*\vscmuserFunction vscmEval.GlobalValues -- return a list of global variables that have bound values

DESCRIPTION
     This function is intended for use as a starting point when browsing.
*/
function GlobalValues() {
    l := nil;
    foreach (sym; vscript.list_iter (symbol_table ())) {
        if (istype (sym, symbolVT) && isbound (sym, nil))
            l = cons (symbol2string (sym), l);
    } /* foreach */
    l = vscript.Sort (apply (vector, l), string_le);
    for (i := 0; i < vector_length (l); ++i)
        l[i] = string2symbol (l[i]);
    l;
} /* function GlobalValues */

package browse_selector(_Create, _Get, windows, globals) {
    windows := "vscmEval.Windows()";
    globals := "vscmEval.GlobalValues()";

    function _Create(expr, region) {
        printf ("_Create entered\n");
        picklist := vcRegion.Create ("", vcPicklist.fac,
                                     list (list (cons ("Global Variables",
                                                       globals),
                                                 cons ("Windows",
                                                       windows),
                                                 cons ("Viewscript Expression",
                                                       nil))));
/*
        tx := vcRegion.Create ("", vtx.fac, "",
                               load_file <- FALSE,
                               height <- 4,
                               width <- 40);
        if (!string_eq (expr, windows) &&
            !string_eq (expr, globals))
            vtx.AppendString (tx, expr);
*/
        tx := vcRegion.Create ("", vcCmdline.fac,
                               (!string_eq (expr, windows) &&
                                !string_eq (expr, globals)) ? expr : "",
                               width <- 50);

        vcPicklist.Notify (picklist, func(val, on_off) {
            if (on_off) {
                print (val);
                print (val == nil);
                vcRegion.Enable (tx, val == nil,
                                 vcRegion.HasBeenReframed (picklist));
                if (val == nil)
                    vcWv.Delay (wv, func() vcRegion.SetFocus (tx, tx););
            } /* if */
        });
        if (string_eq (expr, windows))
            vcPicklist.SetPick (picklist, 1);
        else if (string_eq (expr, globals))
            vcPicklist.SetPick (picklist, 0);
        else
            vcPicklist.SetPick (picklist, 2);
        table := vcTable.Make ("", 2, 1, picklist, tx);
        vcRegion.Align (picklist, vgAlign.UpperLeft);
        vcRegion.AddChild (region, table);
        cons (tx, picklist);
    } /* function _Create */
        
    function _Get(tx_picklist, region, ts) {
        val := vcRegion.Get (cdr (tx_picklist), vscmVT);
        if (val == nil)
            vcRegion.Get (car (tx_picklist), vlStringVT);
        else
            val;
    } /* function _Get */
} /* package browse_selector */

browse_fac := nil;

function browse(data, event, expression) {
    vcWin.WithBusy (vcEvent.Window (event), func() {
        if (load_browser != nil) {
            load_browser ();
            load_browser = nil;
        } /* if */

        title := "Viewscript Browser: " + expression;

        class := vscm_class;
        if (string_eq (expression, browse_selector.windows)) {
            class = root_window_class;
            expression = nil;
        } /* if */

        vcWin.Create (vcRegion.Create (title,
                                       vcBrowse.fac,
                                       cons (expression, class),
                                       exclusive <- TRUE,
                                       hide_root <-
                                       (class == root_window_class)));
    });
} /* function browse */

browser_enabled := TRUE /*isbound (`vcTree, nil)*/;
if (browser_enabled)
    browse_fac =  vcFac.CreateFromPackage ("vscmBrowseSelect",
                                           vscmVT, browse_selector);
else {
    load_browser = nil;
    browse_selector = nil;
} /* else */

/*
 * Build command objects for subsequent key and menu bindings.
 */
log_cmd         := vcWv.Cmd (fac, write_log, "log", "Write Log File",
                             vcWv.Param ("Log name", vlFileWriteVT,
                                         vcWv.Sticky, vcWv.NoDefault));
browser_cmd     := vcWv.Cmd (fac, browse, "browse",
                             "Invoke Viewscript Browser",
                             vcWv.Param ("", vlStringVT, vcWv.Sticky,
                                         browse_selector.globals, browse_fac));
vcCmd.Enable (browser_cmd, browser_enabled);
vbugger_cmd     := vcWv.Cmd (fac, vbugger, "debugger",
                             "Invoke ViewScript Debugger",
                             vcWv.Param ("ViewScript file:", vlFileNameVT,
                                         vcWv.Sticky, vcWv.NoDefault,
                                         vcFilePop.fac, filter <- "*.vs"));
vcCmd.Enable (vbugger_cmd, vlOnOff.Off);
load_file_cmd   := vcWv.Cmd (fac, load_file, "load_file", "Load File",
                             vcWv.Param ("Filename:", vlFileNameVT,
                                         vcWv.Sticky, vcWv.NoDefault,
                                         vcFilePop.fac, filter <- "*.vs"),
                             vcWv.Param ("Package:", vlStringVT, vcWv.Sticky,
                                         "nil", vcWv.Secondary));

/*
 * Construct the menu line.  Note that each menu has one option, thus in
 * reality they become buttons.
 */
menus := vcWv.MakeMenu ("vscmEval",
                        vcWin.RedSquareEntryNoPlotNoQuit,
                        list ("Lo&g session",            log_cmd),
                        list ("&Load a file",            load_file_cmd),
                        list ("ViewScript D&ebugger",    vbugger_cmd),
                        list ("ViewScript B&rowser",     browser_cmd));

vcFac.SetMenus (fac, menus);
