/***************************************************************************
*
*  $Id: lvsErrF.vs,v 1.4 1994/12/14 15:52:10 sandy Stab $
*
*  lvsErrF.vs
*
*  Functional Description:
*    Viewscript for facility lvsErrF
*    
*
*  Environment:
*    UNIX 
*
*  Author:
*    Sandy Lucas         -- Oct, 1994
*
*  Modified by:
*  Who     Version     When       What
*  SAL     0.1         10/03/94   Original coding.
*
*  Copyright (c) 1994 by Viewlogic Systems, Inc.
*
***************************************************************************/

set_stdout(TRUE);

/*-------------------------------------------------------------------------
*
*  GLOBAL VARIABLES
*
--------------------------------------------------------------------------*/
declare lnetbadi, ldevbadi, lauditi, ltermbadi, lmergeneti, lpruneneti;
declare lprunedevi;
declare snetbadi, sdevbadi, sauditi, stermbadi, smergeneti, spruneneti;
declare sprunedevi;

/*-------------------------------------------------------------------------
*
*  FUNCTIONS
*
--------------------------------------------------------------------------*/

/***************************************************************************
*
* function setcheck - 
* changes the icon in the selected picklist line to a
* check mark to indicate that the error object has been visited
*
***************************************************************************/
function setcheck( picklist)
{
  /* find the index of the selected item based on the picklist region name*/
  index := getPicklistIndex(picklist);

  /* find the region given the index */
  reg := vcTable.Get(picklist,index,0);

  /* change the blank icon to the check icon */
  vcTable.Set(reg, 0, 0, vcRegion.Create("icon", vcIcon.fac, "check"));
}

/***************************************************************************
*
* function visit - 
* depending on the netlisttype (either layout or schematic)
* and the picklist string, highlight the appropriate object in the appropriate
* editor
*
***************************************************************************/
function visit( context, event )
{
   setcheck(context.picklist);
   pickstring := vcRegion.Get(context.picklist, vlStringVT);
/*
   printf("netlist type = %s\n", context.netlisttype);
   printf("error type = %s\n", context.errortype);
   printf("filename = %s\n", context.filename);
   printf("pickstring = %s\n", pickstring);
*/
   lvsnav.ErrNav(context.lvsnav, context.netlisttype, context.errortype,
              context.filename, pickstring);
}

/***************************************************************************
*
* function setPicklistIndex -
* store the index into the appropriate global variable depending on the
* region name of the picklist
* this is call from the vcPicklist.Notify function
*
***************************************************************************/
function setPicklistIndex(picklist, index) {
   regname := vcRegion.Name(picklist);
   if (string_eq(regname, "layout/netbad.out"))
      lnetbadi = index;
   else if (string_eq(regname, "layout/devbad.out"))
      ldevbadi = index;
   else if (string_eq(regname, "layout/audit.out"))
      lauditi = index;
   else if (string_eq(regname, "layout/termbad.out"))
      ltermbadi = index;
   else if (string_eq(regname, "layout/mergenet.out"))
      lmergeneti = index;
   else if (string_eq(regname, "layout/prunenet.out"))
      lpruneneti = index;
   else if (string_eq(regname, "layout/prunedev.out"))
      lprunedevi = index;
   else if (string_eq(regname, "schematic/netbad.out"))
      snetbadi = index;
   else if (string_eq(regname, "schematic/devbad.out"))
      sdevbadi = index;
   else if (string_eq(regname, "schematic/audit.out"))
      sauditi = index;
   else if (string_eq(regname, "schematic/termbad.out"))
      stermbadi = index;
   else if (string_eq(regname, "schematic/mergenet.out"))
      smergeneti = index;
   else if (string_eq(regname, "schematic/prunenet.out"))
      spruneneti = index;
   else if (string_eq(regname, "schematic/prunedev.out"))
      sprunedevi = index;
}
    
/***************************************************************************
*
* function getPicklistIndex -
* returns the index stored in the global variable dependent upon the 
* region name of the picklist
*
***************************************************************************/
function getPicklistIndex(picklist) {
   regname := vcRegion.Name(picklist);
   if (string_eq(regname, "layout/netbad.out"))
      return(lnetbadi);
   else if (string_eq(regname, "layout/devbad.out"))
      return(ldevbadi);
   else if (string_eq(regname, "layout/audit.out"))
      return(lauditi);
   else if (string_eq(regname, "layout/termbad.out"))
      return(ltermbadi);
   else if (string_eq(regname, "layout/mergenet.out"))
      return(lmergeneti);
   else if (string_eq(regname, "layout/prunenet.out"))
      return(lpruneneti);
   else if (string_eq(regname, "layout/prunedev.out"))
      return(lprunedevi);
   else if (string_eq(regname, "schematic/netbad.out"))
      return(snetbadi);
   else if (string_eq(regname, "schematic/devbad.out"))
      return(sdevbadi);
   else if (string_eq(regname, "schematic/audit.out"))
      return(sauditi);
   else if (string_eq(regname, "schematic/termbad.out"))
      return(stermbadi);
   else if (string_eq(regname, "schematic/mergenet.out"))
      return(smergeneti);
   else if (string_eq(regname, "schematic/prunenet.out"))
      return(spruneneti);
   else if (string_eq(regname, "schematic/prunedev.out"))
      return(sprunedevi);

}

/***************************************************************************
*
* function error_type -
* converts the filename to an error type for use in labeling dialog boxes
*
***************************************************************************/
function error_type( filename )

   if (string_eq(filename, "netbad.out"))
      return("unmatched nets");
   else if (string_eq(filename, "devbad.out"))
      return("unmatched insts");
   else if (string_eq(filename, "audit.out"))
      return("unmatched attrs");
   else if (string_eq(filename, "termbad.out"))
      return("unmatched terminals");
   else if (string_eq(filename, "mergenet.out"))
      return("merged nets");
   else if (string_eq(filename, "prunenet.out"))
      return("ignored nets");
   else if (string_eq(filename, "prunedev.out"))
      return("ignored devices");
   else return("");


/***************************************************************************
*
* _Create method - creates the top table for the lvsErrF facility
* returns a structure containing the table as the facility object
*
* input data is list whose first item is the full path filename of the error 
* file, the second item is the lvsxprob facility object, the third item is
* the button that was set to invoke the dialog.  This button is reset when
* the cancel function is called.
*
***************************************************************************/
function _Create(args, region) {

  declare line1, line2;
  declare rows, cols;
  declare ptab;

  context := struct {
                reg;
                button;
                filename;
                netlisttype;
                errortype;
                lvsnav;
                picklist;
                fp;
                butnrow;
                tab
             };
  context.reg = region;
  context.filename = car(args);
  context.netlisttype = nil;
  context.errortype = nil;
  context.lvsnav = cadr(args);
  context.button = caddr(args);
  context.picklist = nil;
  context.fp = nil;
  context.butnrow = nil;
  context.tab = nil;

  numlines := vcRegion.PrefGet(context.reg, "n");

  if (fexists(context.filename)) {
      /* parse the filename to extract the type of netlist and type of error*/
      prevtok := nil;
      curtok := nil;
      token := nil;
      foreach(token; string_loop(context.filename, "/")) {
         prevtok = curtok;
         curtok = token;
      }
      
      error_name := error_type(curtok);
      context.netlisttype = prevtok;
      context.errortype = error_name;

      /* create a picklist */
      context.picklist = vcRegion.Create(context.netlisttype + "/" + curtok, 
                                  vcPicklist.fac, nil,
                                  rows <- 1,
                                  columns <- 1,
                                  enhanced_notify <- TRUE,
                                  macro_on <- TRUE);
      vcPicklist.Notify(context.picklist,
          func(entry, on_or_off, index) {
             if (on_or_off == vlOnOff.On)
                setPicklistIndex(context.picklist, index);
          }
      );
      context.fp = fopen(context.filename, "r");
      first := TRUE;
      count := 0;
      for (line1 = fgetline(context.fp); line1 != nil; 
           line1 = fgetline(context.fp)){
         line2 = fgetline(context.fp);
         ptab = vcTable.Create("ptab",1,2);
         vcTable.Add(ptab, vcRegion.Create("icon", vcIcon.fac, "chkbox"));
         vcTable.Add(ptab, vcRegion.Create("label", vcLabel.fac, 
              line1 + " " + line2));
         /* add the concat of line1 and line2 as a picklist choice */
         vcPicklist.Add(context.picklist, ptab, 
                        line1 + " " + line2);
         count = count + 1;
         if (count == numlines)
            break;
      }

      /* close the file if got to the end */
      if (count < numlines) {
         fclose(context.fp);
         context.fp = nil;
      }
      /* select the first item in the picklist to make sure one is selected*/
      vcPicklist.SetPick(context.picklist, 0);
      scroll := vcPicklist.Scroll("scroll", context.picklist, 5, 5, 0);

      /* if there are more lines than displayed, add a VISIT, MORE, and
         CANCEL button */
      if (context.fp != nil)
         context.butnrow = vcRegion.Create("butnrow", vcButnRow.fac,
                      list (list ("Visit", "Visit", cmdVisit, nil),
                            list ("More", "More", cmdMore, nil),
                            list ("Cancel","Cancel",cmdCancel, nil)));

      /* now create a region with a VISIT and CANCEL button */
      else context.butnrow = vcRegion.Create("butnrow", vcButnRow.fac,
                      list (list ("Visit", "Visit", cmdVisit, nil),
                            list ("Cancel","Cancel",cmdCancel, nil)));

      context.tab = vcTable.Create(prevtok + " " + error_name + " (" +curtok +
                      ")", 2, 1);
      vcTable.Add(context.tab, scroll);
      vcTable.Add(context.tab, context.butnrow);

      vcRegion.AddChild(region, context.tab);
      context;
   }
   else {
      vlErr.Out2("lvsErrF", vlErrSev.Error, "LVS error file " +
                   context.filename + "does not exist");
      nil;
   }
}

/* add preferences */
vlPref.Add(prefs, "n", vlIntVT, "Number of error lines to display", 100);

/*-------------------------------------------------------------------------
*
*  COMMANDS
*
--------------------------------------------------------------------------*/

cmdCancel := vcWv.Cmd(lvsErrF.fac, func(context, event) { 
                                   vcWin.Destroy(vcEvent.Window(event));
                                   vcToggleButton.Unset(context.button);
                                   vcRegion.QueueForRefresh(vcRegion.Parent(context.button));
                                 }, "cancel");
vcCmd.EnableMacro(cmdCancel, vlOnOff.On);

cmdVisit := vcWv.Cmd(lvsErrF.fac, visit, "VisitErr");
vcCmd.EnableMacro(cmdVisit, vlOnOff.On);

function more(this, event) 
{
  declare line1, line2, ptab;

  if (this.fp == nil) {
     /* there are no more lines in the file */
     ;
  }
  else {
     numlines := vcRegion.PrefGet(this.reg, "n");

     /* clear the current picklist */
/*  for (i:=1; i<numlines; i=i+1)
     vcPicklist.DeleteRows(this.picklist, i, 1);*/
     vcPicklist.DeleteRows(this.picklist, 0, numlines);

     count := 0;
     for (line1 = fgetline(this.fp); line1 != nil; line1 = fgetline(this.fp)){
        line2 = fgetline(this.fp);
        ptab = vcTable.Create("ptab",1,2);
        vcTable.Add(ptab, vcRegion.Create("icon", vcIcon.fac, "chkbox"));
        vcTable.Add(ptab, vcRegion.Create("label", vcLabel.fac, 
          line1 + " " + line2));
        /* add the concat of line1 and line2 as a picklist choice */
        vcPicklist.Add(this.picklist, ptab, 
                     line1 + " " + line2);
        count = count + 1;
        if (count >= numlines)
           break;
     }

     vcPicklist.SetPick(this.picklist, 0);
     /* close the file if got to the end */
     if (line1 == nil) {
        fclose(this.fp);
        this.fp = nil;
        /* change the butn row - remove the MORE button */
        this.butnrow = vcRegion.Create("butnrow", vcButnRow.fac,
                      list (list ("Visit", "Visit", cmdVisit, nil),
                            list ("Cancel","Cancel",cmdCancel, nil)));
        vcTable.Set(this.tab, 1, 0, this.butnrow);
        vcRegion.QueueForRefresh(this.tab);
     }
  }
}

cmdMore := vcWv.Cmd(lvsErrF.fac, more, "More");


