/****************************************************************************
* $Id: lvsnav.vs,v 1.7 1994/12/27 19:57:06 sandy Stab $
*
* lvsnav.vs
*        
* Viewscript commands and functions for LVS Navigator 
* 
*  Author:
*    Sandy Lucas         -- Oct, 1994
*
*  Modified by:
*  Who     Version     When       What
*  SAL     0.1         10/11/94   Original coding.
*
*  Copyright (c) 1994 by Viewlogic Systems, Inc.
*
****************************************************************************/

set_stdout(TRUE);

/***************************************************************************
*
* GLOBAL VARIABLES 
* 
*
***************************************************************************/
declare layoutSelectObj;
declare schSelectObj;
declare dialogwin;

/***************************************************************************
*
* function EnableLvsnavCmds - enables and disables the commands
*
***************************************************************************/
function EnableLvsnavCmds(on_or_off)
{
   vcCmd.Enable(ProbeSchSelCmd, on_or_off);
   vcCmd.Enable(SchNetNameCmd, on_or_off);
   vcCmd.Enable(SchInstNameCmd, on_or_off);
   vcCmd.Enable(ClearProbeCmd, on_or_off);
   vcCmd.Enable(ReinitCmd, on_or_off);
   vcCmd.Enable(ProbeLaySelCmd, on_or_off);
   vcCmd.Enable(LayNetNameCmd, on_or_off);
   vcCmd.Enable(LayInstNameCmd, on_or_off);
   vcCmd.Enable(OpenCFileCmd, on_or_off);
}

/***************************************************************************
*
* function StartingToolDialog - brings up a dialog box indicating that 
*    program is waiting for Viewdraw to start
*
***************************************************************************/
function StartingToolDialog()
{
  textreg := vcRegion.Create("Note", vcLabel.fac, 
     "Waiting for Viewdraw to start!");
  margin := vcRegion.Create("Note", vcMargin.fac, textreg, margin_size<-20); 
  dialogwin = vcWin.Create(margin, vcWin.NoStatus + vcWin.NoBindings);

  EnableLvsnavCmds(vlOnOff.Off);
}

/***************************************************************************
*
*
***************************************************************************/
function CloseToolDialog()
{

  if (dialogwin != nil) {
     vcWin.Destroy(dialogwin);
     dialogwin = nil;
  }
  EnableLvsnavCmds(vlOnOff.On);
}

/***************************************************************************
*
* function layoutObjPick - set global variable to picklist selected data
* 
*
***************************************************************************/
function layoutObjPick(data, on_or_off) 
{
   if (on_or_off == vlOnOff.On)
      layoutSelectObj = data;
}

function probeit(data, event)
{
   if (layoutSelectObj[3] == '/')
      objname := substring(layoutSelectObj, 4, 0);
   else objname := substring(layoutSelectObj, 3, 0);
   if (layoutSelectObj[0] == 'I')
      XprobeItoSch(data, objname);
   else XprobeNtoSch(data, objname);

}

cmdProbe := vcWv.Cmd(fac, probeit, "Probe");


/***************************************************************************
*
* function laySelectDialog - create a dialog from picklist and parent window
* 
*
***************************************************************************/
function laySelectDialog(picklist, win)
{
  declare picklistwin;

  /* set the notify routine and the first pick */
  vcPicklist.Notify(picklist, layoutObjPick);
  vcPicklist.SetPick(picklist, 0);
 
  /* create a label for the dialog */
  label := vcRegion.Create("label", vcLabel.fac, 
             "Select layout object to be probed to schematic");
  scroll := vcPicklist.Scroll("scroll", picklist, 5, 5, 0);

  /* create the buttons */
  buttons := vcRegion.Create("buttons", vcButnRow.fac,
             list (list ("Probe", "Probe", cmdProbe, nil),
                   list ("Cancel", "Cancel", 
                   func() {
                     vcWin.Close(picklistwin);
                   }, nil)
             ));

  table := vcTable.Create("layoutselect", 3, 1);
  vcTable.Add(table, label);
  vcTable.Add(table, scroll);
  vcTable.Add(table, buttons);

  picklistwin = vcWin.Create(win, table); 
}

/***************************************************************************
*
* function schObjPick - set global variable for schematic picklist data
* 
*
***************************************************************************/
function schObjPick(data, on_or_off)
   if (on_or_off == vlOnOff.On)
      schSelectObj = data;

cmdSchProbe := vcWv.Cmd(fac, 
   func(data, event) {
     objname := substring(schSelectObj, 2, 0);
     if (schSelectObj[0] == 'I')
        XprobeItoLay(data, objname);
     else XprobeNtoLay(data, objname);
   },
   "SchProbe");



/***************************************************************************
*
* function schSelectDialog - create dialog for picklist and parent window
* 
*
***************************************************************************/
function schSelectDialog(picklist, win)
{
  declare picklistwin;

  /* set the notify routine and the first pick */
  vcPicklist.Notify(picklist, schObjPick);
  vcPicklist.SetPick(picklist, 0);
 
  /* create a label for the dialog */
  label := vcRegion.Create("label", vcLabel.fac, 
             "Select schematic object to be probed to layout");
  scroll := vcPicklist.Scroll("scroll", picklist, 5, 5, 0);

  /* create the buttons */
  buttons := vcRegion.Create("buttons", vcButnRow.fac,
             list (list ("Probe", "Probe", cmdSchProbe, nil),
                   list ("Cancel", "Cancel", 
                   func() {
                     vcWin.Close(picklistwin);
                   }, nil)
             ));

  table := vcTable.Create("schematicselect", 3, 1);
  vcTable.Add(table, label);
  vcTable.Add(table, scroll);
  vcTable.Add(table, buttons);

  picklistwin = vcWin.Create(win, table); 
}

ProbeSchSelCmd := vcWv.Cmd(fac, ProbeSchSel, "ProbeSchSel");


/***************************************************************************
*
* function SchNetNameF - function which uppercases the name before calling
*   the real function to do the work
*
***************************************************************************/
function SchNetNameF(data, event, name)
{
   upname := string_upper(name);
   SchNetName(data, event, upname);
}

SchNetNameCmd := vcWv.Cmd(fac, SchNetNameF, "SchNetNameF", 
                 "Xprobe from Schematic",
                 vcWv.Param("Enter Schematic Net Name", vlStringVT,
                            vcWv.Sticky, ""));

/***************************************************************************
*
* function SchInstNameF - function which uppercases the name before calling
*   the real function to do the work
*
***************************************************************************/
function SchInstNameF(data, event, name)
{
   upname := string_upper(name);
   SchInstName(data, event, upname);
}

SchInstNameCmd := vcWv.Cmd(fac, SchInstNameF, "SchInstNameF", 
                 "Xprobe from Schematic",
                 vcWv.Param("Enter Schematic Inst Name", vlStringVT,
                            vcWv.Sticky, ""));

ClearProbeCmd := vcWv.Cmd(fac, ClearProbe, "ClearProbe");

/***************************************************************************
*
* function Reinit - reinitialize xref data and error files for new or
*                   updated run directory
* 
*
***************************************************************************/
function Reinit(lvsnav, event, rundir)
{
   /* verify that the rundir exists */
   if (DirExists(rundir) != 0) {
      ReportErr1(lvsnav, 3036, rundir);
   }
   else {
      /* clear any probes first */
      vcCmd.Run(ClearProbeCmd, lvsnav, event, nil, TRUE);

      /* reinitialize the xref data structures */
      lvsnav =  ReinitXref(lvsnav, rundir);

      /* reinitialize the error buttons */
      lvsreg := GetLvsReg(lvsnav);
      lvsstruct := vcRegion.Get(lvsreg, vscmVT);

      /* set the rundir in the GUI */
      lvs.updateRundir(lvsstruct, rundir);

      vcCmd.Run(lvs.ReinitErrButnCmd, lvsstruct, event, nil, TRUE);
   }
}

ReinitCmd := vcWv.Cmd(fac, Reinit, "Reinit", "Reinitialize",
             vcWv.Param("Enter LVS Run Directory Name", vlStringVT,
                vcWv.Dynamic, getRunDir));

ProbeLaySelCmd := vcWv.Cmd(fac, ProbeLaySel, "ProbeLaySel");

LayNetNameCmd := vcWv.Cmd(fac, LayNetName, "LayNetName", 
                 "Xprobe from Layout",
                 vcWv.Param("Enter Layout Net Name", vlStringVT,
                            vcWv.Sticky, ""));

LayInstNameCmd := vcWv.Cmd(fac, LayInstName, "LayInstName", 
                 "Xprobe from Layout",
                 vcWv.Param("Enter Layout Inst Name", vlStringVT,
                            vcWv.Sticky, ""));

vcWv.MakeEnum("FileModeVT", list ("Append", "Overwrite"));

/***************************************************************************
*
* function AddtoCorrFilePicklist - intermediate function to get from lvsnav C
*          code to corrpt facility (there may be a better way but this works) 
* 
*
***************************************************************************/
function AddtoCorrFilePicklist(str)
{
   corrpt.AddtoPicklist(str);
}

/***************************************************************************
*
* function EnableOpenCFileCmd - enables/disables the OpenCFileCmd
* 
*
***************************************************************************/
function EnableOpenCFileCmd()
   vcCmd.Enable(OpenCFileCmd, vlOnOff.On);
   
/***************************************************************************
*
* function OpenCFile - instantiates the corrpt facility which handles
*          the correspondence file functions
* 
*
***************************************************************************/
function OpenCFile(lvs, event, filename, mode)
{
   declare append;

   if (string_eq(filename, ""))
      /* make sure that the number matches LVSX_NOFILENAME in lvsnav.vmt */
      ReportErr0(lvs, 3029);
   else {
      rundir := getRunDir(lvs);
   
      if (mode == FileMode.Append) {
         append = TRUE;
      }
      else {
         append = FALSE;
      }
      /* create the list of arguments to pass to the corrpt facility create
         method */
      argslist := list(rundir, filename, append, lvs);
   
      /* instantiate the corrpt facility */
      corrreg := vcRegion.Create(filename, corrpt.fac, argslist);
      corrwin := vcWin.Create(corrreg);

      /* disable the OpenCFile Cmd - only one at a time */
      vcCmd.Enable(OpenCFileCmd, vlOnOff.Off);
   }
}

OpenCFileCmd := vcWv.Cmd(fac, OpenCFile, "OpenCFile",
                "Open Correspondence Point File",
                vcWv.Param("Enter File Name", vlStringVT,
                            vcWv.Dynamic, DefaultCorrFileName),
                vcWv.Param("Choose", FileModeVT, vcWv.Sticky, 
                           FileMode.Overwrite));

ToggleZoomCmd := vcWv.Cmd(fac, setZoomPref, "setZoomPref",
                 "Toggle Zoom Preference",
                 vcWv.Param("Zoom Preference", vlOnOffVT, vcWv.Dynamic,
                   getZoomPref));

load("lvsnavmn.vs");
