/*
 * $Log: mfont.vs,v $
Revision 1.17  1995/01/06  19:35:12  gwu
Modified vscript.get_env to vscript.getenv.
This fixes scopus bug #18423.

Revision 1.16  1993/09/15  16:01:34  gwu
Re-implemented Accept, Apply, and Cancel buttons using vcButnRow.fac.
This allows for better looking buttons, as well as better behavior.

Revision 1.15  93/03/09  16:10:28  gwu
Added PC-specific path handling.

Revision 1.14  92/04/21  15:43:56  gwu
Changed the name of SansSerifItalic to Script, and SansSerifBoldItalic
to ScriptBold.

Revision 1.13  92/02/11  14:07:12  gwu
Moved the code that checks for valid font files to apply to schematics
to the compile_font function.  This is a better place to put it, and
allows warnings to be displayed without using error().

Revision 1.12  92/02/08  17:02:48  gwu
Added a font picklist in the compile dialog box.  Verify a font-to-be-
applied before executing the system call.

Revision 1.11  92/02/03  17:31:40  gwu
Increased the amount of spacing for the font region in the decompile
dialog box.

Revision 1.10  92/01/29  18:06:02  gwu
Reduced number of calls to the ugly error() function.  Grouped the
error and information messages in one area.  Tried to make input
parsing more robust.

Revision 1.9  92/01/23  18:57:28  gwu
Gave the Accept and Apply buttons behavior identical to that in
Viewdraw.  Modified the dialog boxes to be popups.

Revision 1.8  92/01/15  16:43:13  gwu
Return nil if the primary working directory is not set in the
viewdraw.ini file.

Revision 1.7  92/01/12  12:40:25  gwu
Removed all forwarding of input into the Makefont dialog boxes.
Disassociated the vcWin.System text window from the Window_ID.
Added the vcRegion.DestroyNotify call to cause Makefont to be
aware of kills from the window manager.

Revision 1.6  92/01/09  19:12:01  gwu
Altered the behaviour of all of the command lines because of the
switch to vcCmdline.CallOnChange.  Fixed some bugs.

Revision 1.5  92/01/08  18:08:02  gwu
Changed the wording of some static text, and modified all
command lines to use vcCmdline.CallOnChange instead of
vcCmdline.CallOnReturn.

Revision 1.4  92/01/02  18:49:59  gwu
Added macro recordability, prefixes for all global variables, and
the ability for the compile process to retain information (i.e.
font and schematic names) from the decompile process.

Revision 1.3  91/12/23  18:39:33  gwu
Forward all input from Viewdraw into the Makefont dialog box when the
dialog box is open.  Also, allow only one Makefont dialog box to be
open at any given time, no matter how many Viewdraw schematics are
in existence.  These should prevent detrimental interaction between
Makefont dialog box processes from occurring.

Revision 1.2  91/12/19  16:46:33  gwu
Added support for the new 'plot' font, added the ability to pop open
a Viewdraw schematic containing the first sheet of a converted font,
and included some improvements to getting user input.

Revision 1.1  91/12/06  19:22:16  gwu
Initial Revision

 */
/* 
 * FILE: mfont.vs
 * Viewscript code for the Makefont GUI under Viewdraw 5.0.
 * This will accept the user's input, do preliminary checks to ensure
 * file existence, and then call makefont in a viewsys.log window.
 * The font schematic(s) will always be put in the primary working
 * directory as specified in the viewdraw.ini file.
 *
 * Copyright (c) 1991 Viewlogic Systems, Inc.
 *
 */
vmfDCMPL_title     := "Convert Stroke Font to Font Schematic";
vmfDCMPL_font_names:= "Select Font:";
vmfDCMPL_alt_name  := "or Enter Font File Name:";
vmfDCMPL_get_font  := "Read From Directory:";
vmfDCMPL_write_sch := "Write Font Schematic:";

vmfCMPL_title      := "Convert Font Schematic to Stroke Font";
vmfCMPL_sch_name   := "Read Font Schematic:";
vmfCMPL_number_pg  := "Number of Schematic Sheets:";
vmfCMPL_font_names := "Write Font:";
vmfCMPL_write_font := "or Enter Font File Name:";
vmfCMPL_font_res   := "Font Resolution:";
vmfCMPL_font_vers  := "Font Version:";
vmfCMPL_use_font   := "Use Stroke Font in Current Schematic:";

vmfDefault         := "fixed";               /* Default cmdline file name.  */
vmfSCH_EXT         := ".1";                  /* Schematic file extension.   */
vmfSTK_EXT         := ".stk";                /* Stroke font file extension. */

vmfPath := "makefont";

vmfDflt_Font_pick     := 0;                  /* Any number between 0 and 12 */
vmfDflt_Res_pick      := 0;                  /* Low = 0   | High = 1        */
vmfDflt_Version_pick  := 0;                  /* Fixed = 0 | Proportional = 1*/
vmfDflt_Use_font_pick := 0;                  /* No = 0    | Yes = 1         */

vmfFont_list := /* The font picklist.  When adding new fonts, include the  */
                /* formal name first, and then the actual file name last.  */
                /* The file name should not have the .stk extension on it. */
 list (list (cons ("Fixed", "fixed"), cons ("Roman", "roman"),
             cons ("RomanItalic", "romani"), cons ("RomanBold", "romanb"),
             cons ("RomanBoldItalic", "romanbi")),
       list (cons ("SansSerif", "sf"), cons ("Script", "script"),
             cons ("SansSerifBold", "sfb"), 
             cons ("ScriptBold", "scriptb"), cons ("Gothic", "gothic")),
       list (cons ("OldEnglish", "oldeng"), cons ("Kanji", "kanji"),
             cons ("Plot", "plot")));

vmfFont_res_list  := list (list (cons ("Low", "16"), 
                                 cons ("High", "1")));
vmfFont_type_list := list (list (cons ("Fixed", "300"), 
                                 cons ("Proportional", "500")));
vmfYes_No_list    := list (list (cons ("No", FALSE), 
                                 cons ("Yes", TRUE)));

vmfAccept := "Accept";
vmfApply  := "Apply";
vmfCancel := "Cancel";

vmfErr_StartingMF    := "Executing MakeFont...";
vmfErr_FinishedMF    := "Executing MakeFont...  Done.";
vmfErr_ErrFinishedMF := "Executing MakeFont...  Done with Errors.";
vmfErr_NeedMoreInfo  := "Cannot run MakeFont.  Fill in all necessary information before continuing.";
vmfErr_StkNotExist   := "Cannot run MakeFont.  Stroke font file does not exist in the specified directory.";
vmfErr_SchExtWrong   := "Cannot run MakeFont.  Font schematic filename must have an extension of";
vmfErr_CannotUseFont := "Cannot apply font to current schematic.  Verify that the stroke font exists in the local directory or in the WDIR path.";
vmfErr_UnknownFont1  := "Invalid font file name.  Cannot use '";
vmfErr_UnknownFont2  := "' in current schematic.";
vmfErr_DoRightThing  := "Select a valid font name from the font list, and click on Apply.";
vmfErr_SchSheetsGr1  := "Invalid entry.  Number of schematic sheets must be at least 1.";
vmfErr_NoDirSpec     := "Invalid entry.  Path specification not allowed in schematic name.";
vmfErr_OneDialogOnly := "Only one Makefont dialog box may be open at any time.\nPlease complete the previous Makefont task before starting another.";

declare vmfTask;
declare vmfFont_name;
declare vmfFont_path;
declare vmfSch_name;
declare vmfNum_pages;
declare vmfFont_resolution;
declare vmfFont_version;
declare vmfUse_font_now;
declare vmfAccept_or_Apply;
declare vmfWindow_ID;
declare vmfAlert_ID;

declare stk_path_cmdline;
declare alt_stk_path_cmdline;
declare schname_cmdline;
declare num_page_cmdline;
declare font_pick;

/* This function provides the same information as the vgStkFont 
 * enumeration.  It returns an ID number for a certain font name,
 * and nil if the font is unrecognized.
 */
function vmfGet_vgStkFontT (stroke_font)
{
  if (string_eq (stroke_font, "fixed.stk"))
     return (0);
  if (string_eq (stroke_font, "roman.stk"))
     return (1);
  if (string_eq (stroke_font, "romani.stk"))
     return (2);
  if (string_eq (stroke_font, "romanb.stk"))
     return (3);
  if (string_eq (stroke_font, "romanbi.stk"))
     return (4);
  if (string_eq (stroke_font, "sf.stk"))
     return (5);
  if (string_eq (stroke_font, "script.stk"))
     return (6);
  if (string_eq (stroke_font, "sfb.stk"))
     return (7);
  if (string_eq (stroke_font, "scriptb.stk"))
     return (8);
  if (string_eq (stroke_font, "gothic.stk"))
     return (9);
  if (string_eq (stroke_font, "oldeng.stk"))
     return (10);
  if (string_eq (stroke_font, "kanji.stk"))
     return (11);
  if (string_eq (stroke_font, "plot.stk"))
     return (12);
  return (nil);

} /* vmfGet_vgStkFontT */

/* This function was taken from Judy's Excellent vspUtils. 
 */
function vmfCmdline (prompt_val, def_val, get_val_func, do_border, reg_name)
{
      /* Label region for prompt */
      prompt := vcRegion.Create1 (prompt_val, vcLabel.fac, prompt_val, nil);

      /* Command line input area */
      cmdline := vcRegion.Create1 (string_append(reg_name, ".cmdline"), 
                                   vcCmdline.fac, def_val,
                                   list (max_width <- 50,
                                         width <- 40,
                                         macro_on <- TRUE));

      /* Set up function to call when carriage return is hit */
      /* vcCmdline.CallOnReturn (cmdline, get_val_func); */
      vcCmdline.CallOnChange (cmdline, get_val_func);

      /* Tie the two regions together */
      prompt_n_line := vcTable.Create (prompt_val, 1, 2);
      vcTable.Col (prompt_n_line, -1, vcTable.Border.Invisible, 4, 6, 8,
                   vgAlign.LowerLeft);
      vcTable.Add (prompt_n_line, prompt);
      vcTable.Add (prompt_n_line, cmdline);

      if (do_border) {
              /* Wrap them with a border */
              border := vcRegion.Create1 (string_append(prompt_val,".border"),
                                          vcBorder.fac, prompt_n_line, nil);
              /* Return the entire widget */
              return(border);
      }
      else
              return(prompt_n_line);
} /* vmfCmdline */

/* This function was taken from Judy's Excellent vspUtils.
 */
function vmfAdd_but (table, str, row, col, fun)
{
      label  := vcRegion.Create1 (str, vcLabel.fac, str, nil);
      border := vcRegion.Create1 (str, vcBorder.fac, label, nil);
      button := vcRegion.Create1 (str, vcButton.fac, func () fun ();,
                                  list (macro_on <- TRUE));
      vcRegion.AddChild (label, button);
      vcTable.Set (table, row, col, border);
} /* vmfAdd_but */

/* Spawn a new viewdraw window with a newly created schematic only
 * if Makefont completes successfully.  The schematic () function
 * is defined in the Viewdraw 5.0 Viewscript code.
 */
function vmfOpen_schematic (compl_mode, compl_code)
{
     /* Call schematic () only if the Makefont process has exited, */
     /* and has a completion code of 0, which means success.       */
     if ((compl_mode == vcSysMode.Exited) && (compl_code == 0))
        schematic (vmfSch_name);

     if ((string_eq (vmfAccept_or_Apply, "apply")) && (vmfWindow_ID != nil)) {
        vcWin.ClearStatus (vmfWindow_ID);
        if (compl_code == 0)
            vcWin.AddStatus (vmfWindow_ID, vmfErr_FinishedMF);
        else
            vcWin.AddStatus (vmfWindow_ID, vmfErr_ErrFinishedMF);
     } /* if */

} /* vmfOpen_schematic */

/* After the makefont compile process has occurred, check to see if
 * the task was successful.  If it was, and the user wishes to use
 * the font in the current schematic, load and apply the new font.
 */
function vmfUse_font (compl_mode, compl_code)
{
     if ((compl_mode == vcSysMode.Exited) && (compl_code == 0) &&
         (vmfUse_font_now == TRUE))
          vmfApply_font ();

     if ((string_eq (vmfAccept_or_Apply, "apply")) && (vmfWindow_ID != nil)) {
        vcWin.ClearStatus (vmfWindow_ID);
        if (compl_code == 0)
            vcWin.AddStatus (vmfWindow_ID, vmfErr_FinishedMF);
        else
            vcWin.AddStatus (vmfWindow_ID, vmfErr_ErrFinishedMF);
     } /* if */

} /* vmfUse_font */

/* Spawn a viewys.log shell process to run makefont.  If the task is to
 * decompile a font (font -> schematic), and if makefont has completed
 * successfully, open a new Viewdraw schematic window with the new sch.
 * If the task is to compile a font, then load the font into the current
 * schematic if the user desires.
 */
function vmfInvoke_makefont (invoke_task, command)
{
     if (string_eq (invoke_task, "compile"))
         vcWin.System (nil, command, nil, TRUE, vcSysOutput.Viewtext,
                       nil, vmfUse_font,  nil, nil);
     else if (string_eq (invoke_task, "decompile"))
         vcWin.System (nil, command, nil, TRUE, vcSysOutput.Viewtext,
                       nil, vmfOpen_schematic, nil, nil);

     vcWin.ClearStatus (vmfWindow_ID);
     vcWin.AddStatus (vmfWindow_ID, vmfErr_StartingMF);

} /* vmfInvoke_makefont */

/* Look for the primary working directory as specified in the first
 * viewdraw.ini file  found.  Use  vsGetini to  find the .ini file,
 * then parse the file to find the first [p], [pw], or 0, directory.
 * If  a formal  function to get the primary  working  directory  is 
 * implemented, it should replace this function.
 */
function vmfGet_primary (ini_file)
{
  declare primary;
  previous_index := 0;
  line_count := 0;

  if ((path := vscript.vsGetini (ini_file)) != nil) {

    fp := fopen (path, "r");

    while (!((line := fgetline (fp)) == nil)) {

        ++line_count;

        line_length := string_length (line);

        for (index := 0; index < line_length; ++index) {
            
            if (string_ref (line, index) == ' ') {
                token := substring (line, previous_index, index);
                if (string_eq (token, "DIR")) {
                    previous_index := index + 1;
                    for (++index; index < line_length; ++index) {
                        if (string_ref (line, index) == ' ') {
                            token := substring (line, previous_index, index);
                            if ((string_eq (token, "[p]")) ||
                                (string_eq (token, "[pw]")) ||
                                (string_eq (token, "0"))) {
                                previous_index := index + 1;
                                for (++index; index < line_length; ++index) {
                                    if ((string_ref (line, index) == ' ') ||
                                        (index ==  (line_length - 1))) {
                                        primary = substring (line,
                                                             previous_index,
                                                             index);
                                    } /* if */
                                } /* for */
                            } /* if */
                        } /* if */
                    } /* for */
                } /* if */
                else {
                    index := line_length;
                } /* else */
            } /* if */
        } /* for */
        previous_index := 0;
    } /* while */
    fclose (fp);
    /* Check to see if primary directory has been set. */
    /* If it has not, then return nil. */
    if (primary == nil) return (primary);
    /* Remove whitespace at end of primary string, if any. */
    string_length := string_length (primary);
    for (x := 0; (x < string_length && string_ref (primary, x) != ' '); ++x);
    primary = substring (primary, 0, x);
    return (primary);
  } /* if */
  else
    return (primary);
} /* vmfGet_primary */

/* Get the WDIR font path as a default.  If the file_name does not
 * exist in the first WDIR, check all WDIRs for the file_name, and
 * return the path.  This is tricky because a value cannot be re- 
 * turned from within the foreach loop, so default_wdir must be set,
 * and be returned outside of the loop.                                  
 */
function vmfGet_wdir_path (file_name)
{
     declare default_wdir;
     declare wdir_not_found;

     wdir_not_found = TRUE;
     foreach (wdir; vscript.env_path ("WDIR")) {
        path_length := string_length (wdir);
        if (vsp.is_pc ()) {
           if (not (string_ref (wdir, (--path_length)) == '\\'))
              wdir = wdir + "\\";
        }
        else {
           if (not (string_ref (wdir, (--path_length)) == '/'))
              wdir = wdir + "/";
        }

        if ((fexists (wdir + file_name)) && (wdir_not_found)) {
           wdir_not_found = FALSE;
           default_wdir = substring (wdir, 0, path_length + 1);
        } /* if */
     } /* foreach */

     if (default_wdir == nil)
        return ("");
     else
        return (default_wdir);
} /* vmfGet_wdir_path */

/* Get current working directory, or the best guess of one.  This is
 * not very easy to do in Viewscript because there really are not the
 * functions to do so.  First check WPROJECT env. variable, then look 
 * for the primary specified in the first viewdraw.ini found, then
 * check for the local /sch directory in which to place the schematic.
 */
function vmfGet_work_path ()
{
     declare default_wk_path;

     /* Get the environment variable set by Viewproject */
     work_path := vscript.getenv ("WPROJECT");

     /* If WPROJECT is not set, check for the viewdraw.ini file. */
     /* The first if check is not accepted in vload because the  */
     /* WPROJECT variable will probably  not be set, and  vload  */
     /* will do a type check and require work_path == nil.       */
     if ((work_path == nil) || (string_eq (work_path, ""))) {

        work_path := vmfGet_primary ("viewdraw.ini");

        /* No project specified in viewdraw.ini.  Check local directory */
        if ((work_path == nil) || (string_eq (work_path, ""))) {

          if ((work_path := vscript.vsSrchdirs ("sch", vscript.vsDIR)) != nil){
             path_length := string_length (work_path);
             default_wk_path = substring (work_path, 0, path_length - 3);
             return (default_wk_path);
          } /* if */
          else {
             if ((work_path := vscript.vsSrchdirs (".", vscript.vsDIR))!=nil) {
                 default_wk_path = work_path; 
                 return (default_wk_path);
             } /* if */
             else {
                 return (work_path);
             } /* else */
          } /* else */
        } /* if */
        wpath_length := string_length (work_path);
        if (vsp.is_pc ()) {
           if (not (string_ref (work_path, (--wpath_length)) == '\\'))
              work_path := work_path + "\\";
        }
        else {
           if (not (string_ref (work_path, (--wpath_length)) == '/'))
              work_path := work_path + "/";
        }
        return (work_path);
     } /* if */
     else {
        /* Check to see if there is an / ending for the WPROJECT data... */
        wpath_length := string_length (work_path);
        if (vsp.is_pc ()) {
           if (not (string_ref (work_path, (--wpath_length)) == '\\'))
              work_path := work_path + "\\";
        }
        else {
           if (not (string_ref (work_path, (--wpath_length)) == '/'))
              work_path := work_path + "/";
        }

        return (work_path);
     } /* else */
} /* vmfGet_work_path */

/* The default stroke font file will be fixed.stk, and the default
 * schematic name will be fixed.1.  The WDIR will be the default path
 * to get the .stk file, and the current working directory will be
 * the place to write the font schematic.
 */ 
function vmfDecompile_default_init ()
{
     vmfFont_name = vmfDefault + vmfSTK_EXT;     /* fixed.stk is the default */
     vmfFont_path = vmfGet_wdir_path (vmfFont_name);/* Get wdir path of fixed*/
     vmfSch_name = vmfDefault + vmfSCH_EXT;      /* fixed.1 is the default   */
} /* decompile_global_init */

function vmfCompile_default_init ()
{
     /* If "decompile" has not been run, use the default schematic name. */
     if (vmfSch_name == nil)
        /* Set the default schematic name. */
        vmfSch_name = vmfDefault + vmfSCH_EXT;
     vmfNum_pages = "1";
     /* If "decompile" has not been run, use the default stroke font path. */
     if (vmfFont_name == nil)
        vmfFont_path = vmfGet_work_path () + vmfDefault + vmfSTK_EXT;
     else
        vmfFont_path = vmfGet_work_path () + vmfFont_name;
     /* Font resolution is low */
     vmfFont_resolution = "16";
     /* Font type is fixed */
     vmfFont_version = "300";
     /* Do not use font as default */
     vmfUse_font_now = FALSE;

} /* compile_global_init */

function vmfGet_font_cmdline (val)
{
     vmfFont_path = val;
     vcWin.ClearStatus (vmfWindow_ID); 

} /* vmfGet_font_cmdline */

function vmfGet_alt_font_cmdline (val)
{
     vmfFont_name = val;
     vcWin.ClearStatus (vmfWindow_ID);

     val_length := string_length (vmfFont_name);
     for (i := val_length - 1;
              (i >= 0) && (not (string_ref (vmfFont_name, i) == '.')); --i);
     if (i >= 0) {
        /* Change the name of the target font schematic to show changes. */
        /* Remove everything after the `.' in the stroke font path.      */
        for (i := (string_length (vmfFont_name) - 1);
                 (i >= 0) && (not (string_ref (vmfFont_name, i) == '.')); --i);
        font_name_prefix := substring (vmfFont_name, 0, i);

        new_sch_name := font_name_prefix + vmfSCH_EXT;
     } /* if */
     else
        new_sch_name := vmfFont_name + vmfSCH_EXT;

     /* Remove everything before the `/' in the stroke font path.  We */
     /* do not want any directory specification in the sch file name. */
     stk_length := string_length (new_sch_name);
     if (vsp.is_pc ())
         for (i := stk_length - 1;
              (i >= 0) && (not (string_ref (new_sch_name, i) == '\\')); --i);
     else
         for (i := stk_length - 1;
              (i >= 0) && (not (string_ref (new_sch_name, i) == '/')); --i);
   
     if (i >= 0)
         new_sch_name := substring (new_sch_name, i + 1, stk_length);

     vcCmdline.Set (schname_cmdline, new_sch_name);

     vmfSch_name = new_sch_name;

} /* vmfGet_alt_font_cmdline */

function vmfGet_schname_cmdline (val)
{
     /* This flag must be used because vcWin.AddStatus will not work */
     /* with the vcCmdline.Set during the compilation process.       */
     illegal_dirspec := FALSE;

     vmfSch_name = val;
     vcWin.ClearStatus (vmfWindow_ID);

     /* Check to see if a directory specification is part of the input. */
     /* Directory specs are not allowed in the schematic file name, so  */
     /* output a warning to the user.                                   */
     sch_length := string_length (vmfSch_name);
     if (vsp.is_pc ()) 
     for (i := sch_length - 1;
              (i >= 0) && (not (string_ref (vmfSch_name, i) == '\\')); --i);
     else
     for (i := sch_length - 1;
              (i >= 0) && (not (string_ref (vmfSch_name, i) == '/')); --i);

     if (i >= 0) {
         vcWin.AddStatus (vmfWindow_ID, vmfErr_NoDirSpec);
         /* The following flag is set for the compilation process */
         illegal_dirspec := TRUE;
     } /* if */

     if (string_eq (vmfTask, "compile")) {
        /* Change the name of the target stroke font path command line. */
        /* Remove everything after the `.' after the schematic name.    */
        for (i := (string_length (vmfSch_name) - 1);
                  (i >= 0) && (not (string_ref (vmfSch_name, i) == '.')); --i);
        if (i >= 0)
            stk_name_prefix := substring (vmfSch_name, 0, i);
        else
            stk_name_prefix := vmfSch_name;

        /* Remove everything before the last `/' if pathname exists. */
        sch_path_length := string_length (stk_name_prefix);
        if (vsp.is_pc ())
           for (i := sch_path_length - 1;
              (i >= 0) && (not (string_ref(stk_name_prefix, i) == '\\')); --i);
        else
           for (i := sch_path_length - 1;
              (i >= 0) && (not (string_ref(stk_name_prefix, i) == '/')); --i);

        if (i >= 0)
              /* Directory specification exists before schematic file name */
              stk_name_prefix := substring (stk_name_prefix, i + 1,
                                            sch_path_length);

        /* Remove old file name (if any), and replace with new file name. */
        font_path_length := string_length (vmfFont_path);
        if (font_path_length < 1) {
            /* Empty stroke font cmdline */
            new_stk_path := stk_name_prefix + vmfSTK_EXT;
        } /* if */
        else {
            if (vsp.is_pc ())
               for (i := font_path_length - 1;
                 (i >= 0) && (not (string_ref(vmfFont_path, i) == '\\')); --i);
            else
               for (i := font_path_length - 1;
                 (i >= 0) && (not (string_ref(vmfFont_path, i) == '/')); --i);

            if (i >= 0) {
                  /* Directory specification before file */
                  dir_path := substring (vmfFont_path, 0, i + 1);
                  new_stk_path := dir_path + stk_name_prefix + vmfSTK_EXT;
            } /* if */
            else {
                  /* No directory specification - file only */
                  new_stk_path := stk_name_prefix + vmfSTK_EXT;
            } /* else */
        } /* else */

        vcCmdline.Set (stk_path_cmdline, new_stk_path);

        if (illegal_dirspec)
            vcWin.AddStatus (vmfWindow_ID, vmfErr_NoDirSpec);

        /* Finally, set the global font_path to the new path. */
        vmfFont_path = new_stk_path;
     } /* if */
} /* vmfGet_schname_cmdline */

function vmfGet_num_cmdline (val)
{
     vmfNum_pages = val;
     vcWin.ClearStatus (vmfWindow_ID); 

     /* This will verify that the value is indeed an integer. */
     /* Viewscript will complain if val cannot be converted.  */
     if (not (string_eq (vmfNum_pages, ""))) {
        num_sheets := string2number (val);

        if (num_sheets < 1) {
            vcWin.ClearStatus (vmfWindow_ID);
            vcWin.AddStatus (vmfWindow_ID, vmfErr_SchSheetsGr1);
        } /* if */
     } /* if */

} /* vmfGet_num_cmdline */

function vmfDecompile_font ()
{
     declare full_font_path;

     if ((vmfPath == nil) || (string_eq (vmfPath, "")) ||
         (vmfFont_name == nil) || (string_eq (vmfFont_name, "")) ||
	 (vmfSch_name == nil) || (string_eq (vmfSch_name, ""))) {
         vcWin.ClearStatus (vmfWindow_ID);
         vcWin.AddStatus (vmfWindow_ID, vmfErr_NeedMoreInfo);
     } /* if */
     else {
        /* Verify that there is a sch. suffix (.1) in vmfSch_name. */
        /* i will be >= 0 if an `.' already exists, < 0 if not.    */
        val_length := string_length(vmfSch_name);
        for (i := val_length - 1;
                (i >= 0) && (not (string_ref (vmfSch_name, i) == '.')); --i);
        if (i < 0)
            error (vmfErr_SchExtWrong, vmfSCH_EXT);
        else {
            if (!(string_eq(substring(vmfSch_name,i, val_length), vmfSCH_EXT)))
                error (vmfErr_SchExtWrong, vmfSCH_EXT);
        } /* else */

        fpath_length := string_length (vmfFont_path);
        if (fpath_length != 0)
            if (vsp.is_pc ()) {
                if (not (string_ref (vmfFont_path, (--fpath_length)) == '\\'))
                    vmfFont_path = vmfFont_path + "\\";
            }
            else {
                if (not (string_ref (vmfFont_path, (--fpath_length)) == '/'))
                    vmfFont_path = vmfFont_path + "/";
            }

        full_font_path = vmfFont_path + vmfFont_name;

        if (fexists (full_font_path)) {
             /* Build shell function call */
             function_call := vmfPath + " -d " + full_font_path + " " + 
                              vmfSch_name;

             vmfInvoke_makefont (vmfTask, function_call);

             if (string_eq (vmfAccept_or_Apply, "accept"))
                 /* Destroy the dialog box if Accept */
                 vmfCancel_CB();

        } /* if */
        else {
            vcWin.ClearStatus (vmfWindow_ID);
            vcWin.AddStatus (vmfWindow_ID, vmfErr_StkNotExist);
        }

     } /* else */
} /* vmfDecompile_font */

function vmfCompile_font ()
{
     declare font_vgStkFontT;
     declare local_errmsg;

     if ((vmfPath == nil) || (string_eq (vmfPath, "")) ||
         (vmfSch_name == nil) || (string_eq (vmfSch_name, "")) ||
         (vmfNum_pages == nil) || (string_eq (vmfNum_pages, "")) || 
	 (vmfFont_path == nil) || (string_eq (vmfFont_path, "")) ||
         (vmfFont_resolution == nil) || (string_eq (vmfFont_resolution, "")) ||
         (vmfFont_version == nil) || (string_eq (vmfFont_version, ""))) {
         vcWin.ClearStatus (vmfWindow_ID);
         vcWin.AddStatus (vmfWindow_ID, vmfErr_NeedMoreInfo);
     } /* if */
     else {
        /* Verify that there is a sch. suffix (.1) in vmfSch_name. */
        /* i will be >= 0 if an `.' already exists, < 0 if not.    */
        val_length := string_length(vmfSch_name);
        for (i := val_length - 1;
                (i >= 0) && (not (string_ref (vmfSch_name, i) == '.')); --i);
        if (i < 0)
            error (vmfErr_SchExtWrong, vmfSCH_EXT);
        else {
            if (!(string_eq(substring(vmfSch_name,i, val_length), vmfSCH_EXT)))
                error (vmfErr_SchExtWrong, vmfSCH_EXT);
        } /* else */

        /* Check to see if target font file can be written */
        /* Exception will be raised by vscript if not.     */
        font_file := fopen (vmfFont_path, "w");
        fclose (font_file);

        /* Build shell function call */
        function_call := vmfPath + " -c " + vmfSch_name + " " + vmfNum_pages
                         + " " + vmfFont_path + " " + vmfFont_resolution 
                         + " " + vmfFont_version;

        if (vmfUse_font_now == TRUE) {

             /* Before applying the font, see if it exists */
             font_path_length := string_length (vmfFont_path);
             if (vsp.is_pc ())
               for (i := font_path_length - 1;
                (i >= 0) && (not (string_ref (vmfFont_path, i) == '\\')); --i);
             else
               for (i := font_path_length - 1;
                (i >= 0) && (not (string_ref (vmfFont_path, i) == '/')); --i);

             vmfFont_name = substring (vmfFont_path, i+1, font_path_length);

             /* Now get the vgStkFontT for the font_name */
             font_vgStkFontT = vmfGet_vgStkFontT (vmfFont_name);

             if (font_vgStkFontT != nil) {
                  vmfInvoke_makefont (vmfTask, function_call);

                  if (string_eq (vmfAccept_or_Apply, "accept"))
                      /* Destroy the dialog box if Accept */
                      vmfCancel_CB();
             } /* if */
             else {
                   /* Signify error in dialog box */
                   local_errmsg = vmfErr_UnknownFont1 + vmfFont_name +
                                  vmfErr_UnknownFont2 + "  " + 
                                  vmfErr_DoRightThing;
                   vcWin.ClearStatus (vmfWindow_ID);
                   vcWin.AddStatus (vmfWindow_ID, local_errmsg);
             } /* else */
        } /* if */
        else {
             vmfInvoke_makefont (vmfTask, function_call);

             if (string_eq (vmfAccept_or_Apply, "accept"))
                 /* Destroy the dialog box if Accept */
                 vmfCancel_CB();
        } /* else */
     } /* else */
} /* vmfCompile_font */

function vmfApply_font ()
{
     declare apply_font_path;
     declare font_vgStkFontT;

     /* Before applying the font, find the font name in a WDIR directory */
     font_path_length := string_length (vmfFont_path);
     if (vsp.is_pc ())
         for (i := font_path_length - 1;
             (i >= 0) && (not (string_ref (vmfFont_path, i) == '\\')); --i);
     else   
         for (i := font_path_length - 1;
             (i >= 0) && (not (string_ref (vmfFont_path, i) == '/')); --i);

     vmfFont_name = substring (vmfFont_path, i + 1, font_path_length);

     /* Now get the vgStkFontT for the font_name */
     font_vgStkFontT = vmfGet_vgStkFontT (vmfFont_name);

     if (!(font_vgStkFontT == nil)) {
       apply_font_path = vmfGet_wdir_path (vmfFont_name);

       if (apply_font_path == nil)
          error (vmfErr_CannotUseFont, vmfFont_name);
       /* "C" callback in Viewdraw for reloading a font */
       /* and refreshing the window.                    */ 
       reloadfont (font_vgStkFontT);
     } /* if */
     else
       error (vmfErr_UnknownFont, vmfFont_name);

} /* vmfApply_font */

function vmfAccept_CB ()
{
     vcWin.ClearStatus (vmfWindow_ID); 
     vmfAccept_or_Apply = "accept";
     if (string_eq (vmfTask, "compile"))
         vmfCompile_font ();
     else
         vmfDecompile_font ();

} /* vmfAccept_CB */

function vmfApply_CB ()
{
     vcWin.ClearStatus (vmfWindow_ID); 
     vmfAccept_or_Apply = "apply";
     if (string_eq (vmfTask, "compile"))
         vmfCompile_font ();
     else
         vmfDecompile_font ();

} /* vmfApply_CB */

function vmfCancel_CB () 
{
     vcWin.Destroy (vmfWindow_ID);
     vmfWindow_ID = nil;

} /* vmfCancel_CB */

function vmfResolution_CB (val, on_off, data) 
{
     vmfFont_resolution = val;
     vcWin.ClearStatus (vmfWindow_ID); 

} /* vmfResolution_CB */

function vmfVersion_CB (val, on_off, data) 
{
     vmfFont_version = val;
     vcWin.ClearStatus (vmfWindow_ID); 

} /* vmfVersion_CB */

function vmfUse_CB (val, on_off, data) 
{
     vmfUse_font_now = val;
     vcWin.ClearStatus (vmfWindow_ID); 

} /* vmfUse_CB */

function vmfFont_CB (val, on_off, data) 
{
     vmfFont_name = val + vmfSTK_EXT;
     vcWin.ClearStatus (vmfWindow_ID); 

     if (string_eq (vmfTask, "decompile")) {

          vcCmdline.Set (alt_stk_path_cmdline, vmfFont_name);

          /* Change the name of the target font schematic to show changes. */
          /* Remove everything after the `.' in the stroke font path.      */
          for (i := (string_length (vmfFont_name) - 1);
                 (i >= 0) && (not (string_ref (vmfFont_name, i) == '.')); --i);
          font_name_prefix := substring (vmfFont_name, 0, i);

          new_sch_name := font_name_prefix + vmfSCH_EXT;
          vcCmdline.Set (schname_cmdline, new_sch_name);

          vmfSch_name = new_sch_name;
     } /* if */
     else { /* Compile */

          /* Change the name of the target stroke font to show changes. */
          /* Remove everything after the `/' in the stroke font path.   */
          if (vsp.is_pc())
              for (i := (string_length (vmfFont_path) - 1);
                (i >= 0) && (not (string_ref (vmfFont_path, i) == '\\')); --i);
          else
              for (i := (string_length (vmfFont_path) - 1);
                (i >= 0) && (not (string_ref (vmfFont_path, i) == '/')); --i);

          if (i > 0) {
               font_path_prefix := substring (vmfFont_path, 0, i + 1);
               new_stk_path := font_path_prefix + vmfFont_name;
          } /* if */
          else
               new_stk_path := vmfFont_name;

          vcCmdline.Set (stk_path_cmdline, new_stk_path);

          vmfFont_path = new_stk_path;
     } /* else */

} /* vmfFont_CB */

function vmfAdd_font_name_pick (dialog_table) 
{
     font_pick_region := vcTable.Create ("font_region", 1, 2);
     font_pick = vcRegion.Create1 ("font_pick", vcPicklist.fac, vmfFont_list,
                                   list (enhanced_notify <- TRUE,
                                         macro_on <- TRUE));

     if (string_eq (vmfTask, "decompile"))
          vcTable.Add (font_pick_region, vcRegion.Create1 ("font_label",
                                                           vcLabel.fac,
                                                           vmfDCMPL_font_names,
                                                           nil));
     else /* Compile */
          vcTable.Add (font_pick_region, vcRegion.Create1 ("font_label",
                                                           vcLabel.fac,
                                                           vmfCMPL_font_names,
                                                           nil));
     vcTable.Add (font_pick_region, font_pick);
     vcTable.Add (dialog_table, font_pick_region);

     /* Default font in picklist is Fixed.stk */
/*
     vcPicklist.SetPick (font_pick, 0);*/
     vcPicklist.Notify (font_pick, vmfFont_CB);        
} /* vmfAdd_font_name_pick */

function vmfAdd_font_resolution_pick (dialog_table) 
{
     res_pick_region := vcTable.Create ("res_region", 1, 5);
     resolution_pick := vcRegion.Create1 ("font_res_pick", vcPicklist.fac,
                                          vmfFont_res_list,
                                          list (enhanced_notify <- TRUE,
                                                macro_on <- TRUE));
     vcTable.Add (res_pick_region, vcRegion.Create1 ("resolution_label",
                                                      vcLabel.fac,
                                                      vmfCMPL_font_res,
                                                      nil));
     /* This region is for spacing purposes only */
     vcTable.Add (res_pick_region, vcRegion.Create1 ("resolution_label",
                                                      vcLabel.fac,
                                           "                                ",
                                                      nil));
     vcTable.Add (res_pick_region, resolution_pick);
     vcTable.Add (dialog_table, res_pick_region);
     vcPicklist.SetPick (resolution_pick, vmfDflt_Res_pick);
     vcPicklist.Notify (resolution_pick, vmfResolution_CB);
} /* vmfAdd_font_resolution_pick */

function vmfAdd_font_version_pick (dialog_table) 
{
     vers_pick_region := vcTable.Create ("version_region", 1, 4);
     version_pick := vcRegion.Create1 ("font_vers_pick", vcPicklist.fac,
                                        vmfFont_type_list,
                                        list (enhanced_notify <- TRUE,
                                              macro_on <- TRUE));
     vcTable.Add (vers_pick_region, vcRegion.Create1 ("version_label",
                                                      vcLabel.fac,
                                                      vmfCMPL_font_vers,
                                                      nil));
     /* This region is for spacing purposes only */
     vcTable.Add (vers_pick_region, vcRegion.Create1 ("version_label",
                                                      vcLabel.fac,
                                                      "           ",
                                                      nil));
     vcTable.Add (vers_pick_region, version_pick);
     vcTable.Add (dialog_table, vers_pick_region);
     vcPicklist.SetPick (version_pick, vmfDflt_Version_pick);
     vcPicklist.Notify (version_pick, vmfVersion_CB);
} /* vmfAdd_font_version_pick */

function vmfAdd_use_font_pick (dialog_table) 
{
     use_font_region := vcTable.Create ("use_region", 1, 6);
     use_pick := vcRegion.Create1 ("use_font_pick", vcPicklist.fac,
                                   vmfYes_No_list,
                                   list (enhanced_notify <- TRUE,
                                         macro_on <- TRUE));
     vcTable.Add (use_font_region, vcRegion.Create1 ("use_label",
                                                     vcLabel.fac,
                                                     vmfCMPL_use_font,
                                                     nil));
     /* This region is for spacing purposes only */
     vcTable.Add (use_font_region, vcRegion.Create1 ("version_label",
                                                      vcLabel.fac,
                                                      "            ",
                                                      nil));
     vcTable.Add (use_font_region, use_pick);
     vcTable.Add (dialog_table, use_font_region);
     vcPicklist.SetPick (use_pick, vmfDflt_Use_font_pick);
     vcPicklist.Notify (use_pick, vmfUse_CB);
} /* vmfAdd_use_font_pick */

function vmfWindow_destroyed (region)
{
     vmfWindow_ID = nil;

} /* vmfWindow_destroyed */

function vmfMake_decompile_dialog () 
{
     /* Initialized defaults */
     vmfDecompile_default_init();

     dialog_table := vcTable.Create (vmfDCMPL_title, 4, 1);

     /*******************************/
     /*** Stroke Font Information ***/
     /*******************************/

     font_region := vcTable.Create ("font_region", 6, 1);

     vmfAdd_font_name_pick (font_region);

     /* Get typed-in alternate name of stroke font */
     alt_stk_path_cmdline = vmfCmdline (vmfDCMPL_alt_name, vmfFont_name,
                                        vmfGet_alt_font_cmdline, FALSE,
                                        "alt_stk_path");
     vcTable.Add (font_region, alt_stk_path_cmdline);

     /* Get the stroke font path to edit from */
     stk_path_cmdline = vmfCmdline (vmfDCMPL_get_font, vmfFont_path,
                                    vmfGet_font_cmdline, FALSE, "stk_path");
     vcTable.Add (font_region, stk_path_cmdline);

     font_wborder := vcRegion.Create1 ("font_border", vcBorder.fac,
                                        font_region, nil);

     vcTable.Add (dialog_table, font_wborder);

     /************************************/
     /*** SCHEMATIC INFORMATION REGION ***/
     /************************************/

     sch_region := vcTable.Create ("sch_region", 3, 1);

     /* Get the name of the font schematic (No default name) */
     schname_cmdline = vmfCmdline (vmfDCMPL_write_sch, vmfSch_name, 
                                   vmfGet_schname_cmdline, FALSE, "schname");
     vcTable.Add (sch_region, schname_cmdline);

     sch_wborder := vcRegion.Create1 ("sch_border", vcBorder.fac,
                                      sch_region, nil);

     vcTable.Add (dialog_table, sch_wborder);

     vcTable.Add (dialog_table,
                  vcRegion.Create ("button_row", vcButnRow.fac,
                                   list (list ("accept", "Accept",
                                               vmfAccept_CB, nil),
                                         list ("apply", "Apply",
                                               vmfApply_CB, nil),
                                         list ("cancel", "Cancel",
                                               vmfCancel_CB, nil))));

     return (dialog_table);   
} /* vmfMake_decompile_dialog */

function vmfMake_compile_dialog () 
{
     /* Initialize defaults */
     vmfCompile_default_init ();

     /* Create dialog region table */
     dialog_table := vcTable.Create (vmfCMPL_title, 5, 1);

     /************************************/
     /*** SCHEMATIC INFORMATION REGION ***/
     /************************************/

     sch_region := vcTable.Create ("sch_region", 4, 1);

     /* Get the name of the font schematic (No default name) */
     schname_cmdline = vmfCmdline (vmfCMPL_sch_name, vmfSch_name, 
                                   vmfGet_schname_cmdline, FALSE, "schname");
     vcTable.Add (sch_region, schname_cmdline);

     /* Get the number of sheets for the schematic (default 1) */
     num_page_cmdline = vmfCmdline (vmfCMPL_number_pg, vmfNum_pages, 
                                    vmfGet_num_cmdline, FALSE, "num_page");
     vcTable.Add (sch_region, num_page_cmdline);

     sch_wborder := vcRegion.Create1 ("sch_border", vcBorder.fac,
                                       sch_region, nil);

     vcTable.Add (dialog_table, sch_wborder);

     /*******************************/
     /*** FONT INFORMATION REGION ***/
     /*******************************/

     font_region := vcTable.Create ("font_region", 6, 1);

     vmfAdd_font_name_pick (font_region);

     /* Get the path and name of the target stroke font */
     stk_path_cmdline = vmfCmdline (vmfCMPL_write_font, vmfFont_path,
                                    vmfGet_font_cmdline, FALSE, "stk_path");
     vcTable.Add (font_region, stk_path_cmdline);

     /* Add the font resolution pick list */
     vmfAdd_font_resolution_pick (font_region);

     /* Add the font version pick list */
     vmfAdd_font_version_pick (font_region);

     font_wborder := vcRegion.Create1 ("font_border", vcBorder.fac,
                                        font_region, nil);
     vcTable.Add (dialog_table, font_wborder);

     use_font_region := vcTable.Create ("use_font_region", 1, 1);

     /* Add the use font pick list */
     vmfAdd_use_font_pick (use_font_region);

     use_wborder := vcRegion.Create1 ("use_border", vcBorder.fac,
                                       use_font_region, nil);
     vcTable.Add (dialog_table, use_wborder);

     vcTable.Add (dialog_table,
                  vcRegion.Create ("button_row", vcButnRow.fac,
                                   list (list ("accept", "Accept",
                                               vmfAccept_CB, nil),
                                         list ("apply", "Apply",
                                               vmfApply_CB, nil),
                                         list ("cancel", "Cancel",
                                               vmfCancel_CB, nil))));

     /* Return the whole region */
     return (dialog_table);   
} /* vmfMake_compile_dialog */

function vmfMakefont_edit (win, event)
{
     /* Open a dialog box only if no other Makefont dialogs are up. */
     if (vmfWindow_ID == nil) {
        vcWin.Busy (vcEvent.Window(event), TRUE);
        vmfTask = "decompile";
        primary_region := vmfMake_decompile_dialog ();
        /* Create the dialog box */
        vmfWindow_ID = vcWin.Popup (vcEvent.Window(event), primary_region, 
                                    primary_region, vcWin.NoFlags);
        vcWin.SetTitle (vmfWindow_ID, vmfDCMPL_title);
        vcWin.Busy (vcEvent.Window(event), FALSE);
        /* Trap all Ctrl-Cs or Window Manager kills. */
        vcRegion.DestroyNotify (primary_region, vmfWindow_destroyed);
     } /* if */
     else
        error (vmfErr_OneDialogOnly, 1);

} /* vmfMakefont_edit */

function vmfMakefont_compile (win, event) 
{
     /* Open a dialog box only if no other Makefont dialogs are up. */
     if (vmfWindow_ID == nil) {
        vcWin.Busy (vcEvent.Window(event), TRUE);
        vmfTask = "compile";
        primary_region := vmfMake_compile_dialog ();
        /* Create the dialog box */
        vmfWindow_ID = vcWin.Popup (vcEvent.Window(event), primary_region, 
                                    primary_region, vcWin.NoFlags);
        vcWin.SetTitle (vmfWindow_ID, vmfCMPL_title);
        vcWin.Busy (vcEvent.Window(event), FALSE);
        /* Trap all Ctrl-Cs or Window Manager kills. */
        vcRegion.DestroyNotify (primary_region, vmfWindow_destroyed);
     } /* if */
     else
        error (vmfErr_OneDialogOnly, 1);

} /* vmfMakefont_compile */

edit_font_cmd :=
                vcWv.Cmd (vcWin.fac, vmfMakefont_edit, "makefont_edit");

compile_font_cmd := 
                vcWv.Cmd (vcWin.fac, vmfMakefont_compile, "makefont_compile");
