/*
 * $Revision: 1.7 $
 *
 * This facility can be used as a parameter editor, prompting for an
 * entity/arch, or a configuration.
 */


/*
 * ei == EAC info
 */
function show_vhdl(ei) {
    if (ei.vsi == nil) {
        vcWv.RequireFac (wv, "vansetup");
        ei.vsi = vcRegion.Create ("vansetup", vansetup.fac, "vsslib.ini",
                                  show_lib_browser <- FALSE,
                                  rows <- 8);
        vansetup.CallOnSelect (ei.vsi, func(pathname, on_off) {
            if (on_off)
                vcRegion.Set (ei.cmdline,
                              vansetup.GetSimulationString (ei.vsi));
        });
    } /* if */
    vcTable.Set (ei.table, 1, 0, ei.vsi);
} /* function show_vhdl */

function show_vsm_files(ei) {
    if (ei.fs == nil) {
        ei.fs = vcRegion.Create ("vsm_files", vcFileSelect.fac, "",
                                 filter <- "*.vsm");
        vcFileSelect.CallOnChange (ei.fs, func(pathname) {
            vcRegion.Set (ei.cmdline, vsRelPath (pathname));
        });
    } /* if */
    vcTable.Set (ei.table, 1, 0, ei.fs);
} /* function show_vsm_files */

function strip_quotes(s) {
    l := string_length (s) - 1;
    if ((l > 0) && (s[0] == '"') && (s[l] == '"'))
        substring (s, 1, l);
    else
        s;
} /* function strip_quotes */

function _Create(eac, region) {
    cmdline := vcRegion.Create ("eac", vcCmdline.fac, strip_quotes (eac));
    picklist := vcRegion.Create ("picklist", vcPicklist.fac,
                                 list (list (cons ("VSM netlists",
                                                   show_vsm_files),
                                             cons ("VHDL design units",
                                                   show_vhdl))));
    table := vcTable.Make ("", 2, 1,
                           vcTable.Make ("", 1, 2, cmdline, picklist));

    vcRegion.AddChild (region, table);

    ei := struct {
        cmdline = cmdline;
        vsi = nil;              /* vansetup region */
        fs = nil;               /* vcFileSelect region */
        table = table           /* table with (1, 0) set to vsi or fs */
    };

    vcPicklist.Notify (picklist, func(show_function, on_off) {
        if (on_off) {
            w := vcRegion.Window (region);
            if (w == nil)
                show_function (ei);
            else
                vcWin.WithBusy (w, func() show_function (ei););
        } /* if */
    });
    vcPicklist.SetPick (picklist,
                        vcRegion.PrefGet (region, "vhdl_first") ? 1 : 0);
    ei;
} /* function _Create */

vlPref.Replace (prefs, "vhdl_first", vlTrueFalseVT,
                "Show VHDL entity/architecture/configuration initially",
                FALSE);

function _Get(ei, region, type) {
    design_unit := vcRegion.Get (ei.cmdline, type);
    if (lindex (design_unit, '(') ||
        lindex (design_unit, ')'))
        "\"" + design_unit + "\"";
    else
        design_unit;
} /* function _Get */

function run_viewvhdl(window, event, design_unit, cmd_file,
                      graphical_interface, vhdl_src_window, vhdl_debug)
{
    cmd := "viewsim " + design_unit + " " + cmd_file;
    
    if (!graphical_interface)   cmd += " -nographics";
    if (!vhdl_src_window)       cmd += " -nosrcwindow";
    if (!vhdl_debug)            cmd += " -nodebug";

    /*
     * KLUDGE KLUDGE KLUDGE
     * If this is being run from the cockpit, they'll be a TxShell
     * region waiting for us, and we should direct the output to
     * there.
     */
    txshell := nil;
    if (isbound (`vframe, nil))
        foreach (w; vcWin.Loop ()) {
            r := vcWin.OwnerRegion (w);
            if (vcRegion.Fac (r) == vframe.fac)
                foreach (r; vcRegion.Children (r, TRUE))
                    if ((vcRegion.Fac (r) == vcTxShell.fac) &&
                        string_eq (vcRegion.Name (r), "viewvhdl"))
                    {
                        txshell = r;
                        vcTxShell.PrintString (txshell,
                                               sprintf ("[`%s']\n", cmd));
                        break;
                    } /* if */
        } /* foreach */

    function print_to_txshell(str)
        vcTxShell.PrintString (txshell, str);
    function exit_handler(mode, code) {
        if (txshell != nil) {
            mode = string_lower (convert (vlStringVT, mode));
            vcTxShell.PrintString (txshell,
                                   sprintf ("[`%s' %s with status %d]\n",
                                            cmd, mode, code));
        } /* if */
    } /* function exit_handler */

    vcWin.System (window, cmd, nil, FALSE,
                  (txshell == nil) ? vcSysOutput.Viewtext :
                  print_to_txshell, nil, exit_handler, nil, nil);
} /* function run_viewvhdl */

viewvhdl_cmd := vcWv.Cmd (vcWin.fac, run_viewvhdl, "viewvhdl", "ViewVHDL",
                          vcWv.Param ("Design Unit", vlStringVT, vcWv.Sticky,
                                      "", vanEAC.fac,
                                      vhdl_first <- TRUE),
                          vcWv.Param ("Command File", vlStringVT,
                                      vcWv.Sticky, "",
                                      vcFilePop.fac, filter <- "*.cmd"),
                          vcWv.Param ("Graphical Interface", vlOnOffVT,
                                      vcWv.Sticky, TRUE),
                          vcWv.Param ("VHDL Source Window", vlOnOffVT,
                                      vcWv.Sticky, TRUE),
                          vcWv.Param ("VHDL Debugging", vlOnOffVT,
                                      vcWv.Sticky, TRUE));


/*
 * This magic is required so that this editor can accept NoDefault
 * as an initial value.  For reasons I don't understand, this requires
 * that CallOnChange-like functionality be provided, using the specific
 * interface below, as mis-documented in the man page for
 * vcDialog.RegisterNoDefault.
 */
vcDialog.RegisterNoDefault (fac, "", func(reg, f, param) {
    ei := vcRegion.GetData (reg);
    vcCmdline.CallOnChange (ei.cmdline, func(str) {
        f (param);
    });
});
