/*
 * This facility edits the ordering of a set of strings.  It takes
 * in either a list or a string to be tokenized into a list, and
 * makes a widget that allows the user to adjust the order, but not
 * the contents of the list.
 */

function table_entry_label_string(t, r, c)
    vcRegion.Name (vcRegion.FindFac (vcTable.Get (t, r, c), vcLabel.fac));
    
dummy := vcRegion.Create ("", vcLabel.fac, "");

function swap_01(region, t1, t2) {
    e1 := vcTable.Get (t1, 0, 1);
    e2 := vcTable.Get (t2, 0, 1);
    a := vgRect.Create (0, 0, 0, 0);
    vgRect.Max (vcRegion.Area (e1), vcRegion.Area (e2), a);
    vcTable.Set (t1, 0, 1, dummy);
    vcTable.Set (t2, 0, 1, dummy);
    vcTable.Set (t2, 0, 1, e1);
    vcTable.Set (t1, 0, 1, e2);
    a := vgRect.Create (0, 0, 0, 0);
    vgRect.Max (vcRegion.Area (e1), a, a);
    vgRect.Max (vcRegion.Area (e2), a, a);
    vcRegion.QueueForRefresh (region);
} /* function swap_rows */

function _RequestSize(so, region, wv);

function make_list_of_strings(list_of_strings) {
    /* If it's a string, tokenize it into a list */
    if (!iscons (list_of_strings)) {
        l := nil;
        foreach (s; string_loop (list_of_strings, " "))
            l = cons (s, l);
        list_of_strings = reverse_destructive (l);
    } /* if */
    list_of_strings;
} /* function make_list_of_strings */

function _Create(list_of_strings, region) {
    list_of_strings = make_list_of_strings (list_of_strings);

    len := length (list_of_strings);
    last_table := nil;
    tables := make_vector (len);

    so := struct {
        len = len;
        tables = tables;
        grabbed = nil;
        call_on_change = nil;
        old_cursor = nil
    };

    i := 0;

    foreach (s; vscript.list_iter (list_of_strings)) {
        position := i;
        table := vcTable.Create ("", 2, 2);
        vcTable.Col (table, -1, vcTable.Border.Invisible, 1, 3, 5,
                     vgAlign.UpperLeft);
        tables[position] = table;

        function swap(event) {
            swap_01 (region, tables[position], tables[position - 1]);
            if (so.call_on_change != nil) {
                str := _Get (so, region, vlStringVT);
                foreach (f; vscript.list_iter (so.call_on_change))
                    f (str);
            } /* if */
        } /* function swap */

        if (last_table != nil) {
            vcTable.Set (last_table, 1, 1, table);

            icon   := vcRegion.Create (s, vcIcon.fac, "swap");
            border := vcRegion.Create (s, vcBorder.fac, icon,
                                       rounded <- FALSE);
            button := vcRegion.Create (s, vcButton.fac, swap);
            vcRegion.AddChild (button, border);
            vcButton.HighlightRegion (button, border);

            vcTable.Set (table, 0, 0, button);
        } /* if */

        vcTable.Set (table, 0, 1, vcRegion.Create (s, vcLabel.fac, s));
        last_table = table;
        ++i;
    } /* foreach */

    vcRegion.AddChild (region, tables[0]);
    so;
} /* function _Create */

function CallOnChange(so_region, f) {
    so := vcRegion.Data (so_region, vscmVT);
    so.call_on_change = cons (f, so.call_on_change);
} /* function CallOnChange */

function _Get(so, region, ts) {
    str := table_entry_label_string (so.tables[0], 0, 1);
    for (i := 1; i < so.len; ++i)
        str += " " + table_entry_label_string (so.tables[i], 0, 1);
    str;
} /* function _Get */

function _Set(so, list_of_strings) {
    i := 0;
    foreach (s; vscript.list_iter (make_list_of_strings (list_of_strings))) {
        if (!string_eq (s, table_entry_label_string (so.tables[i], 0, 1))) {
            done := FALSE;
            for (j := i + 1; (j < so.len) && !done; ++j)
                if (string_eq (s,
                               table_entry_label_string (so.tables[j], 0, 1)))
                {
                    swap_01 (region, so.tables[i], so.tables[j]);
                    done = TRUE;
                } /* if */
            if (!done)
                printf ("vcSetOrder.Set(%s): couldn't find swap for %s\n",
                        list_of_strings, s);
        } /* if */
        ++i;
    } /* foreach */
} /* function _Set */
