/*
 *
 * Viewscript function for creating a dialog box of analog components.
 *
 * RSM - 10/29/91
 * 
 * Copyright (c) 1991 by Viewlogic Systems, Inc.
 */
num_analog_windows := 0;  /* keep track of the number of analog windows */

vsize := vgSize.Create(0,0,0,0,0,1000); /* used to create a vertical rubber */

/* create an analog dialog box */
function create_analog(display,event,group_name)
{
    function vrubber()
    {
        r := vcRubber.Create("",0,0);
        vcRegion.PrefSet(r,list(cons(`size,vsize)));
        return(r);
    }

    dialog_window := nil;
    
    /* this routine is called to add component icons to picklist */
    function add_picklist_entries(comp_picklist,group_name)
    {
        /* need to extract just the name part of the fspec */
        function add_entry(icon,symbol)
        {
            fname := vscript.vsMerge("","analog",icon,"");
            table := vcTable.Create(icon,4,1);
            vcTable.Add(table,vrubber());
            vcTable.Add(table,vcRegion.Create1(icon,vcIcon.fac,fname,nil));
            vcTable.Add(table,vrubber());
            vcTable.Add(table,vcRegion.Create1(symbol,vcLabel.fac,symbol,nil));
            vcPicklist.Add(comp_picklist,table,symbol);
        }
        
        if (string_eq(group_name,"transistors"))
        {
            
            add_entry("npn","npn"); add_entry("pnp","pnp");
            add_entry("nmos3","nmoss"); add_entry("pmos3","pmoss");
            add_entry("nmos4","nmos"); add_entry("pmos4","pmos");
            add_entry("nfet","nfet"); add_entry("pfet","pfet");
            add_entry("gnd","gnd");
        }
        else if (string_eq(group_name,"diodes"))
        {
            add_entry("hdiode","dio1.2"); add_entry("vdiode","dio1.1");
            add_entry("hzdiode","dio2.2"); add_entry("vzdiode","dio2.1");
            add_entry("hsdiode","sdiode.1"); add_entry("vsdiode","sdiode.2");
            add_entry("bridge","bridge"); add_entry("varactor","dio3");
            add_entry("gnd","gnd");
        }
        else if (string_eq(group_name,"r,l,c"))
        {
            add_entry("hres","r.1"); add_entry("vres","r.2");
            add_entry("hcap","c.2"); add_entry("vcap","c.1");
            add_entry("hpcap","cap2.2"); add_entry("vpcap","cap2.1");
            add_entry("hind","ind.1"); add_entry("vind","ind.2");
            add_entry("trfmr","transf"); add_entry("tcore","tcore");
            add_entry("tline","tline"); add_entry("gnd","gnd"); 
        }
        else if (string_eq(group_name,"macros"))
        {
            add_entry("opamp","opamp"); add_entry("switch","vcsw");
            add_entry("crystal","crystal"); add_entry("gparam","param"); 
            add_entry("gnd","gnd");
        }
        else if (string_eq(group_name,"instruments"))
        {
            add_entry("bode","bodeplot"); add_entry("oscope","trans");
            add_entry("dcsweep","sweep"); add_entry("four","fourier");
            add_entry("biaspt","op"); add_entry("temp","temp");
            add_entry("monte","mc"); add_entry("step","step");
            add_entry("noise","noise"); add_entry("sens","sens"); 
            add_entry("xfer","xferfunc"); add_entry("lib","lib");
            add_entry("saveb","sbias"); add_entry("loadb","lbias");
            add_entry("vprobe","probe"); add_entry("iprobe","uammeter");
            add_entry("options","options"); add_entry("csdf","csdf");
        }
        else if (string_eq(group_name,"sources"))
        {
            add_entry("vsin","sin"); add_entry("isin","isin");
            add_entry("vpulse","pulse"); add_entry("ipulse","ipulse");
            add_entry("vpwl","pwl"); add_entry("ipwl","ipwl");
            add_entry("vfm","sffm"); add_entry("ifm","isffm");
            add_entry("vac","ac"); add_entry("iac","iac");
            add_entry("vexp","exp"); add_entry("iexp","iexp");
            add_entry("vcvs","vcvs"); add_entry("vccs","vccs");
            add_entry("ccvs","ccvs"); add_entry("cccs","cccs");
            add_entry("dc","dc"); add_entry("gnd","gnd");
        }
        else return(FALSE);

        return(TRUE);
    }

    /* create buttons for the analog dialog box */
    function create_buttons()
    {
        buttons := vcTable.Create ("Buttons",1,7);
        
        vcTable.Set (buttons,0,1,
                    vcLabel.ButtonCreate ("Accept",
                                                  "Accept",
                                      func () {vcWin.Close (dialog_window);},
                                                    nil));
        vcTable.Set (buttons,0,3,
                     vcLabel.ButtonCreate ("More",
                                           "More...",
                                  func() vcMenuPop.create(analog_menu,event);,
                                           nil));
        vcTable.Set (buttons,0,5,
                    vcLabel.ButtonCreate ("Cancel",
                                           "Cancel",
                                           func ()
                                           {
                                               comp_to_add = nil;
                                               vcWin.Close (dialog_window);
                                           },
                                           nil));
        return(buttons);
    } /* create_buttons */

    function create_scroll_picklist(picklist, min, max)
    {
        declare rows, cols;
        vcPicklist.Dimensions (picklist, &rows, &cols);

        if (rows > max)
            rows = max;

        if (rows < min)
            rows = min;

        if (rows < max)
            max = rows;

        vcPicklist.Scroll ("", picklist, min, rows, max);
    }

    /* set the appropriate values when a component is selected */
    function set_comp(val, on_off)
    {
        if (on_off == vlOnOff.On) {

            if (loadblock(val, vqDataType.Symbol) == 0) {

                comp_to_add = val;
                analog_comp_name = val;
            }
            else {

                status_msg (vcEvent.Window(event),"Symbol not found - " + val);
                comp_to_add = nil;
                analog_comp_name = nil;
            }
        }
    }

    /* this is called when an analog  window is destroy */
    function win_destroyed(w)
    {
        /* decrement the count and if 0 set the analog_comp_name to nil */
        num_analog_windows = num_analog_windows - 1;
        if (num_analog_windows == 0)
            analog_comp_name = nil;
    }

    /* only create an analog dialog box if there is an analog directory */
    vcWin.Busy(getwindow(display),TRUE);
    
    /* create the main symbols dialog box table */
    main_dialog := vcTable.Create (group_name, 2, 1);
    vcTable.Row(main_dialog,-1,vcTable.Border.Invisible,
                5,5,5,vgAlign.MiddleCenter);
    vcTable.Col(main_dialog,-1,vcTable.Border.Invisible,
                5,5,5,vgAlign.MiddleCenter);
    
    /* create a picklist for the icons */
    comp_picklist := vcRegion.Create1("",vcPicklist.fac,nil,
                                     list(cons(`exclusive,TRUE),
                                          cons(`rows,5),
                                          cons(`columns,2)));
    
    if (add_picklist_entries(comp_picklist,group_name) == TRUE)
    {
        vcPicklist.Notify (comp_picklist, set_comp);
        
        /* add entries to the main dialog box */

        vcTable.Add(main_dialog,create_scroll_picklist(comp_picklist,5,5));
/* RSM_FIX_LATER - wait till Josh fixes scroll problem 
        vcTable.Add(main_dialog,comp_picklist);
*/
        vcTable.Add(main_dialog,create_buttons());
        
        /* create a popup window */
        dialog_window = vcWin.Popup (getwindow(display),main_dialog, 
                                     main_dialog,vcWin.NoStatus+vcWin.NoCmdline);
        vcWin.SetTitle(dialog_window,"Analog - "+group_name);
        vcWin.DestroyNotify(dialog_window,win_destroyed);
        
        /* increment the window count to keep track of how many analog 
           windows are opened */
        num_analog_windows = num_analog_windows + 1;
    }
    else
    {
        vcRegion.Destroy(main_dialog);
        vcRegion.Destroy(comp_picklist);
        status_msg(vcEvent.Window(event),
                   "No analog class '"+group_name+"' found");
    }
    
    vcWin.Busy(getwindow(display),FALSE);
} /* create_analog */




