/*
 * $RCSfile: vdbAnno.vs,v $ $Revision: 1.9 $
 *
 * Functional Description:
 *     This file should be loaded by ViewDraw.  It provides the
 *     back-annotation functions required by ViewDatabook II.
 *
 * Copyright (c) 1994 by Viewlogic Systems, Inc.
 */

/* hash table mapping UID to attributes we want to annotate */
declare FinderUidToAttrs;

/* hash table mapping attr names to attr objs -- used when replacing attrs */
declare FinderAttrNameToObj;


/* why isn't this in vscript? */
/*
function caddddr(l)
    car(cddddr(l));
*/

/*
 * DESCRIPTION
 *      Called before any annotation work starts.  Allocates hash tables
 *      that are used during annotation process.
 */
function FinderSetup()
{
    FinderUidToAttrs = vlSym.StrCreate (101); /* size is a guess */
    vlSym.Dynamic (FinderUidToAttrs, 2.0, 2.0);

    FinderAttrNameToObj = vlSym.StrCreate (11); /* size is a guess */
    vlSym.Dynamic (FinderAttrNameToObj, 2.0, 2.0);

}


/*
 * DESCRIPTION
 *      Called after annotation work is done.  Don't actually free the hash
 *      tables, but rely on standard garbage collection mechanism.
 */
function FinderCleanup ()
{
    FinderUidToAttrs = nil;
    FinderAttrNameToObj = nil;

}


function FinderStoreAttrs(uid_list, attr_list)
{
    /* sort attrs by name */
    attr_list = vscript.Sort (attr_list,
                              func(a,b) string_le (car (a), car (b)););

    foreach (uid; vscript.list_iter (uid_list))
        vlSym.Store (FinderUidToAttrs, uid, attr_list);

}


function FinderDoAnnotate(display, event)
{
    declare uid, attr_value_list;

    block := vdraw.DisplayGetBlock (display);

    foreach (comp;
             vdraw.BlockIterObj (block, vqFlag.Component, vqAllOrSel.All))
    {
        uid = vdraw.ObjGetUID (block, comp);
        /* see if we want to annotate this component */
        if ((attr_value_list = vlSym.Fetch (FinderUidToAttrs, uid)) != nil)
            /* do the annotation */
            FinderCompAddAttrs (display, event, block, comp, attr_value_list);
    }

    /* clean up any selections left around */
    vdraw.BlockDeselectAll (block);

} /* FinderDoAnnotate */


function vdbBackAnnotate(data)
{
    /*win := vdraw.getcurrentwindow();*/
    win := vdbCurrentWindow;
    vdraw_rgn := vcWin.OwnerRegion (win);
    display := vcRegion.GetData (vdraw_rgn);

    FinderSetup();
    eval_string (data, current_package ());
    FinderDoAnnotate (display, vdbDummyEvent (win, vdraw_rgn));
    vdraw.DisplayRefresh (display);
    FinderCleanup ();

} /* vdbBackAnnotate */


/* Provides compatible API to vdraw.ObjAddAttr to make my life easier.
 */
function FinderObjAddOat(block, comp, name_value, x, y, visibility)
    vdraw.ObjAddOat (block, comp, name_value);


function FinderCompAddAttrs(display, event, block, comp, attr_value_list)
{
    /* Keep track of attrs added.  Used when placing component. */
    declare added_attrs_list;

    symblock := vdraw.CompGetBlock(block,comp);
    DEBUG ("AddAttr: comp=%s, symblock=%s\n", comp, symblock);

    /* start annotating at lower left of bounding box */
    bbox_loc := vdraw.ObjGetBboxLoc(symblock,symblock,vdraw.vqBoxPt.LowerLeft);
    DEBUG ("\tsymblock bbox lower left %s\n", bbox_loc);
    sym_x := vdraw.LocGetX (symblock, bbox_loc);
    sym_y := vdraw.LocGetY (symblock, bbox_loc);
    loc := vdraw.ObjGetLoc (block, comp);
    DEBUG ("\tcomp loc %s\n", loc);
    comp_bbox_loc := vdraw.ObjGetBboxLoc(block,comp,vdraw.vqBoxPt.LowerLeft);
    DEBUG ("\tcomp bbox lower left %s\n", comp_bbox_loc);

    comp_x := vdraw.LocGetX (block, loc);
    comp_y := vdraw.LocGetY (block, loc);
    x := sym_x + comp_x;
    y := sym_y + comp_y;
    /*y := vdraw.LocGetY (block, comp_bbox_loc);*/

    /* Determine text size, default is Viewdraw's value setting, but if
       attribute placehoilders exist on symbol block for this component
       then get the size of the first one and assume that they are all 
       the same.  Use this value for the text_size.
       */
    old_text_size := vdraw.ParamGetValue(block,vqValue.TSize);
    declare text_size;

    /*
     * remember existing attrs, their text size + location
     */
    vlSym.Clear (FinderAttrNameToObj);

    function Remember(block, comp, attr, y_offset)
    {
        vlSym.Store (FinderAttrNameToObj,
                     vdraw.AttrGetName (block, attr), attr);
        if (vdraw.ObjGetVis (block, attr) != vqAttrVisibility.Invisible)
        {
            if (null (text_size))
                text_size = vdraw.ObjGetSize(block,attr);
            lly := y_offset + vdraw.LocGetY(block,vdraw.ObjGetLoc(block,attr));
            if (lly < y)
                y = lly;
            DEBUG ("\t%s: found non-invisible attr %s at %s\n", block,
                   vdraw.AttrGetName (block, attr),
                   vdraw.ObjGetLoc (block, attr));
        }
    }

    foreach (attr; vdraw.ObjIterAttr (symblock, symblock))
        Remember(symblock, comp, attr, comp_y);

    foreach (attr; vdraw.ObjIterAttr (block, comp))
        Remember(block, comp, attr, 0);

    if (vdbCfg.bOatsGetLoaded)
        foreach (attr; vdraw.ObjIterOat (block, comp))
            Remember(block, comp, attr, 0);

    /* snap text to next grid below y */
    DEBUG ("\ty is now %d\n", y);
    if (null (text_size))
        text_size = old_text_size;
    grid_size := vdraw.ParamGetValue(block, vqValue.Grid);
    y -= y % grid_size;
    DEBUG ("\ty is now %d, after snap to grid_size %d\n", y, grid_size);
    /* set vdraw's text size to our favorite text size...will reset later */
    vdraw.ParamSetValue(block, vqValue.TSize, text_size);

    /* loop over incoming attrs */
    foreach (attr_values; vscript.list_iter (attr_value_list))
    {
        declare this_x, this_y;
        declare attr;

        <name, value, visibility, oat> := attr_values;
        DEBUG ("\tprocessing %-15s %-15s %-10s %s\n",
               name, value, visibility, oat);

        visibility = convert(vqAttrVisibilityVT, visibility);
        name_value := name + "=" + value;
        add_attr := (oat) ? FinderObjAddOat : vdraw.ObjAddAttr;

        if (null (attr = vlSym.Fetch (FinderAttrNameToObj, name))) {
            DEBUG ("\tdidn't find %s, adding...\n",name);
            this_x = x;
            if (visibility != vqAttrVisibility.Invisible)
                y -= text_size;
            this_y = y;
            attr = add_attr (block, comp, name_value, x, y, visibility);
        }
        else {
            loc := vdraw.ObjGetLoc (block, attr);
            DEBUG ("\tfound attr at loc %s\n", loc);
            this_x = vdraw.LocGetX (block, loc);
            this_y = vdraw.LocGetY (block, loc);

            owner := vdraw.ObjGetOwnerObj (block, attr);
            if (owner == comp)
            {
                DEBUG ("\towned by component\n");
                /* arg...vdraw 531 bug requires that we check whether text
                 * string is actually different before setting it
                 */
                if (!string_eqi(vdraw.ObjGetTextString(block,attr),name_value))
                    vdraw.ObjSetTextString (block, attr, name_value);
                if (!vdbCfg.bMaintainAttrVis
                    && visibility != vdraw.ObjGetVis (block, attr))
                    vdraw.ObjSetVis (block, attr, visibility);
            }
            else
            {
                /* this means an attr placeholder exists at the symbol */
                /* level.  Its location is relative to the symbol. */
                /* Add attr at old attr location, with appropriate */
                /* visibility. */
                if (vdbCfg.bMaintainAttrVis)
                    visibility = vdraw.ObjGetVis (block, attr);
                attr = add_attr (block, comp, name_value,
                                 comp_x + vdraw.LocGetX (block, loc),
                                 comp_y + vdraw.LocGetY (block, loc),
                                 visibility);
            }
        }

        added_attrs_list = cons(attr, added_attrs_list);

    } /* foreach attr to annotate */

    /* reset default text size */
    vdraw.ParamSetValue(block, vqValue.TSize, old_text_size);

    /* Return list of added attrs.  This is for use when placing comp. */
    added_attrs_list;

} /* FinderCompAddAttrs */
