/*
 * $RCSfile: vdbde.vs,v $ $Revision: 1.11 $
 *
 * DESCRIPTION
 *     Definitions shared by ViewDatabook II and ViewDraw.
 *     Should be loaded by both.
 *
 * Copyright (c) 1995 by Viewlogic Systems, Inc.
 */

/*
 * ITC stuff.
 */
vitcTOOLID_VDB := "vitcTOOLID_vdb";
sVdrawNameField := "vdb"; /* used as ITC "generic name" field in vdraw cntx */

/* Message types. */
sMsgADD_COMP       := "ADD_COMP_WITH_ATTRS"; /* vdb -> vdraw */
sMsgTRACK_COMPS    := "TRACK_COMPS"; /* vdb -> vdraw */
sMsgBACK_ANNO      := "BACK_ANNOTATE"; /* vdb -> vdraw */
sMsgBACK_ANNO_DONE := "BACK_ANNOTATE_DONE"; /* vdraw -> vdb */
sMsgREQ_SEL_COMP   := "SELECTED_COMP_DATA_REQUEST"; /* vdb -> vdraw */
sMsgSEL_COMP       := "SELECTED_COMP_DATA"; /* vdraw -> vdb */
sMsgGET_READY      := "GET_READY"; /* vdraw -> vdb */
sMsgREADY          := "READY";    /* vdb -> vdraw */

/* std msg types */
sMsgCMD            := vitc.MsgCMD;
sMsgPING           := vitc.MsgPING;
sMsgTOOL_STARTED   := vitc.MsgTOOLSTARTED;
sMsgTOOL_STOPPED   := vitc.MsgTOOLSTOPPED;

/* command names */
sAddCompCmdName := "vdb_addcomp";

/* attr names */
sVllmLibNameAttr := "VLLM_LIB";

/*
 * DESCRIPTION
 *      Return a unique filename.
 *
 */
function vdbTempFilename()
/*
 * Abandoned vscript.vsUnique.  It takes only a directory name, and does a
 * whole lot of extra work.  I want a hidden (dot) file.
 *
 * Note that there is a race condition between when I return the filename,
 * and when the file gets created.  This is also true with vscript.vsUnique...
 *
 * Dag.  Can't use vscript.vsExists with vdraw 5.2.
 */
{
    declare fname;
    if (vcWv.OS (wv) == vcOs.Unix)
    {
        fname = ".vdb_temp";
        fname = vscript.vsAbsPath (fname);
    }
    else
    {
        sTmpdir = vscript.getenv ("TEMP");
        if (null (sTmpdir))
            sTmpdir = "c:\\";
        fname = vscript.vsUnique (sTmpdir);
    }

    /* return value */
    fname;
}

/*
{
    declare sTmpdir;

    if (vcWv.OS (wv) == vcOs.Unix)
    {
        sTmpdir = vscript.getenv ("TMPDIR");
        if (null (sTmpdir))
            sTmpdir = "/tmp";
    }
    else
    {
        sTmpdir = vscript.getenv ("TEMP");
        if (null (sTmpdir))
            sTmpdir = "c:\\";
    }
    vscript.vsUnique (sTmpdir);
}
*/

/* --------------------------------------------------------------
 * vdbPrintStatus
 *   Print the specified string on the status line of the 
 *   specified window.  If the string is nil, the status line
 *   will be cleared.
 */
function vdbPrintStatus(window, str)
{
    if (window != nil) {
        vcWin.ClearStatus(window);

        if (str != nil)
            vcWin.AddStatus(window, str);
    }
} /* vdbPrintStatus */


/* --------------------------------------------------------------
 * vdbMakeButtonRow
 *   Create a button row.  The btns argument should be a list
 *   of 'cons("Button Label", function_to_call_when_pressed)'.
 *   This routine creates command objects for each of the buttons 
 *   so that they will be macro recordable.  If this button row
 *   is used in a popup window, the button functions must close
 *   the popup.
 */
function vdbMakeButtonRow(btns)
{

    btnList := nil;

    for (b := btns; b != nil; b = cdr(b)) {

        label := car(car(b));
        fnc := cdr(car(b));

        cmd := vcWv.Cmd (vcWin.fac, fnc, label);

        btnList = cons(list (label, label, cmd, nil), btnList);
    }
    btnList = reverse_destructive(btnList);

    vcRegion.Create ("vdbMakeButtonRow", vcButnRow.fac, btnList);
} /* vdbMakeButtonRow */


/* --------------------------------------------------------------
 * vdbCloseAlertBox
 *   Close the window associated with the event.
 */
function vdbCloseAlertBox(data, event)
{
    if (event != nil)
        vcWin.Close(vcEvent.Window(event));

} /* vdbCloseAlertBox */


/* --------------------------------------------------------------
 * vdbAlert
 *   The alert routine is used to graphically notify the user of 
 *   a wide range of status, warning, or error messages.
 *   
 *   If stream is non-nil, message will be appended to the 
 *   previous messages for this stream.  Multiple calls to this
 *   routine will grow the alert box
 *   
 *   If window is non-nil, the alert box will be popped-up over it.
 *   Otherwise a new window is created.
 *   
 *   The message may be one or more lines of text.
 *   
 *   Severity may be one of the following.  If Status, no icon will
 *   be displayed.  (Just the message text.)
 *   
 *       vlErrSev.Error         - icon is the Bang symbol
 *       vlErrSev.Status        - no icon 
 *       vlErrSev.Note          - icon is the Info symbol 
 *       vlErrSev.Warning       - icon is the Bang symbol
 *       vlErrSev.Fatal         - icon is the Bang symbol
 *       vlErrSev.Internal      - icon is the Bang symbol
 *       vlErrSev.Disabled
 *   
 *   If buttons is nil, a single Dismiss button will be displayed
 *   at the bottom of the window.  If buttons is non-nil, the 
 *   action routines for the buttons are responsible for dismissing
 *   the alert box.
 *   
 *   The alert box has the following layout.
 *   
 *       --------------------------------------------------
 *      |                                                  |
 *      |     icon                message                  |
 *      |                                                  |
 *      |    severity                                      |
 *      |                                                  |
 *      |                    button row                    |
 *       --------------------------------------------------
 *   
 */
function vdbAlert(stream, window, message, severity, buttons)
{

    if ((stream != nil) && vcWin.IsAlive(stream.msg_window)) {

        /* Add this message to the previous ones and redisplay them all */

        stream.message = stream.message + message;

        msg := vcRegion.Create("", vcStaticText.fac, stream.message);

        vcTable.Set(stream.display, 0, 2, msg);

        vcWin.Resize(stream.msg_window, TRUE);
        vcWin.Show(stream.msg_window);
    }

    else {

        /* Create a new alert box */

        function stretch() vcRubber.Create("", 0, 0);

        if (severity == vlErrSev.Status)
            icon := stretch();          /* No icon will be displayed */
        else if (severity == vlErrSev.Note)
            icon := vcRegion.Create("info", vcIcon.fac, "info");
        else
            icon := vcRegion.Create("bang", vcIcon.fac, "bang");

        if (severity == vlErrSev.Status)
            sev_text := stretch();      /* No severity text string displayed */

        else {
            sev_text := vcRegion.Create("severity", vcLabel.fac,
                                   convert(vlStringVT, severity));

            vcRegion.Font(sev_text, vgFont.mediumb);
        }

        alert := vcTable.Make("left", 3, 1, icon, sev_text, stretch());

        vcTable.Col(alert, 0, vcTable.Border.Invisible, 0, 0, 0, 
                    vgAlign.MiddleCenter);

        msg := vcRegion.Create("", vcStaticText.fac, message);

        filler := vcSteel.Create("", 10, 0);

        display := vcTable.Make("", 1, 3, alert, filler, msg);

        border := vcRegion.Create("", vcBorder.fac, display,
                                  rounded <- FALSE,
                                  top_space <- 10,
                                  bottom_space <- 10,
                                  left_space <- 10,
                                  right_space <- 10);


        if (buttons == nil) {
            /* Create a dismiss button */

            dismissCmd := vcWv.Cmd(vcWin.fac,
                                   func(d, e) 
                                   vcWin.Close(vcEvent.Window(e));,
                                   "whack the window");

            buttons := vcRegion.Create("dismiss", vcButnRow.fac,
                                       list (list ("dismiss", "Dismiss",
                                                   dismissCmd, nil)));
        }

        dbox := vcTable.Make("ViewDatabook Alert", 2, 1, border, buttons);


        win_flags := vcWin.NoCmdline + vcWin.NoStatus + vcWin.NoBindings;

        if (window != nil)
            msg_window := vcWin.Popup(window, dbox, buttons, win_flags);
        else
            msg_window := vcWin.Create(dbox, win_flags);


        stream = struct {
            display = display;
            msg_window = msg_window;
            message = message
        };
    }

    /* Return the stream struct so it can be passed in with the next call */
    stream;

} /* vdbAlert */


/*\vscmuserDocpage GenericCntxCreate -- utility function for creating contexts
 *
 * SYNOPSIS
 *      vitcCntxVT GenericCntxCreate(args...)
 *
 * DESCRIPTION
 *      Returns an ITC context which is completely wildcarded,
 *      except for
 *      1.  customizations specified in the parameter list,
 *      2.  Framework ID, which is FidDFLT
 *      3.  user ID, which is UidDFLT
 *
 *      The customizations are specified using the `field <- value'
 *      syntax.  Each such customization is specified as a separate
 *      parameter to GenericCntxCreate.  See the examples.
 *
 * VALID FIELD NAMES
 *      Field Name  Default           Meaning
 *      ----------  -------           -----------------
 *      tid         vitc.TidANY       any tool ID ("*")
 *      name        vitc.NameANY      any name
 *      fid         vipc.FidDFLT      default framework
 *      hnm         vipc.HnmANY       any hostname
 *      uid         vipc.UidDFLT      default user ID
 *      gid         vipc.GidANY       any group ID
 *      pid         vipc.PidANY       any process ID
 *      cid         vipc.CidANY       any channel ID
 *
 * EXAMPLE 1, A GENERIC MATCH CONTEXT
 *      One needs a "match context" to use during ITC registration.
 *      This context specifies the tools from which we are willing
 *      to receive messages.  One would generally want the limit to
 *      be "any tool run by me within this framework".  In this case, one
 *      would use the generic match context with no customizations:
 *
 *          match_cntx := GenericCntxCreate ();
 *
 * EXAMPLE 2, A SIMPLE DESTINATION CONTEXT
 *      When creating a "destination context" for use with
 *      vitc.Send, one generally wants to wildcard everything except
 *      the tool ID and perhaps the name field.  To send a message
 *      to any tool called "MyTool", one would use:
 *
 *          dest_cntx := GenericCntxCreate (tid <- "MyTool");
 *
 * EXAMPLE 3, A COMPLEX DESTINATION CONTEXT
 *      Perhaps the tool "MyTool" takes full advantage of the name
 *      field, and creates one ITC tool for each design component on
 *      which it is operating.  This would mean that if "MyTool"
 *      were operating on design "amp" and design "pre-amp"
 *      simultaneously, it would have created two ITC tools.  These
 *      two tools would have the same tool ID and different name
 *      fields.  To distinguish between them, one might use:
 *
 *          dest_cntx := GenericCntxCreate (tid <- "MyTool",
 *                                          name <- "amp");
 */
function GenericCntxCreate(args...)
{
    tid  := vitc.TidANY;  /* must be vitc not vipc */
    name := vitc.NameANY; /* or nil; must also be vitc not vipc  */
    fid  := vipc.FidDFLT;
    hnm  := vipc.HnmANY;
    uid  := vipc.UidDFLT;
    gid  := vipc.GidANY;
    pid  := vipc.PidANY;
    cid  := vipc.CidANY;

    foreach (arg; vscript.list_iter (args))
    {
        a := car (arg);
        v := cdr (arg);
        if      (a == `gid)     gid = v;
        else if (a == `pid)     pid = v;
        else if (a == `cid)     cid = v;
        else if (a == `uid)     uid = v;
        else if (a == `fid)     fid = v;
        else if (a == `hnm)     hnm = v;
        else if (a == `tid)     tid = v;
        else if (a == `name)    name = v;
	else error("Invalid field specifier", a);
    }
    vitc.CntxCreate (gid, uid, fid, tid, name, hnm, pid, cid);

} /* GenericCntxCreate */


/*\BeginDebugVS*/
function print_list(str, l)
{
    printf ("%s = (\n", str);
    foreach (elem; vscript.list_iter (l))
        printf ("\t%s\n", elem);
    printf (")\n");
}
/*\EndDebugVS*/


/*\BeginDebugVS
function with_timer(msg, fn)
{
    t1 := vscript.vsTimer (vsTimerUnits.Hundredths);
    ret := fn ();
    t2 := vscript.vsTimer (vsTimerUnits.Hundredths);
    printf ("    (%s, elapsed time: %.2f)\n", msg, (t2 - t1)/100.0);
    ret;

}
\EndDebugVS*/
