/*
 * $Id: vdcomp.vs,v 1.15 92/04/30 09:40:30 surma Stab Locker: surma $
 *
 * Add Componenet Dialog Box.
 *
 * (c) Copyright 1991, 1992 by Viewlogic Systems, Inc.
 */

/* These variables need to be global */

addcomp_num_cols := 4;  /* can be overriden in vdrawus.vs */

loaded_symbols := "*Symbols in memory*";

addcomp_cmdline  := nil;
addcomp_dialog   := nil;  
directory        := nil;
library          := "";
filter           := "*";
comp_to_add      := nil;
w                := nil;
analog_comp_name := nil; /* RSM - added this variable */
complist_rrow    := 1;

/* Some handy shortcuts for specify different types of regions */
function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);
function rubber(x,y) vcRubber.Create("",x,y);
function cmdline(default) vcRegion.Create1("",vcCmdline.fac,default,
                                           list (cons (`width, 30)));

/* function picklist(l) vcRegion.Create1("", vcPicklist.fac, l, nil); */

function build_pathpicklist2(directory)
{
    /* This function differs from build_pathpicklist() in that it includes
     * an entry for the symbols in memory.
     */

    declare count, pick;

    picklist := vcRegion.Create1 ("", vcPicklist.fac, nil,
                                  list (cons (`exclusive, TRUE),
                                        cons (`rows, 5),
                                        cons (`columns, 1)));
    
    vcPicklist.Add (picklist, loaded_symbols, "*memory*");

    count = 1; pick = 0;

    foreach (str; libraries ()) {

        vcPicklist.Add (picklist, str, str);
        if (string_eq (str, directory))
            pick = count;

        ++count;
    }

    vcPicklist.SetPick (picklist, pick);

    return (picklist);

} /* function build_pathpicklist2 */


function clear_dialog()
{
    vcWin.ClearStatus (w);

    vcCmdline.Set (addcomp_cmdline, "");
    comp_to_add = nil;
    filter = "*";

} /* function clear_dialog */


function create_buttons()
{
    buttons := vcTable.Create ("Buttons", 1, 2);

    vcTable.Add (buttons,
                 vcLabel.ButtonCreate ("Clear", "Clear", 
                                       clear_dialog, nil));

    vcTable.Add (buttons,
                 vcLabel.ButtonCreate ("Cancel", "Cancel",
                                       func ()
                                       {
                                           clear_dialog ();
                                           vcWin.Close (w);
                                       },
                                       nil));
    return (buttons);

} /* function create_buttons */


/* This function creates a component object */
function create_comp(display, event, str)
{
    status := 1;

    if (oktorun (display, "comp", TRUE) != 0)
        return (FALSE);

    /* check to see if the string passed in is empty */
    if (string_eq (str, "")) {

        /* If it is and the dialog box is closed then open it and bind 
         * the middle button. Otherwise create a component with the string
         * in the command line buffer 
         */

        if (analog_comp_name == nil) {   /* RSM - add this if here */

            if (addcomp_dialog == nil)
                display_addcomp_dialog (display, event, "*");

            else if (vcWin.IsHidden (w) == TRUE) {

                vcWin.Show (w);
                status = 1;
            }
            else {

                str = vcRegion.Get (addcomp_cmdline, vlStringVT);
                
                if (!string_eq (str, ""))
                    status = comp (display, event, str);
                else
                    status = 1;
            }
        }
        else 
            status = comp (display, event, analog_comp_name); /* RSM */
    }
    else if (   (lindex (str, '*') == FALSE)
             && (lindex (str, '?') == FALSE))
        /* if string isn't empty then create a component with it */
        status = comp (display, event, str);
    else
        display_addcomp_dialog (display, event, str);

    /* if component successfully created then move it */
    if (status == 0)
        move_created_object (display, event);

} /* function create_comp */

comp_cmd := vcWv.Cmd (fac, FILE_CHECK1 (create_comp), "comp", "Add Component",
                      vcWv.Param ("Symbol Name",
                                  vlStringRestOfLineVT, vcWv.Constant, ""));

do_comp_cmd := vcWv.Cmd (fac, FILE_CHECK0 (func(d, e)
                             create_comp (d, e, "");),
                         "do_comp");

vcCmd.EnableMacro (do_comp_cmd, "off");


function add_comp(display, event)
{
    declare name;

    if ((comp_to_add != nil) && (!string_eq (comp_to_add, ""))) {

        if (analog_comp_name != nil)   /* Use no library name here. */
            name = analog_comp_name;

        else if (!string_eq (library, ""))
            name = library + ":" + comp_to_add;

        else
            name = comp_to_add;

        vcCmd.Run (comp_cmd, display, event, list (name), TRUE);
    }

} /* function add_comp */


function load_symbol(name, data_type)
{
    if (!string_eq (library, ""))
        name = library + ":" + name;

    return (loadblock (name, data_type));

} /* function load_symbol */


/* This is the main routine the displays the dialog box of symbol names */
function display_addcomp_dialog(display, event, filter)
{
    declare header;

    /* only change directory if one specified */
    colon := rindex (filter, ':');

    if (colon != FALSE) {
        directory = substring (filter, 0, colon);
        directory = "(" + directory + ")";
        filter = substring (filter, (colon + 1), (string_length (filter)));
    }
    else {
        slash := rindex (filter, '/');
        if (slash == FALSE)
            slash = rindex (filter, '\\');
        
        if (slash != FALSE) {
            directory = substring (filter, 0, slash);
            filter = substring (filter, (slash + 1), (string_length (filter)));
        }
    }

    function create_scroll_picklist(picklist, min, max)
    {
        declare rows, cols;
        vcPicklist.Dimensions (picklist, &rows, &cols);

        if (rows > max)
            rows = max;

        if (rows < min)
            rows = min;

        if (rows < max)
            max = rows;

        vcPicklist.Scroll ("", picklist, min, rows, max);
    }

    /* set the command line value whenever the user picks a symbol */
    function set_line(val, on_off)
    {
        if (on_off == vlOnOff.On) {

            vcCmdline.Set (addcomp_cmdline, val);
/*            comp_to_add = val;   /* cvtalias (display, event, 0, val); */
            comp_to_add = convertalias (val); 
            analog_comp_name = nil; /* RSM */

            vcWin.ClearStatus (w);

            /* Try to load symbol. */

            if (load_symbol (comp_to_add, vqDataType.Symbol) == 0)
                vcWin.AddStatus (w,
                                 "Move cursor into schematic to place "
                                 + convertalias (comp_to_add));
            else
                vcWin.AddStatus (w, convertalias (comp_to_add) + " not found");

            filter = "*";
        }
    }

    function create_files_picklist(val, on_off, filter)
    {
        scroll_file_picklist := nil;
        declare temp_col;

        if (on_off == vlOnOff.On) {

            if ((string_length (val) > 0) && (string_ref (val, 0) == '('))
                library = substring (val, 1, string_length (val) - 1);
            else
                library = "";

            directory = val;

            if (!string_eq (library, ""))
                set_lastlib (library);
            else
                set_lastlib (directory);

            file_picklist := nil;
            area := nil;

            if (w != nil) {

                vcWin.ClearStatus (w);
                vcWin.AddStatus (w, "Building symbol list...");

                vcWin.Busy (w, TRUE);
            }

            if ((scroll_file_picklist =
                 vcTable.Get (addcomp_dialog, complist_rrow, 0)) != nil) {

                area = vcRegion.Area (scroll_file_picklist);
                vcRegion.Destroy (scroll_file_picklist);
            }

            /* create picklist of files */

            if (string_eq (val, "*memory*")) {
                file_picklist =
                    build_ldfilelist (vqDataType.Symbol, filter, "");

                library = "";
            }
            else {
               temp_col = fopen_num_cols;
               fopen_num_cols = addcomp_num_cols;
                file_picklist =
                    build_filepicklist (vqDataType.Symbol, val, filter, "");
               fopen_num_cols = temp_col;
            }

            vcPicklist.Notify (file_picklist, set_line);
            
            /* calculate the correct min, max and ideal rows */

            scroll_file_picklist =
                create_scroll_picklist (file_picklist, 5, 0);

            if (area != nil) {

                vcTable.Set (addcomp_dialog,
                             complist_rrow, 0, scroll_file_picklist);

                /* vcRegion.Reframe (scroll_file_picklist, area); */
                /* vcRegion.Refresh (scroll_file_picklist, area); */
            }

            if (w != nil) {

                vcWin.Busy (w, FALSE);
                vcWin.ClearStatus (w);
            }
        }
        return (scroll_file_picklist);
    }

    function clear_and_create_files_picklist(val, on_off)
    {
        /* Clear command line, null comp_to_add, and reset filter */

        clear_dialog ();
        return (create_files_picklist (val, on_off, "*"));
    }

    vcWin.Busy (vcEvent.Window (event), TRUE);

    /* create the main symbols dialog box table */
    addcomp_dialog = vcTable.Create ("Add Component", 5, 1);

    /* create table for symbol name command line  */
    text_entry := vcTable.Create ("", 1, 2);

    vcTable.Add (text_entry, label ("Enter name: "));

    addcomp_cmdline = cmdline ("");

    function onReturn(e, str)
    {
        if (   (lindex (str, '*') == FALSE)
            && (lindex (str, '?') == FALSE)) {

            analog_comp_name = nil; /* RSM */
/*            comp_to_add = str;   /* cvtalias (display, event, 0, str); */
            comp_to_add = convertalias (str);
            
            vcWin.ClearStatus (w);

            /* Preload symbol */
            if (load_symbol (comp_to_add, vqDataType.Symbol) == 0)
                vcWin.AddStatus (w,
                                 "Move cursor into schematic to place "
                                 + convertalias (comp_to_add));
            else
                vcWin.AddStatus (w, convertalias (comp_to_add) + " not found");

            filter = "*";
        }
        else {

            comp_to_add = nil;
            filter = str;
            create_files_picklist (directory, vlOnOff.On, filter);
        }

    } /* function onReturn */
        
    vcCmdline.CallOnReturn (addcomp_cmdline, onReturn);

    vcTable.Add (text_entry, addcomp_cmdline);
    
    /* create picklist of loading symbols */
    /* scroll_file_picklist := create_files_picklist ("loaded", vlOnOff.On, filter); */

    if (directory == nil) {
        scroll_file_picklist := create_files_picklist ("*memory*", vlOnOff.On, filter);
    }
    else {
        scroll_file_picklist := create_files_picklist (directory, vlOnOff.On, filter);
    }

    /* create picklist of paths */

    path_picklist := build_pathpicklist2 (directory);

    vcPicklist.Notify (path_picklist, clear_and_create_files_picklist);

    scroll_path_picklist := create_scroll_picklist (path_picklist, 4, 8);

    /* add regions to main dialog box region */

    header = label ("Select component to add");
    vcTable.Add (addcomp_dialog, header);

    vcTable.Add (addcomp_dialog, scroll_file_picklist);
    vcTable.Add (addcomp_dialog, text_entry);

    header = label ("Directories and (Libraries)");
    vcTable.Add (addcomp_dialog, header);

    vcTable.Add (addcomp_dialog, scroll_path_picklist);

    buttons := create_buttons ();

    vcTable.Add (addcomp_dialog, buttons);

    if (vcWv.OS (wv) == vcOs.Unix)
        w = vcWin.Create (addcomp_dialog,
                          (vcWin.NoCmdline
                          + vcWin.NoMenus + vcWin.NoBindings));
    else
        w = vcWin.Popup (getwindow (display),
                         addcomp_dialog,
                         addcomp_dialog,
                         (vcWin.NoCmdline
                         + vcWin.NoMenus + vcWin.NoBindings));

    /* if main window closed and dialogs still opened, close it */
    vcWin.DestroyNotify (getwindow (display),
                         func(a)
                         {
                             if (addcomp_dialog != nil) {

                                 vcWin.Close (w);
                                 w = nil;
                             }
                         });
    
    /* set the addcomp_dialog box to nil if window closed */

    vcWin.DestroyNotify (w, func(a) {addcomp_dialog = nil; w = nil;});

    vcWin.Busy (vcEvent.Window (event), FALSE);
} /* function display_addcomp_dialog */


/* RSM - 11/27/91 */
function load_analog(display, event, class)
{
    if (!isbound (`create_analog, vdraw))
        load ("vdanalog.vs", vdraw);

    create_analog (display, event, class);
}
analog_cmd := vcWv.Cmd(fac, FILE_CHECK1 (load_analog), "analog", "Analog",
                       vcWv.Param ("Analog Class", vlStringVT, STICKY, ""));

function open_addcomp_dialog(display, event)
{
    if (addcomp_dialog == nil)
        display_addcomp_dialog (display, event, "*");
}
open_sym_dbox := vcWv.Cmd (fac, FILE_CHECK0 (open_addcomp_dialog), 
                           "opensymdbox");


function strip_lib(libstr)
{
    declare comp_str;

    colon := rindex (libstr, ':');

    if (colon != FALSE)
        comp_str = substring (libstr, (colon + 1), (string_length (libstr)));
    else
        comp_str = libstr;
}

function build_ldfilelist(data_type, filter, name)
{
    declare str, comp_str, sav_str;

    count :=  0;
    templist := nil;
    sorted_list := nil;
    masterlist := nil;

    /* hash table of aliases from data.des */
    ddes_table := open_data_des ();

    foreach (str; ldfiles (data_type, filter, 0)) {

        if (ddes_table != nil) {
            sav_str = str;
            str = convertalias (str);
            comp_str = strip_lib (str);

            /* if strings are same, then try it again without the library
             */
            if (vdraw.vlStrMat (str, sav_str))
                comp_str = convertalias (comp_str);

        }
        else
            comp_str = strip_lib (str);

        /* apply the filter to the list */
        if (vdraw.vlStrMat (filter, str) || vdraw.vlStrMat (filter, comp_str))
            templist = cons (str, templist);
    }

    if (templist != nil)
        sorted_list = vscript.Sort (templist, string_le);
        
    sorted_list = remove_duplicates (sorted_list);

    templist = nil;
    
    while (!null (sorted_list)) {
        str = car (sorted_list);
        
        /* This fancy stuff is done to make n columns.  Before, when 
           vcPicklist.Add was being used, the number of columns was 
           hard-coded in the vcRegion.Create statement.  Now, when 
           building the list prior to the create, we must make the
           list itself reflect the # of columns.  Just change the 
           values of the mod statements to change the number of cols.
           */

        templist = cons (cons (str, str), templist);

        if (++count % addcomp_num_cols == 0) {
            templist = reverse_destructive (templist);
            masterlist = cons (templist, masterlist);
            templist = nil;
        }

        sorted_list = cdr (sorted_list);
    }

    if (count % addcomp_num_cols != 0) {
        templist = reverse_destructive (templist);
        masterlist = cons (templist, masterlist);
    }

    masterlist = reverse_destructive (masterlist);
    picklist := vcRegion.Create ("", vcPicklist.fac, masterlist);

    picklist;

} /* function build_ldfilelist */
