;;;;
;;;; $Revision: 1.8.1.2 $  $Date    : 93/12/18 16:44:43 $
;;;;
;;;; Class definition file for the ViewData design collection class.
;;;;
;;;; Copyright (c) 1994 by Viewlogic Systems, Inc.
;;;;


(vdm|ClassFileSetVersion class_file 5.3)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ComposeFunc for ViewDraw design collection 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; error stream - all errors from "vdrawcol" command will be reported here
(declare err-stream nil)
(set! err-stream (vlErr|StreamCreate))

;; error handler - sends a line from stderr to the common error stream
(define (vdrawError string)
  (vlErr|Out err-stream (vlErr|Create2 "vdrawcol" vlErrSev|Error string)))

;; buffer-lines - all of the errors and inputs need to be handled one 
;;                line at a time so this wrapper is used around the
;;                output and error stream handlers.
(define (buffer-lines handler)
  (let ((input-buffer ""))
    (lambda (string)
      (set! input-buffer (+ input-buffer string))
      (do ((index -1))
          ((equal index nil) #t)
        (set! index (string_search input-buffer "\n"))
        (cond ((not (equal index nil))
               (handler (substring input-buffer 0 (+ index 1)))
               (set! input-buffer 
                     (substring input-buffer (+ index  1) 0))))))))
            

;; vdrawFindLocation - get VDM location ID from a context and location name 
;;                 NOTE: case-insensitive compare is used between given 
;;                       location name and the known VDM location aliases
(define (vdrawFindLocation context collection loc-name obj)
  (let ((loc-handle nil))

    (vpm|MapLoopLocation
         (lambda (handle)
           (cond ((and (string_eqi (vpm|LocationGetAlias handle) loc-name)
                       (vdm|ObjFind obj nil handle))
                  (set! loc-handle handle))))
         (vdm|CntxGetMap context))
    (if (null loc-handle)
        (vlEx|Throw2 vlEx|FAILURE "vdrawcol" vlErrSev|Error 
              (+ "The object \"" (vdm|ObjGetName obj)  
                 "\" could not be found in location \"" loc-name "\". "
                 loc-name " may be an alias from your viewdraw.ini "
                 "file that does not have a corresponding "
                 "entry in your vpmus.scm file."))
        loc-handle)))


;; vdrawObjCreate - create a VDM obj 
(define 
  (vdrawObjCreate context collection obj-name version class-name loc-name 
                  attrs)
  (let ((class (vdm|CntxFindClass context class-name))
        (obj nil)
        (found #f))
    (if (null class)
        (vlEx|Throw2 vlEx|FAILURE "vdrawcol" vlErrSev|Error 
                (+ "Class \"" class-name "\", needed by object \"" 
                   obj-name "\" could not be found.")))

    (set! obj (vdm|ObjCreate obj-name version class nil context attrs))
    
    ;; If the location name is nil, try to find the object in the
    ;; collection's location.  Otherwise, use the location name as an
    ;; alias and let the vdm|ObjFind function perform the search.
    (if (null loc-name)
        (set! found (vdm|ObjFind obj (vdm|ObjGetLocation collection) nil))
        (set! found (vdm|ObjFind obj loc-name nil)))
        

    ;; If the object is not found in neither the alias'ed location,
    ;; nor in the collection's location, then search through the
    ;; locations using case insensitive aliasing.
    (if (not found)
        (vdrawFindLocation context collection loc-name obj))

    ; return
    obj))

;; get-path - this function guesses based on the collection object's 
;;            location which filesystem directory we should "cd" to
;;            before attempting to collect the schematic sheets.  This
;;            is necessary so that vdrawcol is sure to get the correct 
;;            viewdraw.ini file (normally kept in the same location as
;;            design).  Kind of ugly, but I don't know another way...
;(define (get-path context collection)
;  (let ((path (vdm|ObjGetPhysicalDescriptor collection)))
;    (vsUppath (car (cdr (vsParse path))))))
;    
;; the vdrawComposeFunc - each line of input to the stdout handler 
;;                        is assumed to be a Scheme expression that 
;;                        evaluates to a VDM object to be included 
;;                        in the collection.

(define (vdrawComposeFunc context collection)

  ;; print a little message if anyone has registered
  (vdm|CntxCallOutputHandlers context
       (+ "Collecting all objects belonging to ViewDraw design \""
          (vdm|ObjGetName collection)
          "\" ...\n"))

  ;; invoke "vdrawcol" program to get a list of the objects to be included
  (let ((obj-list nil)) 
    (vcSys|CommandSynchronous 

           ; command
           (+ ; "cd " (get-path context collection) "; "
              "vdrawcol " (vdm|ObjGetName collection))

           ; environment 
           nil

           ; output handler
           (buffer-lines 
            (lambda  (string)
              (begin
                (let ((obj ((eval_string2
                             (+ "(lambda (context collection) " 
                                string ")")
                             (current_package) vscmLanguage|SCM)
                            context collection)))
                  (cond ((and (not (null obj)) (not (member obj obj-list)))
                         (set! obj-list (cons obj obj-list))))))))

           ; error handler
           (buffer-lines vdrawError)

           ; exit handler
           (lambda (mode code)
             (cond ((or (equal mode vcSysMode|Signaled)
                        (and (equal mode vcSysMode|Exited)
                             (not (equal code 0))))
                    (set! obj-list nil)
                    (vdm|CntxCallOutputHandlers context
                         "Error encountered - collection not completed.\n"))
                   (#t (vdm|CntxCallOutputHandlers context
                            "Objects collected.\n"))))
           nil         ; host
           nil         ; remote wd
           20000       ; timeout (seconds)
           0)          ; timeout (microsecs)
    obj-list))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; vdrawDesign - a collection class for ViewDraw designs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define vdrawDesign
  (vdm|ClassCreate "ViewDraw Design" class_file
       `(AttrList          . ("NAME"))
       `(AttrFilter        . (".*"))
       `(NameTemplate      . "$NAME")
       `(PhysicalTemplate  . "vdrawcol/$NAME.vcl")
       `(CollectionMembers . ("vdmWildcard"))
       `(ComposeFunc       . ,vdrawComposeFunc)
       `(Type              . ,vdmClassType|Composite)))


