/* The following prefixes are used in these files:
   c_   Command
   cb_  C callback (not defined in viewscript)
   cg_  Command generator, a function to create several related commands
   f_   Action function (called from command)
   fl_  Function returning a long list (see vfpgadef.vs)
   g_   Global (used for modes and long static lists)
   gr_  Global region
   m_   Menu
   p_   Function which defines a command parameter
   r_   Region
   s_   Static variable (in the C source files)
   w_   Window
*/


/*** GLOBAL ROUTINES AND DECLARATIONS ***/
IniFileName := "vfpga.ini";  /* Preferences edited with F13 stored here */

vcWv.MakeEnum ("vfpgaSymTypeVT", list ("Automatic", "User", "Library"));
vcWv.MakeEnum ("vfpgaJ2VDelayVT", list ("Min", "Typ", "Max"));
vcWv.MakeEnum ("vfpgaFitPinVT", list ("Attempt", "Keep", "Ignore"));
vcWv.MakeEnum ("vfpgaVdesOptVT", list ("None", "Area", "Speed"));
vcWv.MakeEnum ("vfpgaSelJamVT", list ("Any", "Yes", "No"));
vcWv.MakeEnum ("vfpgaSelEraVT", list ("Any", "Yes", "No", "EE", "UV"));
vcWv.MakeEnum ("vfpgaJedXvalVT", list ("Zero", "One"));
vcWv.MakeEnum ("vfpgaJedChksumVT", list ("None", "Full", "Dummy"));
vcWv.MakeEnum ("vfpgaJedDocfmtVT", list ("Brief", "Long"));
vcWv.MakeEnum ("vfpgaSideVT", list ("Left", "Right"));
vcWv.MakeMask ("vfpgaMenuMaskVT",list ("Actel","Altera","PLD","Xilinx","FPGA"));

declare gr_log, gr_stat, g_scale, gr_kill; g_selel := "";

/* Print literal string into log window.  Handles only trailing \r's. */
delete_to_eol := FALSE;
function PrintString (s) {
    if (!null (gr_log)) {
        vtx.ReadOnly(gr_log, FALSE);
        if (delete_to_eol) {
            vtx.BeginningOfLine(gr_log);
            vtx.DeleteToEol(gr_log); 
            vtx.Update(gr_log);
            delete_to_eol = FALSE;
        }
        i := lindex(s, '\r');
        if (i) delete_to_eol = TRUE;
        else i = string_length (s);
        vtx.FastAppend(gr_log, substring(s, 0, i));
        vtx.FastAppendFinished(gr_log);
        vtx.Update(gr_log);
        vtx.EndOfBuffer(gr_log); 
        vtx.ReadOnly(gr_log, TRUE); 
    } 
}


/* Called by main() after prefs have been loaded */
function startup () {

    if (!AreClocksSynched())
        cb_mainErr(1209, "Warning", nil, "System clocks differ");

   /* Unset ABEL4 variables */
   vscript.putenv ("ABEL4DEV", nil);
   vscript.putenv ("ABEL4DB", nil);

    /* Check for ABEL environment and set it up if not correct */
   path5 := vscript.getenv ("ABEL5DEV");
   if (!null (path5))
      if (!fexists (vscript.vsConvPath (path5 + "/abel5lib.dev"))) path5 = nil;
   if (null (path5)) {
      path5 = vscript.vsSrchdirs ("lib5", vscript.vsDIR);
      if ((!null (path5)) &&
         (fexists (vscript.vsConvPath (path5 + "/abel5lib.dev")))) {
            vscript.putenv ("ABEL5DEV", path5);
            cb_mainDbg (1, "setenv ABEL5DEV " + path5); }
      else path5 = nil; }
   if (null (path5)) cb_mainErr (1203, "Error", nil,
      "Unable to set up Data I/O environment");
   else {
      s := vscript.getenv ("ABELPROT");
      if ((vcWv.UI(wv) == vcUi.Motif) || (vcWv.UI(wv) == vcUi.OpenLook))
          if (null (s)) {
              vscript.putenv ("ABELPROT", "viewlogic");
              cb_mainDbg (1, "setenv ABELPROT viewlogic"); }
      s = vscript.getenv ("ABEL5DB");
      if (null (s)) {
         p := vscript.vsConvPath (path5 + "/devices");
         vscript.putenv ("ABEL5DB", p);
         cb_mainDbg (1, "setenv ABEL5DB " + p); } } 

   /* Declare command set for PLD tools */
   if ((g_menumask & vfpgaMenuMask.PLD) == vfpgaMenuMask.PLD)
      menuValidTypes ("PLD",
         list ("ABEL", "DIO", "VHDL", "JEDEC", "LPM", "Partition"),
         list (c_opt_devsel, c_opt_fit, c_opt_fuseasm, c_opt_j2vhdl,
            c_opt_part, c_util_diosch, c_cre_part, c_cre_jedec, c_cre_timing));

   prefDefAttrs ();

   /* Check for FPGA vendor environments and set up if not correct */
   if ((g_menumask & vfpgaMenuMask.Actel) == vfpgaMenuMask.Actel) actSetup ();
   if ((g_menumask & vfpgaMenuMask.Altera) == vfpgaMenuMask.Altera) altSetup ();
   if ((g_menumask & vfpgaMenuMask.Xilinx) == vfpgaMenuMask.Xilinx) xilSetup ();

   menuFileload (FALSE);
}


function enable_kill (flag) {
    vcRegion.Enable(gr_kill, flag, TRUE);
}

function set_scale_tools (num) {
    /* Reset the scale bar to zero */
    g_scale.current_count = convert (vlUnsignedIntVT, 0);
    vcScalebar.Set (g_scale.scalebar, g_scale.current_count);
    /* Store the number of tools */
    g_scale.tool_count = num;
}

function increment_scale () {
    if (g_scale.tool_count == 0) delta := 100;
    else delta := 100 / g_scale.tool_count;
    g_scale.current_count = g_scale.current_count +
        convert (vlUnsignedIntVT, delta);
    vcScalebar.Set (g_scale.scalebar, g_scale.current_count);
}

function full_scale () {
    vcScalebar.Set (g_scale.scalebar, convert (vlUnsignedIntVT, 100));
}

/* Create root, log and status regions */
function  _Create (arg, region) {
   r_root := vcTable.Create ("r_root", 2, 1);
   vcTable.Add (r_root, statCreate ());
   gr_log = vcRegion.Create ("gr_log", vtx.fac, "", macro_on <- TRUE,
                             scratch <- TRUE, max_lines <- 1000);
   vtx.ReadOnly(gr_log, TRUE);
   vcTable.Add (r_root, gr_log);
   g_scale = struct { 
       scalebar;        /* Scalebar region                     */
       tool_count;      /* Total number of tools in chain      */
       current_count    /* Percent of tools completed in chain */
   };
   g_scale.scalebar = vcRegion.Create ("scale", vcScalebar.fac, 
                                       convert (vlUnsignedIntVT, 0),
                                       min <- 0, max <- 100, macro_on <- TRUE,
                                       read_only <- TRUE);
   gr_kill = vcWv.LabelButtonCreate("b_kill", "Kill Process", nil,
                                    vcWv.Cmd (fac, func (d, e) {
                                        cb_makeKillchain(); },
                                              "KillChain"), nil);
   r_kill := vcTable.Create("r_kill", 1, 5);
   vcTable.Add (r_kill, vcRegion.Create ("", vcLabel.fac,
      "Tool completion percentage:"));
   vcTable.Add(r_kill, vcRubber.Create ("", 0, 0));
   vcTable.Add(r_kill, g_scale.scalebar);
   vcTable.Add(r_kill, vcRubber.Create ("", 0, 0));
   /* Don't add kill button on PC */
   if ((vcWv.UI(wv) == vcUi.Motif) || (vcWv.UI(wv) == vcUi.OpenLook))
       vcTable.Add(r_kill, gr_kill);
   else
       vcTable.Add(r_kill, vcRubber.Create ("", 0, 0));
   enable_kill (FALSE);
   vcTable.Add (r_root, r_kill);
   vcRegion.AddChild (region, r_root);
   r_root; }


load ("vfpgadef.vs");  /* lists: PLD devices, FPGA techs, manufacturers, etc */
load ("vfpgafac.vs");  /* additional facilities */
load ("vfpgacom.vs");  /* commands and functions */
load ("vfpgawin.vs");  /* preferences, status window, menu greying  */
load ("vfpgafun.vs");  /* Tool command line building functions */
load ("vfpgacfg.vs");  /* Tool configuration */
load ("vfpgaact.vs");  /* Actel dialog boxes and tool chains */
load ("vfpgaalt.vs");  /* Altera dialog boxes and tool chains */
load ("vfpgaxil.vs");  /* Xilinx dialog boxes and tool chains */
load ("vfpgamn.vs");   /* menus */
if (((vlPref.Val (prefs, "testmode")) & 128) != 0)
    load ("vfpgatst.vs");  /* testing */
if ((path := vscript.vsSrchdirs ("vfpgaus.vs", vscript.vsFILE)) != nil)
   load (path);


/*** HELP TEXT FOR COMMANDS ***/


vcCmd.SetHelp (c_file_sch,    "Load a schematic file into memory");
vcCmd.SetHelp (c_file_vhdl,   "Specify the name of top level VHDL file");
vcCmd.SetHelp (c_file_abel,   "Specify the name of top level ABEL file");
vcCmd.SetHelp (c_file_jed,    "Specify the name of top level JEDEC file");
vcCmd.SetHelp (c_file_edit,   "Run appropriate editor on the selected file");
vcCmd.SetHelp (c_util_diotut, "Copy files for PLD tutorial");
vcCmd.SetHelp (c_util_x3tut,  "Copy files for Xilinx 3000 FPGA tutorial");
vcCmd.SetHelp (c_util_act2tut,"Copy files for Act-2 FPGA tutorial");
vcCmd.SetHelp (c_util_diosch, "Create schematic of DIO primitives");
vcCmd.SetHelp (c_util_pldpins,"Display information about PLD package pinouts");
vcCmd.SetHelp (c_util_burndev,"Create window for device programmer interface");
vcCmd.SetHelp (c_util_delint, "Clean up by deleting intermediate files");
vcCmd.SetHelp (c_cre_sym,     "Create top level symbol");
vcCmd.SetHelp (c_cre_func,    "Create simulation model with no timing");
vcCmd.SetHelp (c_cre_part,    "Create schematic of multiple PLDs");
vcCmd.SetHelp (c_cre_jedec,   "Create JEDEC file for PLD device programming");
vcCmd.SetHelp (c_cre_timing,  "Create JEDEC and timing model for PLD");
vcCmd.SetHelp (c_opt_ahdl2pla,"Set options for ABEL compiler (ahdl2blf)");
vcCmd.SetHelp (c_opt_plaopt,  "Set options for PLA optimizer (blifopt)");
vcCmd.SetHelp (c_opt_vhdl,    "Set options for VHDL analyzer (vhdl)");
vcCmd.SetHelp (c_opt_part,    "Set options for PLD partitioner (autopart)");
vcCmd.SetHelp (c_opt_fit,     "Set options for pin assignment (fit, fit*)");
vcCmd.SetHelp (c_opt_fuseasm, "Set options for creating JEDEC (fuseasm)");
vcCmd.SetHelp (c_opt_devsel,  "Set options for PLD device selector (devsel)"); 
vcCmd.SetHelp (c_opt_j2vhdl,  "Set options for PLD timing modeller (j2vhdl)");
