/* This file contains the following major sections:
   - A bunch of functions which return lists for picklists:
     * fl_techlist: synthesis technologies 
     * fl_manlist: PLD manufacturers 
     * fl_jedfmtlist: PLD JEDEC formats 
     * fl_partstrat: partitioner strategies
     * fl_plastrat: PLA Optimization strategies
     * fl_vhdlerr: VHDL analyzer error levels
     * fl_seltemp: device selector temperatures
     * fl_seltech: device selector technologies
     * fl_selpack: device selector packages
     * fl_selsort: device selector sort order
   - Functions which return lists of PLD devices:
     * fl_devlist: all PLD devices 
     * fl_fitlist: only those PLD devices supported by a fitter 
     * fl_j2vlist: only those PLD devices supported by the j2vhdl translator 
     * fl_partlist: only those PLD devices supported by autopart 
*/

/* The following functions return lists.  HACK ALERT!
   These lists should be passed into facilities as preferences.
   Preferences can not take lists as arguments (only strings).
   The string passed in, until version 5.2, is limited to 512 characters.
   Some of these lists are longer than 512 characters.
   Therefore, the facilities take a preference, which is a string, which is
   the name of a function from this file.  The facility asks Viewscript to
   interpret the function, returning the list, and this list is used to
   build the picklist.  Credit to Josh for the first suggestion (which breaks
   at 512 characters until 5.2) and Kenstir for the one which really works.
*/


function fl_techlist () { list (
   cons ("Xilinx 2000", "x2000"),
   cons ("Xilinx 3000", "x3000"),
   cons ("Xilinx 4000", "x4000"),
   cons ("Actel Act-1", "act1"),
   cons ("Actel Act-2", "act2"),
   cons ("Actel Act-3", "act3"),
   cons ("Altera", "altera")); }

function fl_manlist () { list (
   cons ("Altera Corp.",                          "ALT"),
   cons ("Advanced Micro Devices/MMI",            "AMD"),
   cons ("Architectures only",                    "ARCHITECT"),
   cons ("Atmel Corp.",                           "ATM"),
   cons ("AT&T Microelectronics",                 "ATT"),
   cons ("Cypress Semiconductor",                 "CYP"),
   cons ("Exel Microelectronics, Inc.",           "EXL"),
   cons ("Gould AMI Semiconductors",              "GOU"),
   cons ("Hyundai Electronics",                   "HYU"),
   cons ("International CMOS Technology, Inc.",   "ICT"),
   cons ("Intel Corporation",                     "INT"),
   cons ("Lattice Corp.",                         "LAT"),
   cons ("National Semiconductor Corp.",          "NAT"),
   cons ("Philips Semiconductor",                 "PHL"),
   cons ("Plus Logic",                            "PLO"),
   cons ("PLX",                                   "PLX"),
   cons ("QuickLogic Corporation",                "QLC"),
   cons ("Ricoh Corp.",                           "RIC"),
   cons ("SGS-Thompson Microelectronics",         "SGS"),
   cons ("Texas Instruments",                     "TEX"),
   cons ("Toshiba",                               "TOS"),
   cons ("TriQuint semiconductor",                "TRQ")); }

function fl_jedfmtlist () { list (
   cons ("Default Format",     "default"),
   cons ("Brief JEDEC Format", "brief"),
   cons ("Hex JEDEC Format",   "hex"),
   cons ("Full JEDEC Format",  "full"),
   cons ("82 PROM Format",     "82"),
   cons ("83 PROM Format",     "83"),
   cons ("87 PROM Format",     "87"),
   cons ("88 PROM Format",     "88"),
   cons ("POF Format",         "pof")); }

function fl_partstrat () { list (
   cons ("New schematic, use partition device", "newauto"),
   cons ("Update current schematic, use partition device", "oldauto"),
   cons ("New schematic, no partitioning", "newnone")); }

/* Caution: notice that the right hand side strings have underscores; this
   is required by the VertPick package, which uses spaces as delimiters */
function fl_plastrat () { list (
   cons ("Default", "default"),
   cons ("Reduce by Pin, Choose polarity", "bypin_choose"),
   cons ("Reduce by Pin, Fixed polarity", "bypin_fixed"),
   cons ("Reduce as Group, Choose polarity", "group_choose"),
   cons ("Reduce as Group, Fixed polarity", "group_fixed"),
   cons ("D/T Flip-flop, Auto synthesize", "dt"),
   cons ("No reduce, Merge only", "none")); }

function fl_vhdlerr () { list (
   cons ("Errors, Warnings, Extensions, Notes", "0"),
   cons ("Errors, Warnings, Extensions", "1"),
   cons ("Errors, Warnings", "2"),
   cons ("Errors", "3")); }

function fl_seltemp () { list (
   cons ("Commercial", "COM"),
   cons ("Military", "MIL"),
   cons ("Industrial", "IND"),
   cons ("Standard", "STD")); }

function fl_seltech () { list (
   cons ("TTL", "TTL"),      cons ("CMOS", "CMOS"),
   cons ("ECL", "ECL"),      cons ("GAAS", "GAAS")); }

function fl_selpack () { list (
   cons ("DIP", "DIP"),      cons ("CDIP", "CDIP"),
   cons ("LCC", "LCC"),      cons ("PLCC", "PLCC"),
   cons ("PGA", "PGA"),      cons ("SOIC", "SOIC"),
   cons ("FPAK", "FPAK"),      cons ("MISC", "MISC")); }

function fl_selsort () { list (
   cons ("Device",       "dev"),     cons ("Hold Time",    "hold"),
   cons ("Preload",      "jamload"), cons ("Manufacturer", "mfg"),
   cons ("Package",      "pack"),    cons ("Pins",         "pins"),
   cons ("Power",        "power"),   cons ("Price",        "price"),
   cons ("Setup Time",   "setup"),   cons ("Speed",        "speed"),
   cons ("Technology",   "tech"),    cons ("Temperature",  "spc")); }


/*** SUPPORTED PLD DEVICES ***/


function fl_devlist () { list (
   "ANY",         "E0310",       "E0320",       "E0600",       "E0600C",
   "E0630",       "E0630C",      "E0900",       "E0900C",      "E10P8",
   "E12P6",       "E14P4",       "E16P2",       "E16P8",       "E16P8F",
   "E16RP4",      "E16RP4F",     "E16RP6",      "E16RP6F",     "E16RP8",
   "E16RP8F",     "E1800",       "E204",        "E224",        "E224C",
   "E242",        "E5AC312",     "E5AC324",     "EC12C4A",     "EC16C4A",
   "EC16ET6",     "EC16LC4N",    "EC16LC8N",    "EC16LD4N",    "EC16LD8N",
   "EC16LM4N",    "EC16P4A",     "EC16P8C",     "EC16P8N",     "EC16P8NC",
   "EC16PE8",     "EC16RC4N",    "EC16RC8N",    "EC16RD4N",    "EC16RD8N",
   "EC16RM4N",    "EC16TE6",     "EC20EG8A",    "EC20EV8A",    "EC20EV8C",
   "EC20G8M",     "EC20P8M",     "F100",        "F103",        "F105",
   "F151",        "F153",        "F155",        "F157",        "F159",
   "F161",        "F162",        "F163",        "F167",        "F167C",
   "F168",        "F168C",       "F173",        "F173C",       "F179",
   "F179C",       "F253",        "F2605",       "F2678",       "F273",
   "F30S16",      "F405",        "F42VA12",     "F42VA12C",    "F473",
   "F506",        "F507",        "F529",        "F6001",       "F6001C",
   "F6002",       "F6002C",      "F839",        "F9800",       "F9804C0",
   "F9804C1",     "F9804C2",     "F9804C4",     "F9806",       "FMAPL128",
   "FMAPL144",    "ISPGDS14",    "ISPGDS18",    "ISPGDS22",    "MACH110A",
   "MACH120A",    "MACH130A",    "MACH210A",    "MACH215A",    "MACH220A",
   "MACH230A", "MACH435A", "P10H8", "P10P8",    "P12H10",      "P12H6",
   "P12L10",      "P12L6",       "P12P10",      "P12P6",       "P14H4",
   "P14H8",       "P14L4",       "P14L8",       "P14P4",       "P14P8",
   "P16C1",       "P16H2",       "P16H6",       "P16H8",       "P16HD8",
   "P16L2",       "P16L6",       "P16L8",       "P16L8C",      "P16LD8",
   "P16N8",       "P16P2",       "P16P6",       "P16P8",       "P16R4",
   "P16R4C",      "P16R6",       "P16R6C",      "P16R8",       "P16R8C",
   "P16RA8",      "P16RP4",      "P16RP6",      "P16RP8",      "P16SV8",
   "P16V8",       "P16V8HD",     "P16V8HDC",    "P16VP8",      "P16Z8",
   "P18CV8",      "P18G8",       "P18H4",       "P18L4",       "P18N8",
   "P18P8",       "P18U8",       "P18V10G",     "P18V8",       "P18V8Z",
   "P19L8L",      "P19L8R",      "P19R4L",      "P19R4R",      "P19R6L",
   "P19R6R",      "P19R8L",      "P19R8R",      "P20ARP10",    "P20ARP4",
   "P20ARP6",     "P20ARP8",     "P20C1",       "P20CG10",     "P20CG10C",
   "P20G10",      "P20G10C",     "P20H2",       "P20H8",       "P20L10",
   "P20L10C",     "P20L2",       "P20L8",       "P20L8C",      "P20L8FNC",
   "P20P2",       "P20P8",       "P20R4",       "P20R4C",      "P20R4FNC",
   "P20R6",       "P20R6C",      "P20R6FNC",    "P20R8",       "P20R8C",
   "P20R8FNC",    "P20RA10",     "P20RA10C",    "P20RA10G",    "P20RA10N",
   "P20RP4",      "P20RP6",      "P20RP8",      "P20RS10",     "P20RS4",
   "P20RS8",      "P20S10",      "P20SV8",      "P20V8",       "P20V8LCC",
   "P20VP8",      "P20VP8LC",    "P20X10",      "P20X10C",     "P20X4",
   "P20X4C",      "P20X8",       "P20X8C",      "P20XV10",     "P20XV10C",
   "P22AP10",     "P22CV10C",    "P22CV10Z",    "P22IP6",      "P22RX8A",
   "P22V10",      "P22V10C",     "P22V10G",     "P22V10GC",    "P22V10GI",
   "P22V10I",     "P22V10IC",    "P22V10TI",    "P22VP10",     "P22VP10C",
   "P23S8",       "P23SV8",      "P241",        "P24L10",      "P24R10",
   "P24V10",      "P2500",       "P2500C",      "P26CV12",     "P26V12",
   "P29M16",      "P29M16C",     "P29MA16",     "P29MA16C",    "P32VX10",
   "P32VX10A",    "P32VX10C",    "P330",        "P331",        "P332",
   "P333",        "P335",        "P336",        "P337",        "P338",
   "P339",        "P448",        "P448C",       "P448Z",       "P464",
   "P464C",       "P464Z",       "P48N22",      "P5000",       "P5016A",
   "P5016S",      "P5024A",      "P5024S",      "P5032A",      "P5032AC",
   "P5032S",      "P5032SC",     "P508",        "P5100",       "P6L16",
   "P750",        "P750C",       "P8L14",       "PML501",      "PML601",
   "RA10P4",      "RA10P8",      "RA10R8",      "RA11P4",      "RA11P8",
   "RA11R8",      "RA12P4",      "RA12P8",      "RA5P16",      "RA5P8",
   "RA6P16",      "RA8P4",       "RA8P8",       "RA9P4",       "RA9P8",
   "RA9R8",       "RE8P4",       "XC723644",    "XC727268",    "XC727284"
   ); }

function fl_fitlist () { list (
   "E0310",       "E0320",       "E0600",       "E0600C",      "E0900",
   "E0900C",      "E10P8",       "E12P6",       "E14P4",       "E16P2",
   "E16P8F",      "E16RP4F",     "E16RP6F",     "E16RP8F",     "E1800",
   "E224",        "E224C",       "EC12C4A",     "EC16C4A",     "EC16ET6",
   "EC16LC4N",    "EC16LC8N",    "EC16LD4N",    "EC16LD8N",    "EC16LM4N",
   "EC16P4A",     "EC16P8C",     "EC16P8N",     "EC16P8NC",    "EC16PE8",
   "EC16RC4N",    "EC16RC8N",    "EC16RD4N",    "EC16RD8N",    "EC16RM4N",
   "EC16TE6",     "EC20EG8A",    "EC20EV8A",    "EC20EV8C",    "F100",
   "F103",        "F105",        "F151",        "F153",        "F155",
   "F157",        "F159",        "F161",        "F162",        "F163",
   "F167",        "F167C",       "F168",        "F168C",       "F173",
   "F173C",       "F179",        "F179C",       "F253",        "F2605",
   "F2678",       "F273",        "F405",        "F473",        "F506",
   "F529",        "F839",        "F9800",       "F9806",       "FMAPL128",
   "FMAPL144",    "MACH110A",    "MACH120A",    "MACH130A",    "MACH210A",
   "MACH215A",    "MACH220A",    "MACH230A", "MACH435A", "P10H8", "P10P8",
   "P12H10",      "P12H6",       "P12L10",      "P12L6",       "P12P10",
   "P12P6",       "P14H4",       "P14H8",       "P14L4",       "P14L8",
   "P14P4",       "P14P8",       "P16C1",       "P16H2",       "P16H6",
   "P16H8",       "P16HD8",      "P16L2",       "P16L6",       "P16L8",
   "P16L8C",      "P16LD8",      "P16N8",       "P16P2",       "P16P6",
   "P16P8",       "P16R4",       "P16R4C",      "P16R6",       "P16R6C",
   "P16R8",       "P16R8C",      "P16RA8",      "P16RP4",      "P16RP6",
   "P16RP8",      "P16SV8",      "P16V8",       "P16VP8",      "P16Z8",
   "P18CV8",      "P18G8",       "P18H4",       "P18L4",       "P18N8",
   "P18P8",       "P18U8",       "P18V10G",     "P18V8",       "P18V8Z",
   "P19L8L",      "P19L8R",      "P19R4L",      "P19R4R",      "P19R6L",
   "P19R6R",      "P19R8L",      "P19R8R",      "P20ARP10",    "P20ARP4",
   "P20ARP6",     "P20ARP8",     "P20C1",       "P20CG10",     "P20CG10C",
   "P20G10",      "P20G10C",     "P20H2",       "P20H8",       "P20L10",
   "P20L10C",     "P20L2",       "P20L8",       "P20L8C",      "P20L8FNC",
   "P20P2",       "P20P8",       "P20R4",       "P20R4C",      "P20R4FNC",
   "P20R6",       "P20R6C",      "P20R6FNC",    "P20R8",       "P20R8C",
   "P20R8FNC",    "P20RA10",     "P20RA10C",    "P20RA10G",    "P20RA10N",
   "P20RP4",      "P20RP6",      "P20RP8",      "P20SV8",      "P20V8",
   "P20V8LCC",    "P20VP8",      "P20VP8LC",    "P20X10",      "P20X10C",
   "P20X4",       "P20X4C",      "P20X8",       "P20X8C",      "P22AP10",
   "P22CV10C",    "P22CV10Z",    "P22RX8A",     "P22V10",      "P22V10C",
   "P22V10G",     "P22V10GC",    "P22V10GI",    "P22V10I",     "P22V10IC",
   "P22V10TI",    "P22VP10",     "P22VP10C",    "P23S8",       "P23SV8",
   "P24L10",      "P24R10",      "P24V10",      "P2500",       "P2500C",
   "P26CV12",     "P26V12",      "P32VX10",     "P32VX10A",    "P32VX10C",
   "P336",        "P337",        "P338",        "P339",        "P448",
   "P448C",       "P448Z",       "P464",        "P464C",       "P464Z",
   "P48N22",      "P5000",       "P5016A",      "P5016S",      "P5024A",
   "P5024S",      "P5032A",      "P5032AC",     "P5032S",      "P5032SC",
   "P508",        "P5100",       "P6L16",       "P750",        "P750C",
   "P8L14",       "PML501",      "PML601",      "RA10P4",      "RA10P8",
   "RA10R8",      "RA11P4",      "RA11P8",      "RA11R8",      "RA12P4",
   "RA12P8",      "RA5P16",      "RA5P8",       "RA6P16",      "RA8P4",
   "RA8P8",       "RA9P4",       "RA9P8",       "RA9R8",       "RE8P4",
   "XC723644",    "XC727268",    "XC727284"
   ); }

function fl_j2vlist () { list (
   "E0310",       "E0320",       "E0600",       "E0600C",      "E0900",
   "E0900C",      "E12P6",       "E14P4",       "E16P2",       "E16P8",
   "E16RP4",      "E16RP6",      "E16RP8",      "E1800",       "E224",
   "E224C",       "EC16P4A",     "EC16P8C",     "EC16P8N",     "EC16P8NC",
   "EC16PE8",     "EC16RC4N",    "EC16RC8N",    "EC16RD4N",    "EC16RD8N",
   "EC20EV8A",    "EC20EV8C",    "F100",        "F105",        "F151",
   "F153",        "F155",        "F157",        "F159",        "F161",
   "F162",        "F163",        "F167",        "F167C",       "F168",
   "F168C",       "F173",        "F173C",       "F179",        "F179C",
   "F2605",       "F2678",       "F42VA12",     "F42VA12C",    "F506",
   "F529",        "F839",        "F9800",       "MACH110A",    "MACH120A",
   "MACH130A",    "MACH210A",    "MACH215A",    "MACH220A",    "MACH230A",
   "P10H8",       "P10P8",       "P12H10",      "P12H6",       "P12L10",
   "P12L6",       "P12P10",      "P12P6",       "P14L4",       "P14L8",
   "P14P4",       "P14P8",       "P16C1",       "P16H2",       "P16H6",
   "P16H8",       "P16HD8",      "P16L2",       "P16L6",       "P16L8",
   "P16L8C",      "P16N8",       "P16P2",       "P16P6",       "P16P8",
   "P16R4",       "P16R4C",      "P16R6",       "P16R6C",      "P16R8",
   "P16R8C",      "P16RA8",      "P16RP4",      "P16RP6",      "P16RP8",
   "P16SV8",      "P16V8",       "P16Z8",       "P18CV8",      "P18G8",
   "P19L8L",      "P19L8R",      "P19R4L",      "P19R4R",      "P19R6L",
   "P19R6R",      "P19R8L",      "P19R8R",      "P20ARP10",    "P20ARP4",
   "P20ARP6",     "P20ARP8",     "P20C1",       "P20CG10",     "P20CG10C",
   "P20G10",      "P20G10C",     "P20H2",       "P20H8",       "P20L10",
   "P20L10C",     "P20L2",       "P20L8",       "P20L8C",      "P20L8FNC",
   "P20P2",       "P20P8",       "P20R4",       "P20R4C",      "P20R4FNC",
   "P20R6",       "P20R6C",      "P20R6FNC",    "P20R8",       "P20R8C",
   "P20R8FNC",    "P20RA10",     "P20RA10C",    "P20RA10G",    "P20RA10N",
   "P20RP4",      "P20RP6",      "P20RP8",      "P20RS8",      "P20X10",
   "P20X10C",     "P20X4",       "P20X4C",      "P20X8",       "P20X8C",
   "P22RX8A",     "P22V10",      "P22V10C",     "P22V10G",     "P22V10GC",
   "P22V10GI",    "P2500",       "P2500C",      "P29M16",      "P29M16C",
   "P29MA16",     "P29MA16C",    "P330",        "P5000",       "P5016A",
   "P5016S",      "P5024A",      "P5024S",      "P5032A",      "P5032AC",
   "P5032S",      "P5032SC",     "P750",        "P750C",       "P8L14"
   ); }

function fl_partlist () { list (
   "E0310",       "E0320",       "E0600",       "E0600C",      "E0900",
   "E0900C",      "E10P8",       "E12P6",       "E14P4",       "E16P2",
   "E16P8F",      "E16RP4F",     "E16RP6F",     "E16RP8F",     "E224",
   "E224C",       "EC12C4A",     "EC16C4A",     "EC16ET6",     "EC16LC4N",
   "EC16LC8N",    "EC16LD4N",    "EC16LD8N",    "EC16LM4N",    "EC16P4A",
   "EC16P8C",     "EC16P8N",     "EC16P8NC",    "EC16PE8",     "EC16RC4N",
   "EC16RC8N",    "EC16RD4N",    "EC16RD8N",    "EC16RM4N",    "EC16TE6",
   "EC20EG8A",    "EC20EV8A",    "EC20EV8C",    "F100",        "F103",
   "F105",        "F151",        "F153",        "F155",        "F157",
   "F159",        "F161",        "F162",        "F163",        "F167",
   "F167C",       "F168",        "F168C",       "F173",        "F173C",
   "F179",        "F179C",       "F253",        "F2605",       "F2678",
   "F273",        "F405",        "F473",        "F506",        "F529",
   "F839",        "F9800",       "F9806",       "P10H8",       "P10P8",
   "P12H10",      "P12H6",       "P12L10",      "P12L6",       "P12P10",
   "P12P6",       "P14H4",       "P14H8",       "P14L4",       "P14L8",
   "P14P4",       "P14P8",       "P16C1",       "P16H2",       "P16H6",
   "P16H8",       "P16HD8",      "P16L2",       "P16L6",       "P16L8",
   "P16L8C",      "P16LD8",      "P16N8",       "P16P2",       "P16P6",
   "P16P8",       "P16R4",       "P16R4C",      "P16R6",       "P16R6C",
   "P16R8",       "P16R8C",      "P16RA8",      "P16RP4",      "P16RP6",
   "P16RP8",      "P16SV8",      "P16V8",       "P16VP8",      "P16Z8",
   "P18CV8",      "P18G8",       "P18H4",       "P18L4",       "P18N8",
   "P18P8",       "P18U8",       "P18V10G",     "P18V8",       "P18V8Z",
   "P19L8L",      "P19L8R",      "P19R4L",      "P19R4R",      "P19R6L",
   "P19R6R",      "P19R8L",      "P19R8R",      "P20ARP10",    "P20ARP4",
   "P20ARP6",     "P20ARP8",     "P20C1",       "P20CG10",     "P20CG10C",
   "P20G10",      "P20G10C",     "P20H2",       "P20H8",       "P20L10",
   "P20L10C",     "P20L2",       "P20L8",       "P20L8C",      "P20L8FNC",
   "P20P2",       "P20P8",       "P20R4",       "P20R4C",      "P20R4FNC",
   "P20R6",       "P20R6C",      "P20R6FNC",    "P20R8",       "P20R8C",
   "P20R8FNC",    "P20RA10",     "P20RA10C",    "P20RA10G",    "P20RA10N",
   "P20RP4",      "P20RP6",      "P20RP8",      "P20SV8",      "P20V8",
   "P20V8LCC",    "P20VP8",      "P20VP8LC",    "P20X10",      "P20X10C",
   "P20X4",       "P20X4C",      "P20X8",       "P20X8C",      "P22AP10",
   "P22CV10C",    "P22CV10Z",    "P22RX8A",     "P22V10",      "P22V10C",
   "P22V10G",     "P22V10GC",    "P22V10GI",    "P22V10I",     "P22V10IC",
   "P22V10TI",    "P22VP10",     "P22VP10C",    "P23S8",       "P23SV8",
   "P24L10",      "P24R10",      "P24V10",      "P26CV12",     "P26V12",
   "P32VX10",     "P32VX10A",    "P32VX10C",    "P336",        "P337",
   "P338",        "P339",        "P448",        "P448C",       "P448Z",
   "P464",        "P464C",       "P464Z",       "P48N22",      "P508",
   "P6L16",       "P8L14",       "RA10P4",      "RA10P8",      "RA10R8",
   "RA11P4",      "RA11P8",      "RA11R8",      "RA12P4",      "RA12P8",
   "RA5P16",      "RA5P8",       "RA6P16",      "RA8P4",       "RA8P8",
   "RA9P4",       "RA9P8",       "RA9R8",       "RE8P4"
   ); }




