/* ---------------------------------------------------------------------- *\
 * Module -- viewtime.vs
 *
 *      Facility:      Viewtime
 *
 *      Environment:   sun4, pv 5.2
 *
 *      Author:        Mani Alagappan,  Nov 1993
 *                     Jeff Hare
 *
 *      Description:   Viewtime facility GUI implementation
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */

load ("vtvutils.vs");
load ("vtvfopen.vs");

timing_cmd := vcWv.Cmd (vcWin.fac, func(d, e, design, case)window_open (design, case, "");,
                        "_timing", "Open Timing Analysis",
                        vcWv.Param ("Design Name",
                                    string, STICKY, ""),
                        vcWv.Param ("Case Name",
                                    string, STICKY, ""));
 
ToBeDone := vcWv.Cmd (fac, 
                      func(d, e) add_status (d,e,"Note: This command not yet implemented");,
                      "_noop", "NoCmd");


/*===========================================================================*/
/*==                      open_timing                                      ==*/
/*===========================================================================*/
/*\Function open_timing -- Function called from viewtime to create dialog box.
*/
function open_timing(design, case, batchfile)
{
 
   data := nil;
   event := nil;

   if ((string_eq (design, "") == TRUE) ||
       (string_eq (case, "")   == TRUE)){
       build_fopen_dialog (data, event, design);
   }

   else
       window_open (design, case, batchfile);
}
 

function WinCallOnClose(w)
{
   r := vcWin.Region (w);
   vtv := vcRegion.Data(r, vtvVT);
   v := viewtime.GetVtime (vtv);

   shutdown := FALSE;

   if (shutdown == FALSE){

      design := " Design: " + vtime.GetBlockName(v) + 
         "                       Case: " + vtime.GetCaseName(v) + "\n";
      str := "The following Timing Analysis has been modified.\n\n " +
design + "\n You may 
Save changes now and Close the window, 
Discard the changes and Exit the window, or 
Cancel this command.";

      e := vcEvent.Create (0, 0, ' ', "", w, r, nil);
      table := vcTable.Create ("Close?", 2, 1);
      vcTable.Add (table, vcRegion.Create ("msg", vcStaticText.fac, str));
      
      buttons := vcTable.Create ("buttons", 1, 3);
      
      default_button := vcWv.LabelButtonCreate ("SaveClose",

                                                "Save and Close",
                                                nil, 
                                                func(){
                                                   q := vtime.GetQueue (v);
                                                   vtq.Add(q,e,"save");
                                                   vtq.Add(q,e,"exit yes");
                                                   /*vtime.Closed (v);*/
                                                },
                                                list (macro_on <- TRUE,
                                                      close_window <- TRUE));
      vcTable.Add (buttons, default_button);
      
      close_button := vcWv.LabelButtonCreate ("close",
                                              "Discard changes, then Close",
                                              nil, 
                                              func(){
                                                 vtq.Add (vtime.GetQueue (v),
                                                          e,
                                                          "exit yes");
                                              },
                                              list (macro_on <- TRUE,
                                                    close_window <- TRUE));
      vcTable.Add (buttons, close_button);
      
      dismiss_button := vcWv.LabelButtonCreate ("cancel",
                                                "Cancel",
                                                nil, 
                                                func() TRUE;,
                                                list (macro_on <- TRUE,
                                                      close_window <- TRUE));
      vcTable.Add (buttons, dismiss_button);

      vcTable.Add (table, buttons);
      vcWin.Popup (w, table, default_button,
                   vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
   }
   else
      vcWin.Destroy (w);
   
}

function WinDestroyNotify(w) TRUE; /*   printf ("Win %s destroyed\n", w);*/

/*
 * This function creates a ViewTime Window with motive facility.
 */
function window_open(design, case, batchfile) {


   w := FacCreate (list (design, case)); /* add case to list */
   if (w == nil)
      return (FALSE);

   vcWin.CallOnClose (w, WinCallOnClose);
   vcWin.DestroyNotify (w, WinDestroyNotify);
   vcWin.SetTitle (w, "ViewTime - " + vtime.Version());

   r := vcWin.Region (w);
   d_vtime := vcRegion.Data (r, vtimeVT);
   q := vtime.GetQueue (d_vtime);
   vtq.Hold (q, "Wait for motive");
   if (batchfile != nil && string_eq (batchfile,"") != TRUE)
      vtq.Add (q,vcEvent.Create (0, 0, ' ', "", w, r, nil),"execute " + batchfile);
   w;
} /* function window_open */

/*===========================================================================*/
/*==                     F i l e M e n u                                   ==*/
/*===========================================================================*/
OpenCmd := vcWv.Cmd(fac, 
                    func(d, e, str){ 
                       design    := "";
                    case      := "";
                       batchfile := "";
                       
                       for (str = string_token(str, &token, " ");
                            str != nil;
                            str = string_token(str, &token, " ")){
                          
                          if (string_eqi (token, "") != TRUE)
                             if (string_eqi (design,"") == TRUE)
                                design = string_copy (token);
                             else if (string_eqi (case,"") == TRUE)
                             case  = string_copy (token);
                             else
                                batchfile = string_copy (token);
                       }
                       open_timing (design, case, batchfile);
                    }, 
                    "open",
                    "Open another timing anaylysis",
                    vcWv.Param ("Design [case] [Batchfile]",
                                vlStringRestOfLineVT, 
                                vcWv.Constant, ""));

FileOpenCmd := vcWv.Cmd(fac,
                        func (d,e) open_timing ("","","");,
                        "_fopen");
vcCmd.SetHelp    (FileOpenCmd, "Bring up Design/Case Setup Dialog Box to analyze other design(s)/case(s)");

MtvCmd    := vtime.cmdMtv;
FileMenu := vcWv.MakeMenu ("File",
                           list ("Open...", FileOpenCmd),
                            list ("Save", vtime.cmdSave),
                            list ("SaveAs...", vtime.cmdSaveAs),
                            list ("Execute...", vtime.cmdExecute));

/*===========================================================================*/
/*==                      S e t u p M e n u                                ==*/
/*===========================================================================*/
SetupMenu := vcWv.MakeMenu ("Setup",
                            list ("Parameters...", vtime.cmdParameters),
                            list ("Model Libraries...", vtime.cmdChgSrch),
                            list ("Create Pinlist...",  vtime.cmdVtm),
                            list ("Clock Reference File...", vtime.cmdReffile),
                            list ("Restart/Reload Design", vtime.cmdRestart)
/*                            list ("Load Design",     vtime.cmdDesign), */
/*                            list ("Net Constants",   ToBeDone),*/
/*                            list ("MCP Definitions", ToBeDone)*/);
                            
/*===========================================================================*/
/*==                        A n a l y z e M e n u                          ==*/
/*===========================================================================*/
AnalyzeMenu := vcWv.MakeMenu ("Analyze",
                           list("Verify", vtime.cmdVerify),
                           list("Trace Path", vtime.cmdTrace));

/*===========================================================================*/
/*==                      R e s u l t s M e n u                            ==*/
/*===========================================================================*/
ResultsMenu := vcWv.MakeMenu ("Results",
                              list ("Verification Errors", vtime.cmdVisiterrors),
                              list ("Verification Report", vtime.cmdVrepfile),
                              list ("Delay Histogram", vtime.cmdHst),
/*                              list ("Display Global Slack", ToBeDone),  */
/*                              list ("Annotate Errors", ToBeDone), */
                              list ("Path Trace Report", vtime.cmdTrepfile),
                              list ("Show Path Display", vtime.cmdPathDisplay),
                              list ("Show Waveform", vtime.cmdWaveform));  

/*===========================================================================*/
/*==                        B r o w s e M e n u                            ==*/
/*===========================================================================*/
BrowseFileCmd         := vcWv.Cmd (fac,
                                   func(d, e, str) vtx.CreateWindow (str, TRUE);,
                                   "browse",
                                   "File Select",
                                   vcWv.Param ("", string, STICKY, "",
                                               vcFileSelect.fac)
                                   );

IFileMenu := vcWv.MakeMenu ("Input Files",
                             list ("PIN - Design Pinlist", vtime.cmdPinfile),
                             list ("REF - Clock Reference", vtime.cmdReffile),
                             list ("MOD - Compiled Models", vtime.cmdModfile),
                             list ("BSI - Boolean States", vtime.cmdBsifile),
                             list ("IDD - Interconnect Delays", vtime.cmdIddfile),
                             list ("LOC - X,Y Placements", vtime.cmdLocfile),
                             list ("STM - Motive Setup Parameters", vtime.cmdStmfile),
                             list ("MCP - Multi-Cycle Path", vtime.cmdMcpfile));

OFileMenu := vcWv.MakeMenu ("Output Files",
                             list ("ALG - Aborted Paths Log", vtime.cmdAlgfile),
                             list ("GSR - Global Slack Report", vtime.cmdGsrfile),
                             list ("LLG - Logic Loops", vtime.cmdLlgfile),
                             list ("REP - Verification Report", vtime.cmdRepfile),
                             list ("TIM - Timing Model", vtime.cmdTimfile),
                             list ("HST - Session History", vtime.cmdHstfile));

VFileMenu := vcWv.MakeMenu ("VTM Files",
                             list ("Libraries - Model Search Order", vtime.cmdChgSrch),
                             list ("CTL - Control File", vtime.cmdCtlfile),
                             list ("RCF - Revised Control File", vtime.cmdRcffile));

BrowseMenu := vcWv.MakeMenu ("Browse",
                             list ("Input Files", IFileMenu),
                             list ("Output Files", OFileMenu),
                             list ("VTM Files",    VFileMenu),
                             list ("File Browser", BrowseFileCmd));


/*===========================================================================*/
/*==                         I n f o M e n u                               ==*/
/*===========================================================================*/
InfoMenu := vcWv.MakeMenu ("Info",
                             list ("HPins",              vtime.cmdHpins),
                             list ("LNets",              vtime.cmdLnets),
                             list ("Instances",          vtime.cmdLocinfo),
                             list ("Nets",               vtime.cmdNetinfo),
                             list ("Models",             vtime.cmdModinfo),
                             list ("MCPs",               vtime.cmdMcpinfo),
                             list ("Wireload Groups",    vtime.cmdWireloadinfo),
                             list ("General Statistics", vtime.cmdStats));




/*===========================================================================*/
/*==                     P r o c e s s M e n u                             ==*/
/*===========================================================================*/
ProcessMenu := vcWv.MakeMenu ("Process",
                              list ("Setup Clock Ref File",          vtime.cmdReffile),
                              list ("Setup Model Search Order",      vtime.cmdChgSrch),
                              list ("Create Pinlist/Compile Models", vtime.cmdVtm),
                              list ("Verify Design",                 vtime.cmdVerify),
                              list ("Show Verification Errors",      vtime.cmdVisiterrors),
                              list ("Show Verification Report",      vtime.cmdVrepfile),
                              list ("Trace Path",                    vtime.cmdTrace),
                              list ("Show Path Trace Report",        vtime.cmdTrepfile),
                              list ("Show Path Display",             vtime.cmdPathDisplay),
                              list ("Show Waveform",                 vtime.cmdWaveform));  





/*===========================================================================*/
/*==                         O t h e r s                                   ==*/
/*===========================================================================*/
DstatCmd  := vtime.cmdDstat;
ExplCmd   := vtime.cmdExpl;
ClkgatCmd := vtime.cmdClkgat;
MtvquitCmd := vtime.cmdMtvquit;
TMenu     := vcWv.MakeMenu ("UnusedMenu",
                            list("Explaination Mode", vtime.cmdExpl),
                            list("Clock Gating",      vtime.cmdClkgat),
                            list("Detailed Status",   vtime.cmdDstat));

DisplayMenu := vcWv.MakeMenu ("DisplayMenu",
                              list("Peel",  vtime.cmdPeel),
                              list("Merge", vtime.cmdMerge));


QPrint    := vtime.cmdPrint;
QStop     := vtime.cmdStop;
QHold     := vtime.cmdHold;
QStep     := vtime.cmdStep;
QClear    := vtime.cmdClear;
QContinue := vtime.cmdContinue;
QRelease  := vtime.cmdQrelease;
QStep     := vtime.cmdStep;

QMenu := vcWv.MakeMenu ("First",
/*                        list("Print", QPrint),  */
                        list("Stop", QStop),
                        list("Step", QStep),
                        list("Continue", QContinue),
                        list("Clear", QClear),
/*                        list("Hold", QHold),      */
                        list("Release", QRelease));



ViewtimeMenu  := vcWv.MakeMenu ("ViewTime",
                                vcWin.RedSquareEntry,
                                list("File",    FileMenu),
                                list("Setup",   SetupMenu),
                                list("Analyze", AnalyzeMenu),
                                list("Results", ResultsMenu),
                                list("Browse",  BrowseMenu),
                                list("Info",    InfoMenu),
                                list("Queue",   QMenu),
                                list("Process", ProcessMenu));


vcFac.SetMenus(fac ,ViewtimeMenu);

