/*
 *  $Revision $ $Date $
 *
 *  Library Browser facility used by ViewLibrarian
 *
 *  Copyright (c) 1995 by Viewlogic Systems, Inc. */

vcWv.RequireFac(wv, "vcPullDown");
vcWv.RequireFac(wv, "vcSetOrder");
vcWv.RequireFac(wv, "vcExclude");
load("vlibutil.vs");
load("vlibdrc.vs");

function AddVisitFuncs(d) {
    function visit_obj(d, obj) {
        if (!null(d.objLastVisited))
            vcBrowse.ObjUnSelect(d.rgnBrowser, d.objLastVisited);
        vcBrowse.ObjSelect(d.rgnBrowser, obj);
        d.objLastVisited = obj;
    }

    function parse_lib_loc(src, sev, num, loc, data) {
        libdotview := vscript.re_getmatch (vcAlert.last_regexp, 1);
        declare str, libname, gvname;
        str = string_token(libdotview, &libname, ".");
        str = string_token(str, &gvname, ".");

        lib := vllm.CntxFindLib(d.cntx, libname);
        if (null(gvname) || string_eq(gvname, "")) {
            vcBrowse.ObjSelect(d.rgnBrowser, lib);
            vcRegion.SetFocus(vlSym.Fetch(d.symViewReg, gvname),
                              vlSym.Fetch(d.symViewReg, gvname));
        }
        else
            visit_obj(d, lib);
        nil; 
    }
    
    function parse_comp_loc(src, sev, num, loc, data) {
        libname  := vscript.re_getmatch (vcAlert.last_regexp, 1);
        compname := vscript.re_getmatch (vcAlert.last_regexp, 2);
        
        lib := vllm.CntxFindLib(d.cntx, libname);
        vcBrowse.ObjExpand(d.rgnBrowser, lib);
        comp := vllm.LibFindComp(lib, compname);
        visit_obj(d, comp);
        nil; 
    }

    function parse_db_loc(src, sev, num, loc, data) {
        libname  := vscript.re_getmatch (vcAlert.last_regexp, 1);
        dbname := vscript.re_getmatch (vcAlert.last_regexp, 2);
        lib := vllm.CntxFindLib(d.cntx, libname);
        db := vllm.LibFindDb(lib, dbname);
        nil;
    }

    function parse_view_loc(src, sev, num, loc, data) {
        libname  := vscript.re_getmatch (vcAlert.last_regexp, 1);
        compdotview := vscript.re_getmatch (vcAlert.last_regexp, 2);
        
        declare str, compname, vname, vqual;
        str = string_token(compdotview, &compname, ".");
        str = string_token(str, &vname, ".\\");
        str = string_token(str, &vqual, "\\");
        viewname := vname;
        if (!null(vqual)) viewname = viewname + "\\" + vqual;
        
        lib := vllm.CntxFindLib(d.cntx, libname);
        vcBrowse.ObjExpand(d.rgnBrowser, lib);
        comp := vllm.LibFindComp(lib, compname);
        vcBrowse.ObjSelect(d.rgnBrowser, comp);
        vcBrowse.ObjExpand(d.rgnBrowser, comp);
        v := vllm.CompFindView(comp, vname, vqual);
        if (vllm.ViewGetBase(v) == vllmBase.Indirect)
            visit_obj(d, v);
        else
            vcRegion.SetFocus(vlSym.Fetch(d.symViewReg, viewname),
                              vlSym.Fetch(d.symViewReg, viewname));
        nil;
    }

    vcAlert.AddVisit("^(.*) attr (.*)$", parse_view_loc);
    vcAlert.AddVisit("^(.*) comp (.*)$", parse_comp_loc);
    vcAlert.AddVisit("^(.*) db (.*)$",   parse_db_loc);
    vcAlert.AddVisit("^([^ ]*)$",        parse_lib_loc);
}

function Update(d, update_obj) {
    w := vcRegion.Window(d.rgnBrowser);
    vcWin.WithBusy(w, func() {
        /* If this region needs to be updated, it means that
           the init file needs to be updated as well */
        if (null(update_obj)) {
            d.bInitFileDirty = TRUE;
            obj := d.cntx;
            /* Clear the selected object list */
            vlSym.Clear(d.symSelectedObjs);
            d.n_selected_libs = 0;
            d.n_selected_comps = 0;
            d.n_selected_ind_views = 0;
        }
        else
            obj := update_obj;
        vcBrowse.ObjRecalcChildren(d.rgnBrowser, obj, FALSE); 
        enable_cmds(d);
    });
    
}

function Modified(r) {
    d := vcRegion.GetData(r);
    modified := d.bInitFileDirty || (vlSym.Size(d.symDirtyDbs) > 0);
}

function SetOtherRegionDirty(d, name) {
    rgnViewlib := vcWin.OwnerRegion(vcRegion.Window(d.rgnBrowser));
    vlibtop.SetRegionDirty(rgnViewlib, name);
}

/********************************************************************/
/****************  Library Command Definitions **********************/
/********************************************************************/

cmdLibraryAdd :=
   vcWv.Cmd (fac, func(d,e,n) {
       lib := vllm.CntxAddLib(d.cntx, n, FALSE);
       Update(d, nil);
       SetOtherRegionDirty(d, "Configuration");
   },
             "LibraryAdd", "Library Add",
             vcWv.Param ("Library Name", vlStringVT, vcWv.Sticky, ""));
vcCmd.EnableApply(cmdLibraryAdd, TRUE);
vcCmd.VerifyParams(cmdLibraryAdd, func(d,e,n) {
    if (string_eq(string_trim(n), ""))
        vlibThrow(1006, "Library name must not be empty"); 
    if (!null(vllm.CntxFindLib(d.cntx, n)))
        vlibThrow(1007, "Library `" + n + "' already exists"); 
});


cmdLibraryCopy :=
   vcWv.Cmd (fac, func(d,e,n) {
       orig_lib := GetOneSelectedObject(d, vllmLibVT);
       l := vllm.CntxAddLib(d.cntx, n, vllm.LibAllowEclipsing(orig_lib));
       foreach (db; vllm.LibLoopDb(orig_lib)) {
           newdb := vllm.LibAddDb(l, vllm.DbGetName(db),
                                  vllm.DbGetLocation(db), vllm.DbGetType(db),
                                  vllm.DbGetFileName(db), FALSE); 
       }
       foreach (gv; vllm.LibLoopGlobalView(orig_lib))
           vllm.LibAddGlobalView(l, vllm.ViewGetName(gv), ViewGetValueStr(gv));
       foreach (ecn; vllm.LibLoopExcludedCompName(orig_lib))
           vllm.LibExcludeComp(l, ecn);
       vllm.LibReload(l);
       Update(d, nil);
       SetOtherRegionDirty(d, "Configuration");
   },
             "LibraryCopy", "Library Copy",
             vcWv.Param ("New Library Name", vlStringVT, vcWv.Sticky, ""));

vcCmd.VerifyParams(cmdLibraryCopy, func(d,e,n) {
    if (string_eq(string_trim(n), ""))
        vlibThrow(1006, "Library name must not be empty");
    if (!null(vllm.CntxFindLib(d.cntx, n)))
        vlibThrow(1007, "Library `" + n + "' already exists"); 
});

function confirmLibraryDelete(d,e) {
    BuildConfirmRegion(GetSelectedObjects(d, vllmLibVT),
                       "The following libraries will be deleted, OK?",
                       vllm.LibGetName);
}

cmdLibraryDelete := 
   vcWv.CmdWithConfirm (fac, func(d,e) {
       foreach(lib; vscript.list_iter(GetSelectedObjects(d, vllmLibVT))) {
           SelectObject(d, lib, library_class, FALSE);
           vllm.CntxDeleteLib(d.cntx, lib); 
       }
       Update(d, nil);
       SetOtherRegionDirty(d, "Configuration"); },
                        confirmLibraryDelete, TRUE,
                        "LibraryDelete", "Library Delete");

cmdLibraryProps :=
   vcWv.Cmd (fac, func(d,e,eclipse,comps) {
       /* build a list of the library's global views */
       lib := GetOneSelectedObject(d, vllmLibVT);
       vllm.LibSetEclipsing(lib, eclipse);
       vllm.LibClearExcludedComps(lib);
       foreach(cn; vscript.list_iter(vcExclude.GetExcludeList(comps))) 
           vllm.LibExcludeComp(lib, cn);
       vllm.LibReload(lib);
       Update(d, lib); },
             "LibraryProperties", "Library Properties Editor",
             vcWv.Param ("Allow Eclipsing", vlYesNoVT, vcWv.Dynamic, 
                         func(d) {
                             lib := GetOneSelectedObject(d, vllmLibVT);
                             vllm.LibAllowEclipsing(lib); },
                         vcPickEnum.fac),
             vcWv.Param ("Components", vscmVT, vcWv.Dynamic, 
                         func(d) {
                             lib := GetOneSelectedObject(d, vllmLibVT);
                             inclist := nil;
                             foreach (c; vllm.LibLoopSortedComp(lib, FALSE))
                                 inclist = cons(vllm.CompGetName(c), inclist);
                             exclist := nil;
                             foreach (n; vllm.LibLoopExcludedCompName(lib)) 
                                 exclist = cons(n, exclist);
                             cons(inclist, exclist); },
                         vcExclude.fac));

    
cmdLibrarySave :=
   vcWv.Cmd (fac, func(d,e) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           if (d.bInitFileDirty) {
               vllm.WriteInitFile(d.cntx);
               d.bInitFileDirty = FALSE;
           }
           foreach (dbname, dbid; vlSym.Loop(d.symDirtyDbs)) {
               oid := vllm.DbGetVdmObj(dbid);
               vllm.WriteDbFile(d.cntx, dbid);
               vdm.ObjPurgeStatus(oid);
           }
           vlSym.Clear(d.symDirtyDbs); }); },
             "LibrarySave", "Library Save");

cmdLibraryVerify :=
   vcWv.Cmd (fac, func(d,e) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           lib := GetOneSelectedObject(d, vllmLibVT);
           t1 := vscript.vsTimer (vsTimerUnits.Hundredths);
           vlib.DrcRunLibrary(d.drc, d.cntx, lib);
           t2 := vscript.vsTimer (vsTimerUnits.Hundredths);
           show_errors(d, FALSE, (t2-t1)/100.0); }); },
             "LibraryVerify", "Library Verify");


cmdLibraryCheckinRelease := 
   vcWv.Cmd(fac, func(d,e,dest,do_indv,do_ini,notify,msg) {
       lib := GetOneSelectedObject(d, vllmLibVT);
       vcWin.WithBusy(vcEvent.Window(e), func () {
           /* Make sure to save modified data first */
           vcWin.RunCmdline(vcEvent.Window(e), "LibrarySave", e);
           LibCheckinRelease(lib, dest, do_indv, do_ini, msg, notify); }); 
   }, "LibraryCheckinRelease", "Library Checkin & Release",
            vcWv.Param ("Release path", vlStringVT, vcWv.Sticky, ""),
            vcWv.Param ("Release link files", vlYesNoVT, 
                        vcWv.Sticky, vlYesNo.No, vcPickEnum.fac),
            vcWv.Param ("Create ini file", vlYesNoVT, 
                        vcWv.Sticky, vlYesNo.Yes, vcPickEnum.fac),
            vcWv.Param ("Notify", vlYesNoVT, 
                        vcWv.Sticky, vlYesNo.Yes, vcPickEnum.fac),
            vcWv.Param ("Log message", vlStringVT, vcWv.Sticky, "",
                        vtx.fac, height <- 3, width <- 80));

vcCmd.VerifyParams(cmdLibraryCheckinRelease, func(d,e,dest,iv,ini,notify,msg) {
    if (string_eq (msg, ""))
        vlibThrow(1020, "Log message cannot be an empty string"); 
});

menuLibrary :=
    vcWv.MakeMenu ("Library",
                   vcWv.Pin (),
                   list ("&Add",              cmdLibraryAdd),
                   list ("&Copy",             cmdLibraryCopy),
                   list ("&Delete",           cmdLibraryDelete),
                   list ("&Properties",       cmdLibraryProps),
                   list ("&Verify",           cmdLibraryVerify),
                   list ("&SaveModifiedData", cmdLibrarySave),
                   list ("Checkin&Release",   cmdLibraryCheckinRelease));

/********************************************************************/
/**************** Component Command Definitions *********************/
/********************************************************************/

cmdComponentAdd :=
   vcWv.Cmd (fac, func(d,e,n) {
       lib := GetOneSelectedObject(d, vllmLibVT);
       vcBrowse.ObjExpand (d.rgnBrowser, lib);
       c := vllm.LibAddComp(lib, n);
       Update(d, lib);
   },
             "ComponentAdd", "Component Add",
             vcWv.Param ("Component Name", vlStringVT, vcWv.Sticky, ""));

vcCmd.EnableApply(cmdComponentAdd, TRUE);
vcCmd.VerifyParams(cmdComponentAdd, func(d,e,n) {
    if (string_eq(string_trim(n), ""))
        vlibThrow(1010, "Component name must not be empty"); 
    lib := GetOneSelectedObject(d, vllmLibVT);
    if (!null(vllm.LibFindComp(lib, n)))
        vlibThrow(1011, "Component `" + n + "' already exists"); 
});

cmdComponentCopy :=
   vcWv.Cmd (fac, func(d,e,n) {
       c_orig := GetOneSelectedObject(d, vllmCompVT);
       lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c_orig));
       c_new := vllm.LibAddComp(lib, n);
       foreach(v; vllm.CompLoopView(c_orig)) {
           db := vllm.LibFindDb(lib, vllm.ViewGetDbName(v));
           LockDatabase(d, db);
           vllm.DbAddView(db, n,
                          vllm.ViewGetName(v),
                          vllm.ViewGetQualifier(v),
                          vllm.ViewGetData(v));
       }
       Update(d, lib);
   },
             "ComponentCopy", "Component Copy",
             vcWv.Param ("New Component Name", vlStringVT, vcWv.Sticky, ""));

vcCmd.VerifyParams(cmdComponentCopy, func(d,e,n) {
    if (string_eq(string_trim(n), ""))
        vlibThrow(1010, "Component name must not be empty"); 
    c_orig := GetOneSelectedObject(d, vllmCompVT);
    lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c_orig));
    if (!null(vllm.LibFindComp(lib, n)))
        vlibThrow(1011, "Component `" + n + "' already exists"); 
});

function confirmComponentDelete(d,e) {
    BuildConfirmRegion(GetSelectedObjects(d, vllmCompVT),
                       "The following components will be deleted, OK?",
                       vllm.CompGetName);
}

cmdComponentDelete := 
   vcWv.CmdWithConfirm (fac, func(d,e) {
       symLibsAffected := vlSym.StrCreate(3);
       foreach(c; vscript.list_iter(GetSelectedObjects(d, vllmCompVT))) {
           SelectObject(d, c, component_class, FALSE);
           lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c));
           vlSym.Store(symLibsAffected, vllm.LibGetName(lib), lib);
           vllm.LibDeleteComp(lib, c); 
       } 
       foreach (libname, lib; vlSym.Loop(symLibsAffected)) 
           Update(d, lib); 
   }, confirmComponentDelete, TRUE,
                        "ComponentDelete", "Component Delete");

cmdComponentVerify :=
   vcWv.Cmd (fac, func(d,e) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           c := GetOneSelectedObject(d, vllmCompVT);
           t1 := vscript.vsTimer (vsTimerUnits.Hundredths);
           vlib.DrcRunComponent(d.drc, d.cntx, c);
           t2 := vscript.vsTimer (vsTimerUnits.Hundredths);
           show_errors(d, FALSE, (t2-t1)/100.0); }); },
             "ComponentVerify", "Component Verify");

menuComponent :=
    vcWv.MakeMenu ("Component",
                   vcWv.Pin (),
                   list ("&Add",        cmdComponentAdd),
                   list ("&Copy",       cmdComponentCopy),
                   list ("&Delete",     cmdComponentDelete),
                   list ("&Verify",     cmdComponentVerify)); 

/********************************************************************/
/******************* View Command Definitions ***********************/
/********************************************************************/

function GetVtypeList(d) {
    vtlist := nil;
    foreach (vtype; vllm.CntxLoopVtype(d.cntx)) {
        vtypename := vllm.VtypeGetName(vtype);
        vtlist = cons(cons(vtypename, vtypename), vtlist); 
    }
    /* HACK_ALERT: Should be sorted */
    vtlist;
}

function GetSelectedComp(d, alert) {
    /* Need to find either a view or a component selected */
    vlist := GetSelectedObjects(d, vllmViewVT);
    clist := GetSelectedObjects(d, vllmCompVT);
    comp := nil;
    if (length(vlist) > 0) {
        lib := vllm.CntxFindLib(d.cntx, vllm.ViewGetLibName(car(vlist)));
        comp = vllm.LibFindComp(lib, vllm.ViewGetCompName(car(vlist)));
    }
    else if (length(clist) > 0)
        comp = car(clist);
    else if (alert)
        vlibThrow(1003, "Either an attribute or a component must be selected");
    comp;
}

function GetDatabaseList(d) {
    c := GetSelectedComp(d, FALSE);
    lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c));
    dblist := nil;
    foreach(db; vllm.LibLoopDb(lib)) {
        dbname := vllm.DbGetName(db);
        dblist = cons(cons(dbname, dbname), dblist); 
    }
    dblist;
}

function GetSelectedView(d) {
    /* Need to find either a direct or an indirect view */
    if (!null(d.selected_direct_view))
        v := d.selected_direct_view;
    else
        v := GetOneSelectedObject(d, vllmViewVT);
    v;
}

function GetSelectedViewDbName(d) {
    v := GetSelectedView(d);
    vllm.ViewGetDbName(v);
}

function GetSelectedViewName(d) {
    v := GetSelectedView(d);
    vllm.ViewGetName(v);
}

vcWv.MakeEnum("vlibVtypeBaseVT", list("String", "Float"));

function add_vtype(d, e, name, base, units) {
    d.bInitFileDirty = TRUE; 
    vt := vllm.CntxAddVtype(d.cntx, name, convert(vllmBaseVT, base), units);
}

cmdVtypeAdd := 
   vcWv.Cmd (fac, add_vtype, "DefineAttr", "Define Attribute",
             vcWv.Param("Name", vlStringVT, vcWv.Sticky, ""),
             vcWv.Param("Base", vlibVtypeBaseVT, vcWv.Constant, 
                        vlibVtypeBase.String),
             vcWv.Param("Units", vlStringVT, vcWv.Constant, ""));

vcCmd.EnableApply(cmdVtypeAdd, TRUE);
vcCmd.CallOnDialogCreate(cmdVtypeAdd, func(d,e,rgnName,rgnBase,rgnUnits) {
    vcRegion.Enable(rgnUnits, FALSE, TRUE);
    vcPickEnum.CallOnChange(rgnBase, func(newval) {
        vcRegion.Enable(rgnUnits, newval == vlibVtypeBase.Float, TRUE); }); });

vcCmd.VerifyParams(cmdVtypeAdd, func(d, e, name, base, units) {
    if (!null(vllm.CntxFindVtype(d.cntx, name)))
        vlibThrow(1012, "Attribute `" + name + "' already exists"); 
});
 
/*
 *  The DbAddView is actually called in the CmdVerify function to
 *  catch any exceptions.  The action function for this command 
 *  just updates the display.
 */
cmdViewAdd :=
   vcWv.Cmd (fac, func(d,e,dbname,vname,vqual,vstr) {
       if (vstr[0] == '@') {
           c := GetSelectedComp(d, TRUE);
           Update(d, c);
       }
       else {
           vcTable.Set(d.rgnMainRegion, 0, 1, 
                       BuildDirectViews(d, FALSE));
           viewname := vname;
           if (!string_eq(vqual, "")) viewname = viewname + "\\" + vqual;
           vcRegion.SetFocus(vlSym.Fetch(d.symViewReg, viewname),
                             vlSym.Fetch(d.symViewReg, viewname));
       }
   },
             "AttrAdd", "Attribute Add",
             vcWv.Param ("Database", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- "",
                         structure <- GetDatabaseList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Name", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- "",
                         structure <- GetVtypeList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Qualifier", vlStringVT, vcWv.Sticky, ""),
             vcWv.Param ("Value", vlStringVT, vcWv.Sticky, ""));

vcCmd.EnableApply(cmdViewAdd, TRUE);
/* Caution: this routine is run twice: once when the dialog box
 * is about to go away, and once when the command is actually 
 * run.  As a result, we need to avoid calling DbAddView if
 * it's already been called.
 */
vcCmd.VerifyParams(cmdViewAdd, func(d,e,dbname,vname,vqual,vstr) {
    if (string_eq(string_trim(dbname), ""))
        vlibThrow(1008, "Database name must not be empty"); 
    if (string_eq(string_trim(vname), ""))
        vlibThrow(1009, "Attribute name must not be empty"); 

    if (null(vllm.CntxFindVtype(d.cntx, vname))) {
        vcWin.RunCmdline(vcEvent.Window(e), "DefineAttr " + vname, e);
        vlEx.Throw2(vlEx.FAILURE, vllm.CntxGetName(d.cntx), 
                    vlErrSev.Status, "Attribute not defined yet"); 
    }
    
    if (string_eq(string_trim(vstr), ""))
        vlibThrow(1014, "Attribute value must not be empty"); 

    c := GetSelectedComp(d, TRUE);
    lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c));
    db := vllm.LibFindDb(lib, dbname);

    if (string_eqi(vllm.DbGetType(db), "ViewDir"))
        vlibThrow(1021, "Cannot add attributes to ViewDir database");

    v := vllm.CompFindView(c, vname, vqual);
    if (null(v)) {
        /* Perform the actual DbAddView here, to make sure that the view
           can be added (excluded vtype, invalid numeric, duplicate, etc) */
        LockDatabase(d, db); 
        compname := vllm.CompGetName(c);
        vllm.DbAddView(db, compname, vname, vqual, vstr);
        v = vllm.DbFindView(db, compname, vname, vqual); 
        if (null(v)) 
            vlibThrow(1016, "Attribute `" + vname + "' couldn't be added"); 
    }
    else if (!string_eq(ViewGetValueStr(v), vstr))
        vlibThrow(1012, "Attribute `" + vname + "' already exists"); 
});

/*
 *  The DbAddView is actually called in the CmdVerify function to
 *  catch any exceptions.  The action function for this command 
 *  just updates the display.
 */
cmdViewAddLink :=
   vcWv.Cmd (fac, func(d,e,dbname,vname,vqual,loc,class,fname) {
       c := GetSelectedComp(d, TRUE);
       Update(d, c);
   },
             "AttrAddLink", "Attribute Add Link",
             vcWv.Param ("Database", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- "",
                         structure <- GetDatabaseList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Name", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- "",
                         structure <- GetVtypeList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Qualifier", vlStringVT, vcWv.Sticky, ""),
             vcWv.Param ("Location", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac,
                         initial <- "",
                         structure <- GetLocationList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Class", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac,
                         initial <- "",
                         structure <- GetClassList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Filename", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac,
                         initial <- "",
                         structure <- nil,
                         label_fac <- vcCmdline.fac));
vcCmd.EnableApply(cmdViewAddLink, TRUE);
vcCmd.CallOnDialogCreate(cmdViewAddLink, func(d,e,rgnDb,rgnVtype,rgnQual,rgnLoc,rgnClass,rgnFiles) {
    function reset_picklist (new_region) {
        if (!string_eq(vcRegion.Name(new_region), "FileNames")) {
            class := vcRegion.Get(rgnClass, vlStringVT); 
            loc := vcRegion.Get(rgnLoc, vlStringVT);    
            newlist := GetDatabaseFiles(d,class,loc);
            vcRegion.PrefSet(rgnFiles, list (structure <- newlist));
        }
    }
    vcRegion.CallOnFocusChange(rgnClass, reset_picklist);
    vcRegion.CallOnFocusChange(rgnLoc, reset_picklist);
});

/* Caution: this routine is run twice: once when the dialog box
 * is about to go away, and once when the command is actually 
 * run.  As a result, we need to avoid calling DbAddView if
 * it's already been called.
 */
vcCmd.VerifyParams(cmdViewAddLink, func(d,e,dbname,vname,vqual,loc,class,fname) {
    if (string_eq(string_trim(dbname), ""))
        vlibThrow(1008, "Database name must not be empty"); 
    if (string_eq(string_trim(vname), ""))
        vlibThrow(1013, "Attribute name must not be empty"); 

    if (null(vllm.CntxFindVtype(d.cntx, vname))) {
        vcWin.RunCmdline(vcEvent.Window(e), "DefineAttr " + vname, e);
        vlEx.Throw2(vlEx.FAILURE, vllm.CntxGetName(d.cntx), 
                    vlErrSev.Status, "Attribute not defined yet"); 
    }
    
    if (string_eq(string_trim(loc), ""))
        vlibThrow(1017, "Location name must not be empty"); 
    if (string_eq(string_trim(class), ""))
        vlibThrow(1018, "Class name must not be empty"); 
    if (string_eq(string_trim(fname), ""))
        vlibThrow(1019, "Filename must not be empty"); 

    c := GetSelectedComp(d, TRUE);
    lib := vllm.CntxFindLib(d.cntx, vllm.CompGetLibName(c));
    db := vllm.LibFindDb(lib, dbname);

    if (string_eqi(vllm.DbGetType(db), "ViewDir"))
        vlibThrow(1021, "Cannot add attributes to ViewDir database");

    v := vllm.CompFindView(c, vname, vqual);
    vstr := "@" + loc + ":" + class + ":" + fname;
    if (null(v)) {
        /* Perform the actual DbAddView here, to make sure that the view
           can be added (excluded vtype, invalid numeric, duplicate, etc) */
        LockDatabase(d, db); 
        compname := vllm.CompGetName(c);
        vllm.DbAddView(db, compname, vname, vqual, vstr);
        v = vllm.DbFindView(db, compname, vname, vqual); 
        if (null(v)) 
            vlibThrow(1016, "Attribute `" + vname + "' couldn't be added"); 
    }
    else if (!string_eq(ViewGetValueStr(v), vstr))
        vlibThrow(1012, "Attribute `" + vname + "' already exists"); 
});

/*
 *  The LibAddGlobalView is actually called in the CmdVerify 
 *  function to catch any exceptions.  The action function for 
 *  this command just updates the display.
 */
cmdViewAddGlobal :=
   vcWv.Cmd (fac, func(d,e,vname,vstr) {
       lib := GetOneSelectedObject(d, vllmLibVT);
       vcTable.Set(d.rgnMainRegion, 0, 1, 
                   BuildDirectViews(d, TRUE));
       d.bInitFileDirty = TRUE;
   },
             "AttrAddGlobal", "Attribute Add Global",
             vcWv.Param ("Name", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- "",
                         structure <- GetVtypeList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Value", vlStringVT, vcWv.Sticky, ""));

vcCmd.EnableApply(cmdViewAddGlobal, TRUE);
/* Caution: this routine is run twice: once when the dialog box
 * is about to go away, and once when the command is actually 
 * run.  As a result, we need to avoid calling LibAddGlobalView
 * if it's already been called.
 */
vcCmd.VerifyParams(cmdViewAddGlobal, func(d,e,vname,vstr) {
    if (string_eq(string_trim(vname), ""))
        vlibThrow(1013, "Attribute name must not be empty"); 
    if (null(vllm.CntxFindVtype(d.cntx, vname))) {
        vcWin.RunCmdline(vcEvent.Window(e), "DefineAttr " + vname, e);
        vlEx.Throw2(vlEx.FAILURE, vllm.CntxGetName(d.cntx), 
                    vlErrSev.Status, "Attribute not defined yet"); 
    }
    if (string_eq(string_trim(vstr), ""))
        vlibThrow(1014, "Attribute value must not be empty"); 
    lib := GetOneSelectedObject(d, vllmLibVT);
    gv := vllm.LibFindGlobalView(lib, vname);
    if (null(gv)) {
        vllm.LibAddGlobalView(lib, vname, vstr);
        gv = vllm.LibFindGlobalView(lib, vname);
        if (null(gv)) 
            vlibThrow(1016, "Attribute `" + vname + "' couldn't be added"); 
    }
    else if (!string_eq(ViewGetValueStr(gv), vstr))
        vlibThrow(1012, "Attribute `" + vname + "' already exists"); 
});

cmdViewProps :=
   vcWv.Cmd (fac, func(d,e,dbname,vname,vqual,vstr) {
       v := GetSelectedView(d);
       lib := vllm.CntxFindLib(d.cntx, vllm.ViewGetLibName(v));
       db := vllm.LibFindDb(lib, vllm.ViewGetDbName(v));
       compname := vllm.ViewGetCompName(v);
       LockDatabase(d, db);
       if (vllm.ViewGetBase(v) == vllmBase.Indirect)
           SelectObject(d, v, indirect_view_class, FALSE);
       vllm.DbDeleteView(db, v);
       vllm.DbAddView(db, compname, vname, vqual, vstr);
       v = vllm.DbFindView(db, compname, vname, vqual); 
       c := vllm.LibFindComp(lib, compname);
       if (vllm.ViewGetBase(v) == vllmBase.Indirect)
           Update(d, c);
   },
             "AttrProperties", "Attribute Properties",
             vcWv.Param ("Database", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- GetSelectedViewDbName,
                         structure <- GetDatabaseList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Name", vscmVT, vcWv.Dynamic, 
                         func(d) d;, vcPullDown.fac, 
                         initial <- GetSelectedViewName,
                         structure <- GetVtypeList,
                         label_fac <- vcCmdline.fac),
             vcWv.Param ("Qualifier", vlStringVT, vcWv.Dynamic,
                         func(d) {
                             v := GetSelectedView(d);
                             q := vllm.ViewGetQualifier(v);
                             if (null(q)) q := "";
                             q; } ),
             vcWv.Param ("Value", vlStringVT, vcWv.Dynamic,
                         func(d) {
                             v := GetSelectedView(d);
                             ViewGetValueStr(v); } ));

vcCmd.VerifyParams(cmdViewProps, func(d,e,dbname,vname,vqual,vstr) {
    if (string_eq(string_trim(dbname), ""))
        vlibThrow(1008, "Database name must not be empty"); 
    if (string_eq(string_trim(vname), ""))
        vlibThrow(1013, "Attribute name must not be empty"); 
    if (null(vllm.CntxFindVtype(d.cntx, vname))) {
        vcWin.RunCmdline(vcEvent.Window(e), "DefineAttr " + vname, e);
        vlEx.Throw2(vlEx.FAILURE, vllm.CntxGetName(d.cntx), 
                    vlErrSev.Status, "Attribute not defined yet"); 
    }
    if (string_eq(string_trim(vstr), ""))
        vlibThrow(1014, "Attribute value must not be empty"); 
});

function confirmViewDelete(d,e) {
    vlist := GetSelectedObjects(d, vllmViewVT);
    if (!null(d.selected_direct_view))
        vlist = cons(d.selected_direct_view, vlist);
    BuildConfirmRegion(vlist, "The following attributes will be deleted, OK?",
                       CombinedViewName);
}

cmdViewDelete := 
   vcWv.CmdWithConfirm (fac, func(d,e) {
       vlist := GetSelectedObjects(d, vllmViewVT);
       if (!null(d.selected_direct_view))
           vlist = cons(d.selected_direct_view, vlist);
       not_deleted_views := nil;
       foreach(v; vscript.list_iter(vlist)) {
           direct_view := (vllm.ViewGetBase(v) != vllmBase.Indirect);
           lib := vllm.CntxFindLib(d.cntx, vllm.ViewGetLibName(v));
           if (vllm.ViewIsGlobal(v)) {
               vllm.LibDeleteGlobalView(lib, vllm.ViewGetName(v));
               vcTable.Set(d.rgnMainRegion, 0, 1, 
                           BuildDirectViews(d, TRUE));
               d.bInitFileDirty = TRUE;
           }
           else {
               db := vllm.LibFindDb(lib, vllm.ViewGetDbName(v));

               if (string_eqi(vllm.DbGetType(db), "ViewDir")) 
                   not_deleted_views = cons(CombinedViewName(v), 
                                            not_deleted_views);
               else {
                   c := vllm.LibFindComp(lib, vllm.ViewGetCompName(v));
                   LockDatabase(d, db);
                   if (direct_view) {
                       vllm.DbDeleteView(db, v);
                       vcTable.Set(d.rgnMainRegion, 0, 1, 
                                   BuildDirectViews(d, FALSE));
                   }
                   else {
                       SelectObject(d, v, indirect_view_class, FALSE);
                       vllm.DbDeleteView(db, v);
                       Update(d, c); 
                   }
               } 
           } 
       } 
       if (length(not_deleted_views) != 0) {
           msg := "Could not delete these ViewDir attributes:";
           foreach (a; vscript.list_iter(not_deleted_views))
               msg = msg + " " + a;
           error2("viewlib", vlErrSev.Warning, 1022, nil, msg);
       }
   },
                        confirmViewDelete, TRUE,
                        "AttrDelete", "Attribute Delete");

cmdViewVerify :=
   vcWv.Cmd (fac, func(d,e) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           v := GetSelectedView(d);
           t1 := vscript.vsTimer (vsTimerUnits.Hundredths);
           vlib.DrcRunAttr(d.drc, d.cntx, v);
           t2 := vscript.vsTimer (vsTimerUnits.Hundredths);
           show_errors(d, FALSE, (t2-t1)/100.0); }); },
             "AttrVerify", "Attribute Verify");

cmdViewEdit :=
   vcWv.Cmd (fac, func(d,e) {
       v := GetOneSelectedObject(d, vllmViewVT);
       if (vllm.ViewGetBase(v) == vllmBase.Indirect) 
           vcBrowse.ObjEdit(d.rgnBrowser, v);
   },
             "LinkEdit", "Link Edit");

cmdViewManage :=
   vcWv.Cmd (fac, func(d,e) {
       v := GetOneSelectedObject(d, vllmViewVT);
       oid := vllm.ViewGetVdmObj(d.cntx, v);
       rgnViewlib := vcWin.OwnerRegion(vcRegion.Window(d.rgnBrowser));
       vlibtop.SelectTabAndLocate(rgnViewlib, "Management", oid);
   },
             "LinkManage", "Link Manage");

cmdViewCheckinRelease := 
   vcWv.Cmd (fac, func(d,e,dest,msg,notify) {
       v := GetOneSelectedObject(d, vllmViewVT);
       vcWin.WithBusy(vcEvent.Window(e), func () {
           ViewCheckinRelease(d.cntx, v, dest, msg, notify); }); 
   }, "LinkCheckinRelease", "Link Checkin & Release",
            vcWv.Param ("Release path", vlStringVT, vcWv.Sticky, ""),
            vcWv.Param ("Notify", vlYesNoVT, 
                        vcWv.Sticky, vlYesNo.Yes, vcPickEnum.fac),
            vcWv.Param ("Log message", vlStringVT, vcWv.Sticky, "",
                        vtx.fac, height <- 3, width <- 80));

menuView :=
    vcWv.MakeMenu ("Attribute",
                   vcWv.Pin (),
                   list ("Add &Global",     cmdViewAddGlobal),
                   list ("&Add",            cmdViewAdd),
                   list ("Add &Link",       cmdViewAddLink),
                   list ("&Delete",         cmdViewDelete),
                   list ("&Properties",     cmdViewProps),
                   list ("&Verify",         cmdViewVerify),
                   list ("&Edit",           cmdViewEdit),
                   list ("&CheckinRelease", cmdViewCheckinRelease),
                   list ("&Manage",         cmdViewManage));  

cmdPopupViewMenu :=
   vcWv.Cmd(fac, func(d,e) vcMenuPop.Create(menuView, e);,
            "menu-view");

/********************************************************************/
/******************* Utility Command Definitions ********************/
/********************************************************************/

cmdVerifyAll :=
   vcWv.Cmd (fac, func(d,e) {
       vcWin.WithBusy(vcEvent.Window(e), func() {
           t1 := vscript.vsTimer (vsTimerUnits.Hundredths);
           vlib.DrcRunAll(d.drc, d.cntx);
           t2 := vscript.vsTimer (vsTimerUnits.Hundredths);
           show_errors(d, FALSE, (t2-t1)/100.0); }); },
             "VerifyAll", "Verify All");

cmdShowDisabled :=
   vcWv.Cmd (fac, 
             func(d,e) show_errors(d, TRUE, 0.0);,
             "ShowDisabled", "Show Disabled");

cmdEditPrefs := 
   vcWv.Cmd (fac, func(d,e) { 
       vcPref.SetEditor (vlib.DrcPrefs(d.drc), "sort_order", vcSetOrder.fac);
       w := vcPref.CreateWindowAux (vcEvent.Window (e), fac, 
                                    vlib.DrcPrefs(d.drc));
       vcWin.SetTitle (w, "Preference Editor: Library DRC Preferences"); },
             "EditPreferences", "Edit Preferences");

cmdEditSeverities := 
   vcWv.Cmd (fac, func(d,e) { 
       w := vcPref.CreateWindowAux (vcEvent.Window (e), fac, 
                                    vlib.DrcErrPrefs(d.drc));
       vcWin.SetTitle(w, "Preference Editor: Library DRC Error Severities"); },
             "EditSeverities", "Edit Severities");

/* This function enables each command based on which items are selected */
function enable_cmds(d) {

    /* Enable these commands if only one library is selected: */
    vcCmd.Enable (cmdLibraryCopy,     d.n_selected_libs == 1);
    vcCmd.Enable (cmdLibraryProps,    d.n_selected_libs == 1);
    vcCmd.Enable (cmdLibraryVerify,   d.n_selected_libs == 1); 
    vcCmd.Enable (cmdLibraryCheckinRelease,  d.n_selected_libs == 1); 
    vcCmd.Enable (cmdViewAddGlobal,   d.n_selected_libs == 1); 

    /* Only enable component add if the selected library has a database */
    function enable_comp_add() {
        retval := FALSE;
        if (d.n_selected_libs == 1) {
            lib := GetOneSelectedObject(d, vllmLibVT);
            foreach (db; vllm.LibLoopDb(lib))
                retval = TRUE;
        }
        retval;
    }
    vcCmd.Enable (cmdComponentAdd,    enable_comp_add()); 

    /* Enable these commands if at least one library is selected: */
    vcCmd.Enable (cmdLibraryDelete,   d.n_selected_libs >= 1);

    /* Enable these commands only if something has been modified */ 
    vcCmd.Enable (cmdLibrarySave,     (d.bInitFileDirty ||
                                       vlSym.Size(d.symDirtyDbs) > 0)); 

    /* Enable these commands if only one component is selected: */
    vcCmd.Enable (cmdComponentCopy,   d.n_selected_comps == 1);
    vcCmd.Enable (cmdComponentVerify, d.n_selected_comps == 1); 
    vcCmd.Enable (cmdViewAdd,         d.n_selected_comps == 1); 
    vcCmd.Enable (cmdViewAddLink,     d.n_selected_comps == 1); 

    /* Enable these commands if at least one component is selected: */
    vcCmd.Enable (cmdComponentDelete, d.n_selected_comps >= 1);

    /* Enable these commands if only one view is selected: */
    vcCmd.Enable (cmdViewProps,       ((d.n_selected_ind_views == 1) ||
                                       ((!null(d.selected_direct_view)) &&
                                        (!vllm.ViewIsGlobal(d.selected_direct_view)))));
    vcCmd.Enable (cmdViewVerify,      ((d.n_selected_ind_views == 1) ||
                                       (!null(d.selected_direct_view)))); 
    vcCmd.Enable (cmdViewEdit,        d.n_selected_ind_views == 1); 
    vcCmd.Enable (cmdViewManage,      d.n_selected_ind_views == 1); 
    vcCmd.Enable (cmdViewCheckinRelease, d.n_selected_ind_views == 1); 

    /* Enable these commands if at least one view is selected: */
    vcCmd.Enable (cmdViewDelete,      ((d.n_selected_ind_views >= 1) ||
                                       (!null(d.selected_direct_view))));

    /* Update the menus */
    win := vcRegion.Window(d.rgnBrowser);
    if (!null(win)) vcWin.UpdateMenus (win);
}
    

/********************************************************************/
/****************  Browser Class Definitions ************************/
/********************************************************************/

/*
 * These strings are used as drag data, so that they may be easily
 * differentiated.
 */
define_string ("vllmDragLibVT");
define_string ("vllmDragCompVT");
define_string ("vllmDragIndViewVT");

function get_vlib_data() {
    foreach (w; vcWin.Loop ())
        if ((r := vcRegion.FindFac (vcWin.OwnerRegion (w), fac)) != nil)
            return (vcRegion.GetData (r));

    vlibThrow(1005, "Cannot find ViewLibrarian region");
}

/*  
 *  Indirect View Class
 */
function indirect_view_icon (v) {
    "vllmlink";
}

function drag_ind_view (v)
    convert (vllmDragIndViewVT, vllm.ViewGetName (v));

function indirect_view_edit(v, notify, class) {
    d := get_vlib_data();
    oid := vllm.ViewGetVdmObj(d.cntx, v);
    if (!null(oid)) {
        <wk_status, sm_status> := vdm.ObjGetStatus (oid);
        if ((sm_status != vdm.SMStatus.Uncontrolled) &&
            (wk_status == vdm.WorkStatus.Empty))
            vdm.ObjCheckoutLock(oid);
        win := vcRegion.Window(d.rgnBrowser);
        err_strs := "Error invoking viewer on object `" + 
            CombinedViewName(v) + "':\n\n";
        vdm.ObjInvokeEditor (oid, nil,
            func(output)
                /* Buffer the error messages until exit */
                err_strs = err_strs + output;,
            func(mode, code)
            {
                if ((mode == vcSysMode.Exited) && (code == 0))
                    notify();
                else
                {
                    death_msg := ((mode == vcSysMode.Signaled)
                                    ? "killed with signal "
                                    : "exited with status ")
                        + number2string (code);
                    err_strs = err_strs + "\nViewer " + death_msg;
                    notify();
                    
                    /* Important: don't throw an exception here! */
                    vcErr.Report(win, nil, 
                                 vlErr.Create(nil, "viewlib", 1023, 
                                              vlErrSev.Error, err_strs));
                }
            });
    }
}

indirect_view_class := 
   vcBrowse.ClassCreate ("Link",
                         get_name      <- CombinedViewName,
                         menu          <- menuView,
                         drag_data     <- drag_ind_view,
                         get_icon      <- indirect_view_icon,
                         edit          <- indirect_view_edit,
                         vertex_prefs  <-
                         list (raster_font <- vgFont.small,
                               text_color  <- vgColor.Green));
/*  
 *  Component Class
 */
iterator component_children(comp, sub_class) {
    foreach (v; vllm.CompLoopView(comp))
        if (vllm.ViewGetBase(v) == vllmBase.Indirect)
            iterate(v, indirect_view_class);
}

function drag_comp (comp)
    convert (vllmDragCompVT, vllm.CompGetName (comp));

component_class := 
   vcBrowse.ClassCreate ("Component",
                         children      <- component_children,
                         get_name      <- vllm.CompGetName,
                         menu          <- menuComponent,
                         icon          <- "vllmcomp",
                         drag_data     <- drag_comp,
                         vertex_prefs  <-
                         list (raster_font <- vgFont.small,
                               text_color  <- vgColor.LightMagenta));

/*  
 *  Library Class
 */
function drag_lib (lib)
    convert (vllmDragLibVT, vllm.LibGetName (lib));

iterator library_children(lib, sub_class) {
    foreach (c; vllm.LibLoopSortedComp (lib, FALSE))
        iterate (c, component_class);
}

library_class := 
   vcBrowse.ClassCreate ("Library",
                         children      <- library_children,  
                         get_name      <- vllm.LibGetName,
                         menu          <- menuLibrary,
                         drag_data     <- drag_lib,
                         icon          <- "vllmlib",
                         vertex_prefs  <-
                         list (raster_font <- vgFont.small,
                               text_color  <- vgColor.Cyan));
                                
/********************************************************************/
/***************** Direct View Regions/Functions ********************/
/********************************************************************/

function LockDatabase(d, db) {
    oid := vllm.DbGetVdmObj(db);
    <wk_status, sm_status> := vdm.ObjGetStatus (oid);
    if (sm_status != vdm.SMStatus.LockedByYou)
        vdm.ObjCheckoutLock(oid);
    vlSym.Store(d.symDirtyDbs, vllm.DbGetName(db), db);
    SetOtherRegionDirty(d, "Management");
}

function UpdateViewValue(d) {
    if ((d.bDirectViewValueChanged) &&
        (!null (d.rgnLastChanged)) &&
        (!vcRegion.Destroyed (d.rgnLastChanged))) {
        v := vlSym.Fetch(d.symViewObj, vcRegion.Name(d.rgnLastChanged));
        global := vllm.ViewIsGlobal(v);

        /* Get all of the information from the old view, delete
           the old view, then create a new view */
        declare compname;
        vname    := vllm.ViewGetName(v);
        vqual    := vllm.ViewGetQualifier(v);
        oldval   := ViewGetValueStr(v);
        lib := vllm.CntxFindLib(d.cntx, vllm.ViewGetLibName(v));

        if (global) 
            vllm.LibDeleteGlobalView(lib, vname);
        else {
            compname = vllm.ViewGetCompName(v);
            db := vllm.LibFindDb(lib, vllm.ViewGetDbName(v));
            LockDatabase(d, db);
            vllm.DbDeleteView(db, v); 
        }
        newValue := vcRegion.Get(d.rgnLastChanged, vlStringVT);
        if (!null(newValue)) {
            /* if we can't add it, reset to the old value */
            catch (vlEx.FAILURE) {
                if (global)
                    vllm.LibAddGlobalView(lib, vname, newValue);
                else
                    vllm.DbAddView(db, compname, vname, vqual, newValue);
            }
            exception {
                if (global)
                    vllm.LibAddGlobalView(lib, vname, oldval);
                else
                    vllm.DbAddView(db, compname, vname, vqual, oldval);
                vcRegion.Set(d.rgnLastChanged, oldval);
                if (global)
                    d.selected_direct_view = 
                        vllm.LibFindGlobalView(lib, vname);
                else
                    d.selected_direct_view = 
                        vllm.DbFindView(db, compname, vname, vqual);
                vlSym.Store(d.symViewObj, 
                            CombinedViewName(d.selected_direct_view), 
                            d.selected_direct_view);
                vlEx.Throw (vlEx.FAILURE, vlErr.CurrentError());
            }
            if (global) {
                d.bInitFileDirty = TRUE;
                d.selected_direct_view = vllm.LibFindGlobalView(lib, vname);
            }
            else 
                d.selected_direct_view = 
                    vllm.DbFindView(db, compname, vname, vqual);
            vlSym.Store(d.symViewObj, 
                        CombinedViewName(d.selected_direct_view), 
                        d.selected_direct_view);
        }
        else
            vlibThrow(1015, "Attribute value for " + CombinedViewName(v) +
                      " must not be empty"); 
    }
    d.bDirectViewValueChanged = FALSE;
    d.rgnLastChanged = nil;
}
        
function HandleCmdlineChangeFn(d, rgnCmdline) {
    func(sNewValue) {
        d.bDirectViewValueChanged = TRUE;
        d.rgnLastChanged = rgnCmdline;
    };
}

function HandleCmdlineReturnFn(d) {
    func(ev, sNewValue) {  
        UpdateViewValue(d);  
    };
}

readonly_bindings := vcBind.Create();
vcBind.Set(readonly_bindings, '\^A', vcBind.Get(vcCmdline.bindings, '\^A'));
vcBind.Set(readonly_bindings, '\^B', vcBind.Get(vcCmdline.bindings, '\^B'));
vcBind.Set(readonly_bindings, '\^E', vcBind.Get(vcCmdline.bindings, '\^E'));
vcBind.Set(readonly_bindings, '\^F', vcBind.Get(vcCmdline.bindings, '\^F'));
vcBind.Set(readonly_bindings, '\leftdown', 
           vcBind.Get(vcCmdline.bindings, '\leftdown'));
readonly_msg_cmd := vcWv.Cmd(vcCmdline.fac, func(d,e) { 
    vcWin.ClearStatus(vcEvent.Window(e));
    vcWin.AddStatus(vcEvent.Window(e), "Attribute is read-only");
} );
foreach (i; vcWv.Printable ()) 
    vcBind.Set (readonly_bindings, i, readonly_msg_cmd);

function BuildDirectViews(d, global_only) {
    function DirectViewRegion(v) {
        vname := CombinedViewName(v);

        /* Make the cmdline region read-only if it comes from ViewDir */
        is_readonly := FALSE;
        lib := vllm.CntxFindLib(d.cntx, vllm.ViewGetLibName(v));
        db := vllm.LibFindDb(lib, vllm.ViewGetDbName(v));

        if (!vllm.ViewIsGlobal(v) && string_eqi(vllm.DbGetType(db), "ViewDir"))
            is_readonly := TRUE;
            
        rgnViewValue := vcRegion.Create(vname, vcCmdline.fac, 
                                        ViewGetValueStr(v));
        vcCmdline.CallOnChange(rgnViewValue, 
                               HandleCmdlineChangeFn(d, rgnViewValue));
        vcCmdline.CallOnReturn(rgnViewValue, 
                               HandleCmdlineReturnFn (d));
        if (is_readonly) 
            vcRegion.SetBindings(rgnViewValue, readonly_bindings);
        rgnViewName :=  vcRegion.Create("VName", vcStaticText.fac, 
                                        vname + ":",
                                        width <- 15, 
                                        align <- vgAlign.MiddleRight);
        vcRegion.Background (rgnViewValue, vgAllColor.Background, TRUE);
        color := (vllm.ViewIsGlobal(v)) ? 
            vgAllColor.Red : vgAllColor.Foreground;
        vcRegion.Color (rgnViewValue, color);
        font := (vllm.ViewIsGlobal(v)) ? 
            vgFont.smallbif : vgFont.smallbf;
        vcRegion.Font (rgnViewName, font);
        rgnViewEditor := vcTable.Make("view_editor", 1, 2,
                                      rgnViewName, rgnViewValue);
    }

    vlist := nil;
    vlSym.Clear(d.symViewObj);

    if (global_only) {
        liblist := GetSelectedObjects(d, vllmLibVT);
        if (length(liblist) == 1) {
            foreach (gv; vllm.LibLoopGlobalView(car(liblist))) {
                vname := CombinedViewName(gv);
                vlist = cons (vname, vlist);
                vlSym.Store(d.symViewReg, vname, DirectViewRegion(gv));
                vlSym.Store(d.symViewObj, vname, gv);
            }
        }
    }
    else {
        c := GetSelectedComp(d, FALSE);
        if (!null(c)) {
            foreach(v; vllm.CompLoopView(c)) {
                vname := CombinedViewName(v);
                vlSym.Store(d.symViewObj, vname, v);
                if ((vllm.ViewGetBase(v) == vllmBase.String) ||
                    (vllm.ViewGetBase(v) == vllmBase.Float)) {
                    vlist = cons (vname, vlist);
                    vlSym.Store(d.symViewReg, vname, DirectViewRegion(v));
                }
            }
        }
    }
    vlist = vscript.Sort(vlist, string_le); 
    if (length(vlist) == 0) 
        tbl := vcRubber.Create("", 0, 0);
    else {
        tbl := vcTable.Create("Attributes", length(vlist), 1);
        foreach(v; vscript.list_iter(vlist)) {
            r := vlSym.Fetch(d.symViewReg, v);
            if (!null(r)) vcTable.Add(tbl, r);
        }
    }
    scrlDv := vcRegion.Create("scrl", vcScroll.fac, tbl,
                              align <- vgAlign.UpperLeft,
                              horizontal <- vcScroll.Presence.Never,
                              show_border <- TRUE, inner_border <- TRUE,
                              size <- vgSize.Create(200, 100, 200, 60, 
                                                    1000, 1000));
    label := (global_only) ? "Global Attributes" : "Attributes";
    rgn := vcRegion.Create(label, vcGroupBox.fac, scrlDv,
                           label <- label);
    vcRegion.CallOnFocusChange(rgn, func(r) { 
        if (d.bDirectViewValueChanged)
            UpdateViewValue(d);
        if (!null(r))
            d.selected_direct_view = 
                vlSym.Fetch(d.symViewObj, vcRegion.Name(r));
        enable_cmds(d);
    });

    vcBind.Set(bindings, '\rightdown', cmdPopupViewMenu);
    rgn;
}

function show_errors(d, show_disabled, elapsed) {
    stream := vlErr.StreamCreate();
    vlib.DrcShowErrors(d.drc, stream, show_disabled);
    declare n, w, e, i;
    vlErr.StreamSummary(stream, &n, &w, &e, &i);
    function plural(num, str) {
        pstr := (num == 1) ? "" : "s";
        newstr := number2string(num) + " " + str + pstr;
    }
    msg := 
        plural(e, "error") + ", " +
            plural(w, "warning") + ", " +
                plural(n, "note") + "; elapsed time: " +
                    sprintf("%.2fs", elapsed);
    win := vcRegion.Window(d.rgnMainRegion);
    if (!null(win)) {
        vcWin.ClearStatus(win);
        vcWin.AddStatus(win, msg);
    } 
}

function _Destroy(this) {
    vlib.DrcDestroy(this.drc); 
}

function SelectObject(this, obj, class, on_off) {
    add := vlSym.Size(this.symSelectedObjs);
    if (on_off) 
        vlSym.Store (this.symSelectedObjs, obj, class);
    else 
        vlSym.Remove (this.symSelectedObjs, obj);
    add = vlSym.Size(this.symSelectedObjs) - add;
    
    if (class == library_class) 
        this.n_selected_libs = this.n_selected_libs + add;
    else if (class == component_class) 
        this.n_selected_comps = this.n_selected_comps + add;
    else if (class == indirect_view_class)
        this.n_selected_ind_views = this.n_selected_ind_views + add; 
    
    if (on_off) {
        if (class == library_class)
            vcTable.Set(this.rgnMainRegion, 0, 1, 
                        BuildDirectViews(this, TRUE));
        else if (class == component_class)
            vcTable.Set(this.rgnMainRegion, 0, 1, 
                        BuildDirectViews(this, FALSE));
    }
    enable_cmds(this);
}

function _Create(vllmcntx, region) {
    this := struct {
        cntx = vllmcntx;
        bInitFileDirty  = FALSE;                /* data in ini file changed? */
        symDirtyDbs     = vlSym.StrCreate (11); /* all modified databases    */
        symSelectedObjs = vlSym.ObjCreate (11); /* all selected objects      */
        n_selected_libs      = 0;
        n_selected_comps     = 0;
        n_selected_ind_views = 0;
        selected_direct_view = nil;            
        bDirectViewValueChanged = FALSE;        /* has a direct view changed */
        symViewObj = vlSym.StrCreate (67);      /* Maps name to view object  */
        symViewReg = vlSym.StrCreate (67);      /* Maps viewname to region   */
        rgnMainRegion = region;
        rgnLastChanged = nil;
        objLastVisited = nil;
        rgnDirectViews;
        drc = vlib.DrcCreate(vllm.CntxGetName(vllmcntx),
                             vlibDrcUserInit);
        rgnBrowser
    };

    /*  
     *  Context (Search Space) Class
     */
    iterator cntx_children(cntx, sub_class) {
        foreach (lib; vllm.CntxLoopLib (cntx))
            iterate (lib, library_class);
    }
    
    cntx_class :=
        vcBrowse.ClassCreate ("Cntx",
                              parse    <- (func(name) this.cntx;), 
                              get_name <- vllm.CntxGetName,
                              children <- cntx_children);

    this.rgnBrowser = vcRegion.Create("lib_comp", vcBrowse.fac,
                                      cons (this.cntx, "Cntx"), 
                                      hide_root <- TRUE, 
                                      exclusive <- FALSE,
                                      drags     <- TRUE, 
                                      macro_on  <- TRUE,
                                      show_info <- FALSE,
                                      size <- vgSize.Create(200, 100, 200, 60, 
                                                            1000, 1000));
    vcBrowse.CallOnSelect (this.rgnBrowser, func(obj, class, e, on_off) {
        SelectObject(this, obj, class, on_off); });
    this.rgnDirectViews = BuildDirectViews(this, FALSE);
    this.rgnMainRegion = vcTable.Make("Contents", 1, 2, 
                                      this.rgnBrowser, this.rgnDirectViews);
    vcTable.Col (this.rgnMainRegion, vcTable.All, 
                 vcTable.Border.Adjustable, 5, 5, 5,
                 vgAlign.UpperLeft);
    vcRegion.AddChild(region, this.rgnMainRegion);
    AddVisitFuncs(this);
    this;
}

vcCmd.SetHelp(cmdLibraryAdd,      "Create a new library");
vcCmd.SetHelp(cmdLibraryCopy,     "Copy selected library to new name");
vcCmd.SetHelp(cmdLibraryDelete,   "Delete selected libraries");
vcCmd.SetHelp(cmdLibraryProps,    "Edit properties of selected library");
vcCmd.SetHelp(cmdLibrarySave,     "Save modified data to disk");
vcCmd.SetHelp(cmdLibraryVerify,   "Run consistency checks for this library");
vcCmd.SetHelp(cmdLibraryCheckinRelease, 
              "Checkin modified data and release data to a release area");
vcCmd.SetHelp(cmdComponentAdd,    "Create new component in current library");
vcCmd.SetHelp(cmdComponentCopy,   "Copy selected component");
vcCmd.SetHelp(cmdComponentDelete, "Delete selected components");
vcCmd.SetHelp(cmdComponentVerify, "Run consistency checks for this component");
vcCmd.SetHelp(cmdViewAdd,         "Add attribute to selected component");
vcCmd.SetHelp(cmdViewAddLink,     "Add attribute link to selected component");
vcCmd.SetHelp(cmdViewAddGlobal,   "Add global attribute to selected library");
vcCmd.SetHelp(cmdViewProps,       "Edit properties of selected attribute");
vcCmd.SetHelp(cmdViewDelete,      "Delete selected attributes");
vcCmd.SetHelp(cmdViewVerify,      "Run consistency checks for this attribute");
vcCmd.SetHelp(cmdViewEdit,        "Invoke editor for this link");
vcCmd.SetHelp(cmdViewManage,      "Find this object in Management region");
vcCmd.SetHelp(cmdViewCheckinRelease, 
              "Checkin if required, and release this file to a release area");
vcCmd.SetHelp(cmdVerifyAll,       "Run all consistency checks");
vcCmd.SetHelp(cmdShowDisabled,    "Show disabled errors from previous run");
vcCmd.SetHelp(cmdEditPrefs,       "Edit consistency check preferences");
vcCmd.SetHelp(cmdEditSeverities,  "Edit consistency check severities");



