vcWv.RequireFac(wv, "vcPref");
vcWv.RequireFac(wv, "vlibBrowse");
vcWv.RequireFac(wv, "vlibDbBrowse");
vcWv.RequireFac(wv, "vdata");  

function SelectTabAndLocate(r, tabname, obj) {
    d := vcRegion.GetData(r);
    vcNoteTab.SelectByString(d.rgnNotebook, tabname);
    if (string_eq(tabname, "Management") &&
        !null(obj))
        vdata.SelectObject(d.rgnManagement, obj);
}

function SetRegionDirty(r, name) {
    d := vcRegion.GetData(r);

    if (string_eqi(vcRegion.Name(d.rgnCurrentRegion), name)) {
        if (string_eqi(name, "Contents"))
            vlibBrowse.Update(vcRegion.GetData(d.rgnCurrentRegion), nil);
        else if (string_eqi(name, "Configuration"))
            vlibDbBrowse.Update(vcRegion.GetData(d.rgnCurrentRegion), nil);
    }
    else
        vlSym.Store(d.symDirtyRgns, name, name);
}

function _Modified(d, r) {
    modified := vlibBrowse.Modified(d.rgnContents);
    
    if (modified) {
        msg := 
"
   Some library data has changed without being saved. 
   You may discard your changes now by selected Accept,
   or save the changes by selecting Cancel, then choosing
   the Library=>SaveModifiedData menu pick.
";
        vcWin.SetConfirmMessage(vcRegion.Window(r), msg);
    }
    modified;
}

function _Destroy(this) {
    vllm.CntxFree(this.cntx);
}

function InitializePrefs(this) {
    vlPref.Replace(prefs, "notify_users", vlStringVT, 
                   "List of users to notify when releasing", "");
}

function _Create(name, region) {

    this := struct {
        rgnNotebook;                        /* top level region              */
        rgnContents;                        /* components and views          */
        rgnConfiguration;                   /* databases                     */
        rgnManagement;                      /* data management               */
        rgnThis = region;                   /* hang onto region passed in    */
        rgnCurrentRegion;                   /* currently exposed region      */
        symDirtyRgns = vlSym.StrCreate (3); /* modified regions              */
        cntx = vllm.CntxCreate(name)        /* library management context    */
    };
    InitializePrefs(this);
    vllm.CntxInit(this.cntx);
    this.rgnContents =
        vcRegion.Create ("Contents", vlibBrowse.fac, this.cntx,
                         macro_on <- TRUE);
    
    this.rgnConfiguration = 
        vcRegion.Create ("Configuration", vlibDbBrowse.fac, this.cntx,
                         macro_on <- TRUE);
                     
    this.rgnManagement = 
        vcRegion.Create("Management", vdata.fac, 
                        vllm.CntxGetVdmCntx(this.cntx));  
                     
    this.rgnNotebook = vcRegion.Create("notebook", vcNotebook.fac,
                                       vgAlign.UpperCenter, 
                                       macro_on <- TRUE);
    vcNotebook.AddEntry(this.rgnNotebook, "Contents", this.rgnContents);
    vcNotebook.AddEntry(this.rgnNotebook, "Configuration", 
                        this.rgnConfiguration);
    vcNotebook.AddEntry(this.rgnNotebook, "Management", 
                        this.rgnManagement);  
    vcNoteTab.CallOnSelect(this.rgnNotebook, func(r) {
        if (!null(r)) {
            name := vlSym.Remove(this.symDirtyRgns, vcRegion.Name(r));
            if (!null(name)) {
                if (string_eqi(name, "Contents"))
                    vlibBrowse.Update(vcRegion.GetData(r), nil);
                else if (string_eqi(name, "Configuration"))
                    vlibDbBrowse.Update(vcRegion.GetData(r), nil);
            }
            this.rgnCurrentRegion = r;
        }
    });
    vcRegion.AddChild (this.rgnThis, this.rgnNotebook);
    vcRegion.FocusInhibit (this.rgnThis);

    this;
} /* _Create */

facname := "vlib";
load (facname + "bt.vs");
load (facname + "mn.vs");
vcsa.LoadCustomizations (facname, current_package ());

