function vlibThrow(num, args...) 
    error2 ("viewlib", vlErrSev.Error, num, nil, apply (sprintf, args));
 

function GetSelectedObjects(d, type_or_class) {
    objlist := nil;

    foreach (obj, class; vlSym.Loop(d.symSelectedObjs)) {
        if ((type_or_class == nil) || (type_or_class == class) ||
            (istype(type_or_class, vlTypespecVT) && 
             istype(obj, type_or_class)))
            objlist = cons (obj, objlist);
    }
    objlist;
}

function GetOneSelectedObject(d, type_or_class) {
    objlist := GetSelectedObjects(d, type_or_class);
    if (length(objlist) != 1) {
        str := "Exactly one " + convert(vlStringVT, type_or_class) +
            " should be selected, there are " + 
                number2string(length(objlist));
        vlibThrow(1001, str);
    }
    car(objlist);
}

/*
 * Builds a picklist of the selected objects and asks the users to
 * confirm that a given operation can be performed on all of them.
 */
function BuildConfirmRegion(objlist, msg, name_proc) {
    if (objlist != nil) {
        object_table := vcTable.Create ("", length (objlist), 1);
        vcTable.Col (object_table, 0, vcTable.Border.Invisible, 3, 3, 3,
                     vgAlign.LowerLeft);

        foreach (obj; vscript.list_iter (objlist)) {
            name := (name_proc == nil) ? obj.name : name_proc(obj);
            vcTable.Add(object_table, vcRegion.Create("", vcLabel.fac, name));
        }
        scroll := vcTable.Scroll ("Scroll", object_table, object_table,
                                  5, 5, 0);

        stat  := vcRegion.Create ("Stat", vcStaticText.fac, msg);
        result := vcTable.Make ("Table", 2, 1, stat, scroll);
    }
    else
        result := FALSE;
    result;
}

function CombinedViewName(v) {
    str := vllm.ViewGetName(v);
    qual := vllm.ViewGetQualifier(v);
    if ((!null(qual)) && (!string_eq(qual, "")))
        str = str + "\\" + vllm.ViewGetQualifier(v);
    str;
}

function ViewGetValueStr(v) {
    if (vllm.ViewGetBase(v) == vllmBase.Float)
        number2string(vllm.ViewGetData(v));
    else
        vllm.ViewGetData(v);
}

function GetLocationList(d) {
    vdmcntx := vllm.CntxGetVdmCntx(d.cntx);
    llist := nil;
    foreach (l; vpm.MapLoopLocation(vdm.CntxGetMap(vdmcntx))) {
        ln := vpm.LocationGetAlias(l);
        llist = cons(cons(ln, ln), llist); 
    }
    llist;
}

function GetDbClassList(d) {
    vdmcntx := vllm.CntxGetVdmCntx(d.cntx);
    clist := nil;
    foreach (c, cn; vdm.ClassFileLoop(vdm.CntxFindClassFile(vdmcntx,"vllmdb")))
        clist = cons(cons(cn, cn), clist); 
    clist;
}

function GetClassList(d) {
    vdmcntx := vllm.CntxGetVdmCntx(d.cntx);
    clist := nil;
    foreach (cf, cfn; vdm.CntxLoopClassFiles(vdmcntx))
        foreach (c, cn; vdm.ClassFileLoop(cf))
            clist = cons(cons(cn, cn), clist); 
    clist;
}

function GetDatabaseFiles(d, classname, locname) {
    vdmcntx := vllm.CntxGetVdmCntx(d.cntx);
    dblist := nil;
    class := vdm.CntxFindClass(vdmcntx, classname);
    loc := vpm.MapFindLocation(vdm.CntxGetMap(vdmcntx), locname);
    if (!null(class) && !null(loc))
        foreach (oid; vdm.ClassLoopObj(class, loc, vdmcntx, nil)) {
            name := vdm.ObjGetName(oid);
            dblist = cons(cons(name, name), dblist);
        }
    dblist;
}


/*
 * ReleaseObject performs a release operation on a single
 * vdm object.  There are two ways of performing a release:
 * CheckoutNoLock for an RCS-encapsulated file, or Copy
 * for other encapsulations.  
 *
 * CAUTION
 *   Objects encapsulated with storage managers other than
 *   RCS must be files in order for this function to work.
 */
function ReleaseObject(oid, dest) {
    loc := vdm.ObjGetLocation(oid);
    sm := vpm.LocationGetStorageSpec(loc);

    /* HACK ALERT: If the class of the object is ViewDir, do nothing */
    if (string_eqi(vdm.ClassGetName(vdm.ObjGetClass(oid)), "ViewDir"))
        ;
    else if (!string_eqi(sm, "vdmRCS")) {
        from := vdm.ObjGetPhysicalDescriptor(oid);
        to   := vscript.vsAbsPath(dest + "/") + vdm.ObjGetPhysicalName(oid);
        <dev, path, f, ext> := vscript.vsParse(to);
        vscript.vsCreateWithPath(path, vsAttr.Dir);
        if (!vscript.vsCopy(from, to)) {
            msg := "Release failed - attempting to copy " + from +
                " to " + to;
            vlibThrow(1004, msg);
        }
    }
    else {
        /* HACK ALERT: very RCS specific stuff here */
        smstr := vpm.LocationGetString(loc);
        
        /* first part of string is the storage manager */
        declare storage_area;
        string_token(smstr, &storage_area, " ");
        locstr := storage_area + " " + dest;
        destloc := vpm.LocationCreate(vdm.ObjGetName(oid) + "_release",
                                      vpm.LocationGetStorageSpec(loc),
                                      nil, locstr,
                                      vpm.LocationGetClassFilter(loc));
        attr_list := nil;
        foreach (key, val, sticky; vdm.ObjLoopAttr(oid))
            attr_list = cons(cons(string2symbol(key), val), attr_list);
        rel_oid := vdm.ObjCreate(vdm.ObjGetName(oid) + "_release",
                                 vdm.ObjGetVersion(oid),
                                 vdm.ObjGetClass(oid),
                                 destloc, 
                                 vdm.ObjGetCntx(oid),
                                 attr_list);
        vdm.ObjCheckoutNoLock(rel_oid);
    }
}

function LibCheckinRelease(lib,dest,do_indv,do_ini,log_msg,notify) {
    foreach (db; vllm.LibLoopDb(lib)) {
        oid := vllm.DbGetVdmObj(db);
        <wk_status, sm_status> := vdm.ObjGetStatus(oid);
        if ((wk_status == vdm.WorkStatus.New) ||
            (wk_status == vdm.WorkStatus.Modified)) {

            if (sm_status != vdm.SMStatus.LockedByYou)
                vdm.ObjLock(oid);

            vdm.ObjCheckin(oid, log_msg);
            vdm.ObjCheckoutNoLock(oid);
            vdm.ObjPurgeStatus(oid); 
        }
        ReleaseObject(oid, dest);
    }

    if (do_indv) {
        foreach (c; vllm.LibLoopComp(lib)) 
            foreach (v; vllm.CompLoopView(c))
                if (vllm.ViewGetBase(v) == vllmBase.Indirect) 
                    ViewCheckinRelease(vllm.LibGetCntx(lib), v, dest, 
                                       log_msg, FALSE);
    }


    if (do_ini)
        vllm.WriteReleaseInitFile(vllm.LibGetCntx(lib), lib, dest);

    /* call notification proc with log_msg */
    if (notify)
        LibraryReleaseNotify(vllm.LibGetCntx(lib), lib, dest, log_msg);
}

function ViewCheckinRelease(cntx, v, dest, log_msg, notify) {
    oid := vllm.ViewGetVdmObj(cntx, v);
    <wk_status, sm_status> := vdm.ObjGetStatus(oid);
    if ((wk_status == vdm.WorkStatus.New) ||
        (wk_status == vdm.WorkStatus.Modified)) {

        if (sm_status != vdm.SMStatus.LockedByYou)
            vdm.ObjLock(oid);

        vdm.ObjCheckin(oid, log_msg);
        vdm.ObjCheckoutNoLock(oid);
        vdm.ObjPurgeStatus(oid);
    }

    ReleaseObject(oid, dest);
    
    /* call notification proc with log_msg */
    if (notify)
        ViewReleaseNotify(cntx, view, dest, log_msg);
}

function SendEmailMessage(subject, msg, users) {
    if (!null(users)) {
        fname := vscript.vsUnique(vscript.vsAbsPath("."));
        fp := fopen(fname, "w");
        fputline(fp, "To: " + users + "\n");
        fputline(fp, "Subject: " + subject + "\n");
        fputline(fp, msg);
        fclose(fp);

        mail_program := "/usr/lib/sendmail -t";
        cmd := "cat " + fname + " | " + mail_program;
        vcSys.CommandSynchronous(cmd, nil, nil, nil, nil, nil, nil, 60, 0);

        vscript.vsDelete(fname, vsAttr.File);
    }
}


/*\Function LibraryReleaseNotify -- Notify users of library release
 *
 * DESCRIPTION
 *      This function is called after a library is released, to 
 *      notify users that this library has been released.  A 
 *      preference has been defined, called notify_users, which 
 *      allows you to set the notification list in the ViewLibrarian 
 *      preference editor.
 *
 * TO CUSTOMIZE
 *      In order to customize the notification process, add a function
 *      with this same name and argument list in the vlibus.vs.
 *      This ensures that modifications to this file (vlibutil.vs) 
 *      will not affect your customizations.
 */
function LibraryReleaseNotify(cntx, lib, dest, log_msg) {
    prefs := vlPref.Create("vlibtop");
    userlist := vlPref.Val(prefs, "notify_users");
    SendEmailMessage("Library " + vllm.LibGetName(lib) + " updated",
                     log_msg, userlist);
}

/*\Function ViewReleaseNotify -- Notify users of indirect view release
 *
 * DESCRIPTION
 *      This function is called after an indirect view (ie symbol,
 *      VHDL model) has been released, to notify users that this
 *      library has been released.  A preference has been defined,
 *      called notify_users, which allows you to set the notification 
 *      list in the ViewLibrarian preference editor.
 *
 * TO CUSTOMIZE
 *      In order to customize the notification process, add a function
 *      with this same name and argument list in the vlibus.vs.
 *      This ensures that modifications to this file (vlibutil.vs) 
 *      will not affect your customizations.
 */
function ViewReleaseNotify(cntx, view, dest, log_msg) {
    prefs := vlPref.Create("vlibtop");
    userlist := vlPref.Val(prefs, "notify_users");
    SendEmailMessage("Link " + vllm.ViewGetName(v) + " updated",
                     log_msg, userlist);
}

