/*
 *  $Revision: 1.27.6.1 $
 *
 *  ViewScript file for the plotting facility.
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 *
 */

/* Load all constituent widgets for plot worksheet */
load("vsputil.vs");
load("vspdev.vs");
load("vspport.vs");
load("vsppaper.vs");
load("vspextnt.vs");
load("vspoptns.vs");
load("vspgraph.vs");

fac_pack := vsp;

declare plot_go_event;
declare plot_go_window;
declare plot_win_id;
declare plot_reg;

declare vspPlotTable;

/* id for plotting information popup
 */
info_box_id := nil;
info_box_table := nil;

/* Package for highest level facility that is bound with plotting library
 */

tab_width := 4;
tab_height := 1;

if (vcWv.UI (wv) == vcUi.Motif)
    vspACCEPT := "OK";
else
    vspACCEPT := "Accept";
vspAPPLY := "Apply";
vspCANCEL := "Cancel";
vspDISMISS := "Dismiss";

PlotWkshtTitle := "Plotter Worksheet";

/* Keeps track of whether the user hits a ^c or the software closes wrksht.
 */
user_closed := TRUE;

/* Close the window and returns nil */                
function window_close (win_id) {

    if (win_id == plot_win_id)
        user_closed = FALSE;
    if (win_id != nil)
        vcWin.Close (win_id);
    nil;
}

/* Retrieves the current values of all the command lines in the
 * worksheet and sets them in the software.
 */
function get_cmdline_vals () {
        vsppaper.get_cmdline_vals ();
        vspport.get_cmdline_val ();
}

/* Apply all the changes and keep worksheet up- do not spawn plotter.
 */
function apply_changes () {
        get_cmdline_vals ();
        fac_pack.vspWriteInsFile (plot_go_window, plot_go_event);
        fac_pack.vspSaveCurins ();
        vcWin.ClearStatus (plot_win_id);
        vcWin.AddStatus (plot_win_id, "Saved current plot settings");
}

/* Apply all the changes and close the worksheet
 */
function apply_n_close () {
        get_cmdline_vals ();
        vcWin.ClearStatus (plot_win_id);
        vcWin.AddStatus (plot_win_id, "Creating the metafile..");

        /* Must force a refresh, because don't get back to main loop
           until the end of metafile creation, when it's too late */
        vcWin.Redraw (plot_win_id);

        /* Just before we plot, make sure that outdir is set properly */
        if (fac_pack.GetOdirDef ()) {
                wproj := vscript.getenv ("WPROJECT");
                if (wproj != nil)
                        fac_pack.sp_sapopt (string_append ("OUTDIR ", wproj));
                else
                        fac_pack.sp_sapopt (string_append ("OUTDIR ."));
        }

        vcWin.plot_it (plot_go_window, plot_go_event);
        vspport.close_port_win ();
        window_close (plot_win_id);
        plot_win_id = nil;
        plot_go_window = nil;
        plot_go_event = nil;
}

/* Revert to all old values and close the worksheet
 */
function cancel_n_close (close_plot_wksht) {

        fac_pack.vspResetVals ();
        vspport.close_port_win ();
        if (close_plot_wksht)
                plot_win_id = window_close (plot_win_id);
}

/* Builds a column containing all command lines in initial plotting worksheet
 * and other miscellaneous items.  This will be added as a single unit to the
 * plotting worksheet table.
 */
function build_misc_col () {

        misc_col := vcTable.Create("misc_col", 2, 1);

        /* Add the extents picklist */
        vcTable.Add (misc_col, vspextnt.create_extents_pklst());
        
        other_buttons := vcTable.Create ("other_buttons", 1, 2);
        add_but (other_buttons, "OPTIONS...", 0, 0, 
                        func () vspoptns.create_options_wksht(););
        add_but (other_buttons, "GRAPHICS...", 0, 1, 
                        func () vspgraph.create_graphics_wksht(););

        vcTable.Add (misc_col, other_buttons);    

        misc_col;
}


/* This is the function which actually builds the initial plot worksheet.
 */
function create_plot_wksht () {

        mega_table := vcTable.Create ("Entire_plot.tab", 2, 1);

        /* Create a table containing all initial plot worksheet widgets */
        vspPlotTable = vcTable.Create (PlotWkshtTitle, tab_height, 
                                tab_width);

        /* Add device picklist */
        vcTable.Add (vspPlotTable, vspdev.create_device_pklst());

        /* Add paper picklist area */
        vcTable.Add (vspPlotTable, vsppaper.create_paper_area());

        /* Add output picklist */
        vcTable.Add (vspPlotTable, vspport.create_out_list ());

        /* Add column of command lines and other stuff */
        vcTable.Add (vspPlotTable, build_misc_col());

        ttab := vcTable.Create ("App_Ac_Can.tab", 1, 3);
        /* Add the 'Accept', 'Apply' and 'Cancel' buttons */
        add_but (ttab, vspACCEPT, 0, 0,
                 func () apply_n_close (););
        add_but (ttab, vspAPPLY, 0, 1, 
                 func () apply_changes (););
        add_but (ttab, vspCANCEL, 0, 2, 
                 func () cancel_n_close(TRUE););

        vcTable.Add (mega_table, vspPlotTable);
        vcTable.Add (mega_table, ttab);
        mega_table;
}

/* This routine is called when the plot worksheet is killed.  Only if
 * the user killed it with a ^C does it do anything.
 */
function plot_wksht_killed (r)
        if (user_closed) {
                fac_pack.vspResetVals ();
                plot_win_id = nil;
        }

/* Keeps track of the first time the plotting wksht. is brought up during
 * the process lifetime of the facility.  The initial scale value must
 * be calculated at that time.  All subsequent times, the scale is whatever
 * is currently set in the event that the user decides to change it.

never_invoked := TRUE;
 */
 
/* This is called when the 'Plot' button is hit in a facility.  Only one
 * plot popup is allowed on the screen at a time.
 */
function _Create(fac_reg, region) {
        user_closed = TRUE;
        
        if (!fac_pack.vspUserSetScale ())
                fac_pack.vspUpdateScale ();

        /* Make sure that the spcurins.ins file is in place */
        fac_pack.vspCheckCurins ();
        
        /* Save the current dev. des. file (in case of cancel) */ 
        fac_pack.vspSaveCurins ();

        /* Setup the valid com. ports for the machine type */
        vspport.setup_valid_ports ();

        plot_reg = create_plot_wksht ();
        vcRegion.AddChild(region, plot_reg);

        /* To catch ^C in plotting worksheet */
        vcRegion.DestroyNotify (plot_reg, plot_wksht_killed);

        nil;
}

/* Brings up the plotting worksheet if plotting is enabled and there is
 * not already a plotting worksheet up.
 */
function PlotSetup(win, e)
{
    if (plot_win_id != nil)
        vlEx.Throw2 (vlEx.FAILURE, "vspPlot", vlErrSev.Error,
                     "Only 1 plotting worksheet up at once!");

    if (fac_pack.vspPlotEnabled () && plot_win_id == nil) {
        fac_pack.vspSetKludgeWindow (win);
        plot_win_id = vcWin.Create (vcRegion.Create1 ("Plotting Worksheet",
                                                      vspPlot.fac,
                                                      vcWin.OwnerRegion (win),
                                                      nil),
                                    5);
        plot_go_event = e;
        plot_go_window = win;
    }
    else
        vlEx.Throw2 (vlEx.FAILURE, "vspPlot", vlErrSev.Error,
                     "Plotting is not enabled.");
}

/* Close the plotting message box
 */
function dismiss_box () {
        vcWin.Close (info_box_id);
        info_box_id = nil;
}

/* Update the message in the popup information box
 */
function UpdateInfoBox (text_string)
{
        if (info_box_id != nil) {
                vcWin.Show (info_box_id);
                vcRegion.Destroy (vcTable.Get (info_box_table, 0, 0));

                lab_reg := vcRegion.Create1 ("Info_string_update", vcLabel.fac,
                                text_string, nil);

                vcRegion.Font (lab_reg, vgFont.mediumb);

                vcTable.Set (info_box_table, 0, 0, lab_reg);

                /* This is needed, or the label isn't changed until
                   the cursor enters the window */
                vcRegion.Reframe (info_box_table, 
                                vcRegion.Area (info_box_table));
                vcRegion.Refresh (info_box_table, 
                                vcRegion.Area (info_box_table));
        }
        else
                CreateInfoBox (text_string);
}

/* Create a popup with a text string in it and a dismiss button.  Use this
 * rather than running a command if you would like to change the initial
 * message while still keeping the same window.
 */
function CreateInfoBox (text_string)
{
        if (info_box_id != nil) {
                UpdateInfoBox (text_string);
                return (nil);
        }

        info_box_table = vcTable.Create ("Plot Status", 2, 1);

        lab_reg := vcRegion.Create1 ("Info_string", vcLabel.fac, text_string, 
                                        nil);
        vcRegion.Font (lab_reg, vgFont.mediumb);

        vcTable.Add (info_box_table, lab_reg);

        vcTable.Add (info_box_table, vcLabel.ButtonCreate (vspDISMISS,
                                        vspDISMISS, dismiss_box, nil));

        info_box_id = vcWin.Create (info_box_table, 5);
}

