/*
 * $Revision: 1.7.2.1 $
 *
 * Implements the plot device picklist
 */

package vspdev (create_device_pklst) {

/* These are all the plot devices in alphabetical order.  The first string in
 * the list is to be displayed in menu, the second is for internal rep.
 */
dev_lst1 := list (
cons ("CALCOMP PCI", "clcmpci"),
cons ("CANON 75", "canon75"),
cons ("CANON 100", "canon100"),
cons ("CANON 150", "canon150"),
cons ("EPSON 0", "epson_m0"),
cons ("EPSON 1", "epson_m1"),
cons ("EPSON 2", "epson_m2"),
cons ("EPSON 3", "epson_m3"),
cons ("EPSON 4", "epson_m4"),
cons ("EPSON 5", "epson_m5"),
cons ("EPSON 6", "epson_m6"),
cons ("EPSON 7", "epson_m7"),
cons ("EPSON SPECIAL", "epsonsp"),
cons ("HI 41", "hi_41"),
cons ("HI 42", "hi_42"),
cons ("HI 51", "hi_51"),
cons ("HI 52", "hi_52"),
cons ("HP 17440", "hp_17440"),
cons ("HP 7470", "hp_7470"),
cons ("HP 7475", "hp_7475"),
cons ("HP 7550", "hp_7550"),
cons ("HP 7580", "hp_7580"),
cons ("HP 7580e", "hp_7580e"),
cons ("HP 7586", "hp_7586"),
cons ("HP 7586e", "hp_7586e"),
cons ("IBM 60", "ibm60"),
cons ("IBM 120", "ibm120"),
cons ("IBM 120f", "ibm120f"),
cons ("IBM 240", "ibm240"),
cons ("LASERJET 75", "ljpls75"),
cons ("LASERJET 100", "ljpls100"),
cons ("LASERJET 150", "ljpls150"),
cons ("LASERJET 300", "ljpls300"),
cons ("METAFILE", "meta"),
cons ("NEC 101c", "nec101c"),
cons ("NEC 101d", "nec101d"),
cons ("NEC 201c", "nec201c"),
cons ("NEC 201d", "nec201d"),
cons ("PAINTJET 90", "pj90"),
cons ("PAINTJET 180", "pj180"),
cons ("PIC FORMAT", "pic"),
cons ("POSTSCRIPT", "ps"),
cons ("THINKJET EPSON 96", "tjep96"),
cons ("THINKJET EPSON 192", "tjep192"),
cons ("THINKJET HP 96", "tjhp96"),
cons ("THINKJET HP 192", "tjhp192"));

dev_lst2 := list (
cons ("CALCOMP PCI", "clcmpci"),
cons ("CANON 75", "canon75"),
cons ("CANON 100", "canon100"),
cons ("CANON 150", "canon150"),
cons ("EPSON 0", "epson_m0"),
cons ("EPSON 1", "epson_m1"),
cons ("EPSON 2", "epson_m2"),
cons ("EPSON 3", "epson_m3"),
cons ("EPSON 4", "epson_m4"),
cons ("EPSON 5", "epson_m5"),
cons ("EPSON 6", "epson_m6"),
cons ("EPSON 7", "epson_m7"),
cons ("EPSON SPECIAL", "epsonsp"),
cons ("HI 41", "hi_41"),
cons ("HI 42", "hi_42"),
cons ("HI 51", "hi_51"),
cons ("HI 52", "hi_52"),
cons ("HP 17440", "hp_17440"),
cons ("HP 7470", "hp_7470"),
cons ("HP 7475", "hp_7475"),
cons ("HP 7550", "hp_7550"),
cons ("HP 7580", "hp_7580"),
cons ("HP 7580e", "hp_7580e"),
cons ("HP 7586", "hp_7586"),
cons ("HP 7586e", "hp_7586e"),
cons ("IBM 60", "ibm60"),
cons ("IBM 120", "ibm120"),
cons ("IBM 120f", "ibm120f"),
cons ("IBM 240", "ibm240"),
cons ("LASERJET 75", "ljpls75"),
cons ("LASERJET 100", "ljpls100"),
cons ("LASERJET 150", "ljpls150"),
cons ("LASERJET 300", "ljpls300"),
cons ("METAFILE", "meta"),
cons ("NEC 101c", "nec101c"),
cons ("NEC 101d", "nec101d"),
cons ("NEC 201c", "nec201c"),
cons ("NEC 201d", "nec201d"),
cons ("PAINTJET 90", "pj90"),
cons ("PAINTJET 180", "pj180"),
cons ("PIC FORMAT", "pic"),
cons ("POSTSCRIPT", "ps"),
cons ("THINKJET EPSON 96", "tjep96"),
cons ("THINKJET EPSON 192", "tjep192"),
cons ("THINKJET HP 96", "tjhp96"),
cons ("THINKJET HP 192", "tjhp192"),
cons ("VCGL", "vgi"));

declare dev_lst;

/* Called when a device is selected.  Sets the device in the software.
 */
function set_device (val, on_off) {

        if (on_off == vlOnOff.On) {

                fac_pack.sp_sapopt (string_append("DEVICE ", val));

               /* Change the paper selection depending on the new device */
                update_paper_pklst ();

                /* Make sure scale is properly adjusted */
                if (!fac_pack.do_tile ())
                        fac_pack.vspUpdateScale ();

        }
}

/* Initial list for device picklist- contains everything except the
 * array with function calls in it.
 */
devices := list (0, `vertical, "DEVICES", set_device);

/* Converts a device name into an index into the picklist.  This is called
 * when a default value is required upon invocation of the picklist.
 */
function dev2index (dev_name) {

        for (tlst := dev_lst, i := 0; tlst != nil; tlst = cdr (tlst), ++i) {
                if (string_eqi (dev_name, cdar(tlst)))
                        return (i);
        }
        return (nil);
}

/* Create the rest of the structure/list which is required to make a 
 * device picklist.  This is a scollable area and should be in
 * alphabetical order (depending on the order of the 'devices' list).
 */
function create_device_pklst () {
        if (fac_pack.is_pc ())
                dev_lst = dev_lst1;
        else
                dev_lst = dev_lst2;        
        plst := create_pklst (func () return(dev2index(
                fac_pack.get_cur_dev ()));,
                cadr (devices), caddr (devices), dev_lst,
                cadddr (devices), FALSE);

        return (plst);
}

}
