/*
 * $Revision: 1.3.8.1 $
 *
 * Implements the plot extents picklist
 */

package vspextnt (create_extents_pklst) {

/* These constants correspond to the order in which they appear in picklist
 */
EXTENTS_SHT := 0;
EXTENTS_WIN := 1;

vspEXT_TITLE := "EXTENTS";
vspSHEET := "Full Sheet";
vspWINDOW := "Zoomed View";

vspSHT_CMD := "S";
vspWIN_CMD := "W";

/* Called when user selects new plot extent.
 */
function set_extents (val, on_off) {

        if (on_off == vlOnOff.Off)
                return (nil);

        fac_pack.sp_sapopt (string_append("WINSHEET ", val));
        fac_pack.vspUpdateScale ();
}

/* Declaration of extents picklist in plot worksheet.  The first entry 
 * in the picklist declaration is a default value which is dynamically 
 * set. Next is orientation- `vertical or `horizontal.  Next is the title.
 * Finally comes an array of values with an entry for each selection in
 * the picklist.  Each entry contains is string represenation and a
 * function to call when the entry is chosen.
 */
extents := list (0, `vertical, vspEXT_TITLE,
                list (cons(vspSHEET, vspSHT_CMD),
                        cons(vspWINDOW, vspWIN_CMD)),
                set_extents);

/* Creates the extents picklist */
function create_extents_pklst () {
        def := fac_pack.get_def_extents ();

        /* Calculate the scale only if window plot is set.  The
         * scale for sheet plots may be overridden by the user.
         */
        if (string_eqi (def, vspWIN_CMD))
                fac_pack.vspUpdateScale ();

        return(create_pklst (func () return (get_extents_index(def));,
                cadr(extents), caddr (extents), cadddr (extents),
                cadr(cdddr(extents)), FALSE));
}        
        
/* Returns the index into the extents selection picklist, depending 
 * on the choice.  This is useful to highlight the proper entry upon
 * invocation of the software.
 */
function get_extents_index (choice) {

        if (string_eqi (choice, vspSHT_CMD))
                return (EXTENTS_SHT);
        else if (string_eqi (choice, vspWIN_CMD))
                return (EXTENTS_WIN);
        else
                return (nil);
}

}
