/* $Revision: 1.6.8.1 $
 * Implements the Plot Graphics worksheet.  All changes are temporarily stored
 * in Viewscript until the 'Apply' button is hit.
 */

package vspgraph (create_graphics_wksht, set_graphics_modified) {

/* Load all constituent widgets for graphics worksheet */
load("vspfont.vs");
load("vsplwid.vs");
load("vspcaps.vs");
load ("vspscale.vs");
load ("vsprot.vs");
load ("vspexp.vs");
load ("vsptico.vs");

declare graphics_win_id;

declare vspGraphicsTable;

tab_width := 3;
tab_height := 1;
graphics_modified := FALSE;

GraphicsWkshtTitle := "Plot Graphics Worksheet";

function set_graphics_modified () {
        graphics_modified = TRUE;
}

/* Calls all of the graphics worksheet's constituent member's apply_changes
 * routine so that the software is updated.
 */
function update_graphics_changes () {
/* NOTE:  Rotation, Scale, and Expansion are options which take effect 
 *      immediately and therefore are not needed here.
 */

        vspfont.apply_changes ();
        vspcaps.apply_changes ();
        ret := vsptico.apply_changes ();
        return (ret);
}

/* Apply all the changes and close the worksheet
 */
function apply_n_close_graphics () {

        success := TRUE;
/*
        if (graphics_modified) {
 */
                success = update_graphics_changes ();
/*
                graphics_modified = FALSE;
        }
 */

        /* Must always update line width because of the way the scale
         * bar is implemented in core- there is no way to specify a
         * procedure to be called upon change.
         */
        vsplwid.apply_changes ();

        /* Only close window if everything is valid */
        if (success)
                graphics_win_id = window_close (graphics_win_id);

}

/* Discard all changes and close the worksheet
 */
function cancel_n_close_graphics (close_graphics_wksht) {

        if (close_graphics_wksht) {
                /* Since rotation, scale and expansion (and tiling to some
                 * degree) take effect 
                 * immediately, must revert to the old values on a cancel.
                 */
/*
                if (graphics_modified) {
 */
                        vsprot.reset_value ();
                        vspscale.reset_value ();
                        vspexp.reset_value ();
                        vsptico.reset_value ();
                        graphics_modified = FALSE;
/*
                }
 */
                graphics_win_id = window_close (graphics_win_id);
        }
}

/* This is the function which actually builds the graphics worksheet.
 */
function create_graphics_wksht () {

        major_tab := vcTable.Create (GraphicsWkshtTitle, 2, 1);

        /* Create a table containing all plot graphics worksheet widgets */
        vspGraphicsTable = vcTable.Create("graphics.tab", tab_height,
                                tab_width);

        /* Add font picklist */
        tlst := vspfont.create_fontlist ();

        ttab := vcTable.Create ("graph_left_col.tab", 2, 1);


        if (tlst != nil)
                vcTable.Add (ttab, tlst);
        else
                vcTable.Add (ttab, vcRegion.Create1 ("font.phold",
                        vcLabel.fac, "Font\nPlace\nHolder", nil));

        /* Add Line width command line */
        vcTable.Add (ttab, vsplwid.create_lwid_cmdline ());

        vcTable.Add (vspGraphicsTable, ttab);



        ttab = vcTable.Create ("graph_mid_col.tab", 3, 1);

        /* Add scale picklist/command line area */
        vcTable.Add (ttab, vspscale.create_scale_area());

        /* Add tile/corner area */
        vcTable.Add (ttab, vsptico.create_tico_area ());

        rot_tpap_tab := vcTable.Create ("rot_tpap.tab", 1, 2);

        vcTable.Add (rot_tpap_tab, vsptico.create_tile_paper_area ());
        vcTable.Add (rot_tpap_tab, vsprot.create_rotlist());

        /* Add tiled output size and rotation picklists to a shared column */
        vcTable.Add (ttab, rot_tpap_tab);

        vcTable.Add (vspGraphicsTable, ttab);        


        ttab = vcTable.Create ("graph_right_col.tab", 2, 1);

        /* Add dev/std. capability buttons */
        vcTable.Add (ttab, vspcaps.create_caps_tab ());

        /* Add expansion picklist */
        vcTable.Add (ttab, vspexp.create_explist ());

        vcTable.Add (vspGraphicsTable, ttab);

        vcTable.Add (major_tab, vspGraphicsTable);

        ttab = vcTable.Create ("Acc_Can.tab", 1, 2);

        /* Add the 'Accept' and 'Cancel' buttons */
        add_but (ttab,  vspACCEPT, 0, 0,
                func () apply_n_close_graphics (););
        add_but (ttab, vspCANCEL, 0, 1,
                func () cancel_n_close_graphics (TRUE););

        vcTable.Add (major_tab, ttab);

        graphics_win_id = vcWin.Create (major_tab, 5);

        /* Forward all events to graphics worksheet */
        vcWin.Forward (plot_win_id, graphics_win_id);
}


}

