/*
 *  $Revision: 1.13.4.1 $
 *
 *  Implements the paper size picklist area.
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 */

package vsppaper (create_paper_area, get_cmdline_vals) {

declare vspActivePaperLst; /* Points to one of the following */
declare vspUSPaperLst;
declare vspMETPaperLst;
declare xpaper_cmdline, ypaper_cmdline;
declare met_paper_pklst, us_paper_pklst;
declare cur_met_val, cur_us_val;
declare units_pklst;

/* These constants must correspond to those in devinf.h!
 */
USSHEET := 1;
METSHEET := 2;

AREA_TITLE := "PAPER SIZES";
US_TITLE := "US";
MET_TITLE := "Metric";
UNITS_TITLE := "UNITS";

/* The following assignment depends on the order of picklist and
 * assumes that inches is first and millimeters is second.
 */
Iindex := 0;
Mindex := 1;
units_lst := list (cons(US_TITLE, Iindex), cons(MET_TITLE, Mindex));

/* This is called when the user hits the apply button in the main
 * plotting worksheet.  It is required just in case the values have
 * changed and the user neglected to enter a <cr>.
 */
function get_cmdline_vals () {
        set_paper_dims (`xval, vcRegion.Get (xpaper_cmdline, vlStringVT));
        set_paper_dims (`yval, vcRegion.Get (ypaper_cmdline, vlStringVT));
}

function format_str_as_num (str)
    sprintf ("%-.2f", string2number (str));

/* Given the current paper selection and units, displays the corresponding
 * paper dimensions.
 */
function update_cmdlines () {

        val := fac_pack.vspGetPaperDims (vspFIRSTVAL);
        vcCmdline.Set (xpaper_cmdline, format_str_as_num (val));
        val = fac_pack.vspGetPaperDims (vspSECONDVAL);
        vcCmdline.Set (ypaper_cmdline, format_str_as_num (val));
}

/* Set the selected paper size in the software.  For the US paper size picklist
 */
function set_us_paper_size (val, on_off) {

        if (on_off == vlOnOff.On) {
                if (cur_us_val == val)
                    return (nil);

                cur_us_val = val;
                if (vspActivePaperLst != vspUSPaperLst) { 
                        vcPicklist.SetPick (units_pklst, Iindex);
                }
                else {
                        fac_pack.sp_sapopt (string_append("PAPER ", 
                                get_str_from_lst (vspUSPaperLst, val)));
                        update_cmdlines ();
                        if (!fac_pack.do_tile ())
                                fac_pack.vspUpdateScale ();
                }       
        }
}

/* Set the selected paper size in the software.  For metric paper size picklist
 */
function set_met_paper_size (val, on_off) {

        if (on_off == vlOnOff.On) {
                if (cur_met_val == val)
                    return (nil);

                cur_met_val = val;
                if (vspActivePaperLst != vspMETPaperLst)
                        vcPicklist.SetPick (units_pklst, Mindex);
                else {
                        fac_pack.sp_sapopt (string_append("PAPER ", 
                                get_str_from_lst (vspMETPaperLst, val)));
                        update_cmdlines ();
                        if (!fac_pack.do_tile ())
                                fac_pack.vspUpdateScale ();
                }
        }

}

paper  := list (0, `vertical, US_TITLE, set_us_paper_size);

/* Converts a paper name to an index into the paper picklist.
 */
function paper_to_index (val, which) {

        if (which == METSHEET)
                tval := vspMETPaperLst;
        else
                tval := vspUSPaperLst;

        for (i := 0; tval != nil; 
                tval = cdr (tval), ++i) {
                if (string_eqi (val, caar(tval)))
                        return (i);
        }
        nil;
}
        
/* Create a picklist of paper sizes which are relevant to the currently
 * selected device.  Makes list of either metric or US paper sizes.
 */
function make_paper_pklst (which)
{
        declare tcons, vspPaperLst;
        
        if (which == METSHEET) {
                title := MET_TITLE;
                set_func := set_met_paper_size;
        }
        else {
                title := US_TITLE;
                set_func := set_us_paper_size;
        }

        paper_sizes := fac_pack.get_paper_sizes (which);

        for (i := 0, psize := paper_sizes; psize != nil;
                        ++i, psize = cdr(psize)) {
                tcons = cons(car(psize), i);
                if (i == 0)
                        vspPaperLst = cons(tcons, nil);
                else
                        vspPaperLst = cons(tcons, vspPaperLst);
        }

        if (which == METSHEET) {
                vspMETPaperLst = vspPaperLst = reverse(vspPaperLst);
                tval := cur_met_val = paper_to_index (
                                                fac_pack.get_def_paper_size (),
                                                which);
        }
        else {
                vspUSPaperLst = vspPaperLst = reverse(vspPaperLst);
                tval := cur_us_val = paper_to_index (
                                                fac_pack.get_def_paper_size (),
                                                which);
        }

        
        tlst := create_pklst (func () tval;,
                        cadr (paper), title, vspPaperLst, 
                        set_func, TRUE);

        if (which == METSHEET)
                met_paper_pklst = car (tlst);
        else
                us_paper_pklst = car (tlst);

        cadr (tlst);
}

/* Called when the user makes a selection in the units picklist.  Sets the
 * appropriate paper list active and sets the currently selected paper to
 * the last previously chosen selection.  If there is not previous choice,
 * the first entry in the new list is selected.  The x and y cmdlines are
 * updated to reflect the new default.
 */
function set_metric (val, on_off) {
        if (on_off == vlOnOff.On) {

                if (val == Mindex && vspMETPaperLst == nil) {
                        set_metric (Iindex, vlOnOff.On);
                        vcPicklist.SetPick (units_pklst, Iindex);
                        vlEx.Throw2 (vlEx.FAILURE, "vsppaper", vlErrSev.Error,
                                    "No metric paper sizes for device");
                }
                if (val == Iindex && vspUSPaperLst == nil) {
                        set_metric (Mindex, vlOnOff.On);
                        vcPicklist.SetPick (units_pklst, Mindex);
                        vlEx.Throw2 (vlEx.FAILURE, "vsppaper", vlErrSev.Error,
                                    "No US paper sizes for device");
                }

                fac_pack.vspSetMetric (val);

                /* This is where the other picklist should be disabled ! */
                if (val == Mindex) {
                        vspActivePaperLst = vspMETPaperLst;
                        if (cur_met_val != nil)
                                set_met_paper_size (cur_met_val, vlOnOff.On);
                        else {
                                vcPicklist.SetPick (met_paper_pklst, 0);
                                set_met_paper_size (0, vlOnOff.On);
                                }
                        }
                else {
                        vspActivePaperLst = vspUSPaperLst;
                        if (cur_us_val != nil)
                                set_us_paper_size (cur_us_val, vlOnOff.On);
                        else {
                                vcPicklist.SetPick (us_paper_pklst, 0);
                                set_us_paper_size (0, vlOnOff.On);
                                }
                        }
        }
}

/* Converts the units selection to an index into the units picklist
 */
function met_to_index (metric)
    if (metric)
        Mindex;
    else
        Iindex;


/* Creates the units picklist.
 */
function make_units_pklst () {

        tlst := create_pklst (func () met_to_index (fac_pack.vspGetMetric ());,
                              `vertical, UNITS_TITLE, units_lst, set_metric,
                              TRUE);
        units_pklst = car (tlst);
        cadr (tlst);
}

/* This is called when a user types a value into either the x or y cmdlines
 * which accept paper dimensions in the currently set units.  It tries to 
 * find a match of the new dimensions with those paper sizes that are stored
 * in memory.  If such a match occurs, the new selection is highlighted.  If
 * no match is found, it is assumed that the user is not through entering the
 * information in the case that the x value is entered.  If still no match
 * occurs after the y value is entered, an error is reported and both values
 * are reset to reflect the currently chosen paper (in the picklist).
 */
function set_paper_dims (which, val) {
        /* Make sure it's a number */
        convert (float, val);

        if (which == `xval)
                str := string_append (val, " ", vcRegion.Get (
                                ypaper_cmdline,vlStringVT));
        else
                str := string_append (vcRegion.Get (xpaper_cmdline, vlStringVT),
                                " ", val);

        paper_name := fac_pack.vspMatchPaperDims (str);

        /* This is where the new paper is highlighted in the menus */

        if (vspActivePaperLst == vspMETPaperLst) {
                if (is_mem_list (vspMETPaperLst, paper_name)) {
                        vcPicklist.SetPick (met_paper_pklst, paper_to_index (
                                paper_name, METSHEET));
                        fac_pack.sp_sapopt (string_append("PAPER ", 
                                                paper_name));
                }
                else {
                        if (which == `yval) {
                                update_cmdlines ();
                                vlEx.Throw2 (vlEx.FAILURE, "vsppaper",
                                             vlErrSev.Error,
                                             "X and Y dimensions do not match any pre-defined paper");
                        }
                }
        }
        else {
                if (is_mem_list (vspUSPaperLst, paper_name)) {
                        vcPicklist.SetPick (us_paper_pklst, paper_to_index (
                                paper_name, USSHEET));
                        fac_pack.sp_sapopt (string_append("PAPER ", 
                                                paper_name));
                }                        
                else {
                        if (which == `yval) {
                                update_cmdlines ();
                                vlEx.Throw2 (vlEx.FAILURE, "vsppaper",
                                             vlErrSev.Error,
                                             "X and Y dimensions do not match any pre-defined paper");
                        }
                }
        }

}

/* Creates the paper size area consisting of two paper picklists (one
 * metric, one US), a units picklist, and two command lines which display
 * the dimensions of the currently selected paper.
 */
function create_paper_area () {

        vspMETPaperLst = vspUSPaperLst = nil;
        cur_met_val = cur_us_val = nil;

        ttab := vcTable.Create ("paper_pklsts.tab", 1, 3);

        vcTable.Add (ttab, make_paper_pklst (USSHEET));
        vcTable.Add (ttab, make_paper_pklst (METSHEET));

        if (fac_pack.vspGetMetric ())
                vspActivePaperLst = vspMETPaperLst;
        else
                vspActivePaperLst = vspUSPaperLst;

        vcTable.Add (ttab, make_units_pklst ());

        ttab2 := vcTable.Create ("paper_area.tab", 3, 1);
        vcTable.Add (ttab2, vcRegion.Create1 ("paper_area.lab", vcLabel.fac,
                                AREA_TITLE, nil));
        vcTable.Add (ttab2, ttab);

        
        cmdlst := create_double_cmdline_list ("X> ", "Y> ", 
                format_str_as_num (fac_pack.vspGetPaperDims (vspFIRSTVAL)), 
                format_str_as_num (fac_pack.vspGetPaperDims (vspSECONDVAL)),
                set_paper_dims, `xval, `yval, FALSE);
        xpaper_cmdline = car(cmdlst);
        ypaper_cmdline = cadr(cmdlst);
        vcTable.Add (ttab2, caddr (cmdlst));

        vcRegion.Create1 ("paper_area.bord", vcBorder.fac, ttab2, nil);
    }
}
