/*
 * $Revision: 1.3.8.1 $
 */

/* Implements the rotation picklist.  Rotation is the only option
 * in the graphics worksheet which takes effect immediately (i.e.,
 * is set in the software as soon as the user enters a new value).
 * Therefore, there is no apply_changes routine, but there is a
 * reset_value in case of a cancel.
 */

package vsprot (create_rotlist, reset_value) {

declare old_rot;

function reset_value () {
        fac_pack.sp_sapopt (string_append("ROTATION ", old_rot));
}

/* Called when user selects new rotation.  The appropriate scale value is set
 */
function set_rotation (val, on_off) {

        if (on_off == vlOnOff.On) {
                set_graphics_modified ();
                fac_pack.sp_sapopt(string_append("ROTATION ", val));

                fac_pack.vspUpdateScale ();
                vspscale.set_scale_cmdline_val(fac_pack.get_def_scale(
                        vspFIRSTVAL));
        }
}

/* Declaration of rotation picklist in plot worksheet.  The first entry 
 * in the picklist declaration is a default value which is dynamically 
 * set. Next is orientation- `vertical or `horizontal.  Next is the title.
 * Finally comes an array of values with an entry for each selection in
 * the picklist.  Each entry contains is string represenation and a
 * function to call when the entry is chosen.
 */
rotations := list (0, `vertical, "ROTATION", 
                list (cons("0", "0"), cons("90", "90"),
                        cons("180", "180"), cons("270", "270")),
                set_rotation);

/* Creates the rotations picklist */
function create_rotlist () {
        old_rot = fac_pack.get_def_rot ();
        return (create_pklst (func () return (get_rot_index(old_rot));,
                cadr (rotations), caddr (rotations), cadddr (rotations),
                cadr(cdddr (rotations)), FALSE));
}
        
/* Returns the index into the rotation selection picklist, depending 
 * on the choice.  This is useful to highlight the proper entry upon
 * invocation of the software.
 */
function get_rot_index (choice) {

        rot_lst := cadddr(rotations);
        alen := length(rot_lst);
        for (i := 0, tlst := rot_lst; i < alen; ++i, tlst = cdr(tlst)) {
                if (string_eqi (choice, caar(tlst)))
                        return (i);
        }
}

}

