
/*
 * Animation data associated with diagram, keyed on diagram_name + entity_name
 *
 *  diagram_entity:
 *       
 */

vstBkptTable := vlSym.StrCreate (77);
vstDiagramTable := vlSym.StrCreate (77);

vsimContext := vitc.SrcCntxCreate("Viewsim", nil);

vstContext := vitc.CntxCreate(vipc.GidDFLT,vipc.UidDFLT,vipc.FidDFLT,
                              "ViewFSM-Capture", nil, vipc.HnmANY,
                              vipc.PidANY, vipc.CidANY);

function vstVsimQuit() {
    foreach(sym, diagram_data; vlSym.Loop (vstDiagramTable)) {
        vitc.Send ("AnimateFinish", vsimContext, vstContext, diagram_data.name,
                   vipc.FlgDIRECT);
    }

    foreach (sym, bp; vlSym.Loop (vstBkptTable))
        FSMSigTraceCancel (bp.trace_id, FALSE);        

    vlSym.Clear (vstDiagramTable);
    vlSym.Clear (vstBkptTable);
}

function make_bp(type, diagram_name, entity_name, id, trace_id, prefix_name) {
    bp := struct {
        type = type;
        diagram = diagram_name;
        entity = entity_name;
        prefix = prefix_name;
        id = id;
        trace_id = trace_id
    };

    bp;
}

vcQuit.CallOnQuit (vstVsimQuit);

vitc.Register ("NetCommand", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   vsim_exec (sprintf("net %s", data_str));
               });

vitc.Register ("Quit", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   vstVsimQuit ();
               });

vitc.Register ("ExecCommand", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   foreach (w; vcWin.Loop ())
                       if (vcWin.Parent (w) == nil) {
                           r := vcWin.OwnerRegion (w);
                           if (vcRegion.Fac (r) == fac) {
                               event := vcEvent.Create (0, 0, '\0',
                                                        nil, w, r, nil);
                               vcWin.RunCmdline (w, data_str, event);
                               break;
                           }
                       }
               });

vitc.Register ("ListBreakpoints", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name, prefix_name;
                   
                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &prefix_name, " ");

		   bp_data := nil;
 
                   foreach (sym, bp; vlSym.Loop (vstBkptTable)) 
		       bp_data = cons (list (bp.type, bp.diagram, bp.entity,
					     bp.prefix, bp.id), bp_data);
		   vitc.Send ("BreakpointList", vsimContext, vstContext,
			      bp_data, vipc.FlgDIRECT);
               });

vitc.Register ("ClearAllBreakpoints", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name, prefix_name;

                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &prefix_name, " ");

                   del_list := nil;
                   foreach (sym, bp; vlSym.Loop (vstBkptTable)) {
                       if (string_eq (bp.diagram, diagram_name) &&
                           string_eq (bp.entity, entity_name)) {
                           FSMSigTraceCancel (bp.trace_id, FALSE);
                           del_list = cons (sym, del_list);
                       }
                   }

                   foreach(sym; vscript.list_iter (del_list)) 
                       vlSym.Remove (vstBkptTable, sym);
               });

vitc.Register ("BreakInTrans", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name, clock_name, clock_edge;
                   declare trans_id, prefix_name;

                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &clock_name, " ");
                   data_str = string_token (data_str, &clock_edge, " ");
                   data_str = string_token (data_str, &trans_id, " ");
                   data_str = string_token (data_str, &prefix_name, " ");

                   name := sprintf("T_%s_%s_%s_%s", diagram_name, entity_name,
                                   trans_id, prefix_name);

                   id := convert (vlIntVT, trans_id);
                   clk_edge := convert (vlIntVT, clock_edge);
                   
                   ct_name := entity_name + "\\cur_trans";
                   clk_name := entity_name + "\\" + clock_name;
                   
                   function clk_change() {
                       clk_val := vsim_val (clk_name);
                       ct_val := vsim_val (ct_name);

                       ret_value := FALSE;

                       clk_str := convert (vlStringVT, clk_val);
                       if (clk_str[0] != 'X') {
                           clk := convert (vlIntVT, clk_val);
                           
                           if (clk == clk_edge) {
                               ct := convert (vlStringVT, ct_val);
                               if (ct[0] != 'X') {
                                   ct = convert (vlIntVT, ct_val);
                                   if (ct == id) {
                                       ret_value = TRUE;
				       vitc.Send ("BreakHitTrans",
						  vsimContext, vstContext,
						  sprintf("%s %s %s",
							  diagram_name,
							  entity_name,
							  trans_id),
						  vipc.FlgDIRECT);
				   } /* if */
                               }
                           }
                       }
                       ret_value;
                   }

                   function clk_cancel() {
                       FALSE;
                   }

                   if (vlSym.Fetch (vstBkptTable, name) == nil){ 
                       sig_id := FSMSigTrace (clk_name, clk_change, clk_cancel,
                                           clk_cancel);
                       
                       vlSym.Store (vstBkptTable, name,
                                    make_bp ("Transition", diagram_name,
                                             entity_name, id, sig_id,
                                             prefix_name));
                   }
               });
               

vitc.Register ("BreakInState", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name, state_id, prefix_name;
                   declare cur_state_name;

                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &state_id, " ");
                   data_str = string_token (data_str, &cur_state_name, " ");
                   data_str = string_token (data_str, &prefix_name, " ");

                   name := sprintf("S_%s_%s_%s_%s", diagram_name, entity_name,
                                   state_id, prefix_name);

                   cs_name := entity_name + "\\" + cur_state_name;

                   id := convert (vlIntVT, state_id);

                   function cs_change() {
                       val := convert (vlStringVT, vsim_val (cs_name));
                       ret_val := FALSE;
                       if (val[0] != 'X') {
                           value := convert (vlIntVT, vsim_val (cs_name));
                           if (value == id) {
			       vitc.Send ("BreakHitState",
					  vsimContext, vstContext,
					  sprintf("%s %s %s",
						  diagram_name,
						  entity_name,
						  state_id),
					  vipc.FlgDIRECT);
                               ret_val = TRUE;
			   } /* if */
                       }
                       ret_val;
                   }

                   function cs_cancel() {
                       FALSE;
                   }
                   
                   if (vlSym.Fetch (vstBkptTable, name) == nil){ 
                       sig_id := FSMSigTrace (cs_name, cs_change, cs_cancel,
                                           cs_cancel);
                       
                       vlSym.Store (vstBkptTable, name,
                                    make_bp ("State", diagram_name,
                                             entity_name, id, sig_id,
                                             prefix_name));
                   } 
               });

vitc.Register ("DisableAnimation", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name;
                   declare clock_name, clock_edge;
                   declare cur_state_name, cur_trans_name, cur_intr_name;
                   declare prefix_name;

                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &cur_state_name, " ");
                   data_str = string_token (data_str, &cur_trans_name, " ");
                   data_str = string_token (data_str, &cur_intr_name, " ");
                   data_str = string_token (data_str, &clock_name, " ");
                   data_str = string_token (data_str, &clock_edge, " ");
                   data_str = string_token (data_str, &prefix_name, " ");

                   name := sprintf("%s_%s", diagram_name, entity_name);
               
                   diagram_data := vlSym.Fetch (vstDiagramTable, name);
                   if (!null (diagram_data)) {
                       FSMSigTraceCancel (diagram_data.trans_trace_id, TRUE);
                       FSMSigTraceCancel (diagram_data.state_trace_id, TRUE);
                       FSMSigTraceCancel (diagram_data.intr_trace_id, TRUE);
                       vlSym.Remove (vstDiagramTable, name);
                   }
               });

vitc.Register ("EnableAnimation", vsimContext, vstContext,
               func (dtype, src, match, data_str) {
                   declare diagram_name, entity_name;
                   declare clock_name, clock_edge;
                   declare cur_state_name, cur_trans_name, cur_intr_name;
		   declare prefix_name;

                   data_str = string_token (data_str, &diagram_name, " ");
                   data_str = string_token (data_str, &entity_name, " ");
                   data_str = string_token (data_str, &cur_state_name, " ");
                   data_str = string_token (data_str, &cur_trans_name, " ");
                   data_str = string_token (data_str, &cur_intr_name, " ");
                   data_str = string_token (data_str, &clock_name, " ");
                   data_str = string_token (data_str, &clock_edge, " ");
		   data_str = string_token (data_str, &prefix_name, " ");

                   name := sprintf("%s_%s", diagram_name, entity_name);

                   if (null (prefix_name))
                       prefix_name = "";

		   diagram_data := struct {
		       prefix = prefix_name;
		       name = diagram_name;
		       entity = entity_name;
		       clock = clock_name;
		       clock_edge = clock_edge;
		       trans_trace_id = nil;
                       intr_trace_id = nil;
		       state_trace_id = nil
		   };
		   
                   old_data := vlSym.Fetch (vstDiagramTable, name);
                   if (old_data == nil) {
                       if (notify_vfsmcap (cur_trans_name, cur_state_name,
                                           cur_intr_name,
                                           entity_name, diagram_name,
                                           clock_name, clock_edge,
                                           diagram_data)) 
                           vlSym.Store (vstDiagramTable, name, diagram_data);
                   } else {
                       FSMSigTraceCancel (old_data.trans_trace_id, TRUE);
                       FSMSigTraceCancel (old_data.state_trace_id, TRUE);
                       FSMSigTraceCancel (old_data.intr_trace_id, TRUE);
                       
		       vlSym.Remove (vstDiagramTable, name);
		       
                       if (notify_vfsmcap (cur_trans_name, cur_state_name,
                                           cur_intr_name,
                                           entity_name, diagram_name,
                                           clock_name, clock_edge,
                                           diagram_data)) 
                           vlSym.Store (vstDiagramTable, name, diagram_data);
		   } /* } else */
                   nil;
               });
               
function FSMSigTraceCancel(sig_cell, log_flag) {
    sig := car (sig_cell);
    set_car (sig_cell, nil);
    vsim.SigCancel (sig, log_flag);
} /* function FSMSigTraceCancel */

function FSMSigTrace(signals, call_on_change, call_on_cancel, call_on_restart) {
    sig_cell := cons (nil, nil);

    function establish_breakpoint() {
        status := TRUE;
        catch (vlEx.FAILURE) 
            set_car (sig_cell, vsim.SigWatch (signals, call_on_change, func() {
                call_on_restart ();
                
                if (car (sig_cell) == nil) 
                    call_on_cancel ();
                else {
                    vcWv.Timer (wv, 1000, func() {
                        catch (vlEx.FAILURE)
                            establish_breakpoint ();
                        exception {
                            printf ("Could not re-establish: %s",
                                    vlErr.String (vlErr.CurrentError (),
                                                  vlErr.Stderr));
                            call_on_cancel ();
                        } /* exception */
                    }, FALSE);
                } /* else */
            }));
        exception 
            status = FALSE;

        status;
    } /* function establish_breakpoint */

    if (establish_breakpoint ())
        sig_cell;
    else
        nil;
} /* function FSMSigTrace */

function notify_vfsmcap(cur_trans_signal, cur_state_signal, cur_intr_signal,
                        entity_name, diagram_name, clock_name,
                        clock_edge, diagram_data) {

    ret_status := TRUE;

    prefix := diagram_data.prefix;

    if (!string_eq (prefix, "")) {
        ct_name := prefix + "\\" + entity_name + "\\" + cur_trans_signal;
        cs_name := prefix + "\\" + entity_name + "\\" + cur_state_signal;
        ci_name := prefix + "\\" + entity_name + "\\" + cur_intr_signal;
        cur_name := prefix + "\\" + entity_name + "\\cur_state";
        clk_name := prefix + "\\" + entity_name + "\\" + clock_name;
    } else {
        ct_name := entity_name + "\\" + cur_trans_signal;
        cs_name := entity_name + "\\" + cur_state_signal;
        ci_name := entity_name + "\\" + cur_intr_signal;
        cur_name := entity_name + "\\cur_state";
        clk_name := entity_name + "\\" + clock_name;
    }
        
    clk_edge := convert (vlIntVT, clock_edge);

    vitc.Send ("AnimateInit", vsimContext, vstContext,
               sprintf("%s %s %s", diagram_name,
                       diagram_data.entity,
                       diagram_data.prefix), vipc.FlgDIRECT);
    
    function trans_signal_changed() {
        val := vsim_val (ct_name);
        value_str := convert (vlStringVT, val);
        
        if (value_str[0] != 'X') {
            data := sprintf("%s %d", diagram_name, convert (vlIntVT, val));
            vitc.Send ("AnimateTransUpdate", vsimContext, vstContext,
                       data, vipc.FlgDIRECT);
        }

        FALSE;
    }
    
    function watch_cancelled() {
        FALSE;
    }

    function restart_sim() {
        vitc.Send ("AnimateInit", vsimContext, vstContext,
                   sprintf("%s %s %s", diagram_name, entity_name,
                           diagram_data.prefix), vipc.FlgDIRECT);

        FALSE;
    }

    diagram_data.trans_trace_id = FSMSigTrace (ct_name, trans_signal_changed,
                                           watch_cancelled, restart_sim);
    if (diagram_data.trans_trace_id == nil) {
        vitc.Send ("ErrorAnimation", vsimContext, vstContext,
                   sprintf("%s", diagram_name), vipc.FlgDIRECT);
        ret_status = FALSE;
    }

    /*
     * Interrupt animation
     */
    function intr_signal_changed() {
        val := vsim_val (ci_name);
        value_str := convert (vlStringVT, val);
        
        if (value_str[0] != 'X') {
            data := sprintf("%s %d", diagram_name, convert (vlIntVT, val));
            vitc.Send ("AnimateIntrUpdate", vsimContext, vstContext,
                       data, vipc.FlgDIRECT);
            
            cur_val := vsim_val (cur_name);
            curstate := convert (vlIntVT, cur_val); 
            data := sprintf("%s %d", diagram_name, curstate);
            vitc.Send ("AnimateStateUpdate", vsimContext, vstContext,
                       data, vipc.FlgDIRECT);
        }

        FALSE;
    }
    
    diagram_data.intr_trace_id = FSMSigTrace (ci_name, intr_signal_changed,
                                           watch_cancelled, restart_sim);
    if (diagram_data.intr_trace_id == nil) {
        vitc.Send ("ErrorAnimation", vsimContext, vstContext,
                   sprintf("%s", diagram_name), vipc.FlgDIRECT);
        ret_status = FALSE;
    }

    /*
     * State animation
     */
    function clk_signal_changed() {
        clk_val := vsim_val (clk_name);
        cs_val := vsim_val (cs_name);
        ci_val := vsim_val (ci_name);
        cur_val := vsim_val (cur_name);
        
        clk_str := convert (vlStringVT, clk_val);
        if (clk_str[0] != 'X') {
            clk := convert (vlIntVT, clk_val);
            
            if (clk == clk_edge) {
                cs := convert (vlStringVT, cs_val);
                if (cs[0] != 'X') {
                    cs = convert (vlIntVT, cs_val);
                    curstate := convert (vlIntVT, cur_val); 
                    intr_str := convert (vlStringVT, ci_val);
                    if (intr_str[0] != 'X') {
                        intr := convert (vlIntVT, ci_val);
                        
                        /*
                         * If we were triggered via an interrupt
                         * the nxt_state signal is invalid, instead
                         * the cur_state signal is to be used
                         */
                        if (intr == 0)
                            data := sprintf("%s %d", diagram_name, cs);
                        else
                            data := sprintf("%s %d", diagram_name, curstate);
                        
                        vitc.Send ("AnimateStateUpdate", vsimContext, vstContext,
                                   data, vipc.FlgDIRECT);
                    }
                }
            }
        }
        
       FALSE;
    }
    
    diagram_data.state_trace_id = FSMSigTrace (clk_name, clk_signal_changed,
                                            watch_cancelled, restart_sim);
    if (diagram_data.state_trace_id == nil) {
        vitc.Send ("ErrorAnimation", vsimContext, vstContext,
                   sprintf("%s", diagram_name), vipc.FlgDIRECT);
        ret_status = FALSE;
    }
    
    ret_status;
}


