
set_stdout(TRUE);

/*
 * The following ViewScript is for the display and selection of 
 * generic/predefined expressions for use in the Math Expression Evaluator.
 */
CONSTANT := vcWv.Constant;
STICKY   := vcWv.Sticky;
DYNAMIC  := vcWv.Dynamic;

NUM_COLS := 2;
P_HGT    := 20;
P_WDT    := 20;

function label(str) vcRegion.Create1(str, vcLabel.fac, str, nil);

function cmdline_wdt(default, wide) vcRegion.Create1("EnterExprCmdLine" +
                                           convert(string, default),
                                           vcCmdline.fac, default,
                                           list(cons(`width, wide),
                                                cons(`macro_on, FALSE)));

function rubberize(s, x, y) vcRubber.Create(s, x, y);

function macro_off(r)
{
    vcRegion.PrefSet(r,list(macro_on <- FALSE));
    r;
}


/*
 * Checks if the specified expr filename has already an extension.  If it
 * does, then it's reused.  Otherwise, the "xpr" extension is appended and the
 * expr file is reconstructed.  Note that the vscript.vsParse returns a
 * four-element list: device, pathname, filename and extension.
 */
function parse_expr_filename(exprfile)
{
    declare parse_list, ext;

    if (!null (parse_list = vscript.vsParse (exprfile)))
    {
        if (string_eq ("", (ext = cadddr (parse_list))))
            vscript.vsMerge (car (parse_list), cadr (parse_list),
                             caddr (parse_list), "xpr");
        else
            vscript.vsMerge (car (parse_list), cadr (parse_list),
                             caddr (parse_list), ext);
    }
    else
        vlEx.Throw2 (vlEx.FAILURE, "vcMac", vlErrSev.Error,
                     sprintf ("Unable to parse this expr. file name - %s",
                               exprfile));
}

function set_curr_expr_aux (data)
{
    function set_curr_expr (sel_str, on_or_off)
    {
      if ( on_or_off == vlOnOff.On ) {
        data.expr_str = convert (vlStringVT, sel_str);
        vcWin.ClearStatus(win := vcRegion.Window(data.t));
        vcWin.AddStatus(win, "Current Expression:" + data.expr_str);
      }
    }
}

function mk_scroll_pick(lst_name, lst, me_data, n_row, n_col)
{
    if ( n_row <= 0 ) n_row = 5;
    if ( n_col <= 0 ) n_col = 3;
    pick_region := vcRegion.Create1(lst_name, vcPicklist.fac, lst,
                              list (rows      <- n_row,
                                    columns   <- n_col,
                                    exclusive <- TRUE,
                                    range     <- TRUE));
    macro_off(pick_region);
    vcPicklist.Notify(pick_region, set_curr_expr_aux(me_data));
    scroll_picklist := vcScroll.Create(pick_region, P_HGT, P_WDT, nil);
}



/* This command produces the predefined expression list popup menu */
function _Create (init_data, region)
{
    data := struct {t;              /* main table region */
                    mlist;          /* internal pick list items */
                    file_name;      /* currently loaded file name */
                    expr_str;       /* currently selected expression */
                    which_cmd       /* pick list command type */
                   };

    display   := car (init_data);  init_data = cdr (init_data);
    event     := car (init_data);  init_data = cdr (init_data);
    expr_file := car (init_data);

    vcWv.MakeEnum ( "vtListCmdVT", list ("NO_CMD", "ADD_CMD", "RPL_CMD") );

    main_table := vcTable.Create ("Predefined Expressions", 3, 1);
    vcTable.Row (main_table, -1, vcTable.Border.Visible, 20, 20, 20,
                 vgAlign.None);

    data.t         = main_table;
    data.mlist     = nil;
    data.file_name = expr_file;
    data.expr_str  = "";
    data.which_cmd = vtListCmd.NO_CMD;

    /*
     * Whenever the user chooses the Edit->Add option in the menu,
     * the "expression" in the Math Expression Dialog will be added to the
     * picklist and to a list of predefined expressions. The mlist serves as a
     * convenient data structure to have the predefined expressions readily
     * accessible and always available from the facility's data object.
     */
    function update_picklist(data, event)
    {
        /* Get expression in Expression box */
        expr_region := vcTable.Get(data.t, 2, 0);
        str := vcRegion.Get(vcTable.Get(expr_region, 1, 0), string);

        if ( !member (str, data.mlist))
        {
            vcPicklist.Add (vcTable.Get (data.t, 1, 0), str, str);
            data.mlist = cons (str, data.mlist);
        }
        else {
          vcWin.ClearStatus(win := vcEvent.Window(event));
          vcWin.AddStatus(win, "Expression already in list: " + str );
        }
    }

    function start_add (data, event)
    {
      if ( data.which_cmd != vtListCmd.ADD_CMD ) {
        vcOverlay.SetCurrent (vcTable.Get(expr_region, 0, 0), add_cmd_label);
      }
      if ( data.which_cmd == vtListCmd.NO_CMD ) {
        vcTable.Add (main_table, expr_region);
      }
      data.which_cmd = vtListCmd.ADD_CMD;
    }
    add_cmd := vcWv.Cmd (fac, start_add, "_add_cmd");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(add_cmd, vlOnOff.Off);
 */

 

    /*
     * Whenever the user chooses the Edit->Replace option in the menu,
     * the "expression" in the Math Expression Dialog will be used to 
     * replace the currently selected item in the predefined expression
     * picklist.
     */
    function replace_item(data, event)
    {
        /* Get expression in Expression box in math expression dialog */
        expr_region := vcTable.Get(data.t, 2, 0);
        str := vcRegion.Get(vcTable.Get(expr_region, 1, 0), string);

        if ( !member (str, data.mlist) )
        {
          index       := 0;
          picklst     := vcTable.Get (data.t, 1, 0);
          new_mlist   := nil;
          temp        := reverse (data.mlist);
  
          /* Rebuild the data list */
          while (!null (temp))
          {
              if (vcPicklist.IsSet (picklst, index))
                new_mlist = cons (   str    , new_mlist);
              else
                new_mlist = cons (car (temp), new_mlist);
  
              temp = cdr (temp);
              ++index;
          }   
  
          /* Rebuild the picklist */

          new_picklst := mk_scroll_pick("pklist", nil, data, 0, NUM_COLS);
              
          temp = reverse (new_mlist);
          while (!null (temp))
          {
              vcPicklist.Add (new_picklst, car (temp), car (temp));
              temp = cdr (temp);
          }

          vcTable.Set (data.t, 1, 0, new_picklst);
        }
        else {
          vcWin.ClearStatus(win := vcEvent.Window(event));
          vcWin.AddStatus(win, "Expression already in list: " + str);
        }
    }

    function start_rpl (data, event)
    {
      if ( data.which_cmd != vtListCmd.RPL_CMD ) {
        vcOverlay.SetCurrent (vcTable.Get(expr_region, 0, 0), rpl_cmd_label);
      }
      if ( data.which_cmd == vtListCmd.NO_CMD ) {
        vcTable.Add (main_table, expr_region);
      }
      data.which_cmd = vtListCmd.RPL_CMD;
    }
    replace_cmd := vcWv.Cmd (fac, start_rpl, "_replace_cmd");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(replace_cmd, vlOnOff.Off);
 */



    /*
     * Action routine to delete the selected row in the list of predefined
     * expressions stored in the picklist. Note that we need to check if the
     * new list is empty, which can happen if the last expression has been
     * selected.
     */
    function delete(data, event)
    {
        index       := 0;
        picklst     := vcTable.Get (data.t, 1, 0);
        new_mlist   := nil;
        temp        := reverse (data.mlist);

        while (!null (temp))
        {
            if (!vcPicklist.IsSet (picklst, index))
                new_mlist = cons (car (temp), new_mlist);

            temp = cdr (temp);
            ++index;
        }   

        new_picklst := mk_scroll_pick("pklist", nil, data, 0, NUM_COLS);
              
        temp = reverse (new_mlist);
        while (!null (temp))
        {
            vcPicklist.Add (new_picklst, car (temp), car (temp));
            temp = cdr (temp);
        }
 
        vcTable.Set (data.t, 1, 0, new_picklst);
    }
    delete_cmd := vcWv.Cmd (fac, delete, "_delete_cmd");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(delete_cmd, vlOnOff.Off);
 */


    /*
     * Load the expressions from the file.
     */
    function load_file(data, event, file_to_load)
    {
        
        declare  expr_line;
        new_mlist := nil;

        if ( !equal (file_to_load, "") )
        {
          /* Open, write, close */
          filename := parse_expr_filename(file_to_load);
          if ( (filename = vscript.vsGetini (filename)) != nil ) {

            file_ptr := fopen ( filename, "r" );

            if ( file_ptr != nil )
            {
                row0 := label("Current Expression File: \n" + filename);
                vcTable.Set (data.t, 0, 0, row0);

                while ( (expr_line = fgetline (file_ptr)) != nil )
                {
                  expr_line = string_trim(expr_line);
                  new_mlist = cons (expr_line, new_mlist);
                }   
            }   
            fclose (file_ptr);

            new_picklst := mk_scroll_pick("pklist", nil, data, 0, NUM_COLS);

            temp := reverse (new_mlist);
            while (!null (temp))
            {
              vcPicklist.Add (new_picklst, car (temp), car (temp));
              temp = cdr (temp);
            }
  
            vcTable.Set (data.t, 1, 0, new_picklst);
            data.file_name = file_to_load;
          }
          else {
            vcWin.ClearStatus ( win := vcEvent.Window(event) );
            vcWin.AddStatus ( win, "File Not Found: " + file_to_load );
          }
        }
        else {
          vcWin.ClearStatus ( win := vcEvent.Window(event) );
          vcWin.AddStatus ( win, "No file name entered" );
        }
    }
    load_cmd := vcWv.Cmd (fac, load_file, "_load_file",
                          "Load Expression File",
            vcWv.Param("Expression File",string, STICKY, data.file_name,
                                 vcFilePop.fac, filter <- "*.xpr",
                                                read_now <- TRUE));
/*-PRJ
 * To macro record or not - that is the question
 */
    vcCmd.EnableMacro(load_cmd, vlOnOff.Off);


    function write_list (filename, expr_list)
    {
      temp := expr_list;
      /* Open, write, close */
      file_ptr := fopen ( filename, "w" );

        while (!null (temp))
        {
          fputline ( file_ptr, string_trim(car(temp)) + "\n" );
          temp = cdr (temp);
        }   
      fclose (file_ptr);
    }


    /*
     * Save the expressions in the file.
     */
    function save_expr(data, event)
    {

        /* Open, write, close */
        if ( !equal(data.file_name, "") ) {
          filename := vscript.vsGetini (data.file_name);
          write_list (filename, data.mlist);

          row0 := label("Current Expression File: \n" + filename);
          vcTable.Set (data.t, 0, 0, row0);
        }
        else {
          vcWin.ClearStatus ( win := vcEvent.Window(event) );
          vcWin.AddStatus ( win, "No file name entered" );
        }
    }
    save_cmd := vcWv.Cmd (fac, save_expr, "_save_expr",
                          "Save To Expression File");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(save_cmd, vlOnOff.Off);
 */

    /*
     * Save the expressions in the newly named file.
     */
    function save_as_expr(data, event, new_file_name)
    {

        if ( !equal(new_file_name, "") ) {
          /*
           * Parse file for extension. If there is no extension, add
           * .xpr to the end and then use it.
           */
          parsed_file := vscript.vsParse(new_file_name);
          file_ext    := list_ref(parsed_file, 3);
          if ( (file_ext == nil) || (file_ext == "") )
              new_file_name = new_file_name + ".xpr";

          if ( (filename := vscript.vsGetini (new_file_name)) != nil ) {
            function skip(d,e)
                ;
            skip_cmd := vcWv.Cmd (fac, skip);

            buttons :=                        
                vcRegion.Create ("Buttons", vcButnRow.fac,
                                 list (list ("yes",
                    "Delete the existing expression file.",
                                             func () {
                                               data.file_name = new_file_name;
                                               vcCmd.Run(save_cmd, fac,
                                                         event, nil, TRUE);
                                             },
                                             list (close_window <- TRUE)),
                                       list ("no", "Skip saving this list",
                                             skip_cmd,
                                             list (close_window <- TRUE))));
   
            query_t := vcTable.Create ("Delete?", 2, 1);
            strng :=
                sprintf
                 ("The Expression file exists (%s).  Delete it?",
                     filename);
            text := vcRegion.Create ("", vcStaticText.fac, strng);
            vcTable.Add (query_t, text);
            vcTable.Add (query_t, buttons);

            vcWin.Popup (vcEvent.Window (event), query_t, buttons,
                         vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus +
                         vcWin.NoMacros);
          }
          else {
            data.file_name = new_file_name;
            write_list(new_file_name, data.mlist);

            known_name := new_file_name;
            if ( filename != nil ) known_name = filename;
            row0 := label("Current Expression File: \n" + known_name);
            vcTable.Set (data.t, 0, 0, row0);
          }
        }
        else {
          vcWin.ClearStatus ( win := vcEvent.Window(event) );
          vcWin.AddStatus ( win, "No file name entered" );
        }
    }
    save_as_cmd := vcWv.Cmd (fac, save_as_expr, "_save_as_expr",
                             "Save Predefined Expressions As File",
            vcWv.Param("Save As Expression File",string, STICKY, "",
                            vcFilePop.fac, filter <- "*.xpr",
                                                read_now <- TRUE));
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(save_as_cmd, vlOnOff.Off);
 */


    function close_it(data, event) {
        vcWin.Close( vcEvent.Window(event) );
    }
    cancel_it := vcWv.Cmd (fac, close_it, "_cancel_it");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(cancel_it, vlOnOff.Off);
 */


/*     function set_curr_expr (sel_str, on_or_off) */

    function accept_expr (data, event) {
        apply_expr (data, event);
        invisible  (data, event);
    }
    accept_cmd := vcWv.Cmd (fac, accept_expr, "_accept_cmd");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(accept_cmd, vlOnOff.Off);
 */

    function apply_expr (data, event) {
        if ( data.which_cmd == vtListCmd.ADD_CMD )
          update_picklist (data, event);
        else if ( data.which_cmd == vtListCmd.RPL_CMD )
          replace_item (data, event);
        else ;  /* Do nothing */
        vcRegion.QueueForRefresh (data.t);
        vcRegion.ExecuteRefreshQueue();
    }
    apply_cmd := vcWv.Cmd (fac, apply_expr, "_apply_cmd");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(apply_cmd, vlOnOff.Off);
 */

    function invisible (data, event) {
        vcTable.DeleteRows (data.t, 2, 1);
        data.which_cmd = vtListCmd.NO_CMD;
    }
    set_invisible := vcWv.Cmd (fac, invisible, "_set_invisible");
/*-PRJ
 * To macro record or not - that is the question
    vcCmd.EnableMacro(set_invisible, vlOnOff.Off);
 */

    function add_expr_region (data, event) {
      vcTable.Add (main_table, expr_region);
    }

    row0 := label("Current Expression File: \n" + expr_file);
    vcTable.Add (main_table, row0);

/*
    row1 := vcRegion.Create ("pklist", vcPicklist.fac, nil,
                          columns   <- NUM_COLS,
                          exclusive <- TRUE);
    vcPicklist.Notify(row1, set_curr_expr_aux(data));
*/
    row1 := mk_scroll_pick("pklist", nil, data, 0, NUM_COLS);

    vcTable.Add (main_table, row1);

/*-PRJ
    vcTable.Add(main_table, rubberize("SelSigRubber", 0, 5));
 */

    edit_picks := vcWv.MakeMenu ("List", vcWv.Pin(),
                           list ("Add to list", add_cmd),
                           list ("Remove Selected", delete_cmd),
                           list ("Replace Selected", replace_cmd));

    edit_cmds := vcWv.Cmd (fac,
                             func(data, event)
                               vcMenuPop.Create (edit_picks, event);,
                             "Edit_List");
/*-PRJ
 * To macro record or not - that is the question
 */
    vcCmd.EnableMacro(edit_cmds, vlOnOff.Off);

    file_ops   := vcWv.MakeMenu ("File", vcWv.Pin(),
                           list ("Load File", load_cmd),
                           list ("Save List", save_cmd),
                           list ("Save List As ...", save_as_cmd));

    file_cmds := vcWv.Cmd (fac,
                             func(data, event)
                               vcMenuPop.Create (file_ops, event);,
                             "File_Cmds");
/*-PRJ
 * To macro record or not - that is the question
 */
    vcCmd.EnableMacro(file_cmds, vlOnOff.Off);

    menu_ops   := vcWv.MakeMenu ("Options",
                                  list ("File", file_ops),
                                  list ("List", edit_picks),
                                  list ("Quit", cancel_it)
                                );

    options_cmd := vcWv.Cmd (fac,
                             func(data, event)
                               vcMenuPop.Create (menu_ops, event);,
                             "File_List_Ops");
/*-PRJ
 * To macro record or not - that is the question
 */
    vcCmd.EnableMacro(options_cmd, vlOnOff.Off);

    expr_region := vcTable.Create ("Expression Entry", 3, 1);

    /* Create label for command in effect */
    curr_cmd_overlay := vcRegion.Create
                  ("", vcOverlay.fac, vgAlign.MiddleCenter);

    add_cmd_label := label ("Add Expression To List:");
    rpl_cmd_label := label ("Replace Selected Expression With:");
    vcRegion.AddChild (curr_cmd_overlay, add_cmd_label);
    vcRegion.AddChild (curr_cmd_overlay, rpl_cmd_label);
    vcOverlay.SetCurrent (curr_cmd_overlay, add_cmd_label);
    vcTable.Add (expr_region, curr_cmd_overlay);

    /* Expression entry field */
    expr_field := cmdline_wdt ( "", 30 );
    vcTable.Add (expr_region, expr_field);

    /* Create action buttons for Add/Replace commands */
    action_butns := vcRegion.Create("action_buttons", vcButnRow.fac,
                  list(list("Accept", "Accept",
                            accept_cmd,
                            nil),
                       list("Apply", "Apply",
                            apply_cmd,
                            nil),
                       list("Cancel", "Cancel",
                             set_invisible,
                            nil)));
    vcTable.Add (expr_region, action_butns);

    /*
     * NOTE: The "expr_region" is not added at this time as is customary.
     *       This region appears only if the List->Add or List->Replace
     *       commands are executed.
     */


    vcFac.SetMenus (fac, menu_ops);

    vcRegion.AddChild ( region, main_table );

    /* Show all expressions in file */
    if ( !equal(expr_file, "") )
      load_file (data, event, expr_file);

    data;


} /* _Create */


function _Get ( data, region, desired_type )
{
   if ( desired_type == vlStringVT )
     data.expr_str;
   else if ( desired_type == vcRegionVT )
     data.t;
   else if ( desired_type == consVT)
     data.mlist;
   else
     convert ( desired_type, data );
}


function FileName ( region )
{
  data := vcRegion.Data(region, vscmVT);
  data.file_name;
}

function CurrPickValue ( region )
{
  data := vcRegion.Data(region, vscmVT);
  data.expr_str;
}

set_stdout(FALSE);
