
/* MATH Expression Evaluator stuff */

/* ---- testing variable - makes some commands Macro recordable or not ---- */

/* Global Structure for Math Expression Data*/
    math_data := struct {
                         ge_file_name  = "";
                         start_use_pnt = FALSE;        /* Use sweep or pt/mk */
                         end_use_pnt   = FALSE;        /* Use sweep or pt/mk */
                         sel_expr_str  = "";
                         curr_expr     = "";
                         expr_sym_list = nil;
                         expr_val_list = nil;
                         sweep_start   = -HUGEREAL;
                         sweep_end     =  HUGEREAL;

                         parent_win    = nil;
                         parent_data   = nil;
                         me_window     = nil;
                         math_expr_region= nil;
                         ge_dialog     = nil;
                         sig_dialog    = nil;
                         END_STRUCT    = nil
                        };

function null_math_data() {
              math_data.ge_file_name  = "";
              math_data.start_use_pnt = FALSE;        /* Use sweep or pt/mk */
              math_data.end_use_pnt   = FALSE;        /* Use sweep or pt/mk */
              math_data.sel_expr_str  = "";
              math_data.curr_expr     = "";
              math_data.expr_sym_list = nil;
              math_data.expr_val_list = nil;
              math_data.sweep_start   = -HUGEREAL;
              math_data.sweep_end     =  HUGEREAL;

              math_data.parent_win    = nil;
              math_data.parent_data   = nil;
              math_data.me_window     = nil;
              math_data.math_expr_region= nil;
              math_data.ge_dialog     = nil;
              math_data.sig_dialog    = nil;
}

MACRO_RECORD := FALSE;

/* Some handy shortcuts for specify different types of regions */
function label(str) vcRegion.Create1(str, vcLabel.fac, str, nil);
function rubberize(s, x, y) vcRubber.Create(s, x, y);
function cmdline(default) vcRegion.Create1("MathExprCmdLine" +
                                           convert(string, default),
                                           vcCmdline.fac, default,
                                           list(cons(`width, 10),
                                                cons(`macro_on, FALSE)));

VIS_WDT      := 40;
MAX_VIS_WDT  := 80;
MAX_BUF_SIZ  := 300;
function cmdline_wdt(default, wide) vcRegion.Create1("MathExprCmdLine" +
                                           convert(string, default),
                                           vcCmdline.fac, default,
                                           list(cons(`width, wide),
                                                cons(`max_width, MAX_VIS_WDT),
                                                cons(`max_size,  MAX_BUF_SIZ),
                                                cons(`macro_on, FALSE)));

function macro_off(r)
{
    vcRegion.PrefSet(r,list(macro_on <- FALSE));
    r;
}


function get_curr_expr ( data ) {
    data.curr_expr;
}

function set_curr_expr ( data ) {
    expr_data := vcTable.Get(data.math_expr_region, 3, 0);
    data.curr_expr = vcRegion.Get(vcTable.Get(expr_data, 0, 1), string);
}

function get_val_list ( data ) {
    data.expr_val_list;
}

/* Functions to grab the beginning and ending values of the H axis */

function GetMinPntMarkX (vtrace_data) {
    type := HAxisType(vtrace_data, 0);
    min_sweep := GetHAxisVMin(vtrace_data);
    pnt_value := HAxisVPoint();
    value     := HAxisVMark();
    if ( pnt_value < value ) value = pnt_value;
    if ( value != HUGEREAL ) {
        if (type == vtrDispType.Log && value > 0.0)
            value = exp10(value);
    }
    if ( (value == HUGEREAL) || (value < min_sweep) ) value = min_sweep;
    value;
}

function get_start_value ( data ) {
    declare value;
    if ( data.start_use_pnt == FALSE ) {
         value = GetHAxisVMin ( data.parent_data );
         data.sweep_start = -HUGEREAL;
    }
    else {
         value = GetMinPntMarkX ( data.parent_data );
         data.sweep_start = value;
    }
    value;
}

function set_start_value ( data ) {
        expr_data := vcTable.Get(data.math_expr_region, 3, 0);
        vcRegion.Set(vcTable.Get(expr_data, 1, 1), get_start_value(data));
}

function GetMaxPntMarkX (vtrace_data) {
    type := HAxisType(vtrace_data, 0);
    max_sweep := GetHAxisVMax(vtrace_data);
    pnt_value := HAxisVPoint();
    value     := HAxisVMark();
    if ( pnt_value != HUGEREAL ) {
        if ( pnt_value > value ) value = pnt_value;
    }
    if ( value != HUGEREAL ) {
        if (type == vtrDispType.Log && value > 0.0)
            value = exp10(value);
    }
    if ( (value == HUGEREAL) || (value > max_sweep) ) value = max_sweep;
    value;
}

function get_end_value ( data ) {
    declare value;
    if ( data.end_use_pnt == FALSE ) {
         value = GetHAxisVMax ( data.parent_data );
         data.sweep_end   =  HUGEREAL;
    }
    else {
         value = GetMaxPntMarkX ( data.parent_data );
         data.sweep_end   = value;
    }
    value;
}

function set_end_value ( data ) {
        expr_data := vcTable.Get(data.math_expr_region, 3, 0);
        vcRegion.Set(vcTable.Get(expr_data, 2, 1), get_end_value(data));
}


function save_start_value_aux (data) {
        function save_start_value(value) {
            if ( string_length(value) == 0 )
                value = GetHAxisVMin ( math_data.parent_data );
            data.sweep_start = value;
        } /* save_start_value */
}

function save_end_value_aux (data) {
        function save_end_value(value) {
            if ( string_length(value) == 0 )
                value = GetHAxisVMax ( math_data.parent_data );
            data.sweep_end = value;
        } /* save_end_value */
}


/* Load the Generic/Predefined Expression File facility */
vcFac.Create ("vtMthXpr", vscmVT);

set_stdout(TRUE);


/* build up list of math_expr operations for creating op_picklist */
function ops(s) cons(s, s);
function fnc(s) cons(s, s+"()");

arith_ops  := list(
              list(ops("+"), ops("-"), ops("*"), ops("/")),
              list(ops("+="), ops("-="), ops("*="), ops("/=")),
              list(cons("1/x","minverse()"), cons("-x", "ainverse()"),
                   fnc("power"), fnc("sqrt")),
              list(fnc("abs_"),cons("avg", "average()"),
                   fnc("rms"), fnc("sign")),
              list(fnc("ceil"), fnc("floor"), fnc("max"), fnc("min")),
              list(fnc("deriv"), fnc("integ"), fnc("modulo"), (ops("=")))
                  );

trig_ops   := list(
              list(fnc("sin"), fnc("cos"), fnc("tan")),
              list(fnc("asin"), fnc("acos"), fnc("atan")),
              list(fnc("sinh"), fnc("cosh"), fnc("tanh")),
              list(fnc("hypot"), ops(" "), fnc("atan2"))
                  );

lg_exp_bes_ops := list(
              list(fnc("log"),   fnc("exp")),
              list(fnc("log10"), fnc("expn10")),
              list(fnc("db"),    ops(" ")),
              list(fnc("y0"), fnc("j0")),
              list(fnc("y1"), fnc("j1")),
              list(fnc("yn"), fnc("jn"))
                  );

complx_ops := list(
              list(cons("deg", "degrees()"), cons("grad", "gradians()"), 
                   cons("rad", "radians()")),
              list(cons("cdeg", "cdegrees()"), cons("cgrad", "cgradians()"),
                   cons("crad", "cradians()")),
              list(fnc("real"), fnc("imag"), cons("z*", "conjugate()")),
              list(cons("magn", "magnitude()"), fnc("gdelay"))
                  );


/*
 * update the command line value whenever the user picks a math
 * operator or a signal name from their respective picklists.
 */
function build_expr_aux (me_data, picklist_reg) {
    function build_expr(val, on_off)
    {
        expr_data := vcTable.Get(me_data.math_expr_region, 3, 0);
    
        if (on_off == vlOnOff.On) {
    
            vcCmdline._Insert ( vcTable.Get(expr_data, 0, 1), val);
            me_data.curr_expr = vcRegion.Get(vcTable.Get(expr_data, 0, 1), 
                                string);
            /* Reset picklists to NOT set */
            if ( picklist_reg != nil )
                vcPicklist.SetPick(picklist_reg, vcPicklist.NotSet);
        }
        vcRegion.SetFocus(me_data.math_expr_region, expr_data);
        vcRegion.QueueForRefresh (expr_data);
        vcRegion.ExecuteRefreshQueue();
        vcWin.ClearStatus(me_data.parent_win);
    } /* build_expr */
}

function mk_scroll_pick(lst_name, lst, me_data, n_row, n_col)
{
    if ( n_row <= 0 ) n_row = 5;
    if ( n_col <= 0 ) n_col = 3;
    pick_region := vcRegion.Create1(lst_name, vcPicklist.fac, lst, 
                              list (rows      <- n_row,
                                    columns   <- n_col,
                                    exclusive <- TRUE,
                                    range     <- TRUE));
    macro_off(pick_region);
    vcPicklist.Notify(pick_region, build_expr_aux(me_data, pick_region));
    scroll_picklist := vcScroll.Create(pick_region, 20, 20, nil);
}


/*
 * replace the command line value whenever the user accepts a predefined
 * expression from the picklist
 */
function replace_it(data, event) {
    expr_str := vtMthXpr.CurrPickValue (vcWin.Region(data.ge_dialog));
    /* Pass selected string to expression composer */
    if ( !equal (expr_str, "") ) {
      /* Replace the current expression with a blank string */
      expr_data := vcTable.Get(data.math_expr_region, 3, 0);
      vcRegion.Set(vcTable.Get(expr_data, 0, 1), "" );

      (build_expr_aux(data, nil))(expr_str, vlOnOff.On);
    }
    else {
      vcWin.ClearStatus(win := vcEvent.Window(event));
      vcWin.AddStatus(win, "Nothing selected");
    }
} /* replace_expr */


/*
 * The following code is for the display and selection of signals
 * for use in the Math Expression Evaluator.
 */

/* Function to make signal names parseable by ViewScript */
function sig2vscript ( sig_name )
{
  declare  token;
  vs_name  := "#\"";

  sig_name  = string_token ( sig_name, &token, "\\" );
  vs_name = vs_name + token;
  for ( ; 
       (sig_name = string_token ( sig_name, &token, "\\" )) != nil;
      ) {
      vs_name = vs_name + "\\\\" + token;
  }
  vs_name = vs_name + "\"";
  vs_name;
}

function
update_math_expr_sweeps(display)
{ 
    if (math_data.me_window != nil) {
        expr_data := vcTable.Get(math_data.math_expr_region, 3, 0);
        start_reg := vcRegion.Data(vcTable.Get(expr_data, 1, 1), vcCmdlineVT);
        end_reg   := vcRegion.Data(vcTable.Get(expr_data, 2, 1), vcCmdlineVT);
        vcCmdline.Set(start_reg, get_start_value(math_data));
        vcCmdline.Set(end_reg  , get_end_value  (math_data));
    } /* if */
} /* update_math_expr_sweeps */


/* This routine brings up the math_expr dialog box */
function
display_math_expr_dialog(vtrace_data, event)
{
set_stdout(TRUE);
    expr_parent := vcEvent.Window(event);

    math_data.parent_win    = expr_parent;
    math_data.parent_data   = vtrace_data;

    /* This function actually initiates the command to run the parser */
    function do_math_expr_operation()
    {
        expr_data := vcTable.Get(math_data.math_expr_region, 3, 0);
        expr_str  := vcRegion.Get(vcTable.Get(expr_data, 0, 1), string);

        start_val := math_data.sweep_start;
          end_val := math_data.sweep_end  ;

        if ( string_length(expr_str) > 0 ) {
/*-PRJ
          if ( string_length(end_val  ) > 0 )  {
            if ( string_length(start_val) > 0 )
              vcCmd.Run(MathExprCmd, vtrace_data, event,
                   list(expr_str, start_val, end_val  ), TRUE);
            else
              vcCmd.Run(MathExprCmd, vtrace_data, event,
                   list(expr_str, start_val, end_val  ), TRUE);
          }
          else
 */
            vcCmd.Run(MathExprCmd, vtrace_data, event,
                 list(expr_str, start_val, end_val), FALSE);
        }
        else {
          vcWin.ClearStatus(vtrace_data);
          vcWin.AddStatus(vtrace_data, "Please enter a math expression");
        } /* else */

        return(0);
    } /* do_math_expr_operation */

    function accept_expr(/*d, e*/)
    {
        do_math_expr_operation();
    } /* accept_expr */

    /* Hide/Show functions for signal picklist */
    function close_sig_aux(data)
    {
        function close_sig ( win )
        {
          vcOverlay.SetCurrent (sig_overlay, sig_show_but);
          data.sig_dialog = nil;
        }
    }

    function sig_hide(d,e) {
      vcOverlay.SetCurrent (sig_overlay, sig_show_but);
      if ( math_data.sig_dialog != nil ) vcWin.Close(math_data.sig_dialog);
      math_data.sig_dialog = nil;
    }

    sig_hide_cmd := vcWv.Cmd (vcWin.fac, sig_hide, "sig_hide");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(sig_hide_cmd, vlOnOff.Off);

    function sig_show(d,e) {
      vcCmd.Run(SigSelCmd, vtrace_data, event, nil, MACRO_RECORD);
      vcOverlay.SetCurrent (sig_overlay, sig_hide_but);
    }

    sig_show_cmd := vcWv.Cmd (vcWin.fac, sig_show, "sig_show");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(sig_show_cmd, vlOnOff.Off);

    /* Hide/Show functions for predefined expression picklist */
    function destroy_ge_aux(data)
    {
        function destroy_ge ( win )
        {
          data.ge_dialog = nil;
        }
    }

    function close_ge_aux(data)
    {
        function close_ge ( win )
        {
          vcOverlay.SetCurrent (ge_overlay, ge_show_but);
          if ( data.ge_dialog != nil ) {
              data.ge_file_name = vtMthXpr.FileName
                                 (vcWin.Region(data.ge_dialog));
              vcWin.Destroy(data.ge_dialog);
          }
          data.ge_dialog = nil;
        }
    }

    function ge_hide(d,e) {
      /* before "hiding" get the current file loaded */
      if ( math_data.ge_dialog != nil ) vcWin.Close(math_data.ge_dialog);
    }

    ge_hide_cmd := vcWv.Cmd (vcWin.fac, ge_hide, "ge_hide");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(ge_hide_cmd, vlOnOff.Off);

    function ge_show(d,e) {
      ge_region := vcRegion.Create ("Predefined Expressions", vtMthXpr.fac,
                                    list(d,e, math_data.ge_file_name));

      if ( MACRO_RECORD == FALSE )
          math_data.ge_dialog = vcWin.Create (math_data.me_window,
                                              ge_region, vcWin.NoMacros);
      else
          math_data.ge_dialog = vcWin.Create (math_data.me_window,
                                              ge_region);

      vcWin.CallOnClose(math_data.ge_dialog, close_ge_aux(math_data));
      vcWin.DestroyNotify(math_data.ge_dialog, destroy_ge_aux(math_data));
      vcOverlay.SetCurrent (ge_overlay, ge_hide_but);
    }

    ge_show_cmd := vcWv.Cmd (vcWin.fac, ge_show, "ge_show");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(ge_show_cmd, vlOnOff.Off);

/* This command produces the signal list menu */
function
showselsignal(data, event)
{
    declare cols, rows, sig_list, sig_picklist;

    /* Show all visible signals */
    if ( math_data.parent_data != nil )
        sig_list = get_vis_ordinate_sig_names(math_data.parent_data);
    else
        sig_list = nil;
    list_len := length(sig_list);

    if (list_len == 0) {
        vcWin.ClearStatus(win := math_data.parent_win);
        vcWin.AddStatus(win, "No signals to show");
        return(FALSE);
    }

    if (list_len < 5) {
        cols = 1;
        rows = list_len;
    }
    else if (list_len < 10) {
        cols = 2;
        rows = (list_len / cols) + 1;
    }
    else {
        cols = 3;
        rows = (list_len / cols) + 1;
    }

    if (rows <= 12)
        visible := rows;
    else
        visible := 12;

    signal_picks := vcTable.Create("Selectable Signals", 2, 1);

    s_picklist := mk_scroll_pick("sig_picklist", nil,
                                  math_data, visible, visible);

    for (sig := sig_list; sig != nil; sig = cdr(sig))
        vcPicklist.Add(s_picklist, car(sig),
                       sig2vscript(car(sig)) );

    buttons := vcRegion.Create("buttons", vcButnRow.fac,
                     list(
                          list("Cancel", "Cancel", sig_hide_cmd,
                                nil)));

    vcTable.Add(signal_picks, s_picklist);
    vcTable.Add(signal_picks, buttons);

    math_data.sig_dialog = vcWin.Create(math_data.me_window,
                           signal_picks,
                           vcWin.NoCmdline +
                           vcWin.NoMenus +
                           vcWin.NoStatus +
                           vcWin.NoBindings);
    vcWin.DestroyNotify(math_data.sig_dialog, close_sig_aux(math_data));

} /* showselsignal */

SigSelCmd := vcWv.Cmd(fac, showselsignal, "_ShowSelSignal");
/* Never macro recordable */
vcCmd.EnableMacro(SigSelCmd, vlOnOff.Off);


    function get_predef_expr ( d, e )
    {
      if ( math_data.ge_dialog != nil ) {
        replace_it(math_data, e);
      }
      else {
        vcWin.ClearStatus(win := vcEvent.Window(e));
        vcWin.AddStatus(win, "No Predefined Expression Dialog Open");
      }
    }

    get_expr_cmd := vcWv.Cmd (vcWin.fac, get_predef_expr, "get_predef_expr");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(get_expr_cmd, vlOnOff.Off);


    function clear_it(data, event) {
        /* Replace the current expression with a blank string */
        expr_data := vcTable.Get(math_data.math_expr_region, 3, 0);
        vcRegion.Set(vcTable.Get(expr_data, 0, 1), "" );
        math_data.curr_expr = "";
    } /* clear_expr */

    clr_expr_cmd:= vcWv.Cmd(vcWin.fac, clear_it, "_clear_it");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(clr_expr_cmd, vlOnOff.Off);


    /* Use Minimum Sweep or Minimum Point/Mark for start value */
    
    function strt_min(d,e) {
        math_data.start_use_pnt = FALSE;
        set_start_value (math_data);
        vcOverlay.SetCurrent (strt_overlay, strt_pnt_but);
        math_data.sweep_start = -HUGEREAL;
    }

    strt_min_cmd := vcWv.Cmd (vcWin.fac, strt_min, "strt_min");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(strt_min_cmd, vlOnOff.Off);

    function strt_pnt(d,e) {
        math_data.start_use_pnt = TRUE;
        set_start_value (math_data); 
        vcOverlay.SetCurrent (strt_overlay, strt_min_but);
    }

    strt_pnt_cmd := vcWv.Cmd (vcWin.fac, strt_pnt, "strt_pnt");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(strt_pnt_cmd, vlOnOff.Off);

    /* Use Maximum Sweep or Maximum Point/Mark for start value */
    
    function end_max(d,e) {
        math_data.end_use_pnt = FALSE;
        set_end_value (math_data); 
        vcOverlay.SetCurrent (end_overlay, end_pnt_but);
        math_data.sweep_end   =  HUGEREAL;
    }

    end_max_cmd := vcWv.Cmd (vcWin.fac, end_max, "end_max");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(end_max_cmd, vlOnOff.Off);

    function end_pnt(d,e) {
        math_data.end_use_pnt = TRUE;
        set_end_value (math_data); 
        vcOverlay.SetCurrent (end_overlay, end_max_but);
    }

    end_pnt_cmd := vcWv.Cmd (vcWin.fac, end_pnt, "end_pnt");
    if ( MACRO_RECORD == FALSE )
         vcCmd.EnableMacro(end_pnt_cmd, vlOnOff.Off);

    /* End Minimum/Maximum functions for sweep                 */


    /* if symbols_dialog doesn't exist then create it */
    if (math_data.math_expr_region == nil) {

        /* create the main symbols dialog box table */
        math_data.math_expr_region =
                    vcTable.Create("Math Expression Evaluator", 5, 1);
        macro_off(math_data.math_expr_region);

        /* create buttons */
        buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                       list (list ("Accept",
                                   "Accept",
                                   do_math_expr_operation,
                                   list (close_window <-
                                         TRUE)),
                             list ("Apply",
                                   "Apply",
                                   do_math_expr_operation,
                                   nil),
                             list ("Cancel",
                                   "Cancel",
                                    func() vcWin.Destroy(math_data.me_window);,
                                    nil)));

        /* create hide/show regions/buttons for signals */
        sig_overlay := vcRegion.Create
                  ("Signal_Picks", vcOverlay.fac, vgAlign.MiddleCenter);

        sig_hide_but := vcWv.LabelButtonCreate ("sig_hide_but", "Hide Signals",
                                                 nil, sig_hide_cmd, nil);
        sig_show_but := vcWv.LabelButtonCreate ("sig_show_but", "Show Signals",
                                                 nil, sig_show_cmd, nil);

        vcRegion.AddChild (sig_overlay, sig_hide_but);
        vcRegion.AddChild (sig_overlay, sig_show_but);
        vcOverlay.SetCurrent (sig_overlay, sig_show_but);

        /* create hide/show regions/buttons for predefined expressions */
        ge_overlay := vcRegion.Create
                  ("Expression_Picks", vcOverlay.fac, vgAlign.MiddleCenter);

        ge_hide_but := vcWv.LabelButtonCreate ("ge_hide_but",
                                               "Hide Expression",
                                                nil, ge_hide_cmd, nil);
        ge_show_but := vcWv.LabelButtonCreate ("ge_show_but",
                                               "Show Expression",
                                                nil, ge_show_cmd, nil);

        vcRegion.AddChild (ge_overlay, ge_hide_but);
        vcRegion.AddChild (ge_overlay, ge_show_but);
        vcOverlay.SetCurrent (ge_overlay, ge_show_but);

        clr_expr_but := vcWv.LabelButtonCreate ("clr_expr_but",
                                               "Clear Expression",
                                                nil, clr_expr_cmd, nil);

        show_hide_tbl := vcTable.Create("Show/Hide Buttons", 1, 3);
        vcTable.Add(show_hide_tbl,  sig_overlay);
        vcTable.Add(show_hide_tbl, clr_expr_but);
        vcTable.Add(show_hide_tbl,   ge_overlay);


        /* create picklist of expressions operators */
        ops_funcs  := vcTable.Create("Math_Ops_and_Funcs", 2, 4);
            {
                arith_ops  := mk_scroll_pick("Arith_Ops",
                                              arith_ops , math_data, 6, 4);
                vcTable.Set(ops_funcs, 0, 0, label("Arithmetic"));
                vcTable.Set(ops_funcs, 1, 0, arith_ops);
            }
            {
                trig_ops   := mk_scroll_pick("Trig_Ops",
                                              trig_ops  , math_data, 4, 3);
                vcTable.Set(ops_funcs, 0, 1, label("Trigonometric"));
                vcTable.Set(ops_funcs, 1, 1, trig_ops );
            }
            {
                lg_exp_bes_ops := mk_scroll_pick("Log_Exp_Bes_Ops",
                                              lg_exp_bes_ops, math_data, 3, 2);
                vcTable.Set(ops_funcs, 0, 2, label("Log/Exp/Bessel"));
                vcTable.Set(ops_funcs, 1, 2, lg_exp_bes_ops);
            }
            {
                complx_ops := mk_scroll_pick("Complex_Ops",
                                              complx_ops, math_data, 4, 3);
                vcTable.Set(ops_funcs, 0, 3, label("Complex"));
                vcTable.Set(ops_funcs, 1, 3, complx_ops);
            }

        expr_data := vcTable.Create("Expression Data", 3, 3);
        macro_off(expr_data);
        vcTable.Set(expr_data, 0, 0, label("Expression:"));
        {
            NewSigExprCmdline := cmdline_wdt("", VIS_WDT);
            vcTable.Set(expr_data, 0, 1, NewSigExprCmdline);
            vcCmdline.CallOnReturn(NewSigExprCmdline,
                                   func(event, str) { accept_expr(); });
        }
        /* Add command to get selected generic expression */
        get_pre_expr := vcWv.LabelButtonCreate ("get_pre_expr",
                                                "Get Selected Expr.",
                                                 nil, get_expr_cmd, nil);

        vcTable.Set(expr_data, 0, 2, get_pre_expr);
        vcTable.Set(expr_data, 1, 0, label("Start " +
                                            get_h_axis_label(vtrace_data) +
                                           ":"));
        {
            vcTable.Set(expr_data, 1, 1,
                        StartCmdline := cmdline(get_start_value(math_data)));
            vcCmdline.CallOnChange(StartCmdline,
                        save_start_value_aux(math_data));
        }

        /* create use minimum (point,mark)/min X for sweep start value */
        strt_overlay := vcRegion.Create
                  ("Start Value", vcOverlay.fac, vgAlign.MiddleCenter);

        strt_min_but := vcWv.LabelButtonCreate ("strt_min_but",
                                               "Change Min Sweep",
                                                nil, strt_min_cmd, nil);
        strt_pnt_but := vcWv.LabelButtonCreate ("strt_pnt_but",
                                               "Change Min Pt/Mark",
                                                nil, strt_pnt_cmd, nil);
        vcTable.Set(expr_data, 1, 2, strt_overlay);

        vcRegion.AddChild (strt_overlay, strt_min_but);
        vcRegion.AddChild (strt_overlay, strt_pnt_but);
        vcOverlay.SetCurrent (strt_overlay, strt_pnt_but);

        /* create use maximum (point,mark)/max X for sweep  end  value */
        end_overlay := vcRegion.Create
                  ("Start Value", vcOverlay.fac, vgAlign.MiddleCenter);

        end_max_but := vcWv.LabelButtonCreate ("end_max_but",
                                               "Change Max Sweep",
                                                nil, end_max_cmd, nil);
        end_pnt_but := vcWv.LabelButtonCreate ("end_pnt_but",
                                               "Change Max Pt/Mark",
                                                nil, end_pnt_cmd, nil);
        vcTable.Set(expr_data, 2, 2, end_overlay);

        vcRegion.AddChild (end_overlay, end_max_but);
        vcRegion.AddChild (end_overlay, end_pnt_but);
        vcOverlay.SetCurrent (end_overlay, end_pnt_but);

        vcTable.Set(expr_data, 2, 0, label("End " +
                                            get_h_axis_label(vtrace_data) +
                                           ":"));
        {
            vcTable.Set(expr_data, 2, 1,
                        EndCmdline := cmdline(get_end_value(math_data)));
            vcCmdline.CallOnChange(EndCmdline,
                        save_end_value_aux(math_data));
        }


        /* end of min/max sweep regions                                */

        /* add regions to main dialog box region */
        vcTable.Add(math_data.math_expr_region, ops_funcs);
        vcTable.Add(math_data.math_expr_region,
                                        rubberize("MathExprRubber", 0, 5));
        vcTable.Add(math_data.math_expr_region, show_hide_tbl);
        vcTable.Add(math_data.math_expr_region, expr_data);
        vcTable.Add(math_data.math_expr_region, buttons);

        vcRegion.SetFocus(math_data.math_expr_region, expr_data);

        /*
         *  Create a window with no cmdline and no status as a
         * child/popup of the ViewTrace parent window (auto iconification
         * when parent is closed).
         */
        math_data.me_window = vcWin.Create(expr_parent,
                              math_data.math_expr_region,
                              vcWin.NoCmdline +
                              vcWin.NoMenus +
                              vcWin.NoBindings);

        /* if main window closed and dialogs still opened, close it */
        function close_all_win_aux ( data )
        {
            function close_all_win(win) {
                if (data.me_window!= nil)
                    vcWin.Close(data.me_window);
                    vcWin.Destroy(win);
            }
        }

        vcWin.DestroyNotify(expr_parent, close_all_win_aux(math_data));

        /* set the math_expr_region box to nil if window closed */
        function close_me_win_aux ( data )
        {
             function close_me_win (win)
             {
                 ge_file_name := "";
                 expr_reg_clear();
                 if ( math_data.sig_dialog != nil )
                     vcWin.Close(math_data.sig_dialog);
                 if ( math_data.ge_dialog != nil ) {
                     ge_file_name = vtMthXpr.FileName
                                   (vcWin.Region(data.ge_dialog));
                     vcWin.Close(math_data.ge_dialog);
                 }
                 null_math_data();
                 data.ge_file_name = ge_file_name;
             }
        }

        vcWin.DestroyNotify(math_data.me_window,
                            close_me_win_aux(math_data));

    } /* if */
    else {
        /* If it already exists, show it */
        vcWin.Show(math_data.me_window);
    } /* else */

    math_data;
} /* display_math_expr_dialog */

SignalMathExprCmd := vcWv.Cmd(fac, display_math_expr_dialog, "math_expr");
vcCmd.EnableMacro(SignalMathExprCmd, vlOnOff.Off);


function evaluate ( vtrace_data, event, arg_list )
{
  declare var_val_lst;
  expr   := list_ref (arg_list, 0);
  start  := list_ref (arg_list, 1);
  end    := list_ref (arg_list, 2);
  tot_vars := length (arg_list) - 3;
  if ( tot_vars > 0 )
  {
    for (i := 0, var_val_lst = nil; i < tot_vars; i=i+1)
    {
      var_val_lst = cons ((list_ref (arg_list, i+3)), var_val_lst);
    }
  }
  else
    var_val_lst = list();

  expr_eval_check(vtrace_data, event, expr, start, end, var_val_lst);
}

eval_expr_cmd := vcWv.Cmd ( fac, evaluate, "eval_reg_expr",
                           "Evaluate Fully Specified Expression",
                vcWv.Param("Input Data List Format: expr start end [values...]",
                            vscmVT, DYNAMIC, get_val_list) );
/*-PRJ
 * This command needs to be macro recordable
vcCmd.EnableMacro(eval_expr_cmd, vlOnOff.Off);
 */

function parse_expr_reg ( vtrace_data, event, expr_str, start, end )
{
  math_data.expr_sym_list = nil;
  math_data.expr_sym_list = expr_parse(vtrace_data, event, expr_str, start, end);
}

/* Always macro recordable */
ParseEvalCmd:= vcWv.Cmd(fac, parse_expr_reg, "parse_expr_reg",
                                "Math Expression Parser/Evaluator",
                     vcWv.Param("Expression",string,DYNAMIC,get_curr_expr),
                     vcWv.Param("Start",float,DYNAMIC,get_start_value),
                     vcWv.Param("End",  float,DYNAMIC,get_end_value));


function math_expr ( vtrace_data, event, expr_str, start, end )
{
      /* Always macro recordable */
      vcCmd.Run(ParseEvalCmd, vtrace_data, event,
                  list(expr_str, start, end), TRUE);

      if ( !null(math_data.expr_sym_list) )
      {
        /*
         * Build the argument list and function on the fly.
         */

        temp := math_data.expr_sym_list;
        arg_list := " expr_str, start, end";

        arg_list := " expr_str, start, end";
        param_str := " \"expr_str\", \"start\", \"end\"";

        while ( !null(temp) )
        {
          v := car (temp);
          arg_list = arg_list + ", " + v;
          param_str = param_str + ", \"" + v + "\"";

          temp = cdr (temp);
        }

        /*
         * Construct the action function for the command to evaluate
         * the expression.
         * 
         * NOTE: 1. Extra ; at end of func_str is for parser (vcWin.Eval())
         *       2. The command vtrace.eval_expr_cmd is
         *          always macro recordable
         */
        func_str := "func( data, event, " + arg_list + " ) " +
                    "{" +
                    "expr_val_list := cons(list(" + arg_list + " ),nil); " +
                    "vcCmd.Run(vtrace.eval_expr_cmd, data, event, " +
                    "list(list( " + arg_list + " )), TRUE); " +
                    "};";

        action_func := vcWin.Eval ( vcEvent.Window(event),
                       func_str );

        /*
         * Construct the command to evaluate the expression.
         *
         * Construct the function parameter list.
         * Start with the fixed argument information.:
         *        1. expression string
         *        2. sweep start value
         *        3. sweep  end  value
         *        4. add the list of unknown variable values which match
         *           variables in action_func().
         */
        param_lst  := vcWin.Eval ( vcEvent.Window(event),
                     "list(" + param_str + " );" );

        temp = math_data.expr_sym_list;
        param_str = vcWv.Param("Expression",string,CONSTANT,expr_str);
        list_set ( param_lst, 0, param_str );

        param_str = vcWv.Param("Start",float,CONSTANT,start);
        list_set ( param_lst, 1, param_str );

        param_str = vcWv.Param("End",float,CONSTANT,end);
        list_set ( param_lst, 2, param_str );

        arg_tot := length(param_lst);
        for ( i := 3; i < arg_tot; i=i+1)
        {
          v := car(temp);
          /*
           * Add to the function parameter list the variables:
           * vcWv.Param("Variable: var_name",float,STICKY,"")
           * NOTE: replace STICKY with DYNAMIC to take from a list of values.
           */

          if ( i > 3 )
            /* Remaining signal variables - continue through list */
            param_str = vcWv.Param(v,string,DYNAMIC,vtrace.get_next_signal);
          else if ( i == 3 )
            /* This is the first signal variable - start at top of list */
            param_str = vcWv.Param(v,string,DYNAMIC,vtrace.get_first_signal);
          else ;

          list_set ( param_lst, i, param_str );
          temp = cdr(temp);
        }

        /* This command is never macro recordable */
        expr_cmd := vcCmd.Create ( fac, action_func, "bind_var",
                          "Variable Assignment for:",
                           param_lst );
        vcCmd.EnableMacro(expr_cmd, vlOnOff.Off);

        vcCmd.Run(expr_cmd, vtrace_data, event,
                  list(expr_str, start, end), FALSE);
      } /* end if */
} /* math_expr */

/* This command is never macro recorded, commands internal to it are */
MathExprCmd := vcWv.Cmd(fac, func(d,e,s1,start,end)
                     math_expr(d,e,s1,start,end);,
                     "eval_expr","Math Expression Parser",
                     vcWv.Param("Expression",string,DYNAMIC,get_curr_expr),
                     vcWv.Param("Start",float,DYNAMIC,get_start_value),
                     vcWv.Param("End",  float,DYNAMIC,get_end_value));

vcCmd.EnableMacro(MathExprCmd, vlOnOff.Off);


    /*
     * Workaround for overloading binary operators. See mathexpr.c
     */

    vscript.overload ( "+",  add_vc, vtrMathSigVarVT, flonumVT );
    vscript.overload ( "+",  add_cv, flonumVT, vtrMathSigVarVT );

    vscript.overload ( "-",  sub_vc, vtrMathSigVarVT, flonumVT );
    vscript.overload ( "-",  sub_cv, flonumVT, vtrMathSigVarVT );

    vscript.overload ( "*", mult_vc, vtrMathSigVarVT, flonumVT );
    vscript.overload ( "*", mult_cv, flonumVT, vtrMathSigVarVT );

    vscript.overload ( "/",  div_vc, vtrMathSigVarVT, flonumVT );
    vscript.overload ( "/",  div_cv, flonumVT, vtrMathSigVarVT );

    vscript.overload ( "%",  mod_vc, vtrMathSigVarVT, flonumVT );

set_stdout(FALSE);
