/* $Revision: 1.2 $ */

geography := `(Countries
               (USA
                Washington_DC
                (Massachusetts
                 (Boston Allston
                  (Back_Bay Arlington Berkeley Clarendon Dartmouth Exeter
                   Gloucestor Hereford Beacon Marlborough Commonwealth
                   Newbury Boylston St_James Stuart)
                  Roxbury Dorchester Brighton Beacon_Hill)
                 (Cambridge Harvard_Square Central_Sqaure North East Aggasiz
                  Dana))
                (New_York Albany)
                (Connecticut Hartford)
                (New_Hampshire Concord)
                (Vermont Montpelier)
                (Maine Augusta)
                (New_Jersey Trenton)
                (Delaware Dover)
                (Pennsylvania Philadelphia)
                (Ohio Columbus)
                (Michigan Detroit))
               (France Paris)
               (England London)
               (Spain Madrid));
    
iterator place_children(p, sub_class) {
    if (ispair (p)) {
        foreach (e; vscript.list_iter (cdr (p)))
            iterate (e, ispair (e) ? place_class : place_leaf_class);
    } /* if */
} /* iterator place_children */
        
function place_name(p)
    symbol2string (ispair (p)? car (p): p);

function place_edit(p, editing_done, class) {
    while (ispair (p))
        p = car (p);
    cmdline := vcRegion.Create ("PlaceEditor", vcCmdline.fac,
                                symbol2string (p));
    w := vcWin.Create (cmdline);
    vcCmdline.CallOnReturn (cmdline, func(event, str) {
        vcWin.Destroy (w);
    });
    vcWin.DestroyNotify (w, func(w) editing_done (););
} /* function place_edit */

function place_drag(p) {
    while (ispair (p))
        p = car (p);
    symbol2string (p);
} /* function place_drag */

iterator place_info(p) {
    if (ispair (p))
        iterate (sprintf ("NElements: %s", length (p)));
} /* iterator place_info */

function place_menu(obj)
    vcBrowse.obj_menu;

place_class := vcBrowse.ClassCreate ("Geographical Place",
                                     children <- place_children,
                                     get_name <- place_name,
                                     edit <- place_edit,
                                     get_info <- place_info,
                                     drag_data <- place_drag,
                                     color <- vgColor.Green,
                                     get_menu <- place_menu);

function place_leaf_color(p)
    vgColor.Yellow;

place_leaf_class := vcBrowse.ClassCreate ("Geographical Leaf Place",
                                          get_name <- place_name,
                                          edit <- place_edit,
                                          drag_data <- place_drag,
                                          get_color <- place_leaf_color,
                                          menu <- vcBrowse.obj_menu);


struct GeogVT {
    vcRegionVT browse;
    vscmVT file;
};

function _Create(data, region) {
    if (data == nil)
        data = geography;
    browse := vcRegion.Create1 (vcRegion.Name (region),
                                vcBrowse.fac,
                                data,
                                append (list (class <- place_class,
                                              macro_on <- TRUE,
                                              bucket_size <- 10),
                                       vcRegion.PrefCopyChanged (region,
                                                              vcBrowse.fac)));
    file := fopen (vcRegion.Name (region) + ".out", "w");
    fprintf (file, "%s\n", vcBrowse.TypeGetClass (vscmVT));

    vcBrowse.CallOnSelect (browse, func(obj, class, event, flag) {
        fprintf (file, "Select class=%s, obj=%s, path=%s, flag=%s\n",
                 vcBrowse.ClassGetName (class),
                 vcBrowse.ObjGetName (browse, obj, class),
                 vcBrowse.ObjGetPath (browse, obj, class),
                 convert (vlStringVT, flag));
    });
        
    vcRegion.AddChild (region, browse);
    struct GeogVT {browse, file};
} /* function _Create */

function _Destroy(g)
    fclose (g.file);

obj_menu := nil;

function obj_tst(name, f) {
    function my_func(g, event) {
        foreach (obj, class, path; vcBrowse.SelObjLoop (g.browse)) {
            s := sprintf ("%s(%s) = %s\n", name, path,
                         f (g.browse, obj, class));
            vcWin.ClearStatus (vcEvent.Window (event));
            vcWin.AddStatus (vcEvent.Window (event), s);
            fputline (g.file, s);
        } /* foreach */
    } /* function my_func */
    my_cmd := vcWv.Cmd (fac, my_func, name);
    obj_menu = cons (list (name, my_cmd), obj_menu);
} /* function obj_tst */

obj_tst ("ObjIsPresent", vcBrowse.ObjIsPresent);
obj_tst ("ObjSelect", vcBrowse.ObjSelect);
obj_tst ("ObjUnSelect", vcBrowse.ObjUnSelect);
obj_tst ("ObjExpand", vcBrowse.ObjExpand);
obj_tst ("ObjCollapse", vcBrowse.ObjCollapse);
obj_tst ("ObjEdit", vcBrowse.ObjEdit);
obj_tst ("ObjMoveUp", vcBrowse.ObjMoveUp);
obj_tst ("ObjMoveDown", vcBrowse.ObjMoveDown);
obj_tst ("ObjMoveFirst", vcBrowse.ObjMoveFirst);
obj_tst ("ObjRemove", vcBrowse.ObjRemove);
obj_tst ("ObjGetColor", vcBrowse.ObjGetColor);
obj_tst ("ObjNewWindow", vcBrowse.ObjNewWindow);
obj_tst ("ObjViewCreate", vcBrowse.ObjViewCreate);
obj_tst ("ObjGetPath", vcBrowse.ObjGetPath);
obj_tst ("ObjIsSelected", vcBrowse.ObjIsSelected);

obj_tst ("ObjRecalcChildren-self",
         func(b, o, c) vcBrowse.ObjRecalcChildren (b, o, c, TRUE););
obj_tst ("ObjRecalcChildren-no self",
         func(b, o, c) vcBrowse.ObjRecalcChildren (b, o, c, FALSE););

function select_children(g, e) {
    foreach (obj, class, path; vcBrowse.SelObjLoop (g.browse)) {
        foreach (obj, class; vcBrowse.ObjLoop (g.browse, obj, class))
            vcBrowse.ObjSelect (g.browse, obj, class);
    } /* foreach */
} /* function select_children */
select_children_cmd := vcWv.Cmd (fac, select_children, "select_children");
obj_menu = cons (list ("Select Children", select_children_cmd), obj_menu);

function get_parent(g, event) {
    foreach (obj, class, path; vcBrowse.SelObjLoop (g.browse)) {
        <parent, parent_class> := vcBrowse.ObjGetParent (g.browse, obj, class);
        vcBrowse.ObjShow (g.browse, parent, parent_class);
        s := sprintf ("parent(%s) = %s",
                      vcBrowse.ObjGetName (g.browse, obj, class),
                      vcBrowse.ObjGetName (g.browse, parent, parent_class));
        vcWin.ClearStatus (vcEvent.Window (event));
        vcWin.AddStatus (vcEvent.Window (event), s);
        fputline (g.file, s + "\n");
    } /* foreach */
} /* function get_parent */
get_parent_cmd := vcWv.Cmd (fac, get_parent, "get_parent");
obj_menu = cons (list ("ObjGetParent", get_parent_cmd), obj_menu);

function insert(g, event, new_item) {
    foreach (obj, class, path; vcBrowse.SelObjLoop (g.browse))
        vcBrowse.ObjInsert (g.browse, obj, class, string2symbol (new_item),
                            class);
} /* function insert */
insert_cmd := vcWv.Cmd (fac, insert, "insert", "Insert New Item",
                        vcWv.Param ("New Item", vlStringVT, vcWv.Sticky, ""));
obj_menu = cons (list ("ObjInsert", insert_cmd), obj_menu);

function intentional_error(g, event) {
    catch (vlEx.FAILURE) {
        foreach (obj, class, path; vcBrowse.SelObjLoop (g.browse))
            "this is an intentional error" + `cannot_add_to_string;
    } /* catch */
    exception {
        s := vlErr.String (vlErr.CurrentError (), vlErrCntx.LogFile);
        vcWin.ClearStatus (vcEvent.Window (event));
        vcWin.AddStatus (vcEvent.Window (event), s);
        fputline (g.file, s + "\n");
    } /* exception */
} /* function intentional_error */
intentional_error_cmd := vcWv.Cmd (fac, intentional_error,
                                   "intentional_error");
obj_menu = cons (list ("Intentional Error", intentional_error_cmd), obj_menu);

function add_legend(g, event, text, color, icon)
    vcBrowse.AddLegend (g.browse, text, color, icon);
add_legend_cmd := vcWv.Cmd (fac, add_legend, "add_legend", "Add Legend",
                            vcWv.Param ("Text", vlStringVT, vcWv.Sticky, ""),
                            vcWv.Param ("Color", vgAllColorVT, vcWv.Sticky,
                                        vgColor.Red),
                            vcWv.Param ("Icon", vlStringVT, vcWv.Sticky,
                                        "tree"));

obj_menu = cons (vcWv.Pin (), reverse_destructive (obj_menu));



package pref_param(_Create, _Get, _Destroy) {
    function _Create(data, region) {
        p := vcRegion.Create ("prefs", vcPref.fac,
                              list (nil, browse.fac, browse.prefs),
                              all_prefs <- TRUE);
        vcRegion.AddChild (region, p);
        p;
    }

    function _Get(p, region, ts)
        vcRegion.Get (p, ts);

    /*
     * We have to provide a _Destroy event though we don't want one
     * because otherwise we automatically inherit the one from this
     * module, above, which tries to fclose a file, dereferencing
     * a structure we don't have!  This is an architectural flaw in
     * vcFac.CreateFromPackage.
     */
    function _Destroy(p)
        ;
} /* package pref_param */
pref_param_fac := vcFac.CreateFromPackage ("PrefParam", vscmVT, pref_param);

function open_with_prefs(g, event, pref_vals)
    vcWin.Create (vcRegion.Create1 ("Browser Demo", browse.fac, geography,
                                    pref_vals));
open_cmd := vcWv.Cmd (fac, open_with_prefs, "open",
                      "Open Browser with Preferences",
                      vcWv.Param ("", vscmVT, vcWv.Sticky, nil,
                                  pref_param_fac));

the_menu := vcWv.MakeMenu ("geog_menu",
                           vcWin.RedSquareEntryNoPlot,
                           list ("Object", vcMenu.Create ("Obj", obj_menu)),
                           list ("Open", open_cmd),
                           list ("Legend", add_legend_cmd));
vcFac.SetMenus (fac, the_menu);


vcFac.CopyPrefs (vcBrowse.fac, fac);
