/*
 *  $Revision: 1.4.5.1 $
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 *
 */


/*
 * Draw a 2D projection of a wireframe cube on the Yoda and alter it by
 * rotating the cube on the z-axis, changing the drawing color, and changing
 * the projection point.
 */

SIDE    := 500.0;
DELTA   := SIDE / 20;
XSHIFT  := 200;
YSHIFT  := 200;
X := 0;
Y := 1;
Z := 2;


function make_cube() {
 sintheta := 0;
 costheta := 1;
 cube := [[0,           0,      SIDE],
          [0,           SIDE,   SIDE],
          [SIDE,        SIDE,   SIDE],
          [SIDE,        0,      SIDE],
          [0,           0,      0],
          [0,           SIDE,   0],
          [SIDE,        SIDE,   0],
          [SIDE,        0,      0]];
 projection := [[0, 0],
                [0, 0],
                [0, 0],
                [0, 0],
                [0, 0],
                [0, 0],
                [0, 0],
                [0, 0]];
 projector := [SIDE / 2, SIDE / 2, 3 * SIDE];

 /* Canvas for drawing rthe cube */
 r := vcRegion.Create ("Cube", vcCanvas.fac, nil);
 size := vgSize.Create (500, 500, 200, 200, 1000, 1000);
 vcRegion.PrefSet (r, list (size <- size));
 size = nil;
                   
 vp := nil;

 function redraw() {
     vgU.Color (vp, vgColor.Black);
     area := vcRegion.Area (r);
     vgU.PixelToUserRect (vp, area);
     vgU.Box (vp, area);
     vgU.Color (vp, vgColor.Red);
     project_cube ();
     draw_cube ();
 } /* function redraw */

 horiz := vcRegion.Create ("horiz", vcScalebar.fac, projector[X],
                           min <- (-2 * SIDE),
                           max <- (2 * SIDE),
                           increment <- (SIDE / 5),
                           macro_on <- TRUE);
 vcScalebar.CallOnChange (horiz,
                          func(new) {projector[X] = convert (float, new) + 0;
                                     redraw ();});

 vert := vcRegion.Create ("vert", vcScalebar.fac, projector[Y],
                          min <- (-2 * SIDE),
                          max <- (2 * SIDE),
                          increment <- (SIDE / 5),
                          macro_on <- TRUE);
 vcScalebar.CallOnChange (vert,
                          func(new) {projector[Y] = convert (float, new) + 0;
                                     redraw ();});

 depth := vcRegion.Create ("depth", vcScalebar.fac, projector[Z],
                           min <- (2.5 * SIDE),
                           max <- (4 * SIDE),
                           increment <- (SIDE / 5),
                           macro_on <- TRUE);
 vcScalebar.CallOnChange (depth,
                          func(new) {projector[Z] = convert (float, new) + 0;
                                     redraw ();});

 angle := vcRegion.Create ("angle", vcScalebar.fac, 0,
                           min <- -180,
                           max <- 180,
                           increment <- 5,
                           macro_on <- TRUE);
 vcScalebar.CallOnChange (angle,
                          func(new) {a := convert (float, new);
                                     a *= 3.141592654/180;
                                     sintheta = sin (a);
                                     costheta = cos (a);
                                     redraw ();});

 function label(str) vcRegion.Create ("", vcLabel.fac, str);

 vcWin.Create (vcTable.Make ("Cube", 5, 1,
                             vcTable.Make (1, 2, label ("Horizontal"), horiz),
                             vcTable.Make (1, 2, label ("Vertical"), vert),
                             vcTable.Make (1, 2, label ("Depth"), depth),
                             vcTable.Make (1, 2, label ("Angle"), angle),
                             r));

 function project_point(cubepoint, projected_point) {
    x :=  (cubepoint[X] * costheta) + (cubepoint[Y] * sintheta);
    y :=  (cubepoint[Y] * costheta) - (cubepoint[X] * sintheta);
    z :=  cubepoint[Z];
    denom := projector[Z] - z;

    projected_point[X] = x + ((z * (x - projector[X])) / denom);
    projected_point[Y] = y + ((z * (y - projector[Y])) / denom);
 } /* function project_point */

 function ensure_vp() {
     if (vp == nil) {
         area := vgRect.Create (-SIDE * 2, -SIDE * 2, SIDE * 2, SIDE * 2);
         vp = vcRegion.Viewport (r, area, 0);
         vgU.StkSize (vp, 50, 50);
         vgU.Fillstyle (vp, vgFill.Solid);
         vgU.Rasterop (vp, vgRasterop.Replace);
     } /* if */
 } /* function ensure_vp */

 function cube_line(v1, v2)
     vgU.Line (vp,
               (projection[v1])[X],
               (projection[v1])[Y],
               (projection[v2])[X],
               (projection[v2])[Y]);

 function draw_cube() {
     cube_line (0, 1);
     cube_line (1, 2);
     cube_line (2, 3);
     cube_line (3, 0);
     cube_line (4, 5);
     cube_line (5, 6);
     cube_line (6, 7);
     cube_line (7, 4);
     cube_line (0, 4);
     cube_line (1, 5);
     cube_line (2, 6);
     cube_line (3, 7);
 } /* function draw_cube */

 function refresh(event) {

     ensure_vp ();

     /*
      * Erase the whole region with the background color
      */
     r := vcEvent.Rect (event);
     vgU.PixelToUserRect (vp, r);
     vgU.Color (vp, vgColor.Black);
     vgU.Box (vp, r);
     vgU.Color (vp, vgColor.Red);

     draw_cube ();
 } /* function refresh */

 function project_cube() {
     for (i := 0; i < 8; ++i)
         project_point (cube[i], projection[i]);
 } /* function project_cube */

 function rotate_z() {
     for (i := 0; i < 8; ++i) {
         new_x = ((cube[i])[X] * costheta) + ((cube[i])[Y] * sintheta);
         new_y = ((cube[i])[Y] * costheta) - ((cube[i])[X] * sintheta);
         (cube[i])[X] = new_x;
         (cube[i])[Y] = new_y;
     } /* for */
 } /* function rotate_z */

 project_cube ();
 vcCanvas.RefreshProc (r, refresh);
} /* function make_cube */

make_cube();

