/*
 *  $Revision: 1.24.5.1 $
 *
 *  Picklists in Core/Viewscript
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/*
 * Create a table to hold all the picklist examples.  We'll give this
 * Table some wide, visible borders.
 */
table := vcTable.Create ("Picklists", 3, 1);

hspace := vspace := 11;
vcTable.Row (table, -1, vcTable.Border.Adjustable, vspace, vspace, vspace,
             vgAlign.MiddleCenter);
vcTable.Col (table, -1, vcTable.Border.Invisible, hspace, hspace, hspace,
             vgAlign.MiddleCenter);

/*
 * Declare a variable to hold the window ID now so that the make_dump
 * function below can see it.  Later the variable will be set to the
 * created window's ID.
 */
declare w;

/*
 * Create a function that is called whenever a picklist changes state.
 * Note that the picklist ID itself is *not* passed to the function
 * dump_pickdata, but is passed to the "make_dump" function, and held
 * in the lexical closure that is returned from make_dump.  Thus make_dump
 * makes a function that "knows" about a particular picklist "p".
 *
 * The dump_pickdata function *is* passed the value associated with an
 * entry in the picklist, and a boolean flag indicating whether the
 * entry was turned on or off.
 *
 * Note that the one-line status window is not really enough to display
 * the information resulting from a pick on an exclusive pick-list, because
 * the function is called twice:  once to turn the old value (if there was
 * one) off, and another to turn the new value on.  On any reasonable machine,
 * the second call will overwrite the first so fast you won't see it at
 * all.
 */
function make_dump(str, p) {
    function dump_pickdata(val, on_off, index) {
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w,
                        sprintf ("[%s] %s: %s=%s, val=%s, index=%d IsSet=%s\n",
                                 str,
                                 vcRegion.Name (p), val, on_off,
                                 vcRegion.Get (p, vscmVT), index,
                                 vcPicklist.IsSet (p, index)));
    } /* function dump_pickdata */
    dump_pickdata;
}

/*
 * Create an exclusive picklist with 8 choices.
 */
p0 := vcRegion.Create ("Exclusive", vcPicklist.fac,
                       list (list (cons ("One", 1), cons ("Two", 2)),
                             list (cons ("Three", 3), cons ("Four", 4)),
                             list (cons ("Five", 5), cons ("Six", 6)),
                             list (cons ("Seven", 7))),
                       enhanced_notify  <- TRUE,
                       macro_on         <- TRUE);
vcTable.Add (table, p0);

/*
 * Set up a function that's called whenever the picklist changes state.
 */
vcPicklist.Notify (p0, make_dump ("Pick", p0));
vcPicklist.HighlightNotify (p0, make_dump ("Highlight", p0));

p1 := vcRegion.Create ("Non-Exclusive", vcPicklist.fac,
                       list (list (cons ("Yo Mama", 1), cons ("Foobar", 2)),
                             list (cons ("Choice", 3), cons ("AsdfFdsa", 4))),
                       exclusive        <- FALSE,
                       range            <- TRUE,
                       enhanced_notify  <- TRUE,
                       macro_on         <- TRUE);
vcTable.Add (table, vcRegion.Create ("p1", vcBorder.fac, p1));

vcPicklist.Notify (p1, make_dump ("Pick", p1));


p2 := vcRegion.Create ("Exclusive Scroll", vcPicklist.fac,
                       list (cons ("Yo Mama", 1),
                             cons ("Foobar", 2),
                             cons ("Choice", 3),
                             cons ("AsdfFdsa", 4),
                             cons ("Now", 5),
                             cons ("Is", "Is"),
                             cons ("The", "The"),
                             cons ("Time", "Time"),
                             cons ("For", "For"),
                             cons ("All", "All"),
                             cons ("Good", "Good"),
                             cons ("Men", "Men"),
                             cons ("To", "To"),
                             cons ("Come", "Come"),
                             cons ("To", "To"),
                             cons ("The", "The"),
                             cons ("Aid", "Aid"),
                             cons ("Of", "Of"),
                             cons ("Their", "Their"),
                             cons ("Party", "Party")),
                       enhanced_notify  <- TRUE,
                       range            <- TRUE,
                       macro_on         <- TRUE);

scr := vcPicklist.Scroll ("ScrExclusive", p2, 4, 6, 10);
vcRegion.PrefSet (scr, list (show_border <- TRUE));
vcPicklist.HighlightNotify (p2, make_dump ("Highlight", p2));
vcTable.Add (table, scr);

vcPicklist.Notify (p2, make_dump ("Pick", p2));

/*
 * Create a Window for the three picklists.
 */
w = vcWin.Create (table, 0);
