--
--
--  FULL VHDL state machine model for a vending machine
--
--
--  File name: vending.vhdl 
--
--


entity VENDING_MACHINE is

  port(NICKEL_IN, DIME_IN, QUARTER_IN, RESET, CLK: in BIT;
       NICKEL_OUT, DIME_OUT, DISPENSE: out BIT);
end;

architecture BEHAVIOR of VENDING_MACHINE is

-- Define enumerated state types

  type STATE_TYPE is (IDLE, FIVE, TEN, FIFTEEN,
                      TWENTY, TWENTY_FIVE, THIRTY,
                      OWE_DIME);

-- Create signals of the emnunerated type for states

  signal CURRENT_STATE, NEXT_STATE: STATE_TYPE;

begin

-- Define state machine process

  STATE:process(NICKEL_IN, DIME_IN, QUARTER_IN, RESET)

  begin

    -- Default assignments

    NEXT_STATE <= CURRENT_STATE;
    NICKEL_OUT <= '0';
    DIME_OUT <= '0';
    DISPENSE <= '0';

     -- Synchronous reset

    if(RESET = '0') then

      NEXT_STATE <= IDLE;

    else

      -- State transitions and output logic

      case CURRENT_STATE is

        when IDLE =>

          if(NICKEL_IN = '1') then
            NEXT_STATE <= FIVE;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= TEN;
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= TWENTY_FIVE;
          end if;

        when FIVE =>
          if(NICKEL_IN = '1') then
            NEXT_STATE <= TEN;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= FIFTEEN;
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= THIRTY;
          end if;

        when TEN =>
          if(NICKEL_IN = '1') then
            NEXT_STATE <= FIFTEEN;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= TWENTY;
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
          end if;

        when FIFTEEN =>
          if(NICKEL_IN = '1') then
            NEXT_STATE <= TWENTY;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= TWENTY_FIVE;
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
            NICKEL_OUT <= '1';
          end if;

        when TWENTY =>

          if(NICKEL_IN = '1') then
            NEXT_STATE <= TWENTY_FIVE;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= THIRTY;
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
            DIME_OUT <= '1';
          end if;

        when TWENTY_FIVE =>

          if(NICKEL_IN = '1') then
            NEXT_STATE <= THIRTY;
          elsif(DIME_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
            DIME_OUT <= '1';
            NICKEL_OUT <= '1';
          end if;

        when THIRTY =>

          if(NICKEL_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
          elsif(DIME_IN = '1') then
            NEXT_STATE <= IDLE;
            DISPENSE <= '1';
            NICKEL_OUT <= '1';
          elsif(QUARTER_IN = '1') then
            NEXT_STATE <= OWE_DIME;
            DISPENSE <= '1';
            DIME_OUT <= '1';
          end if;

        when OWE_DIME =>

          NEXT_STATE <= IDLE;
          DIME_OUT <= '1';

      end case;

    end if;

  end process;

 
  -- Define process to synchronize state value with clock.

  SYNC:process

  begin

    wait until CLK'event and CLK = '1';
    CURRENT_STATE <= NEXT_STATE;

  end process;

end BEHAVIOR;
